#
#    Project: The EDNA Kernel
#             http://www.edna-site.org
#
#    File: "$Id: EDActionExecuteSystemCommand.py 664 2009-05-06 13:05:34Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License  along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#


from EDImportLib import EDAction
from EDImportLib import EDVerbose
from EDImportLib import EDManagerProcess
from EDImportLib import EDString

from EDImportSystem import PyOs


class EDActionExecuteSystemCommand( EDAction ):
    """
    This class executes a process using the Python os.sytem command.
    It is a temporary solution for the "==>[ERROR]: OSError: [Errno 10] No child processes"
    problem with ALActionProcess (Python bug in subprocess, see bug #61).
    """
    
    
    def __init__( self, _edStringCommand ):
        """
        Initializes the class: the argument _edStringCommand should be
        the complete command line, i.e. path to executable + command line arguments
        """
        EDAction.__init__( self )
        self.m_edStringCommand = _edStringCommand
        self.m_edStringExecutionStatus = None
        self.m_edManagerProcess = EDManagerProcess()
        
        
    def process( self, _edPlugin ):
        """
        Executes the command line using the standard Python os.system command.
        The result of the os.system command is converted into an EDString.
        """
        EDVerbose.DEBUG( "*** EDActionExecuteSystemCommand.process" )
        self.synchronizeOn()
        pyExecutionStatus = PyOs.system( self.m_edStringCommand )
        if ( pyExecutionStatus is None ):
            # If the status returned is None, return an empty string
            self.m_edStringExecutionStatus = EDString()
        else:
            self.m_edStringExecutionStatus = EDString( pyExecutionStatus )
        self.synchronizeOff()
        
        
    def getExecutionStatus( self ):
        """
        Returns the string containing the execution status.
        """
        return self.m_edStringExecutionStatus


    def abort( self, _edObject = None ):
        """
        This method can be used to abort the process, for example when
        the execution time has exceeded the maximum allowed execution time.
        """
        EDVerbose.DEBUG( "*** EDActionExecuteSystemCommand.abort" )
        self.synchronizeOn()
        self.m_edManagerProcess.killProcess( self.m_edStringCommand )
        self.m_bIsAbort = True
        self.synchronizeOff()
        