#
#    Project: The EDNA Kernel
#             http://www.edna-site.org
#
#    File: "$Id: EDUtilsImage.py 664 2009-05-06 13:05:34Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License  along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#


from EDImportLib import EDFile
from EDImportLib import EDString
from EDImportLib import EDList
from EDImportLib import EDVerbose
from EDImportLib import EDDictionary

from EDImportSystem import PyRe

class EDUtilsImage:
    

    def __compileAndMatchRegexpTemplate( _edStringPathToImage ):
        """
        """
        edListResult = None
        pyStrBaseImageName = EDFile.getFileName( _edStringPathToImage )
        pyRegexp           = PyRe.compile(  r'(.*)([^0^1^2^3^4^5^6^7^8^9])([0-9]*)\.(.*)' )
        pyMatch            = pyRegexp.match( pyStrBaseImageName )
        if ( pyMatch != None ):   
            edListResult = EDList( [ EDString( pyMatch.group(0) ), 
                                      EDString( pyMatch.group(1) ), 
                                      EDString( pyMatch.group(2) ), 
                                      EDString( pyMatch.group(3) ), 
                                      EDString( pyMatch.group(4) ),
                                       ] )
        return edListResult
    __compileAndMatchRegexpTemplate = staticmethod( __compileAndMatchRegexpTemplate )


    def getImageNumber( _edStringPathToImage ):
        """
        """
        iImageNumber = None
        edListResult = EDUtilsImage.__compileAndMatchRegexpTemplate( _edStringPathToImage )
        if ( edListResult != None ):
            iImageNumber = edListResult[3].atoi()
        return iImageNumber
    getImageNumber = staticmethod( getImageNumber )

        
    def getTemplate( _edStringPathToImage, _edStringSymbol = "#" ):
        """
        """
        edStringTemplate = None
        edListResult = EDUtilsImage.__compileAndMatchRegexpTemplate( _edStringPathToImage )
        if ( edListResult != None ):
            edStringPrefix      = edListResult[1]
            edStringSeparator   = edListResult[2]
            edStringImageNumber = edListResult[3]
            edStringSuffix      = edListResult[4]
            edStringHashes = ""
            for i in range( len( edStringImageNumber ) ):
                edStringHashes += _edStringSymbol
            edStringTemplate = EDString( edStringPrefix + edStringSeparator + edStringHashes + "." + edStringSuffix )
        return edStringTemplate
    getTemplate = staticmethod( getTemplate )


    def getPrefix( _edStringPathToImage ):
        """
        """
        edStringPrefix = None
        edListResult = EDUtilsImage.__compileAndMatchRegexpTemplate( _edStringPathToImage )
        if ( edListResult != None ):
            edStringPrefix      = EDString( edListResult[1] )
        return edStringPrefix
    getPrefix = staticmethod( getPrefix )


    def getSuffix( _edStringPathToImage ):
        """
        """
        edStringSuffix = None
        edListResult = EDUtilsImage.__compileAndMatchRegexpTemplate( _edStringPathToImage )
        if ( edListResult != None ):
            edStringSuffix      = EDString( edListResult[4] )
        return edStringSuffix
    getSuffix = staticmethod( getSuffix )


