#
#    Project: The EDNA Kernel
#             http://www.edna-site.org
#
#    File: "$Id: EDUtilsXML.py 747 2009-05-29 10:40:22Z levik $"
#
#    Copyright (C) 2008-2009 Diamond Light Source
#                            Chilton, Didcot, UK
#
#    Principal author:       Karl Levik (karl.levik@diamond.ac.uk)
#
#    Contributing authors:   Marie-Francoise Incardona (incardon@esrf.fr)
#                            Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License  along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#


from EDImportLib import EDString
from EDImportLib import EDObject

from EDImportSystem import PyStrIO
from EDImportSystem import PyRe
from EDImportSystem import PyMinidom
     
     
class EDUtilsXML( EDObject ):
    
    def dnaMarshal( _xsDataObject = None):
        """
        Returns a DNA compatible XML representation of an object derived from XSData.
        DNA compatible XML has no whitespace or newlines etc. between elements and single-quotes must be escaped.
        """
        pyStrIO = PyStrIO.StringIO()
        pyStrIO.write( '<?xml version="1.0" encoding="ISO-8859-1"?>' )
        
        pyRegexp = PyRe.compile( 'XSData(.*)' )
        pyMatchObject = pyRegexp.match( _xsDataObject.__class__.__name__ )
        pyStrDNAClassName = pyMatchObject.group( 1 )
        
        _xsDataObject.export( pyStrIO, 0, name_= pyStrDNAClassName )
        pyStrXML = pyStrIO.getvalue()
        pyStrIO.close()
        
        # Remove whitespace and <value> & </value> taqs, escape single-quotes etc
        pyMinidom = PyMinidom.parseString( pyStrXML )
        pyRegexp = PyRe.compile( '\<value\>(.*)\</value\>' )
        pyStrXML = ''
        for line in pyMinidom.toxml().splitlines():
            m = pyRegexp.match( line.strip() )
            if m != None:
                pyStrValue = m.group( 1 )
                pyStrValue = pyStrValue.replace( '\'', '\\\'')
                pyStrXML += pyStrValue
            else:
                pyStrXML += line.strip()
        return EDString( pyStrXML )
    dnaMarshal = staticmethod( dnaMarshal )
