#
#    Project: The EDNA Kernel
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCasePluginUnit.py 664 2009-05-06 13:05:34Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License  along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#


from EDImportLib                         import EDVerbose
from EDImportLib                         import EDString
from EDImportLib                         import EDDiskExplorer

from EDConfiguration                     import EDConfiguration
from EDTestCasePlugin                    import EDTestCasePlugin
from EDUtilsPath                         import EDUtilsPath 

from EDFactoryPlugin import EDFactoryPlugin


class EDTestCasePluginUnit( EDTestCasePlugin ):
    """
    This is the main class for all Plugin Unit tests
    """
    
    def __init__( self, _edStringPluginName, _edStrPluginDir = None, _edStringTestName = None ):
        """
        Initialize the Plugin Unit test
        """
        EDTestCasePlugin.__init__( self, _edStringPluginName, _edStrPluginDir, _edStringTestName )       


    def preProcess( self ):
        """
        Creates the main plugin instance
        """
    	# Create the plugin
        self.m_edPlugin = self.createPlugin()
            
            
    def getPlugin( self ):
        """
        Returns the main plugin instance
        """
        return self.m_edPlugin
    
    
    def createPlugin( self ):
        """
        Creates a plugin instance
        """
        edFactoryPlugin = EDFactoryPlugin()
        edPlugin = edFactoryPlugin.loadPlugin( self.getPluginName() )
        if ( edPlugin == None ):
            EDVerbose.error("*** EDTestCaseEDPlugin.__init__: Could not create plugin: " + self.getPluginName() )
        else:
            edPlugin.setExecutionContext( "EDTestCaseEDPlugin" )
            
        return edPlugin     
        
        
    def getPluginConfiguration( self, _edStringConfigurationFileName, _edPlugin = None ):
        """
        Returns the plugin configuration from a configuration file
        """
        edPlugin = None
        # By default, this method returns the configuration of the plugin attribute member
        # If a plugin is given in parameter, it returns the configuration of the parameter plugin
        if( _edPlugin == None ):
            edPlugin = self.m_edPlugin
        else:
            edPlugin = _edPlugin
        
        xsPluginItem = None
        
        edConfiguration = EDConfiguration( _edStringConfigurationFileName )
        if( edConfiguration != None ):
            edConfiguration.load()
            xsPluginItem = edConfiguration.getPluginItem( self.getPluginName() )
            if( xsPluginItem == None ):
                EDVerbose.warning( "*** EDTestCaseEDPlugin.getPluginConfiguration: Could not get XSPluginItem for: " + self.getPluginName() )
            
        else:
            EDVerbose.warning( "*** EDTestCaseEDPlugin.getPluginConfiguration: Could not load Configuration: " + _edStringConfigurationFileName )
        
        return xsPluginItem
    
    
    def cleanUp( self, _edPlugin ):
        """
        Cleans up some empty directories 
        (mainly working directories that were created automatically, but not filled as Unit tests do not execute the plugin)
        """
        workingDirectory = _edPlugin.getWorkingDirectory()
        if( workingDirectory is not None ):
            fileList = EDDiskExplorer.getFileList( workingDirectory, "*" )
            if( len( fileList ) == 0 ):
                EDVerbose.DEBUG( "Deleting " + workingDirectory + " ..." )
                EDDiskExplorer.deleteFolder( workingDirectory )

                
