#
#    Project: The EDNA Kernel
#             http://www.edna-site.org
#
#    File: "$Id: EDTestSuite.py 719 2009-05-15 15:18:08Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License  along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#

#
# This file contains methods modified from the original ALTestSuite.
#


from ALImportPython    import  PySys
from ALDefinition      import  *
from ALTime            import  ALTime
from ALVerbose         import  ALVerbose
from ALSystem          import  ALSystem
from ALString          import  ALString
from ALAssert          import  ALAssert
from ALTrace           import  ALTrace
from ALDebug           import  ALDebug
from ALLogFile         import  ALLogFile
from ALTest            import  ALTest
from ALInformationTest import  ALInformationTest
from ALTestSuite       import  ALTestSuite 

from EDImportLib import EDVerbose
from EDImportLib import EDString
from EDImportLib import EDTime
from EDImportLib import EDList

from ALManagerTest   import ALManagerTest   as EDManagerTest

from EDUtilsPath     import EDUtilsPath
from EDUtilsTest     import EDUtilsTest

class EDTestSuite( ALTestSuite ):


    __EDFactoryPluginTest = None 

    def __init__( self, _edStringTestSuiteName=None ):
        ALTestSuite.__init__( self, _edStringTestSuiteName )
        if ( _edStringTestSuiteName is None ):
            self.m_oalStringTestName = EDString( self.getClassName() )
        self.m_edListSuite = EDList()
        self.m_bTopLevel = True
        self.m_oTestSuiteNumberTestSuite   = 0
        self.m_oTestSuiteNumberTestTotal   = 0
        self.m_oTestSuiteNumberTest = 0
        self.m_oTestSuiteNumberTestSuccess = 0
        self.m_oTestSuiteNumberTestFailure = 0
        self.m_oTestSuiteNumberTestMethods = 0
        self.m_fDuration = 0.0



    def addTestCaseFromName( self, _edStringTestCaseName ):
        edTestCase = EDUtilsTest.getFactoryPluginTest().loadPlugin( _edStringTestCaseName )
        if ( edTestCase == None ):
            EDVerbose.error("*** EDTestSuite.addTestCaseFromName: Could not create test Case: " + _edStringTestCaseName)
        else:
            self.addTestMethod( edTestCase.execute )


    def addTestSuiteFromName( self, _edStringTestSuiteName ):
        edTestSuite = EDUtilsTest.getFactoryPluginTest().loadPlugin( _edStringTestSuiteName )
        if ( edTestSuite == None ):
            EDVerbose.error("*** EDTestSuitePluginUnit.addTestSuiteFromName: Could not create test suite: " + _edStringTestSuiteName)
        else:
            self.addTestSuite( edTestSuite )


    def addTestSuite( self, _edTestSuite ):
        self.m_edListSuite.append( _edTestSuite )


    def processKernel( self ):
        EDVerbose.DEBUG( "Execution: EDTestSuite.processKernel()" )
        edTime = EDTime()
        edTime.initTime()
        EDManagerTest.create()
        self.m_oNumberTestTotal   = self.m_oalListTest.getNumberObjects( )
        self.m_oNumberTestSuccess = 0
        self.m_oNumberTestFailure = 0
        self.m_oNumberTest        = 0
        for oTestMethod in self.m_oalListTest:
            self.m_oNumberTest += 1
            oalInformationTest = ALInformationTest( self.patchMethodName( oTestMethod ), self.m_oNumberTest )
            EDVerbose.screen( " " )
            EDVerbose.unitTest( "###################################################################" )
            EDVerbose.unitTest( "TEST [ " + EDString( oalInformationTest.getTestNumber() ) + " ] : [ " + oalInformationTest.getTestName() + " ]" )
            EDVerbose.unitTest( " ")
            EDManagerTest.addInformationTest( oalInformationTest )
            try:
                try:
                    oTestMethod()
                    oalInformationTest.setTerminate()
                    oalInformationTest.setSuccess()
                    EDVerbose.unitTest( "TEST [" + EDString(  oalInformationTest.getTestNumber() ) + " ] : [SUCCESS]" )
                except Exception, opyException:
                    oalInformationTest.setTerminate()
                    oalInformationTest.setFailure()
                    oalInformationTest.setException( EDString(opyException) )
                    EDVerbose.unitTest( "Test Failure = [ " + oalInformationTest.getException() + " ] Raised"  )
                    EDVerbose.unitTest( "TEST [ " + EDString(  oalInformationTest.getTestNumber() ) + " ] : [ " + oalInformationTest.getTestName() + " ]: [FAILURE]" )
                    self.raiseControl()

            finally:
                EDVerbose.unitTest( "###################################################################" )
                ofTime = oalInformationTest.getRunTime()
                EDVerbose.unitTest( "Test - " + EDString( oalInformationTest.getTestNumber() ) + " : Module Runtime: %.1f [s]" % ofTime )
                EDVerbose.screen( " " )
        self.m_oNumberTestSuccess = EDManagerTest.getNumberTestSuccess()
        self.m_oNumberTestFailure = EDManagerTest.getNumberTestFailure()
        
        self.m_oTestSuiteNumberTestTotal   = self.m_oNumberTestTotal
        self.m_oTestSuiteNumberTestSuccess = self.m_oNumberTestSuccess
        self.m_oTestSuiteNumberTestFailure = self.m_oNumberTestFailure
        self.m_oTestSuiteNumberTest        = self.m_oNumberTest
        self.m_oTestSuiteNumberTestSuite   = 0
        self.m_oTestSuiteNumberTestMethods = EDManagerTest.getNumberTestMethod()
        for edTestSuite in self.m_edListSuite:
            edTestSuite.setTopLevel( False )
            edTestSuite.execute()
            self.m_oTestSuiteNumberTestTotal   += edTestSuite.m_oTestSuiteNumberTestTotal
            self.m_oTestSuiteNumberTestSuccess += edTestSuite.m_oTestSuiteNumberTestSuccess
            self.m_oTestSuiteNumberTestFailure += edTestSuite.m_oTestSuiteNumberTestFailure
            self.m_oTestSuiteNumberTest        += edTestSuite.m_oTestSuiteNumberTest
            self.m_oTestSuiteNumberTestMethods += edTestSuite.m_oTestSuiteNumberTestMethods
            self.m_oTestSuiteNumberTestSuite   += 1
        self.m_fDuration = edTime.getDuration()


    def outputReportTestSuite(self):
        EDVerbose.screen(   " " )
        EDVerbose.screen(   " " )
        EDManagerTest.create()
        EDVerbose.unitTest( "###################################################################" )
        EDVerbose.unitTest( self.getTestName() + " - END Status [OK]" )
        EDVerbose.unitTest( self.getTestName() + " - Test Report:" )
        if ( self.m_oTestSuiteNumberTestSuite > 0 ):
            EDVerbose.unitTest( "                          TestSuites: " + EDString( self.m_oTestSuiteNumberTestSuite ) )
        EDVerbose.unitTest( "                     Total TestCases: " + EDString( self.m_oTestSuiteNumberTestTotal ) )
        EDVerbose.unitTest( "                  Executed TestCases: " + EDString( self.m_oTestSuiteNumberTest ) )
        EDVerbose.unitTest( "                 TestCases [SUCCESS]: " + EDString( self.m_oTestSuiteNumberTestSuccess ) )
        EDVerbose.unitTest( "                    TestCases [FAIL]: " + EDString( self.m_oTestSuiteNumberTestFailure ) )
        EDVerbose.unitTest( "                 [Total TestMethods]: " + EDString( self.m_oTestSuiteNumberTestMethods ) )
        EDVerbose.unitTest( "                             Runtime: %.1f [s]" % self.m_fDuration )
        EDVerbose.unitTest( "                                 Run: " + ALTime.getTimeStringPSP( self.m_fDuration )  )

    
    def outputReportCumulate(self):
        EDVerbose.outputActionLog()
        EDManagerTest.create()
        EDManagerTest.outputScreen()
        EDVerbose.screen(   " " )
        EDVerbose.screen(   " " )
        EDVerbose.unitTest( "###################################################################" )
        EDVerbose.unitTest( self.getTestName() + " summary report:" )
        if ( self.m_oTestSuiteNumberTestSuite > 0 ):
            EDVerbose.unitTest( "                          TestSuites: " + EDString( self.m_oTestSuiteNumberTestSuite ) )
        EDVerbose.unitTest( "                     Total TestCases: " + EDString( EDManagerTest.getNumberTestCumulative() ) )
        EDVerbose.unitTest( "           Total TestCases [SUCCESS]: " + EDString( EDManagerTest.getNumberTestSuccessCumulative() ) )
        EDVerbose.unitTest( "              Total TestCases [FAIL]: " + EDString( EDManagerTest.getNumberTestFailureCumulative() ) )
        EDVerbose.unitTest( "                 [Total TestMethods]: " + EDString( EDManagerTest.getNumberTestMethodCumulative() ) )
        EDVerbose.unitTest( "                             Runtime: %.1f [s]" % self.m_fDuration )
        EDVerbose.unitTest( "                                 Run: " + ALTime.getTimeStringPSP( self.m_fDuration )  )
        EDVerbose.screen(   " " )
        EDVerbose.screen(   " " )


    def execute( self ):
        EDVerbose.screen(   "" )
        EDVerbose.screen(   "" )
        EDVerbose.unitTest( "STARTING::" + self.getTestName() )
        EDVerbose.unitTest( "###################################################################" )
        try:
            try:
                self.executeKernel()
            except:
                self.raiseControl()
        finally:
            if ( self.isTopLevel() ):
                self.outputReportCumulate()
            else:
                self.outputReportTestSuite()
            #self.outputProfile()
            self.__patchEclipseConsolePipe()


    def __patchEclipseConsolePipe( self ):
        """
        Fix to Eclipse/Python console output not close the pipe quickly
        """
        EDTime.wait(0.1)


    def isTopLevel( self ):
        return self.m_bTopLevel
    
    
    def setTopLevel( self, _bLevel ):
        self.m_bTopLevel = _bLevel

