#
#    Project: The EDNA Prototype
#             http://www.edna-site.org
#
#    File: "$Id: EDUtilsTest.py 664 2009-05-06 13:05:34Z svensson $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImportLib                         import EDObject
from EDImportLib                         import EDString
from EDImportLib                         import EDVerbose
from EDUtilsPath                         import EDUtilsPath
from EDFactoryPluginTest                 import EDFactoryPluginTest


class EDUtilsTest (EDObject):
    """
    """
    m_edFactoryPluginTest = None
    
    def getFactoryPluginTest():
        if ( EDUtilsTest.m_edFactoryPluginTest is None ):
            EDUtilsTest.m_edFactoryPluginTest = EDFactoryPluginTest()
        return EDUtilsTest.m_edFactoryPluginTest
    getFactoryPluginTest = staticmethod( getFactoryPluginTest )

    
    def readAndParseFile( _edStringFileName ):
        """
        Reads a file and parses potential existing environment variables such as:
        - EDNA_TESTS_DATA_HOME
        Returns the content of this file as a string
        """
        edString = None
        try:
            edString = EDUtilsTest.readFile ( _edStringFileName )
        except:
            raise        
        
        if( edString.contains( "${EDNA_TESTS_DATA_HOME}" ) ):
            edString = EDString( edString ).replace( "${EDNA_TESTS_DATA_HOME}" , EDUtilsTest.getTestsDataHome() )

        return edString
    readAndParseFile = staticmethod( readAndParseFile )
    
    
    def readFileAndParseVariable( _edStringFileName, _edStringKey, _edStringValue ):
        """
        Reads a file and parses potential existing environment variables such as:
        Returns the content of this file as a string
        """
        edString = None
        try:
            edString = EDUtilsTest.readFile ( _edStringFileName )
        except:
            raise        
        
        if( edString.contains( _edStringKey ) ):
            edString = EDString( edString ).replace( _edStringKey , _edStringValue )

        return edString
    readFileAndParseVariable = staticmethod( readFileAndParseVariable )
    
    
    def readFileAndParseVariables( _edStringFileName, _edDictionary ):
        """
        Returns the content of this file as a string
        """
        edString = None
        try:
            edString = EDUtilsTest.readFile ( _edStringFileName )
        except:
            raise        
        
        for key in _edDictionary.keys():
            if( edString.contains( key ) ):
                edString = EDString( edString ).replace( key , _edDictionary.getValue( key ) )

        return edString
    readFileAndParseVariables = staticmethod( readFileAndParseVariables )
    
    
    def readFile( _edStringFileName ):
        """
        Reads a file
        Returns its content as a string
        """
        edString = None
        try:
            edString = EDString( open( _edStringFileName, "r" ).read() )
        except:
            EDVerbose.error( "*** EDUtilsTest.readFile: Problem when reading " + _edStringFileName )
            raise        
        return edString
    readFile = staticmethod( readFile )
    
    
    def writeFile( _edStringFileName, _edStringContent ):
        """
        Writes a string into a file
        """
        try:
            open( _edStringFileName, "w" ).write( _edStringContent )
        except:
            EDVerbose.error( "*** EDUtilsTest.writeFile: Problem when writing " + _edStringFileName )
            raise        
    writeFile = staticmethod( writeFile )    
        

    
    def getTestsHome():
        """
        Returns the tests home directory path <EDNA_HOME>/tests
        """
        edStringTestsHome = EDUtilsPath.mergePath( EDUtilsPath.getEdnaHome(), "tests" )
        return edStringTestsHome
    getTestsHome = staticmethod( getTestsHome )
    
    
    def getTestsDataHome():
        """
        Returns the tests data directory path <EDNA_HOME>/tests/data
        """
        edStringTestsDataHome = EDUtilsPath.mergePath( EDUtilsTest.getTestsHome(), "data" )
        return edStringTestsDataHome
    getTestsDataHome = staticmethod( getTestsDataHome )
    

    def getTestsDataImagesHome():
        """
        Returns the tests data image directory path <EDNA_HOME>/prototype/tests/data/images
        """
        edStringTestsDataImagesHome = EDUtilsPath.mergePath( EDUtilsTest.getTestsDataHome(), "images" )
        return edStringTestsDataImagesHome
    getTestsDataImagesHome = staticmethod( getTestsDataImagesHome )

    
    def getPluginTestDataDirectory( _edStringPluginTestName ):
        """
        Given a test case name, returns the corresponding test data directory.
        """
        edStringModuleLocation = EDUtilsTest.getFactoryPluginTest().getModuleLocation( _edStringPluginTestName )
        edStringPluginTestDataDirectory = EDUtilsPath.appendListOfPaths( edStringModuleLocation, [ "..", "data" ] )
        return edStringPluginTestDataDirectory
    getPluginTestDataDirectory = staticmethod( getPluginTestDataDirectory )

        
    def getConfigurationHome( _edStringPluginTestName ):
        """
        Returns the configuration directory path for a given test module
        """
        edStringModuleLocation = EDUtilsTest.getFactoryPluginTest().getModuleLocation( _edStringPluginTestName )
        edStringConfigurationHome = EDUtilsPath.appendListOfPaths( edStringModuleLocation, [ "..", "..", "..", "..", "conf" ] )
        return edStringConfigurationHome
    getConfigurationHome = staticmethod( getConfigurationHome )