#
#    Project: The EDNA Kernel
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCaseEDPluginExecProcessScript.py 671 2009-05-06 14:24:21Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License  along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#



from EDImportLib         import EDCompiler
from EDImportLib         import EDVerbose
from EDImportLib         import EDList
from EDImportLib         import EDStreamFile
from EDImportLib         import EDDiskExplorer

from EDAssert                  import EDAssert
from EDPluginExecProcessScript import EDPluginExecProcessScript
from EDTestCase                import EDTestCase


class EDTestCaseEDPluginExecProcessScript( EDTestCase ):
    """
    """
    def __init__( self, _edStringTestName = None):
        """
        """
        EDTestCase.__init__( self, _edStringTestName )
    
    
    def testGenerateScript( self ):
        """
        """
        edListCommandPreExecution = EDList( ['PRE-EXECUTION1', 'PRE-EXECUTION2 PATATE'] )
        edListCommandExecution = EDList( ['COMMAND1', 'COMMAND2 BANANA'] )
        edListCommandPostExecution = EDList( ['POST-EXECUTION1', 'POST-EXECUTION2 FRAISE'] )
        edPluginExecProcessScript = EDPluginExecProcessScript()
        edPluginExecProcessScript.setScriptShell( "/bin/bash" )
        edPluginExecProcessScript.setScriptBaseName( "TestCaseGenerateScript" )
        edPluginExecProcessScript.setListCommandPreExecution( edListCommandPreExecution )
        edPluginExecProcessScript.setListCommandExecution( edListCommandExecution )
        edPluginExecProcessScript.setListCommandPostExecution( edListCommandPostExecution )
        edPluginExecProcessScript.setRequireCCP4( True )
        edPluginExecProcessScript.setSetupCCP4( "/usr/local/xtal/ccp4-6.0.2/include/ccp4.setup-bash" )
        edPluginExecProcessScript.setScriptExecutable( "cat" )
        edPluginExecProcessScript.configure()
        edStringScript = edPluginExecProcessScript.prepareScript()
        edStringScriptReference = """#!/bin/bash
cd %s
. /usr/local/xtal/ccp4-6.0.2/include/ccp4.setup-bash
PRE-EXECUTION1
PRE-EXECUTION2 PATATE
cat  > TestCaseGenerateScript.log 2> TestCaseGenerateScript.err << EOF-EDPluginExecProcessScript
COMMAND1
COMMAND2 BANANA
EOF-EDPluginExecProcessScript
POST-EXECUTION1
POST-EXECUTION2 FRAISE
""" % ( edPluginExecProcessScript.getWorkingDirectory() )
        EDAssert.equal( edStringScriptReference, edStringScript )
        
        
        
    def testGenerateExecutableScript( self ):
        edListCommands = EDList( ['COMMAND1', 'COMMAND2 BANANA'] )
        edPluginExecProcessScript = EDPluginExecProcessScript()
        edPluginExecProcessScript.setScriptShell( "/bin/bash" )
        edPluginExecProcessScript.setScriptBaseName( "TestCaseGenerateScript" )
        edPluginExecProcessScript.setListCommandExecution( edListCommands )
        edPluginExecProcessScript.setRequireCCP4( True )
        edPluginExecProcessScript.setSetupCCP4( "/usr/local/xtal/ccp4-6.0.2/include/ccp4.setup-bash" )
        edPluginExecProcessScript.setScriptExecutable( "cat" )
        edPluginExecProcessScript.configure()
        edStringScript = edPluginExecProcessScript.prepareScript()
        edPluginExecProcessScript.writeExecutableScript( edStringScript )
        edStringScriptFileName = edPluginExecProcessScript.getScriptFileName()
        edStringScriptFromFile = edPluginExecProcessScript.readProcessFile( edStringScriptFileName )
        edStringScriptReference = """#!/bin/bash
cd %s
. /usr/local/xtal/ccp4-6.0.2/include/ccp4.setup-bash
cat  > TestCaseGenerateScript.log 2> TestCaseGenerateScript.err << EOF-EDPluginExecProcessScript
COMMAND1
COMMAND2 BANANA
EOF-EDPluginExecProcessScript
""" % ( edPluginExecProcessScript.getWorkingDirectory() )
        EDAssert.equal( edStringScriptFromFile, edStringScriptReference )


    def testExecuteScript( self ):
        edListCommandExecution = EDList( ['COMMAND1', 'COMMAND2 BANANA'] )
        edStringLogReference = """COMMAND1
COMMAND2 BANANA
"""
        edPluginExecProcessScript = EDPluginExecProcessScript()
        edPluginExecProcessScript.setListCommandExecution( edListCommandExecution )
        edPluginExecProcessScript.setRequireCCP4( False )
        edPluginExecProcessScript.setScriptExecutable( "cat" )
        from XSDataCommon import XSData
        edPluginExecProcessScript.setXSDataInputClass( XSData )
        edPluginExecProcessScript.execute()      
        edPluginExecProcessScript.synchronize()
        edStringScriptLogFileName = edPluginExecProcessScript.getScriptLogFileName()
        edStringLogFromFile = edPluginExecProcessScript.readProcessLogFile()
        EDAssert.equal( edStringLogFromFile, edStringLogReference )

        
    def process( self ):
        """
        """
        self.addTestMethod( self.testGenerateScript )
        self.addTestMethod( self.testGenerateExecutableScript )
        self.addTestMethod( self.testExecuteScript )

##############################################################################


if __name__ == '__main__':

    # JIT compiler accelerator
    EDCompiler.accelerator()
    
    edTestCaseEDPluginExecProcessScript = EDTestCaseEDPluginExecProcessScript( "TestCase EDPluginExecProcessScript" )
    edTestCaseEDPluginExecProcessScript.execute()
