#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################



from ALDefinition    import *
from ALTime          import ALTime
from ALAction        import ALAction
from ALSemaphore     import ALSemaphore


class ALActionController( ALAction ):
     """
     Parent class for all ALActionController objects.

     By default: a cluster of 64 parallel task are executed at the same time
     """
     def __init__( self, _oiClusterSize = 64, _oBoolPriority = False ):
         """
         Constructor for ALActionController.

         """
         ALAction.__init__( self, _oBoolPriority )
         self.m_oSemaphoreController  = ALSemaphore()
         self.m_oiClusterSize         = _oiClusterSize
         self.m_oiActiveActions       = 0
         self.m_oiSuccessActions      = 0
         self.m_oiFailureActions      = 0
         self.m_oiTotalSuccessActions = 0
         self.m_oiTotalFailureActions = 0



     def synchronizeControllerOn( self ):
         self.m_oSemaphoreController.acquire()


     def synchronizeControllerOff( self ):
         self.m_oSemaphoreController.release()



     def getClusterSize( self ):
         self.synchronizeControllerOn()
         oiValue = self.m_oiClusterSize
         self.synchronizeControllerOff()
         return int( oiValue )


     def setClusterSize( self, _oiClusterSize ):
         self.synchronizeControllerOn()
         self.m_oiClusterSize = int(_oiClusterSize )
         self.synchronizeControllerOff()


     def setNumberActiveActions( self, _oiNumberActions ):
         self.synchronizeControllerOn()
         self.m_oiActiveActions = int( _oiNumberActions )
         self.synchronizeControllerOff()


     def getNumberActiveActions( self ):
         self.synchronizeControllerOn()
         oiValue = self.m_oiActiveActions
         self.synchronizeControllerOff()
         return int( oiValue )


     def controlActiveActions( self ):
         self.synchronizeControllerOn()
         self.m_oiActiveActions += 1
         self.synchronizeControllerOff()


     def isClusterFull( self ):
         obStatus = False
         if ( int(self.getNumberActiveActions())>=int(self.getClusterSize()) ):
             obStatus = True
         return obStatus



     def __resetActiveActions(self):
         self.synchronizeControllerOn()
         self.m_oiTotalSuccessActions += self.m_oiSuccessActions
         self.m_oiTotalFailureActions += self.m_oiFailureActions
         self.m_oiActiveActions       = 0
         self.m_oiSuccessActions      = 0
         self.m_oiFailureActions      = 0
         self.synchronizeControllerOff()



     def getTotalSuccessActions(self):
         self.synchronizeControllerOn()
         oiValue = self.m_oiTotalSuccessActions
         self.synchronizeControllerOff()
         return int(oiValue)


     def getTotalFailureActions(self):
         self.synchronizeControllerOn()
         oiValue = self.m_oiTotalFailureActions
         self.synchronizeControllerOff()
         return int(oiValue)


     def getTotalActions(self):
         self.synchronizeControllerOn()
         oiValue = self.m_oiTotalSuccessActions + self.m_oiTotalFailureActions
         self.synchronizeControllerOff()
         return int(oiValue)



     def controlActionSuccessFailure( self ):
         self.synchronizeControllerOn()
         oiValue = self.m_oiSuccessActions + self.m_oiFailureActions
         self.synchronizeControllerOff()
         return int(oiValue)


     def controlActiondFinished(self):
         return self.controlActionSuccessFailure()


     def isActiveActions( self ):
         obStatus = True
         if (int( self.controlActiondFinished())==int(self.getActiveActions()) ):
             obStatus = False
             self.__resetActiveActions()
         return obStatus



     def getActiveActions( self ):
         self.synchronizeControllerOn()
         oiValue = self.m_oiActiveActions
         self.synchronizeControllerOff()
         return oiValue



     def controlActionSuccess( self, _ovdObject = None ):
         self.synchronizeControllerOn()
         self.m_oiSuccessActions += 1
         self.synchronizeControllerOff()


     def controlActionFailure( self, _ovdObject = None ):
         self.synchronizeControllerOn()
         self.m_oiFailureActions += 1
         self.synchronizeControllerOff()



     def synchronizeCluster( self ):
         self.controlActiveActions()
         if( self.isClusterFull()==True ):
             self.__synchronizeActions()


     def synchronizeController( self ):
         self.__synchronizeActions( )


     def __synchronizeActions( self ):
         while( self.isActiveActions()==True ):
             ALTime.wait( AL_TIME_MINIMUM )
         self.__resetActiveActions()




##############################################################################

