#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################


from ALDefinition                  import *
from ALObject                      import ALObject
from ALString                      import ALString
from ALToken                       import ALToken
from ALVerbose                     import ALVerbose
from ALTime                        import ALTime
from ALList                        import ALList
from ALSystem                      import ALSystem
from ALDebug                       import ALDebug
from ALSlot                        import ALSlot
from ALSignal                      import ALSignal
from ALCommandLine                 import ALCommandLine
from ALCommandLineInterface        import ALCommandLineInterface
from ALCommandLineArgument         import ALCommandLineArgument
from ALResource                    import ALResource
from ALLogFile                     import ALLogFile
from ALSemaphore                   import ALSemaphore
from ALDiskExplorer                import ALDiskExplorer
from ALApplicationCommunicator     import ALApplicationCommunicator
from ALApplicationActionControl    import ALApplicationActionControl
from ALFactoryPlugin               import ALFactoryPlugin
from ALFactoryPluginStatic         import ALFactoryPluginStatic
from ALActionMethodTimer           import ALActionMethodTimer



class ALAppDefinition( ALObject ):
    """
    Interface for Application Objects.

    """
    m_oalSemaphore                      = ALSemaphore()
    m_oalApplicationName                = None
    m_oalApplicationFolder              = None
    m_oalApplicationPluginFolder        = None
    m_oalApplicationVersion             = None
    m_oalCommandLineInterface           = None
    m_oalCommandLine                    = None
    m_oalResource                       = None
    m_oalHelp                           = None
    m_oalMan                            = None
    m_oalDebug                          = False
    m_oalSlotSignal                     = ALSlot()
    m_oalFactoryPlugin                  = ALFactoryPlugin()
    m_oalFactoryPluginDynamic           = True
    m_oalListTimerActions               = ALList()



    def __init__( self, _oalString = AL_FRAMEWORK_APPLICATION_NAME, _dversion = AL_VERSION,
                 _obPlugin = True, _obControlAppName = True ):
        ALObject.__init__( self )
        self.m_oalTime = ALTime()
        # Only for static methods
        ALTime.init()
        if ( _obControlAppName == True):
            ALAppDefinition.m_oalApplicationName = ALString( _oalString ) + "-v" + ALString( _dversion )
        else:
            ALAppDefinition.m_oalApplicationName = ALString( _oalString )
        ALAppDefinition.m_oalApplicationVersion  = ALString( _dversion )
        self.create( _obPlugin )



    def synchronizeOn( _oalObject = None ):
        # Option used to debug thread locks
        if (_oalObject!=None):
            oalStringID = ALString( _oalObject.getID() )
            ALVerbose.aalibDEBUG( "ALAppDefinition::synchronizeOff - " + oalStringID )
            ALVerbose.aalibDEBUG( "ALAppDefinition::synchronizeOff - Semaphore is OFF" )
        ALAppDefinition.m_oalSemaphore.acquire()
    synchronizeOn = staticmethod( synchronizeOn )


    def synchronizeOff( _oalObject = None ):
        # Option used to debug thread locks
        if (_oalObject!=None):
            oalStringID = ALString( _oalObject.getID() )
            ALVerbose.aalibDEBUG( "ALAppDefinition::synchronizeOff - " + oalStringID )
            ALVerbose.aalibDEBUG( "ALAppDefinition::synchronizeOff - Semaphore is OFF" )
        ALAppDefinition.m_oalSemaphore.release()
    synchronizeOff = staticmethod( synchronizeOff )



    def create( self, _obPlugin = True ):
        ALAppDefinition.m_oalCommandLine          = ALCommandLine( ALCommandLineArgument() )
        ALAppDefinition.m_oalApplicationFolder    = ALAppDefinition.m_oalCommandLine.getApplicationFolder()
        ALVerbose.m_oalLogFile                    = ALLogFile( ALAppDefinition.m_oalApplicationName + ".log" )
        ALAppDefinition.m_oalResource             = ALResource( ALAppDefinition.m_oalApplicationName + ".res" )

        self.m_ofTimeOut = AL_TIME_INFINITE # It is: 11d:13h:46m:39s:000ms

        self.m_oalSlotInit          = ALSlot()
        self.m_oalSlotPreProcess    = ALSlot()
        self.m_oalSlotProcess       = ALSlot()
        self.m_oalSlotPostProcess   = ALSlot()
        self.m_oalSlotEnd           = ALSlot()


        if (_obPlugin):
            self.__connectPluginFolder()




    def __connectPluginFolder( self ):
        if (ALDebug.isDebugModeOn()==True or ALDebug.isDebugModeForceOn()==True):
            ALAppDefinition.m_oalDebug = True
            ALVerbose.setVerboseDebugOn()

        # First Plugin Folder
        oalStringFolder = ALString( ALAppDefinition.m_oalApplicationFolder ) + "/plugins"

        if ( ALAppDefinition.m_oalFactoryPlugin.addPluginFolder( oalStringFolder )==False ):
            # Second Plugin Folder
            oalStringFolder = ALDiskExplorer.mergePath( ALAppDefinition.m_oalApplicationFolder, "../" )
            oalStringFolder = oalStringFolder + "/plugins"
            oalStringFolder = ALDiskExplorer.getAbsolutePath( oalStringFolder )

            if ( ALAppDefinition.m_oalFactoryPlugin.addPluginFolder( oalStringFolder )==False ):
                oalStringFolder = ALDiskExplorer.getFolderName( ALAppDefinition.m_oalCommandLine.getArgumentNumber( 0 ) )
                ALAppDefinition.m_oalFactoryPlugin.addPluginFolderFromPath( oalStringFolder )
        ALAppDefinition.m_oalApplicationPluginFolder = oalStringFolder
        self.__connectCommandLineInterfaceFile( oalStringFolder )


    def setCommandLineInterfaceFile( self, _oalStringFile ):
        """
        TO be implemented inside ALAppDefCommandLine.py
        """
        pass


    def __connectCommandLineInterfaceFile( self, _oalStringFolder ):
        oalStringFolder = _oalStringFolder + "/commandLineInterface.xml"
        self.setCommandLineInterfaceFile( oalStringFolder  )


    def setTimeOutPlugins( self, _ofTimeOut ):
        ALAppDefinition.synchronizeOn()
        self.m_ofTimeOut = _ofTimeOut
        ALAppDefinition.synchronizeOff()


    def getTimeOutPlugins( self  ):
        ALAppDefinition.synchronizeOn()
        ofValue = self.m_ofTimeOut
        ALAppDefinition.synchronizeOff()
        return ofValue



    def setFactoryPluginStatic( _oFactoryPluginStatic ):
        ALAppDefinition.synchronizeOn()
        if (_oFactoryPluginStatic is not None ):
            ALAppDefinition.m_oalFactoryPlugin = _oFactoryPluginStatic
            ALAppDefinition.m_oalFactoryPluginDynamic = False
#        try:
#            if (isinstance( _oFactoryPluginStatic, ALFactoryPluginStatic )):
#                ALAppDefinition.m_oalFactoryPlugin = _oFactoryPluginStatic
#                ALAppDefinition.m_oalFactoryPluginDynamic = False
#        except:
#            ALVerbose.DEBUG( "ALApplication::setFactoryPluginStatic() - fix in Jython version" )
        ALAppDefinition.synchronizeOff()
    setFactoryPluginStatic = staticmethod(setFactoryPluginStatic)


    def isFactoryPluginDynamic( ):
        return ALAppDefinition.m_oalFactoryPluginDynamic
    isFactoryPluginDynamic = staticmethod(isFactoryPluginDynamic)



    def getRunTime( self ):
        ALAppDefinition.synchronizeOn()
        oValue = self.m_oalTime.getDuration()
        ALAppDefinition.synchronizeOff()
        return oValue


    # SlotConnection =========================================================
    def connectInit( self, _oMethod ):
        ALAppDefinition.synchronizeOn()
        if (_oMethod is not None ):
            self.m_oalSlotInit.connect( _oMethod )
        ALAppDefinition.synchronizeOff()

    def connectPreProcess( self, _oMethod ):
        ALAppDefinition.synchronizeOn()
        if (_oMethod is not None ):
            self.m_oalSlotPreProcess.connect( _oMethod )
        ALAppDefinition.synchronizeOff()

    def connectProcess( self, _oMethod ):
        ALAppDefinition.synchronizeOn()
        if (_oMethod is not None ):
            self.m_oalSlotProcess.connect( _oMethod )
        ALAppDefinition.synchronizeOff()

    def connectPostProcess( self, _oMethod ):
        ALAppDefinition.synchronizeOn()
        if (_oMethod is not None ):
            self.m_oalSlotPostProcess.connect( _oMethod )
        ALAppDefinition.synchronizeOff()

    def connectEnd( self, _oMethod ):
        ALAppDefinition.synchronizeOn()
        if (_oMethod is not None ):
            self.m_oalSlotEnd.connect( _oMethod )
        ALAppDefinition.synchronizeOff()

    def connectExecute( self, _oMethod ):
        ALAppDefinition.synchronizeOn()
        if (_oMethod is not None ):
            self.m_oalSlotProcess.connect( _oMethod )
        ALAppDefinition.synchronizeOff()



    def callbackSlotInit( self, _oalObject ):
        self.m_oalSlotInit.call( _oalObject )


    def callbackSlotEnd( self, _oalObject ):
        self.m_oalSlotEnd.call( _oalObject )


    def callbackSlotPreProcess( self, _oalObject ):
        self.m_oalSlotPreProcess.call( _oalObject )


    def callbackSlotProcess( self, _oalObject ):
        self.m_oalSlotProcess.call( _oalObject )


    def callbackSlotPostProcess( self, _oalObject ):
        self.m_oalSlotPostProcess.call( _oalObject )



    def existResource( _oalString = AL_FRAMEWORK_APPLICATION_NAME, _dversion = AL_VERSION, _obControlAppName = True ):
        if ( _obControlAppName == True):
            oalAppName = ALString( _oalString ) + "-v" + ALString( _dversion )
        else:
            oalAppName = ALString( _oalString )
        oalResource = ALResource( oalAppName + ".res" )
        return oalResource.exist()
    existResource = staticmethod( existResource )



    def isFirstExecution( _oalString = AL_FRAMEWORK_APPLICATION_NAME, _dversion = AL_VERSION, _obControlAppName = True ):
        if (ALAppDefinition.existResource( _oalString, _dversion, _obControlAppName )==True):
            return False
        else:
            return True
    isFirstExecution = staticmethod( isFirstExecution )



    def connectExecuteTimerAction( self, _oMethod, _oTimeDelay, _oTimeDuration = AL_TIME_INFINITE ):
        ALAppDefinition.executeTimerAction( _oMethod, _oTimeDelay, _oTimeDuration )


    def executeTimerAction( _oMethod, _oTimeDelay, _oTimeDuration = AL_TIME_INFINITE ):
        ALAppDefinition.synchronizeOn()
        oalActionTimer = ALActionMethodTimer(  _oMethod, _oTimeDelay, _oTimeDuration )
        ALAppDefinition.m_oalListTimerActions.add( oalActionTimer )
        oalActionTimer.executeAction()
        ALAppDefinition.synchronizeOff()
    executeTimerAction = staticmethod( executeTimerAction )


    def getApplicationName():
        ALAppDefinition.synchronizeOn()
        oalString =  ALAppDefinition.m_oalApplicationName
        ALAppDefinition.synchronizeOff()
        return oalString
    getApplicationName = staticmethod( getApplicationName )



    # Plugins Factory methods =========================================
    def addPluginObject( _oalPlugin ):
        ALAppDefinition.synchronizeOn()
        ALAppDefinition.m_oalFactoryPlugin.addPlugin( _oalPlugin )
        ALAppDefinition.synchronizeOff()
    addPluginObject = staticmethod( addPluginObject )


    def getPluginObject( _oalToken, _oalFileNameXml = None, _oBoolXsd = False ):
        ALAppDefinition.synchronizeOn()
        oalPlugin =  ALAppDefinition.m_oalFactoryPlugin.getPlugin( _oalToken, _oalFileNameXml, _oBoolXsd )
        ALAppDefinition.synchronizeOff()
        return oalPlugin
    getPluginObject = staticmethod( getPluginObject )


    def addPluginFolder( self, _oalString ):
        ALAppDefinition.synchronizeOn()
        if (_oalString!=ALString.empty()):
            ALAppDefinition.m_oalFactoryPlugin.addPluginFolder( _oalString )
        ALAppDefinition.synchronizeOff()


    def addPluginFile( self, _oalString ):
        ALAppDefinition.synchronizeOn()
        if (_oalString!=ALString.empty()):
            ALAppDefinition.m_oalFactoryPlugin.addPluginFile( _oalString )
        ALAppDefinition.synchronizeOff()


    def addPlugin( self, _oalPlugin ):
        if (_oalPlugin!=None):
            ALAppDefinition.addPluginObject( _oalPlugin )
        else:
            ALVerbose.aalibDEBUG( "ALApplication.addPlugin() = NONE plugin not added" )


    def getPlugin( self, _oalToken, _oalFileNameXml = None, _oBoolXsd = False ):
        return ALAppDefinition.getPluginObject( _oalToken, _oalFileNameXml, _oBoolXsd )



    def getApplicationLogFileName():
        ALAppDefinition.synchronizeOn()
        oalString =  ALString( ALVerbose.m_oalLogFile.getFileName() )
        ALAppDefinition.synchronizeOff()
        return oalString
    getApplicationLogFileName = staticmethod( getApplicationLogFileName )


    def getApplicationFolderName():
        ALAppDefinition.synchronizeOn()
        oalString =  ALString( ALAppDefinition.m_oalApplicationFolder )
        ALAppDefinition.synchronizeOff()
        return oalString
    getApplicationFolderName = staticmethod( getApplicationFolderName )


    def getApplicationPluginFolderName():
        ALAppDefinition.synchronizeOn()
        oalString =  ALString( ALAppDefinition.m_oalApplicationPluginFolder )
        ALAppDefinition.synchronizeOff()
        return oalString
    getApplicationPluginFolderName = staticmethod( getApplicationPluginFolderName )


    def setApplicationPluginFolder( _oalStrFolderName ):
        ALAppDefinition.synchronizeOn()
        if (_oalStrFolderName!=ALString.empty()):
            ALAppDefinition.m_oalFactoryPlugin.addPluginFolder( _oalStrFolderName )
            ALAppDefinition.m_oalApplicationPluginFolder = _oalStrFolderName
        ALAppDefinition.synchronizeOff()
    setApplicationPluginFolder = staticmethod( setApplicationPluginFolder )



    def setHelp( _oalHelp ):
        ALAppDefinition.synchronizeOn()
        ALAppDefinition.m_oalHelp = _oalHelp
        ALAppDefinition.synchronizeOff()
    setHelp = staticmethod( setHelp )



    def setMan( _oalMan ):
        ALAppDefinition.synchronizeOn()
        ALAppDefinition.m_oalMan = _oalMan
        ALAppDefinition.synchronizeOff()
    setMan = staticmethod( setMan )


    # Signal =================================================================
    def getSignal():
        ALAppDefinition.synchronizeOn()
        oalSignal = ALSignal.ok()
        oalSignalExt = ALApplicationCommunicator.getSignal()
        if oalSignalExt != None :
            oalSignal = oalSignalExt.clone()
        ALAppDefinition.synchronizeOff()
        return oalSignal
    getSignal = staticmethod( getSignal )


    def setSignal( _oalSignal ):
        ALAppDefinition.synchronizeOn()
        if (_oalSignal != None):
            ALApplicationCommunicator.setSignal( _oalSignal.clone() )
            ALAppDefinition.m_oalSlotSignal.callSignal( _oalSignal.clone() )
        ALAppDefinition.synchronizeOff()
    setSignal = staticmethod( setSignal )



    # Ressource ================================================================
    def getResource( _oalToken ):
        ALAppDefinition.synchronizeOn( )
        oalRessourceValue = ALAppDefinition.m_oalResource.getResource( _oalToken )
        if ( oalRessourceValue=="None" ):
            oalRessourceValue = None
        ALAppDefinition.synchronizeOff( )
        return oalRessourceValue
    getResource = staticmethod( getResource )


    def setResource( _oalToken, _oalTokenObject ):
        ALAppDefinition.synchronizeOn()
        ALAppDefinition.m_oalResource.setResource( _oalToken, _oalTokenObject )
        ALAppDefinition.synchronizeOff()
    setResource = staticmethod( setResource )



    def resetResource():
        ALAppDefinition.synchronizeOn()
        ALAppDefinition.m_oalResource.reset()
        ALAppDefinition.m_oalResource  = ALResource( ALAppDefinition.m_oalApplicationName + ".res" )
        ALAppDefinition.synchronizeOff()
    resetResource = staticmethod( resetResource )


    def importResource( _oalStrFileName ):
        ALAppDefinition.synchronizeOn()
        ALAppDefinition.m_oalResource.importResource( _oalStrFileName )
        ALAppDefinition.synchronizeOff()
    importResource = staticmethod( importResource )



    def inputResource():
        ALAppDefinition.synchronizeOn()
        ALAppDefinition.m_oalResource.input()
        ALAppDefinition.synchronizeOff()
    inputResource = staticmethod( inputResource )


    def outputResource():
        ALAppDefinition.synchronizeOn()
        ALAppDefinition.m_oalResource.output()
        ALAppDefinition.synchronizeOff()
    outputResource = staticmethod( outputResource )


    def outputProfile():
        ALAppDefinition.synchronizeOn()
        ALVerbose.getProfileFile()
        ALAppDefinition.synchronizeOff()
    outputProfile = staticmethod( outputProfile )


    def outputProfileDataBase():
        ALAppDefinition.synchronizeOn()
        oalFileName = ALVerbose.outputProfileDataBase()
        ALAppDefinition.synchronizeOff()
        return oalFileName
    outputProfileDataBase = staticmethod( outputProfileDataBase )


##############################################################################
