#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################


from ALDefinition            import *
from ALVerbose               import ALVerbose
from ALString                import ALString
from ALDebug                 import ALDebug
from ALAssert                import ALAssert
from ALSystem                import ALSystem
from ALAppDefinitionControl  import ALAppDefinitionControl


class ALAppDefinitionInit( ALAppDefinitionControl ):
    """
    Interface for ALAppDefinitionInit objects.

    """
    def __init__( self, _oalString = AL_FRAMEWORK_APPLICATION_NAME, _dversion = AL_VERSION, _obPlugin = True ):
        ALAppDefinitionControl.__init__( self, _oalString, _dversion, _obPlugin )


    def init( self, _bInit = True ):
        """
        This is the Init method in the ALAppDefinitionInit.
        """
        ALVerbose.aalibDEBUG( "Execution: ALAppDefinitionInit.init()" )
        oBoolStatus = True
        oalString = ALString( ALAppDefinitionInit.m_oalCommandLine.getCommandLine() )
        ALVerbose.aalibDEBUG( oalString )
        ALVerbose.setVerboseOff()


        if (ALDebug.isDebugModeVerboseOn()==True):
            ALVerbose.setVerboseOn()


        if (ALAppDefinitionInit.m_oalCommandLine.existCommand( "--quiet" )):
            ALVerbose.setVerboseOff()


        if (ALAppDefinitionInit.m_oalCommandLine.existCommand( "--profile" )):
            ALAppDefinitionInit.setDebugProfileOn()


        if (ALDebug.isDebugModeOn()==True or ALDebug.isDebugModeForceOn()==True):
            ALAppDefinitionInit.setDebugMode()
            ALVerbose.aalibDEBUG( "Debug Mode [ON]" )


        if (ALDebug.isDebugModeAssertOn()==True ):
            ALAssert.setAssertionModeOn()


        if (_bInit==True):
            ALVerbose.screen( self.m_oalApplicationName + "::Starting" )
            if (ALAppDefinitionInit.m_oalCommandLine.existCommand( "--help" )
                or ALAppDefinitionInit.m_oalCommandLine.existCommand( "-h" )):
                ALVerbose.setVerboseOn()
                ALAppDefinitionInit.outputHelp()
                oBoolStatus = False

            if (ALAppDefinitionInit.m_oalCommandLine.existCommand( "--version" ) or
                ALAppDefinitionInit.m_oalCommandLine.existCommand( "-v" )):
                ALVerbose.setVerboseOn()
                ALAppDefinitionInit.outputVersion()
                oBoolStatus = False

            if (ALAppDefinitionInit.m_oalCommandLine.existCommand( "--man" ) or
                ALAppDefinitionInit.m_oalCommandLine.existCommand( "-m" )):
                ALVerbose.setVerboseOn()
                ALAppDefinitionInit.outputMan()
                oBoolStatus = False

            if (ALAppDefinitionInit.m_oalCommandLine.existCommand( "--no-log" )):
                ALVerbose.setLogFileOff()

            # workaround to avoid problem of the ALString.toXml cleaning function of jython
            if (ALSystem.getOSName()=="Linux" and ALSystem.getMachineVirtuel()=="Java"):
                pass
            else:
                oalObject = ALAppDefinitionInit.getResource( AL_FRAMEWORK_LAST_RUN )
                if (oalObject != None):
                    oalStringLastRun = ALString( oalObject );
                    ALVerbose.screen( "Last Run: " + oalStringLastRun )

            if (ALAppDefinitionInit.m_oalCommandLine.existCommand( "--import-resource" )):
                oalStrResourceFileName = ALAppDefinitionInit.m_oalCommandLine.getCommandArgument( "--import-resource" )
                ALAppDefinitionInit.importResource( oalStrResourceFileName )

        # The OK to continue the application
        return oBoolStatus





##############################################################################
