#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################


from ALString                  import ALString
from ALSystem                  import ALSystem
from ALVerbose                 import ALVerbose
from ALTrace                   import ALTrace
from ALFile                    import ALFile
from ALList                    import ALList
from ALCommand                 import ALCommand
from ALDictionary              import ALDictionary
from ALCommandLineArgument     import ALCommandLineArgument
from ALDiskExplorer            import ALDiskExplorer


class ALCommandLine( ALCommand ):
    """
    Base class for command line objects.
    """
    def __init__( self, _oalCommandLineArgument = None ):
        ALCommand.__init__( self )
        self.m_oalDictCommand         = ALDictionary()
        self.m_oalCommandLineArgument = ALCommandLineArgument()
        if ( isinstance( _oalCommandLineArgument, ALString ) or isinstance( _oalCommandLineArgument, str )):
            self.m_oalStringCommand = _oalCommandLineArgument
            oalList = self.m_oalStringCommand.split()
            self.m_oalCommandLineArgument = ALCommandLineArgument( oalList )
        elif ( isinstance( _oalCommandLineArgument, ALCommandLineArgument ) ):
            self.m_oalCommandLineArgument = _oalCommandLineArgument



    def addCommandLine( self, _oalStringCommand ):
        oalList = _oalStringCommand.split( " " )
        for iElem in oalList:
            self.m_oalCommandLineArgument.add( iElem )



    def setCommandLine( self, _oalStringCommand ):
        self.setCommand( ALString( _oalStringCommand ) )
        self.m_oalCommandLineArgument = ALCommandLineArgument( ALString.empty())
        oalList = self.m_oalStringCommand.split( " " )
        for iElem in oalList:
            self.m_oalCommandLineArgument.add( iElem )




    def getCommand( self ):
        self.synchronizeOn()
        if ( self.m_oalCommandLineArgument != None ):
            self.m_oalStringCommand = ALString( "" )
            for cmdkey in self.m_oalCommandLineArgument:
                self.m_oalStringCommand += ALString( cmdkey ) + " "
        self.synchronizeOff()
        return self.m_oalStringCommand


    def getCommandLine( self ):
        return self.getCommand()


    def addCommand( self, _oalStringToken, _oalObjectMethod ):
        self.synchronizeOn()
        self.m_oalDictCommand.addObject( ALString( _oalStringToken ), _oalObjectMethod )
        self.synchronizeOff()


    def executeCommand( self ):
        self.executeAction()


    def executeCommandSynchronous( self ):
        self.executeActionSynchronous()


    def process( self, _oalObject = None ):
        self.synchronizeOn()
        if ( self.m_oalCommandLineArgument != None ):
            for cmdkey in self.m_oalCommandLineArgument:
                if cmdkey in self.m_oalDictCommand:
                    oalObjectMethod = self.m_oalDictCommand[ cmdkey ]
                    oalObjectMethod( cmdkey )

#                    Maybe in the future???
#                    try:
#                        oalObjectMethod( cmdkey )
#                    except:
#                        ALTrace.trace( self, ALVerbose.isVerboseDebug() )
#                        ALVerbose.warning("ALCommand::process = Exception execution method with argument")
#                        ALVerbose.warning("Trying the execution with NO argument")
#                        try:
#                            oalObjectMethod()
#                        except:
#                            ALTrace.trace( self, ALVerbose.isVerboseDebug() )
        self.synchronizeOff()


    def existCommand( self, _oalString ):
#        ALVerbose.DEBUG( "Executing ALCommandLine::existCommand() = " + _oalString )
#        self.synchronizeOn( self )
        self.synchronizeOn()
        oalString = ALString( _oalString )
        bexist = self.m_oalCommandLineArgument.existArgument( oalString )
        self.synchronizeOff()
        return bexist


    def getArgument( self, _oalString ):
        self.synchronizeOn()
        oalString = ALString( _oalString )
        oalArgument = self.m_oalCommandLineArgument.getArgument( oalString )
        self.synchronizeOff()
        return oalArgument


    def getCommandArgument( self, _oalString ):
        return self.getArgument( _oalString )



    def addCommandArgument( self, _oalStrCommand, _oalStrArgument = None ):
        self.synchronizeOn()
        oalString = ALString( _oalStrCommand )
        self.m_oalCommandLineArgument.append( oalString )
        if ( _oalStrArgument != None ):
            oalString = ALString( _oalStrArgument )
            self.m_oalCommandLineArgument.append( oalString )
        self.synchronizeOff()


    def removeCommandArgument( self, _oalStringToken, _odtControl = False ):
        self.synchronizeOn()
        if ( self.m_oalCommandLineArgument != None ):
            i = 0
            for cmdkey in self.m_oalCommandLineArgument:
                if (cmdkey ==_oalStringToken):
                    self.m_oalCommandLineArgument.remove( i )
                    if (_odtControl==True):
                        self.m_oalCommandLineArgument.remove( i )
                        i = i-1
                    i = i-1
                i = i+1
        self.synchronizeOff()


    def isEmpty( self ):
        self.synchronizeOn()
        bEmpty = False
        if len(self.m_oalCommandLineArgument)<=1:
            bEmpty = True
        self.synchronizeOff()
        return bEmpty


    def synchronizeCommand( self, _oalObject = None ):
        self.waitActionEnd( 0.1 )



    def getArgumentNumber( self, _oValue ):
        self.synchronizeOn()
        oalObject = self.m_oalCommandLineArgument.getArgumentNumber( _oValue )
        self.synchronizeOff()
        return oalObject



    def getApplicationFolder( self ):
        self.synchronizeOn()
        oalStringFolder = self.__getApplicationPathFromCommandLine()
        #To be used by unix systems
        if (oalStringFolder==None):
            oalStringFolder = self.__getApplicationPath()
        #To be used by windows systems
        if (oalStringFolder==None):
            oalStringFolder = self.__getApplicationPath( ALString( ".exe" ) )
        #To be used by MacOSX systems
        if (oalStringFolder==None):
            oalStringFolder = self.__getApplicationPath( ALString( ".app" ) )
        self.synchronizeOff()
        return oalStringFolder


# These methods go to new ALPath class in the future
    def __getApplicationPath( self, _oStrExtension = None ):
        oalStringFolder = None
        oalStrFile = ALSystem.getProgramName()
        if ( _oStrExtension != None ):
            oalStrFile = ALFile.setExtension( oalStrFile, _oStrExtension )
        oalList    = ALSystem.getSystemPath()
        for iPath in oalList:
            oalStrFileName = ALDiskExplorer.getAbsolutePath( iPath ) + "/" + oalStrFile
            if ( ALDiskExplorer.existFile(oalStrFileName)==True):
                oalStringFolder = ALFile.getFolderName( oalStrFileName )
#        ALVerbose.DEBUG( "Application Folder from Path test: " +  ALString( oalStringFolder ) )
        return oalStringFolder


    def __getApplicationPathFromCommandLine( self ):
        oalStringFolder = self.m_oalCommandLineArgument.getArgumentNumber( 0 )
        if ( ALDiskExplorer.existFile(oalStringFolder)==True):
            oalStringFolder = ALDiskExplorer.getAbsolutePath( oalStringFolder )
            oalStringFolder = ALFile.getFolderName( oalStringFolder )
        else:
            oalStringFolder = None
#        ALVerbose.DEBUG("Application Folder from CommandLine test: " + ALString( oalStringFolder ) )
        return oalStringFolder


##############################################################################
