#!/usr/bin/env python

#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################

#
#
#
# Generated by ALExternalPackGenerateDS.py.
#
# This file is generated by a modified version of the original file:
# generateDS.py, proposed by Dave Kuhlman:
#
## LICENSE
#
## Copyright (c) 2003 Dave Kuhlman
#
## Permission is hereby granted, free of charge, to any person obtaining
## a copy of this software and associated documentation files (the
## "Software"), to deal in the Software without restriction, including
## without limitation the rights to use, copy, modify, merge, publish,
## distribute, sublicense, and/or sell copies of the Software, and to
## permit persons to whom the Software is furnished to do so, subject to
## the following conditions:
#
## The above copyright notice and this permission notice shall be
## included in all copies or substantial portions of the Software.
#
## THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
## EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
## MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
## IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
## CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
## TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
## SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
##############################################################################

import sys
import getopt
from xml.dom import minidom
from xml.dom import Node


#
# Support/utility functions.
#

def showIndent(outfile, level):
    for idx in range(level):
        outfile.write('    ')


def showIndentHtml(outfile, level):
    for idx in range(level):
        outfile.write(' &nbsp; &nbsp; &nbsp; &nbsp;')


def quote_xml(inStr):
    s1 = inStr
    s1 = s1.replace('&', '&amp;')
    s1 = s1.replace('<', '&lt;')
    s1 = s1.replace('"', '&quot;')
    return s1

def quote_python(inStr):
    s1 = inStr
    if s1.find("'") == -1:
        if s1.find('\n') == -1:
            return "'%s'" % s1
        else:
            return "'''%s'''" % s1
    else:
        if s1.find('"') != -1:
            s1 = s1.replace('"', '\\"')
        if s1.find('\n') == -1:
            return '"%s"' % s1
        else:
            return '"""%s"""' % s1


class MixedContainer(object):
    # Constants for category:
    CategoryNone = 0
    CategoryText = 1
    CategorySimple = 2
    CategoryComplex = 3
    # Constants for content_type:
    TypeNone = 0
    TypeText = 1
    TypeString = 2
    TypeInteger = 3
    TypeFloat = 4
    TypeDecimal = 5
    TypeDouble = 6
    TypeBoolean = 7
    def __init__(self, category, content_type, name, value):
        self.category = category
        self.content_type = content_type
        self.name = name
        self.value = value
    def getCategory(self):
        return self.category
    def getContenttype(self, content_type):
        return self.content_type
    def getValue(self):
        return self.value
    def getName(self):
        return self.name
    def export(self, outfile, level, name):
        if self.category == MixedContainer.CategoryText:
            outfile.write(self.value)
        elif self.category == MixedContainer.CategorySimple:
            self.exportSimple(outfile, level, name)
        else:    # category == MixedContainer.CategoryComplex
            self.value.export(outfile, level, name)
    def exportSimple(self, outfile, level, name):
        if self.content_type == MixedContainer.TypeString:
            outfile.write('<%s>%s</%s>' % (self.name, self.value, self.name))
        elif self.content_type == MixedContainer.TypeInteger or \
                self.content_type == MixedContainer.TypeBoolean:
            outfile.write('<%s>%d</%s>' % (self.name, self.value, self.name))
        elif self.content_type == MixedContainer.TypeFloat or \
                self.content_type == MixedContainer.TypeDecimal:
            outfile.write('<%s>%f</%s>' % (self.name, self.value, self.name))
        elif self.content_type == MixedContainer.TypeDouble:
            outfile.write('<%s>%g</%s>' % (self.name, self.value, self.name))
    def exportLiteral(self, outfile, level, name):
        if self.category == MixedContainer.CategoryText:
            showIndent(outfile, level)
            outfile.write('MixedContainer(%d, %d, "%s", "%s"),\n' % \
                (self.category, self.content_type, self.name, self.value))
        elif self.category == MixedContainer.CategorySimple:
            showIndent(outfile, level)
            outfile.write('MixedContainer(%d, %d, "%s", "%s"),\n' % \
                (self.category, self.content_type, self.name, self.value))
        else:    # category == MixedContainer.CategoryComplex
            showIndent(outfile, level)
            outfile.write('MixedContainer(%d, %d, "%s",\n' % \
                (self.category, self.content_type, self.name,))
            self.value.exportLiteral(outfile, level + 1)
            showIndent(outfile, level)
            outfile.write(')\n')



TEMPLATE_PAGE_P1 = """
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en">
<head>
    <title>AALib Plugin Data Description</title>
    <meta http-equiv="content-type" content="text/html; charset=utf-8">
    <meta name="description" content="AALib Plugin generation from XSD files">
    <meta name="keywords" content="AALib, xsd, plugin">
    <meta name="author" content="Romeu Pieritz(romeu_pieritz@yahoo.fr)">
    <link href="style.css" rel="stylesheet" type="text/css">
    <base target="main">
</head>
<body>
    <div id="main">
        <div id="block">
          %s
    </div>
"""

TEMPLATE_PAGE_P2 = """
    </div>
</body>
</html>
"""



#
# Class definition
#

class ALCommandLineInterfaceElement(object):
    subclass = None
    def __init__(self, command='', argument='', commandDescription='', klass='', classMethod='', classFileName='', plugin='', pluginMethod='', pluginFileName='', help='', manual='', webAdress='', webUpDate='', copyright='', author='', date=''):
        self.command = command
        self.argument = argument
        self.commandDescription = commandDescription
        self.klass = klass
        self.classMethod = classMethod
        self.classFileName = classFileName
        self.plugin = plugin
        self.pluginMethod = pluginMethod
        self.pluginFileName = pluginFileName
        self.help = help
        self.manual = manual
        self.webAdress = webAdress
        self.webUpDate = webUpDate
        self.copyright = copyright
        self.author = author
        self.date = date
    def factory(*args_, **kwargs_):
        if ALCommandLineInterfaceElement.subclass:
            return ALCommandLineInterfaceElement.subclass(*args_, **kwargs_)
        else:
            return ALCommandLineInterfaceElement(*args_, **kwargs_)
    factory = staticmethod(factory)
    def getCommand(self): return self.command
    def setCommand(self, command): self.command = command
    def getArgument(self): return self.argument
    def setArgument(self, argument): self.argument = argument
    def getCommandDescription(self): return self.commandDescription
    def setCommandDescription(self, commandDescription): self.commandDescription = commandDescription
    def getClass(self): return self.klass
    def setClass(self, klass): self.klass = klass
    def getClassMethod(self): return self.classMethod
    def setClassMethod(self, classMethod): self.classMethod = classMethod
    def getClassFileName(self): return self.classFileName
    def setClassFileName(self, classFileName): self.classFileName = classFileName
    def getPlugin(self): return self.plugin
    def setPlugin(self, plugin): self.plugin = plugin
    def getPluginMethod(self): return self.pluginMethod
    def setPluginMethod(self, pluginMethod): self.pluginMethod = pluginMethod
    def getPluginFileName(self): return self.pluginFileName
    def setPluginFileName(self, pluginFileName): self.pluginFileName = pluginFileName
    def getHelp(self): return self.help
    def setHelp(self, help): self.help = help
    def getManual(self): return self.manual
    def setManual(self, manual): self.manual = manual
    def getWebAdress(self): return self.webAdress
    def setWebAdress(self, webAdress): self.webAdress = webAdress
    def getWebUpDate(self): return self.webUpDate
    def setWebUpDate(self, webUpDate): self.webUpDate = webUpDate
    def getCopyright(self): return self.copyright
    def setCopyright(self, copyright): self.copyright = copyright
    def getAuthor(self): return self.author
    def setAuthor(self, author): self.author = author
    def getDate(self): return self.date
    def setDate(self, date): self.date = date
    def export(self, outfile, level = 0, name_='ALCommandLineInterfaceElement'):
        showIndent(outfile, level)
        outfile.write('<%s>\n' % name_)
        self.exportChildren(outfile, level + 1, name_)
        showIndent(outfile, level)
        outfile.write('</%s>\n' % name_)
    def exportAttributes(self, outfile, level, name_='ALCommandLineInterfaceElement'):
        pass
    def exportChildren(self, outfile, level, name_='ALCommandLineInterfaceElement'):
        showIndent(outfile, level)
        outfile.write('<command>%s</command>\n' % quote_xml(self.getCommand()))
        showIndent(outfile, level)
        outfile.write('<argument>%s</argument>\n' % quote_xml(self.getArgument()))
        showIndent(outfile, level)
        outfile.write('<commandDescription>%s</commandDescription>\n' % quote_xml(self.getCommandDescription()))
        showIndent(outfile, level)
        outfile.write('<class>%s</class>\n' % quote_xml(self.getClass()))
        showIndent(outfile, level)
        outfile.write('<classMethod>%s</classMethod>\n' % quote_xml(self.getClassMethod()))
        showIndent(outfile, level)
        outfile.write('<classFileName>%s</classFileName>\n' % quote_xml(self.getClassFileName()))
        showIndent(outfile, level)
        outfile.write('<plugin>%s</plugin>\n' % quote_xml(self.getPlugin()))
        showIndent(outfile, level)
        outfile.write('<pluginMethod>%s</pluginMethod>\n' % quote_xml(self.getPluginMethod()))
        showIndent(outfile, level)
        outfile.write('<pluginFileName>%s</pluginFileName>\n' % quote_xml(self.getPluginFileName()))
        showIndent(outfile, level)
        outfile.write('<help>%s</help>\n' % quote_xml(self.getHelp()))
        showIndent(outfile, level)
        outfile.write('<manual>%s</manual>\n' % quote_xml(self.getManual()))
        showIndent(outfile, level)
        outfile.write('<webAdress>%s</webAdress>\n' % quote_xml(self.getWebAdress()))
        showIndent(outfile, level)
        outfile.write('<webUpDate>%s</webUpDate>\n' % quote_xml(self.getWebUpDate()))
        showIndent(outfile, level)
        outfile.write('<copyright>%s</copyright>\n' % quote_xml(self.getCopyright()))
        showIndent(outfile, level)
        outfile.write('<author>%s</author>\n' % quote_xml(self.getAuthor()))
        showIndent(outfile, level)
        outfile.write('<date>%s</date>\n' % quote_xml(self.getDate()))

    #Only to export the entire XML tree to a file stream on disk
    def outputFile( self, _outfileName ):
        outfile = open( _outfileName, "w" )
        outfile.write("<?xml version=\"1.0\" ?>\n")
        self.export( outfile, 0, name_='ALCommandLineInterfaceElement' )
        outfile.close()


    #Only to export the entire XML tree to a Html Page in a file stream on disk
    def outputPageHtml( self, _outfileName ):
        odtStrTitle = "    <span class=\"title\">ALCommandLineInterfaceElement:</span> <br> <br> <br>\n"
        outfile = open( _outfileName, "w" )
        outfile.write( TEMPLATE_PAGE_P1 % ( odtStrTitle ) )
        self.exportLiteralHtml( outfile, 0, name_='ALCommandLineInterfaceElement' )
        outfile.write( TEMPLATE_PAGE_P2  )
        outfile.close()


    def outputFileHtml( self, _outfileName ):
        self.outputPageHtml( _outfileName )


    def exportLiteral(self, outfile, level = 0, name_='ALCommandLineInterfaceElement'):
        level += 1
        self.exportLiteralAttributes(outfile, level, name_)
        self.exportLiteralChildren(outfile, level, name_)
    def exportLiteralAttributes(self, outfile, level, name_):
        pass
    def exportLiteralChildren(self, outfile, level, name_):
        showIndent(outfile, level)
        outfile.write('command = %s,\n' % quote_python(self.getCommand()))
        showIndent(outfile, level)
        outfile.write('argument = %s,\n' % quote_python(self.getArgument()))
        showIndent(outfile, level)
        outfile.write('commandDescription = %s,\n' % quote_python(self.getCommandDescription()))
        showIndent(outfile, level)
        outfile.write('klass = %s,\n' % quote_python(self.getClass()))
        showIndent(outfile, level)
        outfile.write('classMethod = %s,\n' % quote_python(self.getClassMethod()))
        showIndent(outfile, level)
        outfile.write('classFileName = %s,\n' % quote_python(self.getClassFileName()))
        showIndent(outfile, level)
        outfile.write('plugin = %s,\n' % quote_python(self.getPlugin()))
        showIndent(outfile, level)
        outfile.write('pluginMethod = %s,\n' % quote_python(self.getPluginMethod()))
        showIndent(outfile, level)
        outfile.write('pluginFileName = %s,\n' % quote_python(self.getPluginFileName()))
        showIndent(outfile, level)
        outfile.write('help = %s,\n' % quote_python(self.getHelp()))
        showIndent(outfile, level)
        outfile.write('manual = %s,\n' % quote_python(self.getManual()))
        showIndent(outfile, level)
        outfile.write('webAdress = %s,\n' % quote_python(self.getWebAdress()))
        showIndent(outfile, level)
        outfile.write('webUpDate = %s,\n' % quote_python(self.getWebUpDate()))
        showIndent(outfile, level)
        outfile.write('copyright = %s,\n' % quote_python(self.getCopyright()))
        showIndent(outfile, level)
        outfile.write('author = %s,\n' % quote_python(self.getAuthor()))
        showIndent(outfile, level)
        outfile.write('date = %s,\n' % quote_python(self.getDate()))
    def exportLiteralHtml(self, outfile, level = 0, name_='ALCommandLineInterfaceElement'):
        level += 1
        self.exportLiteralAttributesHtml(outfile, level, name_)
        self.exportLiteralChildrenHtml(outfile, level, name_)
    def exportLiteralAttributesHtml(self, outfile, level, name_):
        pass
    def exportLiteralChildrenHtml(self, outfile, level, name_):
        showIndentHtml(outfile, level)
        outfile.write('command = <a  href:"#">%s</a><br>\n' % quote_python(self.getCommand()))
        showIndentHtml(outfile, level)
        outfile.write('argument = <a  href:"#">%s</a><br>\n' % quote_python(self.getArgument()))
        showIndentHtml(outfile, level)
        outfile.write('commandDescription = <a  href:"#">%s</a><br>\n' % quote_python(self.getCommandDescription()))
        showIndentHtml(outfile, level)
        outfile.write('klass = <a  href:"#">%s</a><br>\n' % quote_python(self.getClass()))
        showIndentHtml(outfile, level)
        outfile.write('classMethod = <a  href:"#">%s</a><br>\n' % quote_python(self.getClassMethod()))
        showIndentHtml(outfile, level)
        outfile.write('classFileName = <a  href:"#">%s</a><br>\n' % quote_python(self.getClassFileName()))
        showIndentHtml(outfile, level)
        outfile.write('plugin = <a  href:"#">%s</a><br>\n' % quote_python(self.getPlugin()))
        showIndentHtml(outfile, level)
        outfile.write('pluginMethod = <a  href:"#">%s</a><br>\n' % quote_python(self.getPluginMethod()))
        showIndentHtml(outfile, level)
        outfile.write('pluginFileName = <a  href:"#">%s</a><br>\n' % quote_python(self.getPluginFileName()))
        showIndentHtml(outfile, level)
        outfile.write('help = <a  href:"#">%s</a><br>\n' % quote_python(self.getHelp()))
        showIndentHtml(outfile, level)
        outfile.write('manual = <a  href:"#">%s</a><br>\n' % quote_python(self.getManual()))
        showIndentHtml(outfile, level)
        outfile.write('webAdress = <a  href:"#">%s</a><br>\n' % quote_python(self.getWebAdress()))
        showIndentHtml(outfile, level)
        outfile.write('webUpDate = <a  href:"#">%s</a><br>\n' % quote_python(self.getWebUpDate()))
        showIndentHtml(outfile, level)
        outfile.write('copyright = <a  href:"#">%s</a><br>\n' % quote_python(self.getCopyright()))
        showIndentHtml(outfile, level)
        outfile.write('author = <a  href:"#">%s</a><br>\n' % quote_python(self.getAuthor()))
        showIndentHtml(outfile, level)
        outfile.write('date = <a  href:"#">%s</a><br>\n' % quote_python(self.getDate()))
    def build(self, node_):
        attrs = node_.attributes
        self.buildAttributes(attrs)
        for child_ in node_.childNodes:
            nodeName_ = child_.nodeName.split(':')[-1]
            self.buildChildren(child_, nodeName_)
    def buildAttributes(self, attrs):
        pass
    def buildChildren(self, child_, nodeName_):
        if child_.nodeType == Node.ELEMENT_NODE and \
            nodeName_ == 'command':
            command_ = ''
            for text__content_ in child_.childNodes:
                command_ += text__content_.nodeValue
            self.command = command_
        elif child_.nodeType == Node.ELEMENT_NODE and \
            nodeName_ == 'argument':
            argument_ = ''
            for text__content_ in child_.childNodes:
                argument_ += text__content_.nodeValue
            self.argument = argument_
        elif child_.nodeType == Node.ELEMENT_NODE and \
            nodeName_ == 'commandDescription':
            commandDescription_ = ''
            for text__content_ in child_.childNodes:
                commandDescription_ += text__content_.nodeValue
            self.commandDescription = commandDescription_
        elif child_.nodeType == Node.ELEMENT_NODE and \
            nodeName_ == 'class':
            class_ = ''
            for text__content_ in child_.childNodes:
                class_ += text__content_.nodeValue
            self.klass = class_
        elif child_.nodeType == Node.ELEMENT_NODE and \
            nodeName_ == 'classMethod':
            classMethod_ = ''
            for text__content_ in child_.childNodes:
                classMethod_ += text__content_.nodeValue
            self.classMethod = classMethod_
        elif child_.nodeType == Node.ELEMENT_NODE and \
            nodeName_ == 'classFileName':
            classFileName_ = ''
            for text__content_ in child_.childNodes:
                classFileName_ += text__content_.nodeValue
            self.classFileName = classFileName_
        elif child_.nodeType == Node.ELEMENT_NODE and \
            nodeName_ == 'plugin':
            plugin_ = ''
            for text__content_ in child_.childNodes:
                plugin_ += text__content_.nodeValue
            self.plugin = plugin_
        elif child_.nodeType == Node.ELEMENT_NODE and \
            nodeName_ == 'pluginMethod':
            pluginMethod_ = ''
            for text__content_ in child_.childNodes:
                pluginMethod_ += text__content_.nodeValue
            self.pluginMethod = pluginMethod_
        elif child_.nodeType == Node.ELEMENT_NODE and \
            nodeName_ == 'pluginFileName':
            pluginFileName_ = ''
            for text__content_ in child_.childNodes:
                pluginFileName_ += text__content_.nodeValue
            self.pluginFileName = pluginFileName_
        elif child_.nodeType == Node.ELEMENT_NODE and \
            nodeName_ == 'help':
            help_ = ''
            for text__content_ in child_.childNodes:
                help_ += text__content_.nodeValue
            self.help = help_
        elif child_.nodeType == Node.ELEMENT_NODE and \
            nodeName_ == 'manual':
            manual_ = ''
            for text__content_ in child_.childNodes:
                manual_ += text__content_.nodeValue
            self.manual = manual_
        elif child_.nodeType == Node.ELEMENT_NODE and \
            nodeName_ == 'webAdress':
            webAdress_ = ''
            for text__content_ in child_.childNodes:
                webAdress_ += text__content_.nodeValue
            self.webAdress = webAdress_
        elif child_.nodeType == Node.ELEMENT_NODE and \
            nodeName_ == 'webUpDate':
            webUpDate_ = ''
            for text__content_ in child_.childNodes:
                webUpDate_ += text__content_.nodeValue
            self.webUpDate = webUpDate_
        elif child_.nodeType == Node.ELEMENT_NODE and \
            nodeName_ == 'copyright':
            copyright_ = ''
            for text__content_ in child_.childNodes:
                copyright_ += text__content_.nodeValue
            self.copyright = copyright_
        elif child_.nodeType == Node.ELEMENT_NODE and \
            nodeName_ == 'author':
            author_ = ''
            for text__content_ in child_.childNodes:
                author_ += text__content_.nodeValue
            self.author = author_
        elif child_.nodeType == Node.ELEMENT_NODE and \
            nodeName_ == 'date':
            date_ = ''
            for text__content_ in child_.childNodes:
                date_ += text__content_.nodeValue
            self.date = date_
# end class ALCommandLineInterfaceElement




def parse(inFileName):
    doc = minidom.parse(inFileName)
    rootNode = doc.documentElement
    rootObj = ALCommandLineInterfaceElement.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc.unlink()
#    doc = None
    return rootObj


def parseString(inString):
    doc = minidom.parseString(inString)
    rootNode = doc.documentElement
    rootObj = ALCommandLineInterfaceElement.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc.unlink()
#    doc = None
#    sys.stdout.write('<?xml version="1.0" ?>\n')
#    rootObj.export(sys.stdout, 0, name_="ALCommandLineInterfaceElement")
    return rootObj


# AAlib Modifications for marshalling and unmarshalling
# Creating program data constructs from XML
def unMarshalling( _inFileName ):
    return parse(  _inFileName )


# Serializing program data constructs to XML
def marshalling( _oObject, _inFileName ):
    oFile = open( _inFileName, "w" )
    oFile.write('<?xml version="1.0" ?>\n')
    _oObject.export( oFile, 0, name_="ALCommandLineInterfaceElement" )
    oFile.close()

