#!/usr/bin/env python

#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################

#
#
#
# Generated by ALExternalPackGenerateDS.py.
#
# This file is generated by a modified version of the original file:
# generateDS.py, proposed by Dave Kuhlman:
#
## LICENSE
#
## Copyright (c) 2003 Dave Kuhlman
#
## Permission is hereby granted, free of charge, to any person obtaining
## a copy of this software and associated documentation files (the
## "Software"), to deal in the Software without restriction, including
## without limitation the rights to use, copy, modify, merge, publish,
## distribute, sublicense, and/or sell copies of the Software, and to
## permit persons to whom the Software is furnished to do so, subject to
## the following conditions:
#
## The above copyright notice and this permission notice shall be
## included in all copies or substantial portions of the Software.
#
## THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
## EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
## MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
## IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
## CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
## TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
## SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
##############################################################################

import sys
import getopt
from xml.dom import minidom
from xml.dom import Node


#
# Support/utility functions.
#

def showIndent(outfile, level):
    for idx in range(level):
        outfile.write('    ')


def showIndentHtml(outfile, level):
    for idx in range(level):
        outfile.write(' &nbsp; &nbsp; &nbsp; &nbsp;')


def quote_xml(inStr):
    s1 = inStr
    s1 = s1.replace('&', '&amp;')
    s1 = s1.replace('<', '&lt;')
    s1 = s1.replace('"', '&quot;')
    return s1

def quote_python(inStr):
    s1 = inStr
    if s1.find("'") == -1:
        if s1.find('\n') == -1:
            return "'%s'" % s1
        else:
            return "'''%s'''" % s1
    else:
        if s1.find('"') != -1:
            s1 = s1.replace('"', '\\"')
        if s1.find('\n') == -1:
            return '"%s"' % s1
        else:
            return '"""%s"""' % s1


class MixedContainer(object):
    # Constants for category:
    CategoryNone = 0
    CategoryText = 1
    CategorySimple = 2
    CategoryComplex = 3
    # Constants for content_type:
    TypeNone = 0
    TypeText = 1
    TypeString = 2
    TypeInteger = 3
    TypeFloat = 4
    TypeDecimal = 5
    TypeDouble = 6
    TypeBoolean = 7
    def __init__(self, category, content_type, name, value):
        self.category = category
        self.content_type = content_type
        self.name = name
        self.value = value
    def getCategory(self):
        return self.category
    def getContenttype(self, content_type):
        return self.content_type
    def getValue(self):
        return self.value
    def getName(self):
        return self.name
    def export(self, outfile, level, name):
        if self.category == MixedContainer.CategoryText:
            outfile.write(self.value)
        elif self.category == MixedContainer.CategorySimple:
            self.exportSimple(outfile, level, name)
        else:    # category == MixedContainer.CategoryComplex
            self.value.export(outfile, level, name)
    def exportSimple(self, outfile, level, name):
        if self.content_type == MixedContainer.TypeString:
            outfile.write('<%s>%s</%s>' % (self.name, self.value, self.name))
        elif self.content_type == MixedContainer.TypeInteger or \
                self.content_type == MixedContainer.TypeBoolean:
            outfile.write('<%s>%d</%s>' % (self.name, self.value, self.name))
        elif self.content_type == MixedContainer.TypeFloat or \
                self.content_type == MixedContainer.TypeDecimal:
            outfile.write('<%s>%f</%s>' % (self.name, self.value, self.name))
        elif self.content_type == MixedContainer.TypeDouble:
            outfile.write('<%s>%g</%s>' % (self.name, self.value, self.name))
    def exportLiteral(self, outfile, level, name):
        if self.category == MixedContainer.CategoryText:
            showIndent(outfile, level)
            outfile.write('MixedContainer(%d, %d, "%s", "%s"),\n' % \
                (self.category, self.content_type, self.name, self.value))
        elif self.category == MixedContainer.CategorySimple:
            showIndent(outfile, level)
            outfile.write('MixedContainer(%d, %d, "%s", "%s"),\n' % \
                (self.category, self.content_type, self.name, self.value))
        else:    # category == MixedContainer.CategoryComplex
            showIndent(outfile, level)
            outfile.write('MixedContainer(%d, %d, "%s",\n' % \
                (self.category, self.content_type, self.name,))
            self.value.exportLiteral(outfile, level + 1)
            showIndent(outfile, level)
            outfile.write(')\n')



TEMPLATE_PAGE_P1 = """
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en">
<head>
    <title>AALib Plugin Data Description</title>
    <meta http-equiv="content-type" content="text/html; charset=utf-8">
    <meta name="description" content="AALib Plugin generation from XSD files">
    <meta name="keywords" content="AALib, xsd, plugin">
    <meta name="author" content="Romeu Pieritz(romeu_pieritz@yahoo.fr)">
    <link href="style.css" rel="stylesheet" type="text/css">
    <base target="main">
</head>
<body>
    <div id="main">
        <div id="block">
          %s
    </div>
"""

TEMPLATE_PAGE_P2 = """
    </div>
</body>
</html>
"""



#
# Class definition
#

class ALDataManagerProcess(object):
    subclass = None
    def __init__(self, date='', version='', name='', information='', author='', listDataProcess=None):
        self.date = date
        self.version = version
        self.name = name
        self.information = information
        self.author = author
        self.listDataProcess = listDataProcess
    def factory(*args_, **kwargs_):
        if ALDataManagerProcess.subclass:
            return ALDataManagerProcess.subclass(*args_, **kwargs_)
        else:
            return ALDataManagerProcess(*args_, **kwargs_)
    factory = staticmethod(factory)
    def getDate(self): return self.date
    def setDate(self, date): self.date = date
    def getVersion(self): return self.version
    def setVersion(self, version): self.version = version
    def getName(self): return self.name
    def setName(self, name): self.name = name
    def getInformation(self): return self.information
    def setInformation(self, information): self.information = information
    def getAuthor(self): return self.author
    def setAuthor(self, author): self.author = author
    def getListDataProcess(self): return self.listDataProcess
    def setListDataProcess(self, listDataProcess): self.listDataProcess = listDataProcess
    def export(self, outfile, level = 0, name_='ALDataManagerProcess'):
        showIndent(outfile, level)
        outfile.write('<%s>\n' % name_)
        self.exportChildren(outfile, level + 1, name_)
        showIndent(outfile, level)
        outfile.write('</%s>\n' % name_)
    def exportAttributes(self, outfile, level, name_='ALDataManagerProcess'):
        pass
    def exportChildren(self, outfile, level, name_='ALDataManagerProcess'):
        showIndent(outfile, level)
        outfile.write('<date>%s</date>\n' % quote_xml(self.getDate()))
        showIndent(outfile, level)
        outfile.write('<version>%s</version>\n' % quote_xml(self.getVersion()))
        showIndent(outfile, level)
        outfile.write('<name>%s</name>\n' % quote_xml(self.getName()))
        showIndent(outfile, level)
        outfile.write('<information>%s</information>\n' % quote_xml(self.getInformation()))
        showIndent(outfile, level)
        outfile.write('<author>%s</author>\n' % quote_xml(self.getAuthor()))
        if self.listDataProcess:
            self.listDataProcess.export(outfile, level, name_='listDataProcess')

    #Only to export the entire XML tree to a file stream on disk
    def outputFile( self, _outfileName ):
        outfile = open( _outfileName, "w" )
        outfile.write("<?xml version=\"1.0\" ?>\n")
        self.export( outfile, 0, name_='ALDataManagerProcess' )
        outfile.close()


    #Only to export the entire XML tree to a Html Page in a file stream on disk
    def outputPageHtml( self, _outfileName ):
        odtStrTitle = "    <span class=\"title\">ALDataManagerProcess:</span> <br> <br> <br>\n"
        outfile = open( _outfileName, "w" )
        outfile.write( TEMPLATE_PAGE_P1 % ( odtStrTitle ) )
        self.exportLiteralHtml( outfile, 0, name_='ALDataManagerProcess' )
        outfile.write( TEMPLATE_PAGE_P2  )
        outfile.close()


    def outputFileHtml( self, _outfileName ):
        self.outputPageHtml( _outfileName )


    def exportLiteral(self, outfile, level = 0, name_='ALDataManagerProcess'):
        level += 1
        self.exportLiteralAttributes(outfile, level, name_)
        self.exportLiteralChildren(outfile, level, name_)
    def exportLiteralAttributes(self, outfile, level, name_):
        pass
    def exportLiteralChildren(self, outfile, level, name_):
        showIndent(outfile, level)
        outfile.write('date = %s,\n' % quote_python(self.getDate()))
        showIndent(outfile, level)
        outfile.write('version = %s,\n' % quote_python(self.getVersion()))
        showIndent(outfile, level)
        outfile.write('name = %s,\n' % quote_python(self.getName()))
        showIndent(outfile, level)
        outfile.write('information = %s,\n' % quote_python(self.getInformation()))
        showIndent(outfile, level)
        outfile.write('author = %s,\n' % quote_python(self.getAuthor()))
        if self.listDataProcess:
            showIndent(outfile, level)
            outfile.write('listDataProcess = ALDataListDataProcess(\n')
            self.listDataProcess.exportLiteral(outfile, level, name_='listDataProcess')
            showIndent(outfile, level)
            outfile.write('),\n')
    def exportLiteralHtml(self, outfile, level = 0, name_='ALDataManagerProcess'):
        level += 1
        self.exportLiteralAttributesHtml(outfile, level, name_)
        self.exportLiteralChildrenHtml(outfile, level, name_)
    def exportLiteralAttributesHtml(self, outfile, level, name_):
        pass
    def exportLiteralChildrenHtml(self, outfile, level, name_):
        showIndentHtml(outfile, level)
        outfile.write('date = <a  href:"#">%s</a><br>\n' % quote_python(self.getDate()))
        showIndentHtml(outfile, level)
        outfile.write('version = <a  href:"#">%s</a><br>\n' % quote_python(self.getVersion()))
        showIndentHtml(outfile, level)
        outfile.write('name = <a  href:"#">%s</a><br>\n' % quote_python(self.getName()))
        showIndentHtml(outfile, level)
        outfile.write('information = <a  href:"#">%s</a><br>\n' % quote_python(self.getInformation()))
        showIndentHtml(outfile, level)
        outfile.write('author = <a  href:"#">%s</a><br>\n' % quote_python(self.getAuthor()))
        if self.listDataProcess:
            showIndentHtml(outfile, level)
            outfile.write('listDataProcess<br>\n')
            self.listDataProcess.exportLiteralHtml(outfile, level, name_='listDataProcess')
            showIndentHtml(outfile, level)
            outfile.write('<br>\n')
    def build(self, node_):
        attrs = node_.attributes
        self.buildAttributes(attrs)
        for child_ in node_.childNodes:
            nodeName_ = child_.nodeName.split(':')[-1]
            self.buildChildren(child_, nodeName_)
    def buildAttributes(self, attrs):
        pass
    def buildChildren(self, child_, nodeName_):
        if child_.nodeType == Node.ELEMENT_NODE and \
            nodeName_ == 'date':
            date_ = ''
            for text__content_ in child_.childNodes:
                date_ += text__content_.nodeValue
            self.date = date_
        elif child_.nodeType == Node.ELEMENT_NODE and \
            nodeName_ == 'version':
            version_ = ''
            for text__content_ in child_.childNodes:
                version_ += text__content_.nodeValue
            self.version = version_
        elif child_.nodeType == Node.ELEMENT_NODE and \
            nodeName_ == 'name':
            name_ = ''
            for text__content_ in child_.childNodes:
                name_ += text__content_.nodeValue
            self.name = name_
        elif child_.nodeType == Node.ELEMENT_NODE and \
            nodeName_ == 'information':
            information_ = ''
            for text__content_ in child_.childNodes:
                information_ += text__content_.nodeValue
            self.information = information_
        elif child_.nodeType == Node.ELEMENT_NODE and \
            nodeName_ == 'author':
            author_ = ''
            for text__content_ in child_.childNodes:
                author_ += text__content_.nodeValue
            self.author = author_
        elif child_.nodeType == Node.ELEMENT_NODE and \
            nodeName_ == 'listDataProcess':
            obj_ = ALDataListDataProcess.factory()
            obj_.build(child_)
            self.setListDataProcess(obj_)
# end class ALDataManagerProcess


class ALDataListDataProcess(object):
    subclass = None
    def __init__(self, dataProcess=None):
        if dataProcess is None:
            self.dataProcess = []
        else:
            self.dataProcess = dataProcess
    def factory(*args_, **kwargs_):
        if ALDataListDataProcess.subclass:
            return ALDataListDataProcess.subclass(*args_, **kwargs_)
        else:
            return ALDataListDataProcess(*args_, **kwargs_)
    factory = staticmethod(factory)
    def getDataProcess(self): return self.dataProcess
    def setDataProcess(self, dataProcess): self.dataProcess = dataProcess
    def addDataProcess(self, value): self.dataProcess.append(value)
    def insertDataProcess(self, index, value): self.dataProcess[index] = value
    def export(self, outfile, level = 0, name_='ALDataListDataProcess'):
        showIndent(outfile, level)
        outfile.write('<%s>\n' % name_)
        self.exportChildren(outfile, level + 1, name_)
        showIndent(outfile, level)
        outfile.write('</%s>\n' % name_)
    def exportAttributes(self, outfile, level, name_='ALDataListDataProcess'):
        pass
    def exportChildren(self, outfile, level, name_='ALDataListDataProcess'):
        for dataProcess_ in self.getDataProcess():
            dataProcess_.export(outfile, level, name_='dataProcess')

    #Only to export the entire XML tree to a file stream on disk
    def outputFile( self, _outfileName ):
        outfile = open( _outfileName, "w" )
        outfile.write("<?xml version=\"1.0\" ?>\n")
        self.export( outfile, 0, name_='ALDataListDataProcess' )
        outfile.close()


    #Only to export the entire XML tree to a Html Page in a file stream on disk
    def outputPageHtml( self, _outfileName ):
        odtStrTitle = "    <span class=\"title\">ALDataListDataProcess:</span> <br> <br> <br>\n"
        outfile = open( _outfileName, "w" )
        outfile.write( TEMPLATE_PAGE_P1 % ( odtStrTitle ) )
        self.exportLiteralHtml( outfile, 0, name_='ALDataListDataProcess' )
        outfile.write( TEMPLATE_PAGE_P2  )
        outfile.close()


    def outputFileHtml( self, _outfileName ):
        self.outputPageHtml( _outfileName )


    def exportLiteral(self, outfile, level = 0, name_='ALDataListDataProcess'):
        level += 1
        self.exportLiteralAttributes(outfile, level, name_)
        self.exportLiteralChildren(outfile, level, name_)
    def exportLiteralAttributes(self, outfile, level, name_):
        pass
    def exportLiteralChildren(self, outfile, level, name_):
        showIndent(outfile, level)
        outfile.write('dataProcess=[\n')
        level += 1
        for dataProcess in self.dataProcess:
            showIndent(outfile, level)
            outfile.write('ALDataProcess(\n')
            dataProcess.exportLiteral(outfile, level, name_='dataProcess')
            showIndent(outfile, level)
            outfile.write('),\n')
        level -= 1
        showIndent(outfile, level)
        outfile.write('],\n')
    def exportLiteralHtml(self, outfile, level = 0, name_='ALDataListDataProcess'):
        level += 1
        self.exportLiteralAttributesHtml(outfile, level, name_)
        self.exportLiteralChildrenHtml(outfile, level, name_)
    def exportLiteralAttributesHtml(self, outfile, level, name_):
        pass
    def exportLiteralChildrenHtml(self, outfile, level, name_):
        showIndentHtml(outfile, level)
        outfile.write('dataProcess<br>\n')
        level += 1
        for dataProcess in self.dataProcess:
            dataProcess.exportLiteralHtml(outfile, level, name_='dataProcess')
            showIndentHtml(outfile, level)
            outfile.write('<br>\n')
        level -= 1
        showIndentHtml(outfile, level)
        outfile.write('<br>\n')
    def build(self, node_):
        attrs = node_.attributes
        self.buildAttributes(attrs)
        for child_ in node_.childNodes:
            nodeName_ = child_.nodeName.split(':')[-1]
            self.buildChildren(child_, nodeName_)
    def buildAttributes(self, attrs):
        pass
    def buildChildren(self, child_, nodeName_):
        if child_.nodeType == Node.ELEMENT_NODE and \
            nodeName_ == 'dataProcess':
            obj_ = ALDataProcess.factory()
            obj_.build(child_)
            self.dataProcess.append(obj_)
# end class ALDataListDataProcess


class ALDataProcess(object):
    subclass = None
    def __init__(self, command='', imageName='', pid=-1, sessionName='', session='', memoryUsage='', status='', userName='', cpuTime='', windowTitle=''):
        self.command = command
        self.imageName = imageName
        self.pid = pid
        self.sessionName = sessionName
        self.session = session
        self.memoryUsage = memoryUsage
        self.status = status
        self.userName = userName
        self.cpuTime = cpuTime
        self.windowTitle = windowTitle
    def factory(*args_, **kwargs_):
        if ALDataProcess.subclass:
            return ALDataProcess.subclass(*args_, **kwargs_)
        else:
            return ALDataProcess(*args_, **kwargs_)
    factory = staticmethod(factory)
    def getCommand(self): return self.command
    def setCommand(self, command): self.command = command
    def getImageName(self): return self.imageName
    def setImageName(self, imageName): self.imageName = imageName
    def getPid(self): return self.pid
    def setPid(self, pid): self.pid = pid
    def getSessionName(self): return self.sessionName
    def setSessionName(self, sessionName): self.sessionName = sessionName
    def getSession(self): return self.session
    def setSession(self, session): self.session = session
    def getMemoryUsage(self): return self.memoryUsage
    def setMemoryUsage(self, memoryUsage): self.memoryUsage = memoryUsage
    def getStatus(self): return self.status
    def setStatus(self, status): self.status = status
    def getUserName(self): return self.userName
    def setUserName(self, userName): self.userName = userName
    def getCpuTime(self): return self.cpuTime
    def setCpuTime(self, cpuTime): self.cpuTime = cpuTime
    def getWindowTitle(self): return self.windowTitle
    def setWindowTitle(self, windowTitle): self.windowTitle = windowTitle
    def export(self, outfile, level = 0, name_='ALDataProcess'):
        showIndent(outfile, level)
        outfile.write('<%s>\n' % name_)
        self.exportChildren(outfile, level + 1, name_)
        showIndent(outfile, level)
        outfile.write('</%s>\n' % name_)
    def exportAttributes(self, outfile, level, name_='ALDataProcess'):
        pass
    def exportChildren(self, outfile, level, name_='ALDataProcess'):
        showIndent(outfile, level)
        outfile.write('<command>%s</command>\n' % quote_xml(self.getCommand()))
        showIndent(outfile, level)
        outfile.write('<imageName>%s</imageName>\n' % quote_xml(self.getImageName()))
        showIndent(outfile, level)
        outfile.write('<pid>%d</pid>\n' % self.getPid())
        showIndent(outfile, level)
        outfile.write('<sessionName>%s</sessionName>\n' % quote_xml(self.getSessionName()))
        showIndent(outfile, level)
        outfile.write('<session>%s</session>\n' % quote_xml(self.getSession()))
        showIndent(outfile, level)
        outfile.write('<memoryUsage>%s</memoryUsage>\n' % quote_xml(self.getMemoryUsage()))
        showIndent(outfile, level)
        outfile.write('<status>%s</status>\n' % quote_xml(self.getStatus()))
        showIndent(outfile, level)
        outfile.write('<userName>%s</userName>\n' % quote_xml(self.getUserName()))
        showIndent(outfile, level)
        outfile.write('<cpuTime>%s</cpuTime>\n' % quote_xml(self.getCpuTime()))
        showIndent(outfile, level)
        outfile.write('<windowTitle>%s</windowTitle>\n' % quote_xml(self.getWindowTitle()))

    #Only to export the entire XML tree to a file stream on disk
    def outputFile( self, _outfileName ):
        outfile = open( _outfileName, "w" )
        outfile.write("<?xml version=\"1.0\" ?>\n")
        self.export( outfile, 0, name_='ALDataProcess' )
        outfile.close()


    #Only to export the entire XML tree to a Html Page in a file stream on disk
    def outputPageHtml( self, _outfileName ):
        odtStrTitle = "    <span class=\"title\">ALDataProcess:</span> <br> <br> <br>\n"
        outfile = open( _outfileName, "w" )
        outfile.write( TEMPLATE_PAGE_P1 % ( odtStrTitle ) )
        self.exportLiteralHtml( outfile, 0, name_='ALDataProcess' )
        outfile.write( TEMPLATE_PAGE_P2  )
        outfile.close()


    def outputFileHtml( self, _outfileName ):
        self.outputPageHtml( _outfileName )


    def exportLiteral(self, outfile, level = 0, name_='ALDataProcess'):
        level += 1
        self.exportLiteralAttributes(outfile, level, name_)
        self.exportLiteralChildren(outfile, level, name_)
    def exportLiteralAttributes(self, outfile, level, name_):
        pass
    def exportLiteralChildren(self, outfile, level, name_):
        showIndent(outfile, level)
        outfile.write('command = %s,\n' % quote_python(self.getCommand()))
        showIndent(outfile, level)
        outfile.write('imageName = %s,\n' % quote_python(self.getImageName()))
        showIndent(outfile, level)
        outfile.write('pid = %d,\n' % self.getPid())
        showIndent(outfile, level)
        outfile.write('sessionName = %s,\n' % quote_python(self.getSessionName()))
        showIndent(outfile, level)
        outfile.write('session = %s,\n' % quote_python(self.getSession()))
        showIndent(outfile, level)
        outfile.write('memoryUsage = %s,\n' % quote_python(self.getMemoryUsage()))
        showIndent(outfile, level)
        outfile.write('status = %s,\n' % quote_python(self.getStatus()))
        showIndent(outfile, level)
        outfile.write('userName = %s,\n' % quote_python(self.getUserName()))
        showIndent(outfile, level)
        outfile.write('cpuTime = %s,\n' % quote_python(self.getCpuTime()))
        showIndent(outfile, level)
        outfile.write('windowTitle = %s,\n' % quote_python(self.getWindowTitle()))
    def exportLiteralHtml(self, outfile, level = 0, name_='ALDataProcess'):
        level += 1
        self.exportLiteralAttributesHtml(outfile, level, name_)
        self.exportLiteralChildrenHtml(outfile, level, name_)
    def exportLiteralAttributesHtml(self, outfile, level, name_):
        pass
    def exportLiteralChildrenHtml(self, outfile, level, name_):
        showIndentHtml(outfile, level)
        outfile.write('command = <a  href:"#">%s</a><br>\n' % quote_python(self.getCommand()))
        showIndentHtml(outfile, level)
        outfile.write('imageName = <a  href:"#">%s</a><br>\n' % quote_python(self.getImageName()))
        showIndentHtml(outfile, level)
        outfile.write('pid = <a  href:"#">%d</a><br>\n' % self.getPid())
        showIndentHtml(outfile, level)
        outfile.write('sessionName = <a  href:"#">%s</a><br>\n' % quote_python(self.getSessionName()))
        showIndentHtml(outfile, level)
        outfile.write('session = <a  href:"#">%s</a><br>\n' % quote_python(self.getSession()))
        showIndentHtml(outfile, level)
        outfile.write('memoryUsage = <a  href:"#">%s</a><br>\n' % quote_python(self.getMemoryUsage()))
        showIndentHtml(outfile, level)
        outfile.write('status = <a  href:"#">%s</a><br>\n' % quote_python(self.getStatus()))
        showIndentHtml(outfile, level)
        outfile.write('userName = <a  href:"#">%s</a><br>\n' % quote_python(self.getUserName()))
        showIndentHtml(outfile, level)
        outfile.write('cpuTime = <a  href:"#">%s</a><br>\n' % quote_python(self.getCpuTime()))
        showIndentHtml(outfile, level)
        outfile.write('windowTitle = <a  href:"#">%s</a><br>\n' % quote_python(self.getWindowTitle()))
    def build(self, node_):
        attrs = node_.attributes
        self.buildAttributes(attrs)
        for child_ in node_.childNodes:
            nodeName_ = child_.nodeName.split(':')[-1]
            self.buildChildren(child_, nodeName_)
    def buildAttributes(self, attrs):
        pass
    def buildChildren(self, child_, nodeName_):
        if child_.nodeType == Node.ELEMENT_NODE and \
            nodeName_ == 'command':
            command_ = ''
            for text__content_ in child_.childNodes:
                command_ += text__content_.nodeValue
            self.command = command_
        elif child_.nodeType == Node.ELEMENT_NODE and \
            nodeName_ == 'imageName':
            imageName_ = ''
            for text__content_ in child_.childNodes:
                imageName_ += text__content_.nodeValue
            self.imageName = imageName_
        elif child_.nodeType == Node.ELEMENT_NODE and \
            nodeName_ == 'pid':
            if child_.firstChild:
                sval_ = child_.firstChild.nodeValue
                try:
                    ival_ = int(sval_)
                except ValueError:
                    raise ValueError('requires integer -- %s' % child_.toxml())
                self.pid = ival_
        elif child_.nodeType == Node.ELEMENT_NODE and \
            nodeName_ == 'sessionName':
            sessionName_ = ''
            for text__content_ in child_.childNodes:
                sessionName_ += text__content_.nodeValue
            self.sessionName = sessionName_
        elif child_.nodeType == Node.ELEMENT_NODE and \
            nodeName_ == 'session':
            session_ = ''
            for text__content_ in child_.childNodes:
                session_ += text__content_.nodeValue
            self.session = session_
        elif child_.nodeType == Node.ELEMENT_NODE and \
            nodeName_ == 'memoryUsage':
            memoryUsage_ = ''
            for text__content_ in child_.childNodes:
                memoryUsage_ += text__content_.nodeValue
            self.memoryUsage = memoryUsage_
        elif child_.nodeType == Node.ELEMENT_NODE and \
            nodeName_ == 'status':
            status_ = ''
            for text__content_ in child_.childNodes:
                status_ += text__content_.nodeValue
            self.status = status_
        elif child_.nodeType == Node.ELEMENT_NODE and \
            nodeName_ == 'userName':
            userName_ = ''
            for text__content_ in child_.childNodes:
                userName_ += text__content_.nodeValue
            self.userName = userName_
        elif child_.nodeType == Node.ELEMENT_NODE and \
            nodeName_ == 'cpuTime':
            cpuTime_ = ''
            for text__content_ in child_.childNodes:
                cpuTime_ += text__content_.nodeValue
            self.cpuTime = cpuTime_
        elif child_.nodeType == Node.ELEMENT_NODE and \
            nodeName_ == 'windowTitle':
            windowTitle_ = ''
            for text__content_ in child_.childNodes:
                windowTitle_ += text__content_.nodeValue
            self.windowTitle = windowTitle_
# end class ALDataProcess




def parse(inFileName):
    doc = minidom.parse(inFileName)
    rootNode = doc.documentElement
    rootObj = ALDataManagerProcess.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc.unlink()
#    doc = None
    return rootObj


def parseString(inString):
    doc = minidom.parseString(inString)
    rootNode = doc.documentElement
    rootObj = ALDataManagerProcess.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc.unlink()
#    doc = None
#    sys.stdout.write('<?xml version="1.0" ?>\n')
#    rootObj.export(sys.stdout, 0, name_="ALDataManagerProcess")
    return rootObj


# AAlib Modifications for marshalling and unmarshalling
# Creating program data constructs from XML
def unMarshalling( _inFileName ):
    return parse(  _inFileName )


# Serializing program data constructs to XML
def marshalling( _oObject, _inFileName ):
    oFile = open( _inFileName, "w" )
    oFile.write('<?xml version="1.0" ?>\n')
    _oObject.export( oFile, 0, name_="ALDataManagerProcess" )
    oFile.close()

