#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################



from ALImportPython            import PySys
from ALObject                  import ALObject
from ALSystem                  import ALSystem
from ALString                  import ALString
from ALSemaphore               import ALSemaphore
from ALTrace                   import ALTrace
from ALVerbose                 import ALVerbose


LICENSE_LGPL3_PYAALIB_JYAALIB = """\
#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################


"""


class ALDebug( ALObject ):
    """
    Interface for ALDebug control objects.
    """

    m_obStateDebugMode        = False
    m_obStateDebugModeForce   = False
    m_obStateDebugModeAssert  = False
    m_obStateDebugModeVerbose = False
    m_obStateDebugModeLogFile = True
    m_obStateDebugModeAction  = False
    m_obStateDebugModeProfile = False
    m_obStateDebugLogFile     = False
    m_oalStringLogFileName    = None
    m_oalSemaphoreDebug       = ALSemaphore()
    m_obLicenseOutput         = False


    def __init__( self ):
        ALObject.__init__( self )


    def __analyseCommandLine():
        ALDebug.synchronizeOn()
        for cmdkey in PySys.argv:
            if ( cmdkey == "--DEBUG-FORCE" or
                 cmdkey == "--DEBUGFORCE" ):
                ALDebug.m_obStateDebugModeForce   = True

            if ( cmdkey == "--DEBUG" ):
                ALDebug.m_obStateDebugMode = True

            if ( cmdkey == "--DEBUG-TAG" ):
                ALDebug.m_obStateDebugMode = True
                for itag in range(0, len( PySys.argv ) ):
                    if ( PySys.argv[itag] == "--DEBUG-TAG" ):
                        ALVerbose.setDebugTag( PySys.argv[itag+1] )

            if ( cmdkey == "--ASSERT" ):
                ALDebug.m_obStateDebugModeAssert = True

            if ( cmdkey == "--verbose" ):
                ALDebug.m_obStateDebugModeVerbose = True

            if ( cmdkey == "--profile" ):
                ALDebug.m_obStateDebugModeProfile = True

            if ( cmdkey == "--log" ):
                ALDebug.m_obStateDebugModeLogFile = False

            if ( cmdkey == "--DEBUG-FORCE-PLUGIN" or
                 cmdkey == "--DEBUG-PLUGIN" ):
                ALDebug.m_obStateDebugModeAction = True
                
            if ( cmdkey == "--license" and ALDebug.m_obLicenseOutput == False ):
                ALDebug.m_obLicenseOutput = True
                print LICENSE_LGPL3_PYAALIB_JYAALIB
    
        ALDebug.synchronizeOff()

    __analyseCommandLine = staticmethod( __analyseCommandLine )




    def synchronizeOn():
        """
        This method must be used in together with the method synchronizeOff().
        This method makes the code threadsafe until the method synchronizeOff
        is called.
        """
        ALDebug.m_oalSemaphoreDebug.acquire()
    synchronizeOn = staticmethod( synchronizeOn )


    def synchronizeOff():
        """
        This method must be used in together with the method synchronizeOn().
        """
        ALDebug.m_oalSemaphoreDebug.release()
    synchronizeOff = staticmethod( synchronizeOff)



    def controlDebug( _oalObject = None, _oBoolScreen = True ):
        if (ALDebug.isDebugModeForceOn()==True):
            ALDebug.debugForce( _oalObject, False )
        else:
            ALTrace.trace( _oalObject, _oBoolScreen )
    controlDebug = staticmethod( controlDebug )



    def debugForce( _oalObject = None, _oBoolScreen = True ):
        ALTrace.trace( _oalObject, _oBoolScreen )
        raise
    debugForce = staticmethod( debugForce )



    def isDebugModeOn():
        if (ALDebug.m_obStateDebugMode==True):
            return ALDebug.m_obStateDebugMode
        ALDebug.__analyseCommandLine()
        return ALDebug.m_obStateDebugMode
    isDebugModeOn = staticmethod( isDebugModeOn )


    def isDebugModeCommand():
        return ALDebug.isDebugModeOn()
    isDebugModeCommand = staticmethod( isDebugModeCommand )


    def isDebugModeForceOn():
        if (ALDebug.m_obStateDebugModeForce==True):
            return ALDebug.m_obStateDebugModeForce
        ALDebug.__analyseCommandLine()
        return ALDebug.m_obStateDebugModeForce
    isDebugModeForceOn = staticmethod( isDebugModeForceOn )


    def isDebugModeVerboseOn():
        if (ALDebug.m_obStateDebugModeVerbose==True):
            return ALDebug.m_obStateDebugModeVerbose
        ALDebug.__analyseCommandLine()
        return ALDebug.m_obStateDebugModeVerbose
    isDebugModeVerboseOn = staticmethod( isDebugModeVerboseOn )


    def isDebugModeLogFileOn():
        if (ALDebug.m_obStateDebugModeLogFile==False):
            return ALDebug.m_obStateDebugModeLogFile
        ALDebug.__analyseCommandLine()
        return ALDebug.m_obStateDebugModeLogFile
    isDebugModeLogFileOn = staticmethod( isDebugModeLogFileOn )


    def isDebugModeAssertOn():
        if (ALDebug.m_obStateDebugModeAssert==True):
            return ALDebug.m_obStateDebugModeAssert
        ALDebug.__analyseCommandLine()
        return ALDebug.m_obStateDebugModeAssert
    isDebugModeAssertOn = staticmethod( isDebugModeAssertOn )


    def isDebugModeActionOn():
        if (ALDebug.m_obStateDebugModeAction==True):
            return ALDebug.m_obStateDebugModeAction
        ALDebug.__analyseCommandLine()
        return ALDebug.m_obStateDebugModeAction
    isDebugModeActionOn = staticmethod( isDebugModeActionOn )



    def isDebugModeProfileOn():
        if (ALDebug.m_obStateDebugModeProfile==True):
            return ALDebug.m_obStateDebugModeProfile
        ALDebug.__analyseCommandLine()
        return ALDebug.m_obStateDebugModeProfile
    isDebugModeProfileOn = staticmethod( isDebugModeProfileOn )


    def isDebugModePluginOn():
        return ALDebug.isDebugModeActionOn()
    isDebugModePluginOn = staticmethod( isDebugModePluginOn )


###################################################################

    def setDebugModeVerboseOn():
        ALDebug.synchronizeOn()
        ALDebug.m_obStateDebugModeVerbose = True
        ALDebug.synchronizeOff()
    setDebugModeVerboseOn = staticmethod( setDebugModeVerboseOn )


    def setDebugModeActionOn():
        ALDebug.synchronizeOn()
        ALDebug.m_obStateDebugModeAction = True
        ALDebug.synchronizeOff()
    setDebugModeActionOn = staticmethod( setDebugModeActionOn )


    def setDebugModePluginOn():
        ALDebug.setDebugModeActionOn()
    setDebugModePluginOn = staticmethod( setDebugModePluginOn )


    def setDebugModeProfileOn():
        ALDebug.synchronizeOn()
        ALDebug.m_obStateDebugModeProfile = True
        ALDebug.synchronizeOff()
    setDebugModeProfileOn = staticmethod( setDebugModeProfileOn )


###################################################################


    def setDebugModeOn():
        ALDebug.synchronizeOn()
        ALVerbose.setVerboseDebugOn()
        ALDebug.synchronizeOff()
        ALDebug.setDebugModeVerboseOn()
    setDebugModeOn = staticmethod( setDebugModeOn )


    def setDebugModeForceOn():
        ALDebug.setDebugModeOn()
        ALDebug.synchronizeOn()
        ALDebug.m_obStateDebugModeForce=True
        ALDebug.synchronizeOff()
    setDebugModeForceOn = staticmethod( setDebugModeForceOn )


    def setDebugModeOff():
        ALDebug.synchronizeOn()
        ALVerbose.setVerboseDebugOff()
        ALDebug.synchronizeOff()
    setDebugModeOff = staticmethod( setDebugModeOff )


    def setDebugProfileOn():
        ALDebug.synchronizeOn()
        ALVerbose.setProfileOn()
        ALDebug.synchronizeOff()
    setDebugProfileOn = staticmethod( setDebugProfileOn )


    def setDebugProfileOff():
        ALDebug.synchronizeOn()
        ALVerbose.setProfileOff()
        ALDebug.synchronizeOff()
    setDebugProfileOff = staticmethod( setDebugProfileOff )


    def setDebugVerboseOn():
        ALDebug.setDebugModeVerboseOn()
        ALDebug.synchronizeOn()
        ALVerbose.setVerboseOn()
        ALDebug.synchronizeOff()
    setDebugVerboseOn = staticmethod( setDebugVerboseOn )


    def setDebugPluginOn():
        ALDebug.setDebugModeActionOn()
    setDebugPluginOn = staticmethod( setDebugPluginOn )

##############################################################################

    def setDebugLogFile( _oalString = "DebugLogFile.log" ):
        """
        You can lock the log file only one time during the runtime.
        That means: if the log file is locked, it returns the locked name
        """
        ALDebug.synchronizeOn()
        ALDebug.m_oalStringLogFileName = ALString( _oalString )
        ALDebug.m_obStateDebugLogFile = True
        ALVerbose.setLogFileNameUnique( ALDebug.m_oalStringLogFileName )
        ALDebug.m_oalStringLogFileName = ALVerbose.getLogFileName()
        ALDebug.synchronizeOff()
    setDebugLogFile = staticmethod( setDebugLogFile )


    def isDebugLogFile():
        ALDebug.synchronizeOn()
        oObject = ALDebug.m_obStateDebugLogFile
        ALDebug.synchronizeOff()
        return oObject
    isDebugLogFile = staticmethod( isDebugLogFile )


    def getDebugLogFileName():
        ALDebug.synchronizeOn()
        oObject = ALDebug.m_oalStringLogFileName
        ALDebug.synchronizeOff()
        return oObject
    getDebugLogFileName = staticmethod( getDebugLogFileName )

##############################################################################