#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################


from ALImportPython            import PyGlob
from ALImportPython            import PyOS
from ALImportPython            import PyShutil
from ALList                    import ALList
from ALObject                  import ALObject
from ALString                  import ALString
from ALSystem                  import ALSystem
from ALThreadingCondition      import ALThreadingCondition


class ALDiskExplorer( ALObject ):
    """
    Interface for AL disk folder and file manager.
    """

    m_oalThrCondDiskExplorer = ALThreadingCondition()
    m_oalList = None


    def __init__( self ):
        ALObject.__init__( self )


    def existFile( _oalString ):
        """
        Verify the existence of the file
        """
        return ALDiskExplorer.isFile( _oalString  )
    existFile = staticmethod( existFile )


    def isFile( _oalString ):
        """
        Verify the file
        """
#        print _oalString
        oalFilePath = ALDiskExplorer.normPath( _oalString )
        return PyOS.path.isfile( oalFilePath )
    isFile = staticmethod( isFile )


    def isDirectory( _oalString ):
        """
        Verify the directory
        """
        return PyOS.path.isdir( ALDiskExplorer.normPath( _oalString ) )
    isDirectory  = staticmethod( isDirectory )


    def isFolder( _oalString ):
        return ALDiskExplorer.isDirectory( _oalString )
    isFolder  = staticmethod( isFolder )


    def existFolder( _oalString ):
        return ALDiskExplorer.isDirectory( _oalString )
    existFolder  = staticmethod( existFolder )


    def createFolder( _oalString ):
        if (_oalString != None):
            if (ALDiskExplorer.existFolder( _oalString ) == False):
                oalString = ALDiskExplorer.absolutePath( _oalString )
                oalList = oalString.split( "/" )
                iFolder = ALString( "" )
                for i in oalList:
                    iFolder = iFolder + ALString( i )
                    if (ALSystem.getOSName()=="Windows"):
                        if (iFolder[1]==":" and len(iFolder)>2 ):
                            if (ALDiskExplorer.existFolder( iFolder ) == False):
                                PyOS.mkdir( iFolder )
                    else:
                        if (iFolder!=ALString.empty()):
                            if (ALDiskExplorer.existFolder( iFolder ) == False):
                                PyOS.mkdir( iFolder )
                    iFolder = iFolder + "/"
    createFolder  = staticmethod( createFolder )


    def deleteFolder( _oalString ):
        if (_oalString != None):
            if ( ALDiskExplorer.existFolder( _oalString ) == True ):
                PyOS.rmdir( ALDiskExplorer.normPath( _oalString ) )
    deleteFolder  = staticmethod( deleteFolder )


    def deleteFolderTree( _oalString ):
        """
        Warning: it deletes the tree in force...
        """
        if (_oalString == None):
            return
        if ( ALDiskExplorer.isFolder(_oalString)==False):
            return
        oalList = ALDiskExplorer.getCompleteFolderList( _oalString )
        i = len(oalList)-1
        while i>=0:
            oalString = oalList[i]
            if ( ALDiskExplorer.isFile(oalString)==True):
                ALDiskExplorer.deleteFile(oalString)
            elif ( ALDiskExplorer.isFolder(oalString)==True):
                ALDiskExplorer.deleteFolder(oalString)
            i = i-1
        if ( ALDiskExplorer.isFolder(_oalString)==True):
            ALDiskExplorer.deleteFolder(_oalString)
    deleteFolderTree  = staticmethod( deleteFolderTree )
    

    def copyFolderTree( _oalStrSource, _oalStrDestination, _oalSimbolicLinks = 0 ):
        """
        Warning: This function will be modified in the future to overwrite the local
        files in the target folder !!!!!
        """
        ALDiskExplorer.copyFolder( _oalStrSource, _oalStrDestination, _oalSimbolicLinks)
    copyFolderTree = staticmethod( copyFolderTree )

    
    
    
    
    def deleteFile( _oalString ):
        if (_oalString != None):
            if ( ALDiskExplorer.existFile( _oalString ) == True ):
                PyOS.remove( ALDiskExplorer.normPath( _oalString ) )
    deleteFile  = staticmethod( deleteFile )


    def isLink( _oalString ):
        """
        Verify the link
        """
        return PyOS.path.islink(  ALDiskExplorer.normPath( _oalString ) )
    isLink  = staticmethod( isLink )


    def getFileList( _oalStringPath = "./", _oalStringMask = "*" ):
        """
        This method get the file list of a folder by
        a mask - i.e.: "*.py"
        """
        ALDiskExplorer.m_oalThrCondDiskExplorer.synchronizeOn()
        oalStringFind = PyOS.path.join( _oalStringPath, _oalStringMask )
        oalListFile = ALList( PyGlob.glob( oalStringFind ) )
        ALDiskExplorer.m_oalThrCondDiskExplorer.synchronizeOff()
        for i in range( 0, len( oalListFile )):
            oalListFile[i] = ALDiskExplorer.normPath( oalListFile[i] )
        return oalListFile
    getFileList = staticmethod( getFileList )


    def getFolderList( _oalStringPath = "./", _oalStringMask = "*" ):
        return ALDiskExplorer.getFileList( _oalStringPath, _oalStringMask )
    getFolderList = staticmethod( getFolderList )




    
    def copyFilesType( _oOriginFolder, _oTargetFolder, _oExtension = "*" ):
        if (ALDiskExplorer.isFolder(_oTargetFolder)==False):
            ALDiskExplorer.createFolder( _oTargetFolder )
        oalListFile = ALDiskExplorer.getFolderList( _oOriginFolder, _oExtension)
        for i in range( 0, len( oalListFile )):
            oFileName = ALString( oalListFile[i] )
            oFileNameTarget = oFileName.replace( _oOriginFolder, _oTargetFolder )
           # print "  ->" + oFileNameTarget
            ALDiskExplorer.copyFile( oFileName, oFileNameTarget)
    copyFilesType = staticmethod( copyFilesType )
    
    


    def __addFolderInteract( _pyArg, _pyFolder, _pyFiles ):
        oalStrCurFolder       = ALDiskExplorer.getAbsolutePath( _pyFolder )
        oalListFile           = ALList( _pyFiles )
        for iFile in oalListFile:
            oalString = ALDiskExplorer.mergePath( oalStrCurFolder, iFile )
            ALDiskExplorer.m_oalList.addObject( oalString )    
    __addFolderInteract = staticmethod(__addFolderInteract)
    

    def getCompleteFolderList( _oalStringPath = "./" ):
        oalStrFolder = ALDiskExplorer.getAbsolutePath( _oalStringPath )
        if(ALDiskExplorer.existFolder( oalStrFolder )==False):
            ALDiskExplorer.m_oalList = ALList()
            return ALDiskExplorer.m_oalList
        else:
            ALDiskExplorer.m_oalThrCondDiskExplorer.synchronizeOn()
            ALDiskExplorer.m_oalList = ALList()
            ALDiskExplorer.iteractPath( oalStrFolder, ALDiskExplorer.__addFolderInteract, oalStrFolder )
            ALDiskExplorer.m_oalThrCondDiskExplorer.synchronizeOff()
        return ALDiskExplorer.m_oalList
    getCompleteFolderList = staticmethod( getCompleteFolderList )





    def folderName( _oalStringPath ):
        return PyOS.path.dirname( ALDiskExplorer.normPath( _oalStringPath ) )
    folderName = staticmethod( folderName )


    def getFolderName( _oalStringPath ):
        return PyOS.path.dirname( ALDiskExplorer.normPath( _oalStringPath ) )
    getFolderName = staticmethod( getFolderName )



    # FIle name manipulation ###########################################################
    def getFileName( _oalString ):
        """
        File name + extension = NO PATH
        """
        return ALString( PyOS.path.basename( ALDiskExplorer.normalize( _oalString ) ) )
    getFileName = staticmethod( getFileName )


    def getFileNameComplete( _oalString ):
        """
        Path + File name + extension 
        """
        oalString = ALString( ALDiskExplorer.normalize( _oalString ) )
        if (oalString.contains("/")==False and oalString.contains("\\")==False ):
            oalString = "./" + oalString
        return oalString
    getFileNameComplete = staticmethod( getFileNameComplete )

    
    def getFileBaseName( _oalString ):
        """
        FileName - Without extension
        """
        oalString = ALString( PyOS.path.basename( ALDiskExplorer.normalize( _oalString ) ) ) #possible bug with more than one point extension !!!!!
        oalList = ALList( PyOS.path.splitext( ALDiskExplorer.normalize( oalString ) ) )
        oalString2 = ALString()
        for i in range( len( oalList )-1 ):
            oalString2 = oalString2 + ALString( oalList[i] )
        return oalString2
    getFileBaseName = staticmethod( getFileBaseName )


    def getFileExtension( _oalString ):
        oalList = ALList( PyOS.path.splitext( ALDiskExplorer.normalize( _oalString ) ) )
        iLenght = len( oalList )
        oalString = ALString( oalList[iLenght-1] )
        return oalString
    getFileExtension = staticmethod( getFileExtension )


    def isFileExtension( _oalStrFileName, _oalStrExtension ):
        #if ALFile.exist( _osStrFileName )==False:
        #    return False
        #
        if (ALDiskExplorer.isFile( _oalStrFileName )==False):
            return False
        if (ALDiskExplorer.getFileExtension( _oalStrFileName )==_oalStrExtension):
            return True
        else:
            return False
    isFileExtension = staticmethod( isFileExtension )




    def setFileExtension(  _oalStrFile, _oalStrExt ):
        """
        You must provide a complete extension name
        """
        oalList = ALList( PyOS.path.splitext( ALDiskExplorer.normalize( _oalStrFile ) ) )
        oalString2 = ALString()
        for i in range( len( oalList )-1 ):
                oalString2 = oalString2 + ALString( oalList[i] )
        oalString2 = oalString2 + _oalStrExt
        return oalString2
    setFileExtension = staticmethod( setFileExtension )


    def getFileFolderName( _oalString ):
        oalString = PyOS.path.dirname( _oalString )
        return ALDiskExplorer.normalize( oalString )
    getFileFolderName = staticmethod( getFileFolderName )




    def absPath( _oalStringPath ):
        oalStringPath = ALString( _oalStringPath )
        oalAbsPath = PyOS.path.abspath( oalStringPath )
        return ALDiskExplorer.normPath( oalAbsPath )
    absPath = staticmethod( absPath )


    def absolutePath( _oalStringPath ):
        return ALDiskExplorer.absPath( _oalStringPath )
    absolutePath = staticmethod( absolutePath )


    def getAbsolutePath( _oalStringPath ):
        return ALDiskExplorer.absPath( _oalStringPath )
    getAbsolutePath = staticmethod( getAbsolutePath )


    def normPath( _oalStringPath ):
        # Fix for Jython
        try:
            oalStrPath = PyOS.path.normpath( _oalStringPath )
        except:
            oalStrPath = _oalStringPath
        return ALDiskExplorer.normDrive( oalStrPath )
    normPath = staticmethod( normPath )



    def normalize( _oalStringPath ):
        return ALDiskExplorer.normPath( ALString( _oalStringPath ) )
    normalize = staticmethod( normalize )



    def normDrive( _oalStringPath ):
        oalStringPath = ALString( _oalStringPath )
        oalStringPath = oalStringPath.replace( "\\", "/" )
        if ALSystem.getOSType()==ALString( "Cygwin" ):
            if oalStringPath.count("cygdrive") == True :
                lstr = oalStringPath[10]
                oalStringPath = ALString.replace( oalStringPath, "/cygdrive/" + lstr + "/", lstr + ":/" )
        return oalStringPath
    normDrive = staticmethod( normDrive )



    def iteractPath( _oalPath, _oalObjectMethod, _oalArgument) :
        """
        Execute the object method iteractivetily with alll elements
        in the folder
        """
        PyOS.path.walk( _oalPath, _oalObjectMethod, _oalArgument )
    iteractPath = staticmethod( iteractPath )


    def mergePath( _oalPath0, _oalPath1 ):
        oalPath0 = ALDiskExplorer.normPath( _oalPath0 )
        oalPath1 = ALDiskExplorer.normPath( _oalPath1 )
        return ALDiskExplorer.normPath( PyOS.path.join( oalPath0, oalPath1 ) )
    mergePath = staticmethod( mergePath )


    def addPath( _oalPath0, _oalPath1 ):
        oalPath0 = ALDiskExplorer.normPath( _oalPath0 )
        oalPath1 = ALDiskExplorer.normPath( _oalPath1 )
        ALDiskExplorer.m_oalThrCondDiskExplorer.synchronizeOn()
        oalPath = oalPath0 + "/" + oalPath1
        ALDiskExplorer.m_oalThrCondDiskExplorer.synchronizeOff()
        return oalPath
    addPath = staticmethod( addPath )


    def getCwd():
        return ALDiskExplorer.normPath( PyOS.getcwd() )
    getCwd = staticmethod( getCwd )


    def getCurrentFolder():
        return ALDiskExplorer.getCwd()
    getCurrentFolder = staticmethod( getCurrentFolder )


    def setCurrentFolder( _oalPath ):
        oalStrPath = ALDiskExplorer.normPath( _oalPath )
        ALDiskExplorer.m_oalThrCondDiskExplorer.synchronizeOn()
        PyOS.chdir( oalStrPath )
        ALDiskExplorer.m_oalThrCondDiskExplorer.synchronizeOff()
    setCurrentFolder = staticmethod( setCurrentFolder )


    def setMode( _oalPath, _pyMode ):
        oalStrPath = ALDiskExplorer.normPath( _oalPath )
        ALDiskExplorer.m_oalThrCondDiskExplorer.synchronizeOn()

        if (ALSystem.getInterpreter()!="Jython"):
            PyOS.chmod( oalStrPath, _pyMode )

        ALDiskExplorer.m_oalThrCondDiskExplorer.synchronizeOff()
    setMode = staticmethod( setMode )


    # Force the remove!!!
    def removeFolder( _oalStrSource, _oalIgnoreErrors = 0, _oalControlErrors = None ):
        return PyShutil.rmtree( _oalStrSource, _oalIgnoreErrors, _oalControlErrors  )
    removeFolder = staticmethod( removeFolder )



    # TO IMPROVE!!! a lot errors from the original python version = NOT INTELIGENT
    # we must do it with inteligence
    def copyFile( _oalStrSource, _oalStrDestination ):
        try:
            oalStringOrig = ALString( _oalStrSource )
            oalStringDest = ALString( _oalStrDestination )
            return PyShutil.copyfile( oalStringOrig, oalStringDest )
        except:
#            ALVerbose.warning( "Error creating or coping file+folder: " + _oalStrDestination )
            print " [Warning] ALDiskExplorer::copyFile() - Error creating or coping file+folder: " + _oalStrDestination
    copyFile = staticmethod( copyFile )


    def copyFolder( _oalStrSource, _oalStrDestination, _oalSimbolicLinks = 0 ):
        try:
            oalStringOrig = ALString( _oalStrSource )
            oalStringDest = ALString( _oalStrDestination )
            return PyShutil.copytree( oalStringOrig, oalStringDest, _oalSimbolicLinks  )
        except:
#            ALVerbose.warning( "The folder exists: " + _oalStrDestination )
            print " [Warning] ALDiskExplorer::copyFile() - The folder exists: " + _oalStrDestination
    copyFolder = staticmethod( copyFolder )






##############################################################################