#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################


from ALImportPython        import PyOS
from ALImportPython        import PySys
from ALDefinition          import *
from ALSystem              import ALSystem
from ALLog                 import ALLog
from ALString              import ALString
from ALTime                import ALTime
from ALFile                import ALFile
from ALStreamFile          import ALStreamFile
from ALDictionary          import ALDictionary
from ALDiskExplorer        import ALDiskExplorer
from ALProfile             import ALProfile



class ALLogFile( ALLog ):
    """
    Base class to Log output in a file
    This class control the information to log in a file
    """
    m_oalLogFileName      = None
    m_bIsInit             = False
    m_oalLockFileName     = False


    def __init__( self, _oalString = AL_FRAMEWORK_LOG_NAME, _bIsInit = False, _bGenericLog = False ):
        ALLog.__init__( self )
        self.m_oalLogFileNameGeneric = None
        self.__createFile( _oalString,  _bIsInit, _bGenericLog  )



    def __versionName(self, _oalString ):
        oalString = None
        if (ALSystem.getOSName()=="Java" and _oalString==AL_FRAMEWORK_LOG_NAME):
            oalString = "Jy" + _oalString
        elif (_oalString==AL_FRAMEWORK_LOG_NAME):
            oalString = "Py" + _oalString
        else :
            oalString = _oalString
        return ALString( oalString )



    def __fromIndex( self, _oalString ):
        oList = ALString( _oalString ).split( AL_TOKEN_SEPARATOR )
        oalStrTime = ALTime.timeStringValue( float( oList[0] ) )
        oalStrInfo = ALString( oList[1] )
        return oalStrTime, oalStrInfo


    def __createFile( self, _oalString, _bIsInit = False, _bGenericLog = False ):
        if (_bGenericLog == True):
#            oalStringFileName = ALString( ALString( "./" ) + ALTime.getDate()+ "-" + self.__versionName( _oalString ) )
            oalStringFileName =  _oalString
            oalStringFileName = ALDiskExplorer.setFileExtension( oalStringFileName, ".log" )
            self.m_oalLogFileNameGeneric = ALDiskExplorer.getAbsolutePath( oalStringFileName )
            # Creates the folder
            oalFolderName = ALDiskExplorer.getFileFolderName( self.m_oalLogFileNameGeneric )
            ALDiskExplorer.createFolder( oalFolderName )
            self.initFile( self.m_oalLogFileNameGeneric )
        else:
            if (ALLogFile.m_oalLockFileName==True):
                return
            ALLogFile.m_bIsInit = _bIsInit
            oalStringFilePath = ALString()
            oalStringFilePath = ALSystem.getCurrentFolder()
            oalStringFileName = ALString( oalStringFilePath + ALString( "/" ) + ALTime.getDate()+ "-" + self.__versionName( _oalString ) )
            ALLogFile.m_oalLogFileName = PyOS.path.abspath( oalStringFileName )
            if (ALLogFile.m_bIsInit==True) :
                self.initFile()



    def initFile( self, _oalFileName = None ):

        if (ALLogFile.m_bIsInit==True and _oalFileName==None) :
            return
        else:
            self.synchronizeOn()
            ALLogFile.m_bIsInit = True
            oalFileName = ALLogFile.m_oalLogFileName
            if (_oalFileName is not None):
                oalFileName = _oalFileName
            ALLogFile.createStringSystemInfo( oalFileName )

            obStatus = False
            for cmdkey in PySys.argv:
                if (cmdkey == "--systemLog" ):
                    obStatus = True
                oalString = ALString(cmdkey)
                if (oalString.contains("DEBUG")):
                    obStatus = True

            if (obStatus==True):
                oalLogFile = ALStreamFile( oalFileName, "a" )
                oalLogFile.write( "\n\n\n" )
                oalLogFile.write( "<!-- ################################################################################## -->\n" )
                oalLogFile.write( "<lg>\n" )
                oalLogFile.write( "<lgSystem>\n" )
                oalLogFile.write( ALLogFile.getSystemInfo() )
                oalLogFile.write( "</lgSystem>\n" )
                oalLogFile.write( "</lg>\n" )
                oalLogFile.write( "<!-- ################################################################################## -->\n\n" )
                oalLogFile.close()
            else:
                oalLogFile = ALStreamFile( oalFileName, "a" )
                oalLogFile.write( "\n\n\n\n\n" )
                oalLogFile.write( "<!-- ################################################################################## -->\n" )
                oalLogFile.close()

            self.synchronizeOff()



    def log( self, _oalString = None ):
        if (_oalString != None ):
            self.initFile()
        self.synchronizeOn()
        if ( _oalString != None ):
            oalString  = ALString( " <lg> " + ALTime.timeString() +   ALString.setLogXmlToString( ALString( _oalString ) ) + "</lg>" )
            oalLogFile = ALStreamFile( ALLogFile.m_oalLogFileName, "a" )
            oalLogFile.write( oalString )
            oalLogFile.close()
        self.synchronizeOff()


    def addLog( self, _oalString = None ):
        """
        Experimental - it will be changed or moved to other class in the future
        """
        if (_oalString != None ):
            self.initFile()
        self.synchronizeOn()
        if ( _oalString != None ):
            oalString  = ALString( " <lg> " + ALTime.timeString() +   ALString.setLogXmlToString( ALString( _oalString ) ) + "</lg>" )
            if ( self.m_oalLogFileNameGeneric!= None):
                oalLogFile = ALStreamFile( self.m_oalLogFileNameGeneric, "a" )
                oalLogFile.write( oalString )
                oalLogFile.close()
        self.synchronizeOff()


    def outputVerboseAction( self, _oalDictionary, _oalID = None ):
        """
        Experimental
        """
        return self.logVerboseTree(  _oalDictionary, _oalID, self.m_oalLogFileNameGeneric )


    def logVerboseTree( self, _oalDictionary, _oalID = None, _oalFileName = None ):
        self.synchronizeOn()
        oalDict = ALDictionary()
        for i in _oalDictionary.ordered():
            oalDict.addObject( _oalDictionary[i], _oalDictionary[i] )

        oalFileName = ALLogFile.m_oalLogFileName
        if (_oalFileName is not None ):
            oalFileName = _oalFileName

        oalLogFile = ALStreamFile( oalFileName, "a" )
        oalLogFile.write( "\n\n" )
        oalLogFile.write( "<!-- [ Actions Log List ]################################################################ -->\n" )
        oalLogFile.write( "<lg>\n" )
        oalLogFile.write( "<lgActionList>\n" )
        try:
            for i in oalDict.ordered():
                if (_oalID!=None):
                    if (_oalID==i):
                        oalLogFile.write( "  <!-- ################################################################################## -->\n" )
                        oalLogFile.write( "  <lgAction>\n" )
                        oalLogFile.write( "            " + ALTime.timeString( ) + "  [ Action log :: " + i + "]\n" )

                        for j in _oalDictionary.ordered():
                            if ( _oalDictionary[j]==i):
                                oalStrTime, oalStrInfo = self.__fromIndex( j )
                                oalStringFiltered =  ALString.setLogXmlToString( ALString( oalStrInfo ) )

                                oalLogFile.write( "            " + ALTime.timeString( ) +"        ==>" + oalStrTime + " " + oalStringFiltered + "\n" )

                        oalLogFile.write( "  </lgAction>\n" )
                        oalLogFile.write( "  <!-- ################################################################################### -->\n" )
                else:
                    oalLogFile.write( "  <!-- ################################################################################## -->\n" )
                    oalLogFile.write( "  <lgAction>\n" )
                    oalLogFile.write( "            " + ALTime.timeString( ) + "  [ Action log :: " + i + "]\n" )

                    for j in _oalDictionary.ordered():
                        if ( _oalDictionary[j]==i):
                            oalStrTime, oalStrInfo = self.__fromIndex( j )
                            oalStringFiltered =  ALString.setLogXmlToString( ALString( oalStrInfo ) )

                            oalLogFile.write( "            " + ALTime.timeString( ) +"        ==>" + oalStrTime + " " + oalStringFiltered + "\n" )

                    oalLogFile.write( "  </lgAction>\n" )
                    oalLogFile.write( "  <!-- ################################################################################### -->\n" )

        except:
            oalLogFile.write("<lgActionListException>\n" )
            oalLogFile.write( ALTime.timeString( ) +"        ==> Exception in the Jython dictionary version \n" )
            oalLogFile.write("</lgActionListException>\n" )

        oalLogFile.write( "</lgActionList>\n" )
        oalLogFile.write( "</lg>\n" )
        oalLogFile.write( "<!-- ################################################################################### -->\n" )
        oalLogFile.write( "\n\n" )
        oalLogFile.close()

        self.synchronizeOff()

       # return  # only to the release 4
        # System to force to have only visual interface if the code is profiling
        if (_oalFileName is None ):
            return ALProfile().getProfileFileHtml( oalFileName, _oalDictionary )
        else:
            return oalFileName



    def setFileName( self, _oalString ):
        self.synchronizeOn()
        self.__createFile( _oalString )
        self.synchronizeOff()


    def setUniqueFileName( self, _oalString ):
        self.synchronizeOn()
        self.__createFile( _oalString )
        self.synchronizeOff()
        self.setLockFileName()


    def setLockFileName( self ):
        self.synchronizeOn()
        ALLogFile.m_oalLockFileName = True
        self.synchronizeOff()


    def getFileName( self ):
        return ALLogFile.m_oalLogFileName



    def outputProfileFile(self, _oalFileName = None ):
        """
        Experimental
        """
        if (_oalFileName==None):
            _oalFileName = self.m_oalLogFileNameGeneric
        return self.getProfileFile( _oalFileName )



    def getProfileFile( self, _oalFileName = None  ):
        oalFileName = self.getFileName()
        if ( _oalFileName is not None):
            oalFileName = _oalFileName
        if ( ALDiskExplorer.existFile( oalFileName )==False):
            return
        self.synchronizeOn()

        oalStrFileName = ALProfile().getProfileFile( oalFileName )

        self.synchronizeOff()
        return oalStrFileName


##############################################################################


