#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################


from ALImportPython     import PySys
from ALDefinition       import *
from ALString           import ALString
from ALSystem           import ALSystem
from ALVerbose          import ALVerbose
from ALList             import ALList
from ALTime             import ALTime
from ALFile             import ALFile
from ALDiskExplorer     import ALDiskExplorer
from ALManager          import ALManager
from ALSubProcess       import ALSubProcess

try:
    from ALDataManagerProcess  import ALDataManagerProcess
    from ALDataProcess         import ALDataProcess
    from ALDataListDataProcess import ALDataListDataProcess
except ImportError:
    try:
        from ALXsd import ALXsd
        # You must run this module on the m/modules folder to generate the specific code
        ALXsd().generateModule( "ALDataManagerProcess.py", "ALDataManagerProcess.xsd", "xsd:" )
        ALXsd().generateModule( "ALDataProcess.py", "ALDataProcess.xsd", "xsd:" )
        ALXsd().generateModule( "ALDataListDataProcess.py", "ALDataListDataProcess.xsd", "xsd:" )
    except:
        ALVerbose.warning( "Problems to generate class interfaces for: ALDataManagerProcess, ALDataProcess and ALDataListDataProcess" )



class ALManagerProcess( ALManager ):
    """
    Parent class for all ALManagerProcess
    """

    def __init__( self ):
        ALManager.__init__( self )
        self.m_oalStrCommand     = ALString( "ps axu" )
        self.m_oalStrCommandKill = ALString( "kill -9 " )
        if (ALSystem.getOSName()=="Windows"):
            self.m_oalStrCommand     = ALString( "tasklist.exe /fo TABLE  /v  /nh" )
            self.m_oalStrCommandKill = ALString( "taskkill.exe /f /pid " )
        self.m_oalDataManagerProcess = None
        self.__createDataManagerProcess()


    def __createDataManagerProcess(self):
        if (self.m_oalDataManagerProcess!=None):
            del self.m_oalDataManagerProcess
        self.m_oalDataManagerProcess = ALDataManagerProcess()
        self.m_oalDataManagerProcess.setDate( ALTime.getTimeString())
        self.m_oalDataManagerProcess.setVersion(ALString( AL_VERSION ))
        self.m_oalDataManagerProcess.setName( ALString( "ALManagerProcess" ) )
        self.m_oalDataManagerProcess.setInformation( ALString( AL_FRAMEWORK_NAME ) )
        self.m_oalDataManagerProcess.setAuthor( ALString( AL_FRAMEWORK_NAME ) )


    def control( self ):
        self.__generateDataManagerProcess()


    def getListDataProcess( self ):
        if (self.m_oalDataManagerProcess!=None):
            oalListDataProcess = self.m_oalDataManagerProcess.getListDataProcess()
            if (oalListDataProcess!=None):
                return oalListDataProcess.getDataProcess()
            else:
                ALVerbose.warning("ALManagerProcess::getListDataProcess() = No Data Processes avaiable" )
        else:
            return None



    def __patchControlDelay(self):
        """
        To wait the start up of the external process
        """
        if (ALSystem.getOSName()=="Windows"):
            ALTime.wait(1.0)
        elif (ALSystem.getInterpreter()=="Jython"):
            ALTime.wait(1.0)


    def __generateDataManagerProcess( self ):
        oalSubProcess = ALSubProcess( self.m_oalStrCommand )
        oalSubProcess.executeProcess()
        self.__patchControlDelay()
        oalListOutput = oalSubProcess.getOutput()
        self.__parseSubProcessOutput( oalListOutput )



    def __parseSubProcessOutput( self, _oalListOutput ):
        self.__createDataManagerProcess()
        if (ALSystem.getOSName()=="Windows"):
            self.__parseSubProcessOutputWindows( _oalListOutput )
        if (ALSystem.getOSName()=="Linux"):
            self.__parseSubProcessOutputLinux( _oalListOutput )
        if (ALSystem.getOSName()=="MacOS"):
            self.__parseSubProcessOutputMacOS( _oalListOutput )




    def __parseSubProcessOutputWindows( self, _oalListOutput ):
        oalDataListDataProcess = ALDataListDataProcess()
        oalListDataProcess     = oalDataListDataProcess.getDataProcess()

        for iLine in _oalListOutput:
            #print iLine
            oalStringLine = ALString( iLine ).replace("/n","")
            oalListString = ALString( oalStringLine ).split()
            oalList = ALList()
            for iObj in oalListString:
                if (iObj!=""):
                    oalList.append(iObj)

            oalDataProcess = ALDataProcess()
            if (len(oalList)>0): # To guarantee the first item no empty
                if (oalList[0]!=""):
                    if ( oalList[0]=="System" and oalList[1]=="Idle" and oalList[2]=="Process"):
                        oalDataProcess.setImageName( "System Idle Process" )
                        for i in range(1, len(oalList)-2):
                            oalList[i] = oalList[i+2]
                        for i in range(len(oalList)-2, len(oalList)):
                            oalList[i] = ""
                    else:
                        # patch for a stupid Windows task list
                        try:
                            oalStrPID = ALString( oalList[1] ).atoi()
                            oalDataProcess.setImageName( oalList[0] )
                        except:
                            oalDataProcess.setImageName( ALString( oalList[0]+" " + oalList[1]) )
                            for i in range(1, len(oalList)-1):
                                oalList[i] = oalList[i+1]
                            for i in range(len(oalList)-1, len(oalList)):
                                oalList[i] = ""

                    #print ALString( oalList[1] ).atoi()
                    oalDataProcess.setPid( ALString( oalList[1] ).atoi() )
                    oalDataProcess.setSessionName( oalList[2] )
                    oalDataProcess.setSession( oalList[3] )
                    oalDataProcess.setMemoryUsage( oalList[4]+" "+oalList[5] )
                    oalDataProcess.setStatus( oalList[6] )

                    if (oalList[7]=="NT" and oalList[9]=="SERVICE"):
                        oalDataProcess.setUserName( oalList[7]+" "+oalList[8]+" "+oalList[9])
                        for i in range(7, len(oalList)-2):
                            oalList[i] = oalList[i+2]
                        for i in range(len(oalList)-2, len(oalList)):
                            oalList[i] = ""

                    elif (oalList[7]=="NT" and oalList[9]!="SERVICE"):
                        oalDataProcess.setUserName( oalList[7]+" "+oalList[8] )
                        for i in range(7, len(oalList)-1):
                            oalList[i] = oalList[i+1]
                        for i in range(len(oalList)-1, len(oalList)):
                            oalList[i] = ""
                    else:
                        oalDataProcess.setUserName( oalList[7]  )

                    oalDataProcess.setCpuTime( oalList[8] )
                    oalString = ALString()
                    for iLetre in range(9, len(oalList)):
                        oalString = oalString + ALString( oalList[iLetre])
                    oalDataProcess.setWindowTitle( oalString )
                    if (oalString!="N/A"):
                        oalDataProcess.setCommand( oalDataProcess.getWindowTitle() )
                    else:
                        oalDataProcess.setCommand( oalDataProcess.getImageName() )

                oalListDataProcess.append( oalDataProcess )

        oalDataListDataProcess.setDataProcess( oalListDataProcess )
        self.m_oalDataManagerProcess.setListDataProcess( oalDataListDataProcess  )

#
#        <dataProcess>
#            <command>System</command>
#            <imageName>System</imageName>
#            <pid>4</pid>
#            <sessionName>Console</sessionName>
#            <session>0</session>
#            <memoryUsage>272 K</memoryUsage>
#            <status>Running</status>
#            <userName>NT AUTHORITY\SYSTEM</userName>
#            <cpuTime>0:01:05</cpuTime>
#            <windowTitle>N/A</windowTitle>
#        </dataProcess>

#  0          1    2   3      4    5   6        7    8       9    10
#USER       PID %CPU %MEM   VSZ  RSS TTY      STAT START   TIME COMMAND

    def __parseSubProcessOutputLinux( self, _oalListOutput ):
        oalDataListDataProcess = ALDataListDataProcess()
        oalListDataProcess     = oalDataListDataProcess.getDataProcess()

        for iLine in _oalListOutput:
            #print iLine
            oalStringLine = ALString( iLine ).replace("/n","")
            oalListString = ALString( oalStringLine ).split()
            oalList = ALList()
            for iObj in oalListString:
                if (iObj!=""):
                    oalList.append(iObj)

            oalDataProcess = ALDataProcess()
            if (len(oalList)>0): # To guarantee the first item no empty
                if (oalList[0]!="" and oalList[0]!="USER"):
                    oalDataProcess.setUserName( oalList[0] )
                    oalDataProcess.setPid( ALString( oalList[1] ).atoi() )

                    oalDataProcess.setMemoryUsage( oalList[3] )
                    oalStatus = ALString(oalList[7])
#                    if (oalStatus.contains("R")==True or oalStatus.contains("S")==True):
#                        oalStatus = "Running"
#                    else:
#                        oalStatus = "Hang"
#                    if (oalStatus.contains("Z")==True or oalStatus.contains("z")==True):
#                        oalStatus = "Hang"
#                    else:
#                        oalStatus = "Running"



                    oalDataProcess.setStatus( oalStatus )

                    oalDataProcess.setSessionName( oalList[6] )
                    oalDataProcess.setSession( oalList[6] )
                    oalDataProcess.setCpuTime( oalList[9] )

                    oalString = ALString()
                    for iLetre in range(10, len(oalList)):
                        oalString = oalString + ALString( oalList[iLetre]) + " "
                    oalDataProcess.setWindowTitle( oalString )
                    oalDataProcess.setCommand(     oalString )
                    oalDataProcess.setImageName(   oalString )

                    oalListDataProcess.append( oalDataProcess )

        oalDataListDataProcess.setDataProcess( oalListDataProcess )
        self.m_oalDataManagerProcess.setListDataProcess( oalDataListDataProcess  )






    def __parseSubProcessOutputMacOS( self, _oalListOutput ):
        self.__parseSubProcessOutputLinux( _oalListOutput )



    def __patchGetUserName(self):
        oalUserName = ALSystem.getUserName()
        if (ALSystem.getOSName()=="Windows"):
            oalHost     = ALSystem.getHost()
            oalHostList = oalHost.split(".")
            oalUserName = oalHostList[0] + "\\" + oalUserName
        return oalUserName


    def getUserName(self):
        return self.__patchGetUserName()


    def getPid( self, _oalStrProcess ):
        self.__generateDataManagerProcess()

        oalUserName = self.__patchGetUserName()


        oiPid = None
        oalListDataProcess = self.getListDataProcess()
        if (oalListDataProcess==None):
            return oiPid

        oalStrProcess = ALString( _oalStrProcess )


        # Patch to avoid the python kill itself!!!
        if (ALSystem.getMachineVirtual()=="Python"):
            if (_oalStrProcess=="python" or _oalStrProcess=="python.exe"):
                return oiPid



        # Try to find the PID by the command string
        oBoolStatus = False
        oalStrProcessName = oalStrProcess
        for iProc in oalListDataProcess:
            if (iProc.getCommand()==oalStrProcessName and oBoolStatus == False and iProc.getUserName()==oalUserName):
                oiPid = iProc.getPid()
                oBoolStatus = True


        # Try to find the PID by the software string name string
        # Try to find the PID by the imageName string
        # Patch to avoid the python kill itself!!!
        if (ALSystem.getMachineVirtual()=="Python"):
            if (oalStrProcess.contains("python")==True):
                return oiPid


        # Try to find the PID by the software string name string
        if (oBoolStatus == False):
            oalStrList = oalStrProcess.split()
            oalStrProcessRoot = oalStrList[0]
            oalStrProcessRootAbs = ALDiskExplorer.getAbsolutePath( oalStrProcessRoot )
            oalStrProcessName = ALString( oalStrProcess ).replace( oalStrList[0], oalStrProcessRootAbs )
#            print oalStrProcessName

            for iProc in oalListDataProcess:
                if (iProc.getCommand()==oalStrProcessName and oBoolStatus == False and iProc.getUserName()==oalUserName):
                    oiPid = iProc.getPid()
                    oBoolStatus = True

            if (oBoolStatus == False):
                if (ALSystem.getOSName()=="Windows"):
                    oalStrProcessRootAbs = ALFile.setExtension( oalStrProcessRootAbs, ".exe")
                oalStrProcessName = ALString( oalStrProcess ).replace( oalStrList[0], oalStrProcessRootAbs )
#                print oalStrProcessName

                for iProc in oalListDataProcess:
                    if (iProc.getCommand()==oalStrProcessName and oBoolStatus == False and iProc.getUserName()==oalUserName):
                        oiPid = iProc.getPid()
                        oBoolStatus = True


        # Try to find the PID by the imageName string
        if (oBoolStatus == False):
            oalStrList = oalStrProcess.split()
            oalStrProcessName = oalStrList[0]
            oalStrProcessName = ALFile.getBaseName( oalStrProcessName )
            if (ALSystem.getOSName()=="Windows"):
                oalStrProcessName = ALFile.setExtension( oalStrProcessName, ".exe")

#            print oalStrProcessName
            for iProc in oalListDataProcess:
                if (iProc.getImageName()==oalStrProcessName and oBoolStatus == False and iProc.getUserName()==oalUserName):
                    oiPid = iProc.getPid()
                    oBoolStatus = True


        # Try to find the PID by the  string inside the command line - Unix: ps axu
        if (oBoolStatus == False and ALSystem.getOSName()!="Windows"):
            for iProc in oalListDataProcess:
                oalCommand = ALString( iProc.getCommand() )
                if (oalCommand.contains( oalStrProcess )==True and  oBoolStatus == False and iProc.getUserName()==oalUserName):
                    oiPid = iProc.getPid()
                    oBoolStatus = True


        if (oiPid!=None):
            oiPid = int(oiPid)

        return oiPid




    def getDataProcess( self, _oalProcess ):
        oiPid = self.getPid( _oalProcess )
        if (oiPid==None):
            return None
        return self.getDataProcessPid( oiPid )


    def getDataProcessPid( self, _oiPid ):
        oalListDataProcess = self.getListDataProcess()
        if (oalListDataProcess==None):
            return self.__generateDataManagerProcess()

        oalListDataProcess = self.getListDataProcess()
        if (oalListDataProcess==None):
            return None

        for iProc in oalListDataProcess:
            if (iProc.getPid()==_oiPid):
                return iProc
        return None



    def getStatusPid( self, _oiPid, _oBoolControl = True ):
        if(_oiPid<=0):
            return ALString("HANG")
        if (_oBoolControl==True):
            self.__generateDataManagerProcess()
        oStrStatus = ALString( "TERMINATED" )
        oalListDataProcess = self.getListDataProcess()
        if (oalListDataProcess==None):
            return oStrStatus
        oStatus = None

#        print "==========================reading PID"
#        print _oiPid
#        self.output( "taskList.xml" )
#        raise
        for iProc in oalListDataProcess:
            if (iProc.getPid()==_oiPid):
#                print "==========================STATUS PID"
#                print iProc.getStatus()
                oStatus = ALString( iProc.getStatus() )
                if (oStatus.contains("Z")==False and oStatus.contains("z")==False and oStatus.contains("Hang")==False):
                    oStatus = "RUNNING"

        if (oStatus!=None):
            if (ALString( oStatus ).capitalize()=="Running"):
                oStrStatus = ALString("RUNNING")
            else:
                oStrStatus = ALString("HANG")
        return oStrStatus



    def getStatus(self, _oalProcess ):
        oiPid = self.getPid( _oalProcess )
        if (oiPid==None):
            return ALString( "TERMINATED" )
        else:
            return self.getStatusPid( oiPid, False )



    def isRunning(self, _oalProcess ):
        oiPid = self.getPid( _oalProcess )
        if (oiPid==None):
            return False
        oStrStatus = self.getStatusPid( oiPid, False )
        if (oStrStatus==ALString( "RUNNING" )):
            return True
        else:
            return False



    def isRunningPid(self, _oiPid ):
        if (_oiPid==None):
            return False
        oStrStatus = self.getStatusPid( int( _oiPid ) )
        if (oStrStatus==ALString( "RUNNING" )):
            return True
        else:
            return False




    def killProcess(self, _oalProcess ):
        oiPid = self.getPid( _oalProcess )
        if (oiPid==None):
            return
        self.killProcessPid( oiPid )



    def killProcessPid( self, _oiPid ):
        if (_oiPid<=0):
            return
        oalStrCommandKill = self.m_oalStrCommandKill + ALString( _oiPid )

        oalSubProcess = ALSubProcess( oalStrCommandKill )
        oalSubProcess.executeProcess()
        self.__patchControlDelay()
#        #Only for debug
#        oalListOutput = oalSubProcess.getOutput()
#        for iLine in oalListOutput:
#            print iLine




    def output(self, _oalStrFileName ):
        if (self.m_oalDataManagerProcess!=None):
            self.m_oalDataManagerProcess.outputFile( _oalStrFileName )




    def outputTable( self, _oalStream = PySys.stdout ):
        oalListDataProcess = self.getListDataProcess()
        if (oalListDataProcess==None):
            return

        _oalStream.write( "\n\n" )
        _oalStream.write( "Image Name                           PID  SessionName Session  Mem Usage  Status                     User Name      CPUTime Window Title                                Command\n" )
        _oalStream.write( "================================== ====== =========== ======= ========== ======== ============================ ============ =========================================   ============================\n" )
        for iProc in oalListDataProcess:
            _oalStream.write( ALString.setFormat( iProc.getImageName()    , 35 ) )
            _oalStream.write( ALString.setFormat( iProc.getPid()          , 6  ) )
            _oalStream.write( ALString.setFormat( iProc.getSessionName()  , 9  ) )
            _oalStream.write( ALString.setFormat( iProc.getSession()      , 5  ) )
            _oalStream.write( ALString.setFormat( iProc.getMemoryUsage()  , 10 ) )
            _oalStream.write( ALString.setFormat( iProc.getStatus()       , 10 ) )
            _oalStream.write( ALString.setFormat( iProc.getUserName()     , 35 ) )
            _oalStream.write( ALString.setFormat( iProc.getCpuTime()      , 12 ) )
            _oalStream.write( ALString.setFormat( iProc.getWindowTitle()  , 45 ) )
            _oalStream.write( ALString.setFormat( iProc.getCommand()      , 25 ) + "\n" )



##############################################################################
