#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################


from ALImportPython        import *
from ALManager             import ALManager
from ALThread              import ALThread
from ALList                import ALList


class ALManagerSignal( ALManager ):
    """
    Class for managing signals.
    """


    def __init__( self ):
        ALManager.__init__( self )
        self.m_oalListConnection = ALList()


    def connect(self, _oalObject, _oalSignal, _oalClassMethod ):
        """
        Connects a signal between an object and a method in an other object.
        """

        if _oalObject is None:
            raise "RunTimeException", "ALManagerSignal: _oalObject is None"
        if _oalSignal is None:
            raise "RunTimeException", "ALManagerSignal: _oalSignal is None"
        if _oalClassMethod is None:
            raise "RunTimeException", "ALManagerSignal: _oalClassMethod is None"

        # Block the thread
        self.synchronizeOn()

        self.m_oalListConnection.append( [ _oalObject.getID(),
                                           _oalSignal,
                                           _oalClassMethod ] )

        # Release the thread
        self.synchronizeOff()


    def emit( self, _oalObject, _oalSignal, _oalListArguments ):

        # Block the thread
        self.synchronizeOn()

        if _oalObject is None:
            raise "RunTimeException", "ALManagerSignal: _oalObject is None"
        if _oalSignal is None:
            raise "RunTimeException", "ALManagerSignal: _oalSignal is None"

        for connection in self.m_oalListConnection:
            id1 = connection[ 0 ]
            id2 = _oalObject.getID()
            if id1 == id2:
                oslStringSignal1 = connection[ 1 ].getSignal()
                oslStringSignal2 = _oalSignal.getSignal()
                if oslStringSignal1.equals( oslStringSignal2 ):
                    method = connection[ 2 ]
                    if _oalListArguments is None:
                        method()
                        #PyStartNewThread( method )
                    else:
                        method( _oalListArguments )
                        #PyStartNewThread( method, (_oalListArguments, ) )

        # Release the thread
        self.synchronizeOff()


##############################################################################


