#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################


from ALImportPython     import PySys
from ALDefinition       import *
from ALString           import ALString
from ALSystem           import ALSystem
from ALVerbose          import ALVerbose
from ALList             import ALList
from ALManager          import ALManager
from ALSemaphore        import ALSemaphore
from ALTime             import ALTime



class ALManagerTest( ALManager ):
    """
    Parent class for all ALManagerTest objects.
    """
    m_oalListTest                  = ALList()
    m_oalListTestTotal             = ALList()
    m_oalSemaphore                 = ALSemaphore()
    m_oalRunningTest               = None
    m_oiCumulativeTotalTest        = 0
    m_oiCumulativeTotalTestMethod  = 0
    m_oiCumulativeTotalTestSuccess = 0
    m_oiCumulativeTotalTestFailure = 0
    m_oalStringReport              = ALString()

    def __init__( self ):
        ALManager.__init__( self )


    def synchronizeOn():
        ALManagerTest.m_oalSemaphore.synchronizeOn()
    synchronizeOn = staticmethod( synchronizeOn )


    def synchronizeOff():
        ALManagerTest.m_oalSemaphore.synchronizeOff()
    synchronizeOff = staticmethod( synchronizeOff )


    def create():
        ALManagerTest.m_oiCumulativeTotalTest        += ALManagerTest.getNumberTest()
        ALManagerTest.m_oiCumulativeTotalTestMethod  += ALManagerTest.getNumberTestMethod()
        ALManagerTest.m_oiCumulativeTotalTestSuccess += ALManagerTest.getNumberTestSuccess()
        ALManagerTest.m_oiCumulativeTotalTestFailure += ALManagerTest.getNumberTestFailure()
        ALManagerTest.synchronizeOn()
        for iTest in ALManagerTest.m_oalListTest:
            ALManagerTest.m_oalListTestTotal.addObject( iTest )
        del ALManagerTest.m_oalListTest
        ALManagerTest.m_oalListTest  = ALList()
        ALManagerTest.synchronizeOff()
    create = staticmethod( create )



    def outputTable( _oalStream = PySys.stdout ):
        ALManagerTest.synchronizeOn()
        _oalStream.write( ALManagerTest.m_oalStringReport )
        ALManagerTest.synchronizeOff()
    outputTable = staticmethod( outputTable )



    def outputScreen():
        ALManagerTest.synchronizeOn()
        oalString = ALString()

        oalString = oalString + ""
        oalString = oalString +  "\n  ===============================================================================\n"
        oalString = oalString +   "  Final Report - Test Manager\n"
        oalString = oalString +   "               Test Manager Core: [ PyAALib-JyAALib " + ALString( AL_VERSION ) +" ]\n"
        oalString = oalString +   "                        Compiler: [ " + ALString( ALSystem.getCompilerName() ) +" ]\n"
        oalString = oalString +   "                 Virtual Machine: [ " + ALString( ALSystem.getMachineVirtuel() ) +" ]\n"
        oalString = oalString +   "                          System: [ " + ALSystem.getOSName()+" "+ALSystem.getOSType()+" "+ALSystem.getOSVersion()+" "+ALSystem.getMachineVersion()+" ]\n"
        oalString = oalString +   "                            Date: " + ALTime.getTimeString() + "\n"
        oalString = oalString +   "           Total Cumulated Tests: [ " + ALString( ALManagerTest.m_oiCumulativeTotalTest ) +" ]\n"
        oalString = oalString +   "    Total Cumulated Test SUCCESS: [ " + ALString( ALManagerTest.m_oiCumulativeTotalTestSuccess ) +" ]\n"
        oalString = oalString +   "   Total Cumulated Test FAILLURE: [ " + ALString( ALManagerTest.m_oiCumulativeTotalTestFailure ) +" ]\n"
        oalString = oalString +   "    Total Cumulated Test Methods: [ " + ALString( ALManagerTest.m_oiCumulativeTotalTestMethod ) +" ]\n"
        oalString = oalString +   "  -------------------------------------------------------------------------------\n"
        for iTest in ALManagerTest.m_oalListTestTotal:
            oalString = oalString + iTest.outputString()
        oalString = oalString +  "\n  ==============================================================================="
        ALManagerTest.m_oalStringReport = oalString
        ALVerbose.screen( ALManagerTest.m_oalStringReport )
        ALManagerTest.synchronizeOff()
    outputScreen = staticmethod( outputScreen )




    def getAcidIndex():
        ovAcidIndex = ( float (ALManagerTest.m_oiCumulativeTotalTestSuccess) / float( ALManagerTest.m_oiCumulativeTotalTest ) )
        ovAcidIndex = ovAcidIndex * 100.0
        return int( ovAcidIndex )
    getAcidIndex = staticmethod( getAcidIndex )


    def getNumberTestCumulative():
        return ALManagerTest.m_oiCumulativeTotalTest
    getNumberTestCumulative = staticmethod( getNumberTestCumulative )


    def getNumberTestMethodCumulative():
        return ALManagerTest.m_oiCumulativeTotalTestMethod
    getNumberTestMethodCumulative = staticmethod( getNumberTestMethodCumulative )



    def getNumberTestSuccessCumulative():
        return ALManagerTest.m_oiCumulativeTotalTestSuccess
    getNumberTestSuccessCumulative = staticmethod( getNumberTestSuccessCumulative )


    def getNumberTestFailureCumulative():
        return ALManagerTest.m_oiCumulativeTotalTestFailure
    getNumberTestFailureCumulative = staticmethod( getNumberTestFailureCumulative )




    def addInformationTest( _oalInformationTest ):
        ALManagerTest.synchronizeOn()
        ALManagerTest.m_oalRunningTest = _oalInformationTest
        ALManagerTest.m_oalListTest.addObject( _oalInformationTest )
        ALManagerTest.synchronizeOff()
    addInformationTest = staticmethod( addInformationTest )




    def addInformationTestMethod( _oalInformationTest ):
        ALManagerTest.synchronizeOn()
        if (ALManagerTest.m_oalRunningTest!=None):
            ALManagerTest.m_oalRunningTest.addInformationTest( _oalInformationTest )
        ALManagerTest.synchronizeOff()
    addInformationTestMethod = staticmethod( addInformationTestMethod )




    def getNumberTest():
        ALManagerTest.synchronizeOn()
        oiNumberTestTotal = ALManagerTest.m_oalListTest.getNumberObjects()
        ALManagerTest.synchronizeOff()
        return oiNumberTestTotal
    getNumberTest = staticmethod( getNumberTest )


    def getNumberTestMethod():
        ALManagerTest.synchronizeOn()
        oiNumberTestTotal = 0
        for iTest in ALManagerTest.m_oalListTest:
            oiNumberTestTotal += iTest.getNumberTestMethod()
        ALManagerTest.synchronizeOff()
        return oiNumberTestTotal
    getNumberTestMethod = staticmethod( getNumberTestMethod )



    def getNumberTestSuccess():
        ALManagerTest.synchronizeOn()
        oiNumberTestTotal = 0
        for iTest in ALManagerTest.m_oalListTest:
            if (iTest.isSuccess()==True):
                oiNumberTestTotal += 1
        ALManagerTest.synchronizeOff()
        return oiNumberTestTotal
    getNumberTestSuccess = staticmethod( getNumberTestSuccess )


    def getNumberTestFailure():
        ALManagerTest.synchronizeOn()
        oiNumberTestTotal = 0
        for iTest in ALManagerTest.m_oalListTest:
            if (iTest.isSuccess()==False):
                oiNumberTestTotal += 1
        ALManagerTest.synchronizeOff()
        return oiNumberTestTotal
    getNumberTestFailure = staticmethod( getNumberTestFailure )





##############################################################################
