#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################


from    ALDefinition   import  *
from    ALImportPython import  PySys
from    ALImportPython import  PyCopy
from    ALImportPython import  PyXml
from    ALImportPython import  PySemaphore
from    ALRandom       import  ALRandom
from    ALString       import  ALString
from    ALMetaObject   import  ALMetaObject


class ALObject( ALMetaObject ):
    """
    Virtual base class for all AL classes. AL = Asynchronous Action Library
    class namespace definition.

    The self.m_strMagicNumber variable (constant) is used to define
    the name of the class ini.e.:XML files.

    The self.m_strID is used to assign a unique identity string
    to each object in order to distinguish between different
    objects and copies of objects.

    WARNING: the method getId() is deprecated!!
    Must use the getID() instead...
    """


    def __init__( self ):
        """
        Constructor of the main pure virtual class.
        This constructor implements:
        the MagicNumber and the unique ID of the objects.
        """
        self.m_objectID          = self.generateRandomID()
        self.m_objectMagicNumber = self.getMagicNumber()
        self.m_objectName        = ALString( "obj"+ self.m_objectMagicNumber[3:]  )
        self.m_objectVersion     = ALString( AL_VERSION )
        self.m_opySemaphore      = None
        self.createSemaphore()


    def getName( self ):
        """
        To be Deprecated
        """
        return self.getObjectName()


    def getObjectName( self ):
        return ALString( self.m_objectName )


    def getClassMembers( self ):
        return self.__dict__


    def generateRandomID( self ):
        oPid = 44
        oPid = ALRandom.randInt( oPid, oPid * 2 )
        oalString = "%s-%04i-%04i-%04i-%04i-%04i-%04i" % ( self.getClassName(), int( float( AL_VERSION ) * 100 ),
                                                      ALRandom.randInt( oPid,9999 ),
                                                      ALRandom.randInt( 0, 9999 ),
                                                      ALRandom.randInt( oPid,9999 ),
                                                      ALRandom.randInt( 0, oPid),
                                                      ALRandom.randInt( oPid, 9999 ))
        return ALString( oalString )


    def createSemaphore(self):
        self.m_opySemaphore  = PySemaphore()


    def deleteSemaphore(self):
        del self.m_opySemaphore
        self.m_opySemaphore = None


    def getID( self ):
        return ALString( self.m_objectID )


    def getVersion( self ):
        return ALString( self.m_objectVersion )


    def getMagicNumber( self ):
        oalString = self.getClassName()
        self.m_objectMagicNumber = ALString( "mag" + oalString )
        return ALString( self.m_objectMagicNumber )


    def getClassName( self ):
        oalString = ALString( self.__class__ )
        oalString = oalString.replace( "<class '", "" )
        oalString = oalString.replace( "'>", "" )
        return ALString( oalString.split( "." )[1] )


    def synchronizeOn( self, _oalObject = None ):
        """
        This method must be used in together with the method synchronizeOff().
        This method makes the code threadsafe till the method synchronizeOff
        is called.
        """
        self.m_opySemaphore.acquire()


    def synchronizeOff( self, _oalObject = None ):
        """
        This method must be used in together with the method synchronizeOn().
        """
        self.m_opySemaphore.release()



    # Think about ############################################################
    def output( self, _oalStream = None ):
        if (_oalStream is not None):
            _oalStream.output( self )


    def input( self, _oalStream = None ):
        pass


    # Experimental
    def outputClassMembers( self, _oalFile = PySys.stdout ):
        _oalFile.write( "\n\n" )
        for ikey in self.__dict__:
            _oalFile.write( " ClassMember: " + ALString( ikey ) + "\n" )
            _oalFile.write( "        Type: " + ALString( self.__dict__[ ikey ].__class__) + "\n" )



    def outputXsdObject( self,  _oalXml, _oalXmlElement ):
        pass



    def outputXmlObject( self,  _oalXml, _oalXmlElement ):
        oalXmlElementObject = _oalXml.createElement( self.getMagicNumber() )
        _oalXmlElement.appendChild( oalXmlElementObject )
        for ikey in self.__dict__:
            objKey = self.__dict__[ ikey ]
            if ( objKey is not None ):
                if ( ikey!="m_oalDict" and ikey!="m_objectMagicNumber" and ikey!="_keys" ):
                    if (ikey[0:2]=="m_"):
                        oalXmlElement = _oalXml.createElement( ikey[2:] )
                        oalXmlElement.appendChild( _oalXml.createTextNode( ALString(  self.__dict__[ ikey ] ) ) )
                        oalXmlElementObject.appendChild( oalXmlElement )
        return  oalXmlElementObject



## To be fixed for jython on Linux - Windows version is OK
## [ERROR]: File  :: /mntdirect/_scisoft/users/pieritz/private/Workspace-Development/Workspace-2.4.3/PyAALib-JyAALib-HEAD-v2.0/modules/ALObject.py
##  [ERROR]: Method:: outputXmlObject
##  [ERROR]: Line  :: 194
##  [ERROR]: Object:: if ( objKey!= None ):
##  [ERROR]: TypeError: dict.__cmp__(x,y) requires y to be 'dict', not a 'NoneType'
##
#            if ( objKey!= None ):
#                if ( ikey!="m_oalDict" and ikey!="m_objectMagicNumber"  ):
#                    if (ikey[0:2]=="m_"):
#                        oalXmlElement = _oalXml.createElement( ikey[2:] )
#                        oalXmlElement.appendChild( _oalXml.createTextNode( ALString(  self.__dict__[ ikey ] ) ) )
#                        oalXmlElementObject.appendChild( oalXmlElement )





    def inputXmlObject( self, _oalXmlElement ):
        for inode in _oalXmlElement.childNodes:
            if inode.nodeType==PyXml.Node.ELEMENT_NODE:
                if ( ALString( inode.nodeName ) == ALString( "objectID" ) ):
                    for jnode in inode.childNodes:
                        if jnode.nodeType==PyXml.Node.TEXT_NODE:
                            oalStringValue = ALString( jnode.data )
                            self.m_objectID = oalStringValue.cleanXml()
                if ( ALString( inode.nodeName ) == ALString( "objectVersion" ) ):
                    for jnode in inode.childNodes:
                        if jnode.nodeType==PyXml.Node.TEXT_NODE:
                            oalStringValue = ALString( jnode.data )
                            self.m_objectVersion = oalStringValue.cleanXml()
                if ( ALString( inode.nodeName ) == ALString( "objectName" ) ):
                    for jnode in inode.childNodes:
                        if jnode.nodeType==PyXml.Node.TEXT_NODE:
                            oalStringValue = ALString( jnode.data )
                            self.m_objectName = oalStringValue.cleanXml()




    # Only to be virtual
    def outputXml( self,  _oalXml, _oalXmlElement ):
        oalXmlElementObject = self.outputXmlObject( _oalXml, _oalXmlElement )

    def inputXml( self, _oalXmlElement, _oalXml = None ):
        self.inputXmlObject( _oalXmlElement )


# To be deprecated
#    def outputXsd( self,  _oalXml, _oalXmlElement ):
#        oalXmlElementObject = self.outputXsdObject( _oalXml, _oalXmlElement )


    ###########################################################################################
    # Not finished yet
    def outputXsd( self, _oalXsdFile = PySys.stdout ):

        _oalXsdFile.write( "\n\n" )
        oalDict = {}
        for ikey in self.__dict__:
            oalString = ALString( self.__dict__[ ikey ].__class__ )
            oalString = oalString.replace( "<class '", "" )
            oalString = oalString.replace( "<type '", "" )
            oalString = oalString.replace( "'>", "" )
            oClassName =  ALString( oalString.split( "." )[0] )
            oName = oClassName[ 0:2 ]
            if (oName=="AL" and oClassName!="ALString" ):
                oalDict[ oClassName ] =  oClassName

        for ikey in oalDict:
            _oalXsdFile.write( "    <xs:include schemaLocation=\"" + ikey + ".xsd\"/>\n" )


        oClassName = self.getClassName()
        _oalXsdFile.write( "\n\n" )
        _oalXsdFile.write( "    <xs:element name=\"" + oClassName + "\">\n" )
        _oalXsdFile.write( "        <xs:complexType>\n" )
        _oalXsdFile.write( "            <xs:sequence>\n" )

        for ikey in self.__dict__:
            if ( self.__dict__[ ikey ]!= None):
                oName     = str( ikey )
                if (oName[0:2]=="m_"):
                    oName  = oName[2:]
                oalString = ALString( self.__dict__[ ikey ].__class__ )
                oalString = oalString.replace( "<class '", "" )
                oalString = oalString.replace( "<type '", "" )
                oalString = oalString.replace( "'>", "" )
                oClassName= ALString( oalString.split( "." )[0] )
                oCName = oClassName[ 0:2 ]
                if (oCName!="AL"):
                    if (oClassName=="str"):
                        oClassName = "string"
                    elif (oClassName=="bool"):
                        oClassName = "boolean"
                    elif (oClassName=="int"):
                        oClassName = "integer"
                    oClassName = "xs:" + oClassName
                if (oCName=="AL" and oClassName=="ALString"):
                    oClassName = "xs:string"
                _oalXsdFile.write( "                <xs:element name=\"" + oName + "\"\n")
                _oalXsdFile.write( "                            type=\"" + oClassName + "\"/>\n" )

        _oalXsdFile.write( "            </xs:sequence>\n" )
        _oalXsdFile.write( "        </xs:complexType>\n" )
        _oalXsdFile.write( "    </xs:element>\n" )
        _oalXsdFile.write( "\n\n" )



##############################################################################


