#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################


from ALImportPython        import PyOS
from ALImportPython        import PySys
from ALImportPython        import PyImport
from ALDefinition          import *
from ALLog                 import ALLog
from ALObject              import ALObject
from ALSystem              import ALSystem
from ALString              import ALString
from ALTime                import ALTime
from ALStreamFile          import ALStreamFile
from ALDictionary          import ALDictionary
from ALDiskExplorer        import ALDiskExplorer

from ALProfileTemplate          import *


class ALProfile( ALObject ):
    """
    Base class to profile the aalib log tree for multi plugins actions
    This class control the information to profile the ALApplication in a file
    """


    def __init__( self ):
        ALObject.__init__( self )



    def __fromIndex( self, _oalString ):
        oList = ALString( _oalString ).split( AL_TOKEN_SEPARATOR )
        oalStrTime = ALTime.timeStringValue( float( oList[0] ) )
        oalStrInfo = ALString( oList[1] )
        return oalStrTime, oalStrInfo


    def outputProfileFile(self, _oalFileName = None ):
        return self.getProfileFile( _oalFileName )


    def getProfileFileHtml( self, _oalFileName, _oalDictionary = None ):
        self.synchronizeOn()
        oalLogFileName = ALString( _oalFileName ) #mudar a extensao.....
        oalStringHour = ALTime.getHourName()
        oalNewExtension = ALString("-") + ALString( oalStringHour ) + ALString( "-profile.html" )
        oalStrFileName = ALDiskExplorer.setFileExtension( oalLogFileName, oalNewExtension )
        oalStrExtension = ".profiledb"
        oalBaseFolderName = ALDiskExplorer.setFileExtension( oalStrFileName, oalStrExtension )
        ALDiskExplorer.createFolder(oalBaseFolderName)


#        print oalBaseFolderName
        oalStrList = oalBaseFolderName.split("/")
        oalStringFolder  = oalStrList[len(oalStrList)-1]
#        print oalStringFolder
        oalString = oalStringFolder.replace( oalStrExtension, "-Timeline.xml" )
#        print oalString
        oalStrFileNameXml = oalString
#        print oalStrFileNameXml
#
#        print oalLogFileName
#        print oalBaseFolderName
        # Workaround to local profile file on the database
        oalString = oalStrFileNameXml.replace( "-Timeline.xml", ".txt" )
        oalStrNewProfileFile = oalBaseFolderName+"/"+oalString
#        oalStrProfileFile = ALDiskExplorer.setFileExtension(oalLogFileName, "-profile.xml" )
        oalStrProfileFile = oalLogFileName
        ALDiskExplorer.copyFile( oalStrProfileFile, oalStrNewProfileFile )


        oalRelativeFileFolder = "./" + oalStringFolder + "/"+ oalStrFileNameXml

#        print oalRelativeFileFolder

        oalStringDataTime = self.getUmtTimeString( ALLog.getTimeInit() )
        oalData = {
                   '_pageName' : ALDiskExplorer.getFileBaseName( oalStrFileName ),
                   '_widthZone1' : str("70%"),
                   '_widthZone2' : str("20%"),
                   '_widthZone3' : str("10%"),
                   '_dataInit' : oalStringDataTime,
                   '_dataFileNameXml' : oalRelativeFileFolder,
                   '_folderName' : oalStringFolder,
                   }

        oalString = ( ALPROFILE_TEMPLATE_TIMELINE % oalData )

        oalProfileFile = ALStreamFile( oalStrFileName, "w" )
        oalProfileFile.write( oalString )
        oalProfileFile.close()
        oalProfileFile = ALStreamFile( oalBaseFolderName+"/system.txt", "w" )
        oalProfileFile.write( ALLog.getSystemInfo() )
        oalProfileFile.write( "     Time End       :: " + ALLog.getTimeEnd()  + "\n" )
        if ( ALLog.getRuntime()!=0.0 ):
            oalProfileFile.write( "     Runtime[s]     :: " + ALString( ALLog.getRuntime() ) + "\n" )
            oalProfileFile.write( "     Runtime        :: " + ALTime.getTimeStringPSP( ALLog.getRuntime() )+ "\n" )
        oalProfileFile.close()


        #Output the JavaScript Package
        try:
            fp, pathname, description = PyImport.find_module("ALProfilePlugin")
            oalString     = ALString( pathname )

            oalStrReference = ALString( "ALProfilePlugin.py" )
            if (ALSystem.getInterpreter()=="Jython"):
                oalStrReference = ALString( "ALProfilePlugin$py.class" )

            oalStringBlue = ALString(oalString.replace( oalStrReference, "ALProfileImageBlue.png") )
            oalStringRed  = ALString(oalString.replace( oalStrReference, "ALProfileImageRed.png") )
            oalStrNewImageFile = oalBaseFolderName+"/dull-blue-circle.png"
            ALDiskExplorer.copyFile( oalStringBlue, oalStrNewImageFile )
            oalStrNewImageFile = oalBaseFolderName+"/dark-red-circle.png"
            ALDiskExplorer.copyFile( oalStringRed, oalStrNewImageFile )

            oalString2 = ALString( oalString.replace( oalStrReference, "ALProfileBundle.css") )
            oalString3 = ALString( oalBaseFolderName+"/bundle.css" )
            ALDiskExplorer.copyFile( oalString2, oalString3 )

            oalString2 = ALString(oalString.replace( oalStrReference, "ALProfileBundle.js") )
            oalString3 =  ALString(oalBaseFolderName+"/bundle.js" )
            ALDiskExplorer.copyFile( oalString2, oalString3 )

            oalString2 = ALString(oalString.replace( oalStrReference, "ALProfileLabellers.js") )
            oalString3 =  ALString(oalBaseFolderName+"/labellers.js")
            ALDiskExplorer.copyFile( oalString2, oalString3 )


            oalString2 = ALString(oalString.replace( oalStrReference, "ALProfileStyles.css" ) )
            oalString3 = ALString( oalBaseFolderName + "/styles.css")
            ALDiskExplorer.copyFile( oalString2, oalString3 )

            oalString2 = ALString(oalString.replace( oalStrReference, "ALProfileTimeline.js" ))
            oalString3 = ALString( oalBaseFolderName+"/timeline.js" )
            ALDiskExplorer.copyFile( oalString2, oalString3 )

            oalString2 = ALString(oalString.replace( oalStrReference, "ALProfileTimeline-api.js" ) )
            oalString3 = ALString(oalBaseFolderName+"/timeline-api.js")
            ALDiskExplorer.copyFile( oalString2, oalString3 )
        except:
            print " [Warning] Profile problem to copy JavaScript Package from modules"

        self.synchronizeOff()
        self.outputDataBase( oalBaseFolderName, oalRelativeFileFolder, _oalDictionary )
        return oalStrFileName


    def getUmtTimeString( self, _oalString ):
        oalString =  _oalString
        oalString =  oalString.replace("[","")
        oalString =  oalString.replace("]","")
        oalString =  oalString.replace("/","-")
        oalStrList=  oalString.split("-")
        return ALString( oalStrList[1]+" "+oalStrList[2]+" "+oalStrList[0]+" "+oalStrList[3]+" GMT-0000" )




    def outputDataBase( self, _oalBaseFolderName, _oalStrFileNameXml, _oalDictionary ):
        if (_oalDictionary is None):
            return
        self.synchronizeOn()

        # Workaround to bypass import limitation from ALDebug
        obStatusTagDebug = False
        for cmdkey in PySys.argv:
            if ( cmdkey == "--DEBUG-TAG" ):
                obStatusTagDebug = True



        oalDict = ALDictionary()
        for i in _oalDictionary.getInverseOrder():
            oalDict.addObject( _oalDictionary[i], _oalDictionary[i] )

        oalFileName = ALDiskExplorer.getAbsolutePath( _oalStrFileNameXml )
        oalXmlFile = ALStreamFile( oalFileName, "w" )

        oalXmlFile.write( "\n" )
        oalXmlFile.write( "<data>\n" )





        oalBoolStatusGlobal = True
        oiNumberActions = 0
        for i in oalDict.getInverseOrder():
            oiNumberActions += 1
            oalBoolEventTagDebug = False
            oalBoolStatus        = True
            ostrTimeStart        = None  # patch for bug in the log tree
            ostrTimeEnd          = None
            oalStringText = "[SUCCESS] :: Action complete until the end"

            oalDataFileName = _oalBaseFolderName+"/"+i+".txt"
            oalDataFile = ALStreamFile( oalDataFileName, "w" )
            oalDataFile.write( "\n[ Log Information :: " + i +" ]\n" )
            oalDataFile.write( "###############################################################################\n\n" )
            for j in _oalDictionary.ordered():
                if ( _oalDictionary[j]==i):
                    oalStrTime, oalStrInfo = self.__fromIndex( j )
                    oalDataFile.write( oalStrTime+"  "+ oalStrInfo + "\n" )
                    if (oalStrInfo.contains( "[ThreadStart]")):
                        oalString =  oalStrInfo.replace("[DEBUG]: [ALAction] [ThreadStart]  :: [","")
                        ostrTimeStart = self.getUmtTimeString( oalString )

                    if (oalStrInfo.contains( "[ThreadEnd]")):
                        oalString =  oalStrInfo.replace("[DEBUG]: [ALAction] [ThreadEnd]    :: [","")
                        ostrTimeEnd = self.getUmtTimeString( oalString )


                    if (oalStrInfo.contains( "[FAILURE]") or oalStrInfo.contains( "[ERROR]")):
                        oalStringText = "[FAILURE] :: Action exception"
                        oalBoolStatus = False
                        oalBoolStatusGlobal = False

                    if (obStatusTagDebug==True and oalStrInfo.contains( "[DEBUG]") ):
                        oalBoolEventTagDebug = True

                    # patch to fix time stuffs
                    if (ostrTimeStart==None and oalStrInfo.contains( "[ThreadStart]")==False):
                        ostrTimeStart = self.getUmtTimeString( oalStrTime )
                    elif (ostrTimeStart is not None and oalStrInfo.contains( "[ThreadEnd]")==False ):
                        ostrTimeEnd = self.getUmtTimeString( oalStrTime )



            oalDataFile.close()

            # Patch when the thread has a NO end = ex.: from PyQT windows crashes
            if (ostrTimeEnd==None):
                ostrTimeEnd = ALTime.getTimeStringUmt()
            if (ostrTimeStart==None):
                ostrTimeStart = ostrTimeEnd


            oalDataFileName = oalDataFileName.replace( _oalBaseFolderName,"." )

            oalXmlFile.write( "      <event\n" )
            oalXmlFile.write( "         start=\"" + ostrTimeStart + "\" \n" )
            oalXmlFile.write( "         end=\"" + ostrTimeEnd + "\" \n" )
            if (ostrTimeStart!=ostrTimeEnd):
                oalXmlFile.write( "         isDuration=\"true\" \n" )
            oalStrList = ALString(i).split("-")
            oalStrNew = oalStrList[0]
            oalXmlFile.write( "         title=\""+ oalStrNew + "\" \n")
            oalXmlFile.write( "         link=\""+ oalDataFileName + "\" \n")

            if (oalBoolStatus==False):
                oalXmlFile.write( "         color=\"#CC6666\" \n")
                oalXmlFile.write( "         image=\"./dark-red-circle.png\" \n")
            else:
                oalXmlFile.write( "         image=\"./dull-blue-circle.png\" \n")
                if (oalBoolEventTagDebug==True):
                    oalXmlFile.write( "         color=\"#669966\" \n")
            oalXmlFile.write( "         >\n")
            oalStrNew = "[ID]::"+i+" "+oalStringText
            oalXmlFile.write( "         "+oalStrNew+" \n")
            oalXmlFile.write( "      </event>\n" )



        # Writing the runtime total
        ostrTimeStart = self.getUmtTimeString( ALLog.getTimeInit() )
        ostrTimeEnd   = self.getUmtTimeString( ALLog.getTimeEnd() )
        oalStrTitle   = "Application RunTime"
        oalDataFileName =  "./" + ALDiskExplorer.getFileName( _oalStrFileNameXml )
        oalDataFileName = oalDataFileName.replace( "-Timeline.xml", ".txt" )

        oalStringText = "[SUCESS] The Application has NO errors :: Number Actions= " + ALString( oiNumberActions )
        oalXmlFile.write( "      <event\n" )
        oalXmlFile.write( "         start=\"" + ostrTimeStart + "\" \n" )
        oalXmlFile.write( "         end=\"" + ostrTimeEnd + "\" \n" )
        oalXmlFile.write( "         isDuration=\"true\" \n" )
        oalXmlFile.write( "         title=\""+ oalStrTitle + "\" \n")
        oalXmlFile.write( "         link=\""+ oalDataFileName + "\" \n")
        if (oalBoolStatusGlobal==False):
            oalXmlFile.write( "         color=\"#CC6666\" \n")
            oalXmlFile.write( "         image=\"./dark-red-circle.png\" \n")
            oalStringText = "[WARNING] The Application has at least one exception :: Number Actions= " + ALString( oiNumberActions )
        oalXmlFile.write( "         >\n")
        oalXmlFile.write( "         "+oalStringText+" \n")
        oalXmlFile.write( "      </event>\n" )


        oalXmlFile.write( "</data>\n" )
        oalXmlFile.close()
        self.synchronizeOff()




    def getProfileFile( self, _oalFileName  ):
        oalFileName = _oalFileName
        if ( ALDiskExplorer.existFile( oalFileName )==False):
            return
        self.synchronizeOn()

        oalStrFileName = ALDiskExplorer.setFileExtension( ALString( oalFileName ), "-profile.xml" )

        oalLogFile = ALStreamFile( oalFileName, "r" )
        oalList =  oalLogFile.getLines()
        oalLogFile.close()

        oalProfileFile = ALStreamFile( oalStrFileName, "w" )
        oalProfileFile.write( "<?xml version=\"1.0\"?>\n" )
        oalProfileFile.write( "<!-- Generated Log File Profile :: PyAALib-JyAALib -->\n" )
        oalProfileFile.write( "<lgProfile>\n" )
        oalProfileFile.write( "<version>" + self.getVersion() +"</version>\n" )
        oalProfileFile.write( "<lgLineList>\n" )

        for iLine in oalList:
            oalProfileFile.write( iLine )

        oalProfileFile.write( "</lgLineList>\n" )
        oalProfileFile.write( "\n\n</lgProfile>\n" )
        oalProfileFile.close()

        self.synchronizeOff()
        return oalStrFileName


##############################################################################


