#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################


from ALSystem                import  ALSystem
from ALObject                import  ALObject
from ALString                import  ALString
from ALToken                 import  ALToken
from ALDictionary            import  ALDictionary
from ALXml                   import  ALXml
from ALXmlNode               import  ALXmlNode
from ALStreamFile            import  ALStreamFile
from ALStreamSerial          import  ALStreamSerial
from ALStreamXml             import  ALStreamXml
from ALDiskExplorer          import  ALDiskExplorer
from ALFile                  import  ALFile
from ALVerbose               import  ALVerbose



class ALResource( ALObject ):
    """
    Parent class for all ALResource objects.
    """


    def __init__( self, _oalString = "resource.res", _bUserPath = True, _bRead = True ):
        ALObject.__init__( self )
        self.m_oalDict = ALDictionary()
        self.m_oResourceFilePath = ALDiskExplorer.getCurrentFolder()
        self.m_oResourceFileName = ALString( self.m_oResourceFilePath + ALString( "/" ) + ALString( _oalString ) )

        oalStringFolder = ALString( "/Library/" )
        if (ALSystem.getOSName()=="Linux"):
            ALString( "/.Library/" )

        if (_bUserPath == True):
            self.m_oResourceFilePath = ALSystem.getUserPath() + oalStringFolder + ALFile.getBaseName( _oalString )
            self.m_oResourceFileName = ALString( self.m_oResourceFilePath + ALString( "/" ) + ALString( _oalString ) )
        if (_bRead == True):
            try:
                self.input()
            except:
                ALVerbose.warning("Exception reading the resource file: " + self.m_oResourceFileName )


    def getResourceDictionary(self):
        self.synchronizeOn()
        oalDic = ALDictionary( self.m_oalDict )
        self.synchronizeOff()
        return oalDic


    def getResource( self, _oalToken ):
        self.synchronizeOn()
        oalObject = self.m_oalDict.getValue( ALToken( _oalToken ) )
        self.synchronizeOff()
        return oalObject


    def setResource( self, _oalToken, _oalTokenObject ):
        self.synchronizeOn()
        self.__setResource( _oalToken, _oalTokenObject )
        self.synchronizeOff()


    def importResource( self, _oalString ):
        oalResource = ALResource( _oalString, False )
        oalDictionary = oalResource.getResourceDictionary()
        for iToken in oalDictionary.ordered():
            self.setResource( iToken, oalDictionary[iToken])



    def __setResource( self, _oalToken, _oalTokenObject ):
        self.m_oalDict.add( ALToken( _oalToken ), _oalTokenObject )



    def reset( self ):
        self.synchronizeOn()
        self.m_oalDict = ALDictionary()
        self.synchronizeOff()
        self.output()


    def exist( self ):
        self.synchronizeOn()
        obExist = False
        if ALDiskExplorer.existFile( self.m_oResourceFileName ) :
            obExist = True
        self.synchronizeOff()
        return obExist



    def output( self ):
        self.synchronizeOn()
        ALDiskExplorer.createFolder( self.m_oResourceFilePath )

        oalXml = ALXml()
        oalXml.addComment( "AALib Resource File::DO NOT EDIT" )
        self.synchronizeOff()

        oalXml.addObject( self )

        self.synchronizeOn()
        if (ALSystem.getOSName()!="Linux"):
            oalXml.output( self.m_oResourceFileName, True )
        else:
            oalXml.output( self.m_oResourceFileName ) # using to bypass error on Jython linux version
        oalXml.close()

        ALVerbose.aalibDEBUG( "Resource::Saved   = " + self.m_oResourceFileName  )
        self.synchronizeOff()



    def input( self ):
        self.synchronizeOn()
        if ( ALDiskExplorer.existFile( self.m_oResourceFileName ) ) :

            oalXml = ALXml()
            oalXml.input( self.m_oResourceFileName )
            oalXmlElement = oalXml.getElement( self.getMagicNumber() )

            if (oalXmlElement!=None):
                self.inputXml( oalXmlElement, oalXml )
            oalXml.close()

            ALVerbose.aalibDEBUG( "Resource::Loaded  = " + self.m_oResourceFileName )
        else :
            ALVerbose.aalibDEBUG( "Resource::It will be CREATED = " + self.m_oResourceFileName )
        self.synchronizeOff()



    def outputXml( self, _oalXml, _oalXmlElement ):
        oalXmlElementObject = self.outputXmlObject( _oalXml, _oalXmlElement )
        self.m_oalDict.outputXml( _oalXml, oalXmlElementObject )



    def inputXml( self, _oalXmlElement, _oalXml  ):
        self.inputXmlObject( _oalXmlElement )
        oalXmlElementDictionary = _oalXml.getElementObject( _oalXmlElement, self.m_oalDict.getMagicNumber() )
        if (oalXmlElementDictionary!=None):
            self.m_oalDict.inputXml( oalXmlElementDictionary )




    ###########################################################################
    # Serial Output - TO BE DEPRECATED
    def outputSerial( self ):
        oalStreamSerial = ALStreamSerial( self.m_oResourceFileName, "wb" )
        oalStreamSerial.output( self.m_oalDict )
        oalStreamSerial.close()



    def inputSerial( self ):
        if ALDiskExplorer.existFile( self.m_oResourceFileName ) :
            oalStreamSerial = ALStreamSerial( self.m_oResourceFileName, "rb" )
            self.m_oalDict = oalStreamSerial.input()
            oalStreamSerial.close()
            ALVerbose.aalibDEBUG( "Resource::Loaded  = " + self.m_oResourceFileName )
        else :
            ALVerbose.aalibDEBUG( "Resource::CREATED = " + self.m_oResourceFileName )


    # Only to import old resouce files - STRINGS - To Be Deprecated ==========
    def outputSerialASCII( self, _oalObject = None ):
        self.synchronizeOn()
        oalStreamFile = ALStreamFile( "ascii-resource.res", "w" )
        for ikey in self.m_oalDict.ordered():
            oalStreamFile.write( ALString( ikey ) + ' ' + ALString( self.m_oalDict[ikey] ) + '\n')
        oalStreamFile.close()
        ALVerbose.aalibDEBUG( "Resource::Saved   = " + "ascii-resource.res"  )
        self.synchronizeOff()


    def inputSerialASCII( self, _oalObject = None ):
        self.synchronizeOn()
        if ALDiskExplorer.existFile( "ascii-resource.res" ) :
            oalStreamFile = ALStreamFile( "ascii-resource.res", "r" )
            #self.m_oalDict = oalStreamFile.read()
            for iline in oalStreamFile.readLines():
                key, token  = ALString.split( iline )
                self.setResource( ALString( key ), ALString( token ) )
            oalStreamFile.close()
            ALVerbose.aalibDEBUG( "Resource::Loaded  = " + "ascii-resource.res" )
        else :
            ALVerbose.aalibDEBUG( "Resource::CREATED = " + "ascii-resource.res" )
        self.synchronizeOff()


##############################################################################
