#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################


from ALObject          import  ALObject
from ALDebug           import  ALDebug
from ALList            import  ALList
from ALTrace           import  ALTrace
from ALTime            import  ALTime
from ALString          import  ALString
from ALVerbose         import  ALVerbose
from ALTrace           import  ALTrace
from ALAssert          import  ALAssert


class ALTest( ALObject ):
    """
    Interface for AL Tests Object
    """
    def __init__( self, _oalStringTestName = None ):
        ALObject.__init__( self )
        self.m_oalStringTestName  = _oalStringTestName
        self.m_oalListTest        = ALList()
        self.m_oBoolIsVerbose     = False
        self.m_oBoolIsDebug       = False
        self.m_oBoolIsAssert      = False
        self.m_oBoolIsLogFile     = False
        self.m_oBoolIsProfile     = False
        self.m_obForceDebug       = False


    def controlCommandLine( self ):
        if ( ALDebug.isDebugModeOn()==True or ALDebug.isDebugModeForceOn()==True ):
            ALVerbose.setVerboseDebugOn()
 #           ALVerbose.setProfileOn()  # to think about
        if ( ALDebug.isDebugModeVerboseOn()==True ):
            ALVerbose.setVerboseOn()
        if ( ALDebug.isDebugModeLogFileOn()==False ):
            ALVerbose.setLogFileOff()
        if ( ALDebug.isDebugModeAssertOn()==True):
            ALAssert.setAssertionModeOn()
        if ( ALDebug.isDebugModeProfileOn()==True):
           ALVerbose.setProfileOn()


    def outputProfile(self):
        ALVerbose.getProfileFile()


    def getTestName( self ):
        oalString = ALString("Test")
        if (self.m_oalStringTestName!=None):
            oalString = self.m_oalStringTestName
        return oalString


    def addTestMethod( self, _oClassMethod ):
        self.m_oalListTest.append( _oClassMethod )


    def patchMethodName(self, _oalStringText ):
        """
        Fix for Jython name scheme of methods = <method xxxxx....
        and Python is = <bound method xxxx....
        """
        oalStringTest = ALString ( _oalStringText )
        oalStringTest2 = oalStringTest.replace( "<", " " )
        oalString = oalStringTest2.getNextToken("method")
        if (oalString==None):
            oalString = ALString ( "None Object" )
        return oalString


    def __preProcessKernel( self ):
        self.m_oBoolIsVerbose = ALVerbose.isVerbose()
        self.m_oBoolIsDebug   = ALVerbose.isVerboseDebug()
        self.m_oBoolIsLogFile = ALVerbose.isLogFile()
        self.m_oBoolIsAssert  = ALAssert.isAssertionMode()
        self.m_oBoolIsProfile = ALVerbose.isProfile()


    def __postProcessKernel( self ):
        if (self.m_oBoolIsVerbose==True):
            ALVerbose.setVerboseOn()
        else:
            ALVerbose.setVerboseOff()

        if (self.m_oBoolIsDebug==True):
            ALVerbose.setVerboseDebugOn()
        else:
            ALVerbose.setVerboseDebugOff()

        if (self.m_oBoolIsLogFile==True):
            ALVerbose.setLogFileOn()
        else:
            ALVerbose.setLogFileOff()

        if (self.m_oBoolIsAssert==True):
            ALAssert.setAssertionModeOn()
        else:
            ALAssert.setAssertionModeOff()

        if (self.m_oBoolIsProfile==True):
            ALVerbose.setProfileOn()
        else:
            ALVerbose.setProfileOff()



    def processKernel( self ):
        ALVerbose.aalibDEBUG( "Execution: ALTest.processKernel()" )



    def raiseControl( self ):
        oBoolStatus =  ALDebug.isDebugModeOn()
        ALDebug.controlDebug( self, oBoolStatus )


    def executeKernel( self ):
        ALVerbose.aalibDEBUG( "Execution: ALTest.executeKernel()" )
        self.__preProcessKernel()
        self.preProcess()

        self.process()
        self.processKernel()

        self.postProcess()
        self.__postProcessKernel()



    def execute( self ):
        ALVerbose.aalibDEBUG( "Execution: ALTest.execute()" )
        try:
            self.executeKernel()
        except:
            self.raiseControl()




    ##########################################################################
    # To be modified by the user in the child classes
    def preProcess( self ):
        ALVerbose.aalibDEBUG( "Execution: ALTest.preProcess()" )


    def process( self ):
        ALVerbose.aalibDEBUG( "Execution: ALTest.process()" )


    def postProcess( self ):
        ALVerbose.aalibDEBUG( "Execution: ALTest.postProcess()" )



##############################################################################