#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################


from ALImportAction      import *
from ALImportKernel      import *
from ALImportMath        import *
from ALImportSystem      import *


##############################################################################
# ALActionProcess

class ALTestCaseALActionProcess( ALTestCase ):

    def testALActionProcess(self):
        ALVerbose.unitTest( "UnitTest ALActionProcess" )

#        oalStrPython = "C:/Program-Files/Automation/win32/Program-Files/python-2.5.2/python.exe"
        oalStrPython = "C:/Python25/python.exe"
        oalStrCode   = ALDiskExplorer.getAbsolutePath( "./DataForTests/demoApplication.py" )
        if (ALSystem.getOSName()=="Linux"):
            oalStrPython = "python"
        if (ALSystem.getInterpreter()=="Jython"):
            ALVerbose.warning( "JAVA-Jython Windows: some problems to identify external process started by python" )
            oalStrCode = "-h"

        oalCommand   = oalStrPython  + " " + oalStrCode


        ALVerbose.unitTest( "Creating ALActionProcess" )
        ALVerbose.unitTest( "Please Wait - The demon run during 15 seconds" )
        oalActionProcess = ALActionProcess( oalCommand, "./PythonJythonTestDataFor/ActionProcess"  )
        oalActionProcess.setTimeOut(20.0)
        oalActionProcess.executeProcess()

        ALVerbose.unitTest( "ALActionProcess.executeProcess() - Running = External Process is running" )
        oalActionProcess.synchronize()
        ALVerbose.unitTest( "[Total Time Duration]: " + ALString( oalActionProcess.getTimeDuration() ) )

        oalListOutput2      = oalActionProcess.getOutput( )
        oalListOutputError2 = oalActionProcess.getOutputError()
        oalListInput2       = oalActionProcess.getInput()


        ALVerbose.unitTest( "Input to the Child Process: " )
        for i in oalListInput2:
 #           ALVerbose.unitTest( i )
            ALVerbose.screenID( oalActionProcess.getID(), i )
        ALVerbose.unitTest( "Output from the Child Process: " )
        for i in oalListOutput2:
 #           ALVerbose.unitTest( i )
            ALVerbose.screenID( oalActionProcess.getID(), i )
        ALVerbose.unitTest( "Output Error from the Child Process: " )
        for i in oalListOutputError2:
 #           ALVerbose.unitTest( i )
            ALVerbose.screenID( oalActionProcess.getID(), i )
            ALVerbose.unitTest( i )

        # Output Individual Log action File
        ALVerbose.outputActionLog( oalActionProcess.getID(), "./TestCreationFolder/ForLogAction/TestOutputActionLog.log" )
        ALVerbose.outputActionLog( oalActionProcess.getID() )


        del oalActionProcess
        ALVerbose.unitTest()
        ALVerbose.unitTest("\n\n\n")




    def testALActionProcessJava(self):
        ALVerbose.unitTest( "UnitTest ALActionProcessJava" )

        oalStrPython = "java.exe"
        if (ALSystem.getOSName()!="Windows"):
            oalStrPython = "java"

        oalStrCode   = " -h"
        oalCommand   = oalStrPython  + " " + oalStrCode


        ALVerbose.unitTest( "Creating ALActionProcessJava" )

        oalActionProcess = ALActionProcess( oalCommand, "./PythonJythonTestDataFor/ActionProcessJava"  )
        oalActionProcess.setTimeOut(20.0)
        oalActionProcess.executeProcess()

        ALVerbose.unitTest( "ALActionProcess.executeProcess() - Running = External Process is running" )
        oalActionProcess.synchronize()
        ALVerbose.unitTest( "[Total Time Duration]: " + ALString( oalActionProcess.getTimeDuration() ) )

        oalListOutput2      = oalActionProcess.getOutput( )
        oalListOutputError2 = oalActionProcess.getOutputError()
        oalListInput2       = oalActionProcess.getInput()

        ALVerbose.unitTest( "Input to the Child Process: " )
        for i in oalListInput2:
            ALVerbose.unitTest( i )
        ALVerbose.unitTest( "Output from the Child Process: " )
        for i in oalListOutput2:
            ALVerbose.unitTest( i )
        ALVerbose.unitTest( "Output Error from the Child Process: " )
        for i in oalListOutputError2:
            ALVerbose.unitTest( i )

        del oalActionProcess
        ALVerbose.unitTest()
        ALVerbose.unitTest("\n\n\n")





##############################################################################

    def process( self ):
        self.addTestMethod( self.testALActionProcess )
        self.addTestMethod( self.testALActionProcessJava )



##############################################################################


if __name__ == '__main__':

    # JIT compiler accelerator
    ALCompiler.accelerator()

    oalTestCaseALActionProcess = ALTestCaseALActionProcess( "TestCase ALActionProcess" )
    oalTestCaseALActionProcess.execute()
