#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################


from ALImportSystem      import *
from ALImportKernel      import *
from ALImportAction      import *
from ALImportMath        import *


##############################################################################
# Module ALActionMethodTimer

def printLine( _oalObject = None ):
    ALVerbose.screen( " printing line on screen" )


##############################################################################
# Module ALActionPure
class ActionPure( ALAction ):

    def process( self, _oalObject = None ):
        self.synchronizeOn()
        for i in range(0, 10000):
            self.actionControl()
            ALVerbose.screenID( self.getID(), " -----------------Running ActionPure to Test ActionMethodTimer: " + ALString(i) )
            ALTime.sleep( 1.0 )
        self.synchronizeOff()


    def methodTest( self, _oalObject = None ):
        ALVerbose.screenID( self.getID(), "Running ActionPure---TEST" )

##############################################################################
# Module ALActionWait
class ActionWait( ALAction ):

    def process( self, _oalObject = None ):
        ALTime.wait( 5.0 )



##############################################################################

class ClassAA:
    def __init__( self ):
        self.m_oalThreadingCondition  = ALThreadingCondition()

    def methodFirst( self, _oalObject = None ):
        self.m_oalThreadingCondition.synchronizeOn()
        oalString = ALString( _oalObject )
        ALVerbose.screen( "ClassA::methodFirst -> " + oalString  )
        # Signalizing an error to exit the application
        #ALApplicationCommunicator.setSignal( ALSignal.error( "File do not exist :: BATATA in ClassAA::methodFirst" ) )
        self.m_oalThreadingCondition.synchronizeOff()


    # Signalizing an error to exit the application
    # ALApplication.setSignal( ALSignal.error() )
    def executeError( self, _oalObject = None ):
        self.m_oalThreadingCondition.synchronizeOn()
        ALApplication.setSignal( ALSignal.error( "File do not exist :: BATATA in ClassAA::executeError" ) )
        self.m_oalThreadingCondition.synchronizeOff()



class ClassBB:
    def __init__( self ):
        self.m_oalThreadingCondition  = ALThreadingCondition()

    def methodFirst( self, _oalObject = None ):
        self.m_oalThreadingCondition.synchronizeOn()
        oalString = ALString( _oalObject )
        ALVerbose.screen( "ClassB::methodFirst -> " + oalString  )
        ALTime.sleep( 2 )
        self.m_oalThreadingCondition.synchronizeOff()




class ClassCC:
    def __init__( self ):
        self.m_oalThreadingCondition  = ALThreadingCondition()

    def methodFirst( self, _oalObject = None ):
        self.m_oalThreadingCondition.synchronizeOn()
        oalString = ALString( _oalObject )
        ALVerbose.screen( "ClassC::methodFirst -> " + oalString )
        self.m_oalThreadingCondition.synchronizeOff()

#===============================================================================
#Plugins for multiaction application and ApplicationActionControl Test
#===============================================================================

class TestPlugin01( ALPlugin ):

    def __init__( self ):
        ALPlugin.__init__( self )

    def process( self, oalObject = None ):
        self.synchronizeOn()
        ALVerbose.screenID( self.getID(), "Running TestPlugin01010101" )
        self.synchronizeOff()



class TestPlugin02( ALPlugin ):

    def __init__( self ):
        ALPlugin.__init__( self, "tokenPlugin02" )

    def process( self, _oalObject = None ):
        self.synchronizeOn()
        ALVerbose.screenID( self.getID(),"Running TestPlugin02020202" )
        self.synchronizeOff()


    def methodTest( self, _oalObject = None ):
        ALVerbose.screenID( self.getID(),"Running TestPlugin02020202---TEST" )


class TestPlugin03( ALPlugin ):

    def process( self, _oalObject = None ):
        oalToken = self.getToken()
        self.synchronizeOn()
        ALVerbose.screenID( self.getID(),"Running TestPlugin   ||||" + oalToken )
        self.synchronizeOff()


    def methodDoPrint(self):
        ALVerbose.screenID( self.getID(),"printing from TestPlugi3" )


    def methodDoPrint2(self):
        ALVerbose.screenID( self.getID(),"printing from TestPlugi3 - print 2 - static test" )


    def methodDoPrintAction(self):
        ALVerbose.screenID( self.getID(),"printing from TestPlugi3 an instance of Plugin2" )
        oalAction = TestPlugin02()
        oalAction.executeAction()


    def methodDoSleep(self, _oalObject = None ):
        ALVerbose.screenID( self.getID(),"Sleeping 3 seconds" )
        ALTime.sleep( 3 )
        ALVerbose.screenID( self.getID(),"Weak up after 3 seconds" )



class TestPlugin04( TestPlugin03 ):
    pass


class TestPlugin05( ALPlugin ):

    def process( self, _oalObject = None ):

        value = ALApplication.getCommandLineArgument( "--cB" )
        ALVerbose.screenID( self.getID(), " Test Static method: " + ALString( value ) )

        oalToken = self.getToken()

        self.synchronizeOn()
        ALVerbose.screenID( self.getID(),"Running TestPlugin   ||||" + oalToken )
        for i in range(0, 100000):
            j = i
        ALVerbose.screenID( self.getID(),"Total Iteraction 1: " + ALString( j ) )
        for i in range(0, 100000):
            j = i
        ALVerbose.screenID( self.getID(),"Total Iteraction 2: " + ALString( j ) )
        for i in range(0, 100000):
            j = i
        ALVerbose.screenID( self.getID(),"Total Iteraction 3: " + ALString( j ) )
        self.synchronizeOff()




class TestPlugin06( ALPlugin ):

    def process( self, _oalObject = None ):
        self.synchronizeOn()
        oalPlugin = TestPlugin05()
        oalPlugin.executePlugin()
        ALVerbose.unitTest( "Running Plugin 5 - in 6" )
        oalPlugin = TestPlugin05()
        oalPlugin.executePlugin()
        ALVerbose.unitTest( "Running Plugin 5 - in 6" )
        self.synchronizeOff()



class TestPlugin07( ALPlugin ):

    def process( self, _oalObject = None ):
        value = ALApplication.getCommandLineArgument( "--cB" )
        ALVerbose.screenID( self.getID(), "Test Static method: " + ALString( value ) )

        self.synchronizeOn()
        oalPlugin = TestPlugin06()
        oalPlugin.executePlugin()
        ALVerbose.unitTest( "Running Plugin 6 - in 7" )
        oalPlugin = TestPlugin05()
        oalPlugin.executePlugin()
        ALVerbose.unitTest( "Running Plugin 5 - in 7" )
        oalPlugin = TestPlugin06()
        oalPlugin.executePlugin()
        ALVerbose.unitTest( "Running Plugin 6 - in 7" )
        self.synchronizeOff()




def methodFactory( _oalObject = None ):
    ALVerbose.screen( "" )
    ALVerbose.screen( "" )
    ALVerbose.screen( "#####################################################" )
    ALVerbose.screen( "EXECUTION of Application based on Plugins" )
    oalPlugin = ALApplication.getPluginObject( "TestPlugin01" )
    if (oalPlugin!=None):
        oalPlugin.executePlugin()
        oalPlugin.outputScreenInformation()
        ALVerbose.screen( "Test APPLICATION PluginFactory: " + oalPlugin.getToken() )

    oalPlugin = ALApplication.getPluginObject( "tokenPlugin02" )
    if (oalPlugin!=None):
        oalPlugin.executePlugin()
        oalPlugin.methodTest()
        oalPlugin.outputScreenInformation()
        ALVerbose.screen( "Test APPLICATION PluginFactory: " + oalPlugin.getToken() )

    oalPlugin = ALApplication.getPluginObject( "tokenPlugin03" )
    if (oalPlugin!=None):
        oalPlugin.executePlugin()
        oalPlugin.outputScreenInformation()
        ALVerbose.screen( "Test APPLICATION PluginFactory: " + oalPlugin.getToken() )

    oalPlugin = ALApplication.getPluginObject( "tokenPlugin04" )
    if (oalPlugin!=None):
        oalPlugin.executePlugin()
        oalPlugin.outputScreenInformation()
        ALVerbose.screen( "Test APPLICATION PluginFactory: " + oalPlugin.getToken() )


def methodFactoryFromFolder( _oalObject = None ):
    ALVerbose.screen( "" )
    ALVerbose.screen( "" )
    ALVerbose.screen( "#####################################################" )
    ALVerbose.screen( "EXECUTION of Application based on Plugins FROM FOLDER" )
    oalPlugin = ALApplication.getPluginObject( "PluginTest01" )
    if (oalPlugin!=None):
        oalPlugin.executePlugin()
        oalPlugin.outputScreenInformation()
        ALVerbose.screen( "#######APPLICATION PluginFactory: " + oalPlugin.getToken() )

    oalPlugin = ALApplication.getPluginObject( "PluginTest02" )
    if (oalPlugin!=None):
        oalPlugin.executePlugin()
        oalPlugin.methodTest()
        oalPlugin.outputScreenInformation()
        ALVerbose.screen( "#######APPLICATION PluginFactory: " + oalPlugin.getToken() )

    oalPlugin = ALApplication.getPluginObject( "PluginTest03" )
    if (oalPlugin!=None):
        oalPlugin.executePlugin()
        oalPlugin.outputScreenInformation()
        ALVerbose.screen( "#######APPLICATION PluginFactory: " + oalPlugin.getToken() )

    oalPlugin = ALApplication.getPluginObject( "PluginTest04" )
    if (oalPlugin!=None):
        oalPlugin.executePlugin()
        oalPlugin.outputScreenInformation()
        ALVerbose.screen( "#######APPLICATION PluginFactory: " + oalPlugin.getToken() )



def methodFactoryFromFile( _oalObject = None ):
    ALVerbose.screen( "" )
    ALVerbose.screen( "" )
    ALVerbose.screen( "#####################################################" )
    ALVerbose.screen( "EXECUTION of Application based on Plugins FROM FILE" )
    oalPlugin = ALApplication.getPluginObject( "ClassPluginFile" )
    if (oalPlugin!=None):
        oalPlugin.executePlugin()
        oalPlugin.outputScreenSignature()
        ALVerbose.screen( "#######APPLICATION PluginFactory FROM FILE: " + oalPlugin.getToken() )


def methodFactoryAll( _oalObject = None ):
    ALVerbose.screen( "" )
    ALVerbose.screen( "" )
    ALVerbose.screen( "#####################################################" )
    ALVerbose.screen( "EXECUTION of Application based on Plugins ALL LIST in the FACTORY" )
#    ALApplication.outputScreenPlugin()
#    ALApplication.outputScreenPluginGroup()
#    ALApplication.outputScreenPluginGroupTree()



class ALTestCaseALApplication( ALTestCase ):
    """
    TestSuite for ALApplication Class
    """
    def testALApplication( self ):
        ALVerbose.unitTest( "Test ALApplication" )

        ALApplication( "DemoTestCaseALApplication", "2.0" ).execute()

        ALVerbose.unitTest()
        ALVerbose.unitTest("\n\n\n")



    def testALApplicationControlDebug( self ):
        ALVerbose.unitTest( "Test ALApplication ControlDebug" )


        # Standart Debug options hardcoded
        ALApplication.setDebugProfileOn()
        ALApplication.setDebugVerboseOn()
        ALApplication.setDebugPluginOn()

        ALApplication( "DemoTestCaseALApplication", "2.1" ).execute()

        ALVerbose.unitTest( "Setting the profile OFF in testALApplicationControlDebug" )
        ALApplication.setDebugProfileOff()

        ALVerbose.unitTest()
        ALVerbose.unitTest("\n\n\n")



    def testALApplicationProfile( self ):
        ALVerbose.unitTest( "Test ALApplication Profile" )

        # Standart Debug options hardcoded
        ALApplication.setDebugProfileOn()

        ALApplication( "DemoTestCaseALApplication", "2.2" ).execute()

        ALVerbose.unitTest( "Setting the profile OFF in testALApplicationProfile" )
        ALApplication.setDebugProfileOff()

        ALVerbose.unitTest()
        ALVerbose.unitTest("\n\n\n")


    def testALApplicationFolderPlugins( self ):
        ALVerbose.unitTest( "Test ALApplication FolderPlugins" )


        oalApplication = ALApplication( "DemoTestCaseALApplication", "2.3" )
        oalApplication.setApplicationPluginFolder( "./DataForTests/FolderPlugins" )
        oalApplication.execute()
        del oalApplication

        ALVerbose.unitTest()
        ALVerbose.unitTest("\n\n\n")



    def testALApplicationCommandLineInterface( self ):
        ALVerbose.unitTest( "Test ALApplication CommandLineInterface" )

        oalApplication = ALApplication( "DemoTestCaseALApplication", "2.3" )
        # Using a file to define the command line execution and mapping
        oalApplication.setCommandLineInterfaceFile( "./DataForTests/commandLineInterface.xml"  )
        oalApplication.execute()
        del oalApplication

        ALVerbose.unitTest()
        ALVerbose.unitTest("\n\n\n")



    def testALApplicationFolderPluginsGroup( self ):
        ALVerbose.unitTest( "Test ALApplication FolderPluginsGroup" )

        oalApplication = ALApplication( "DemoTestCaseALApplication", "2.4" )
        oalApplication.setApplicationPluginFolder( "./DataForTests/FolderPlugins" )
        oalApplication.setCommandLine( "--help --ALL"  )
        oalApplication.setCommandLine( "--plugins"  )
        oalApplication.execute()
        del oalApplication

        ALVerbose.unitTest()
        ALVerbose.unitTest("\n\n\n")



    def testALApplication2( self ):
        ALVerbose.unitTest( "UnitTest ALApplication2" )


        objA = ClassAA()
        objB = ClassBB()
        objC = ClassCC()
        # Workaround to have an command line to test the stuffs
       # oalCommandLineArgument = ALCommandLineArgument( [ "--verbose", "--version", "--cB", "1000", "--cA", "file_name.txt", "--cC", "banane" ] )
       # oalCommandLineArgument = ALCommandLineArgument( [ "--quiet", "--version", "--cB", "1000", "--cA", "file_name.txt", "--cC", "banane" ] )
        #oalCommandLineArgument = ALCommandLineArgument( [ "-m",  "--cB", "1000", "--cA", "file_name.txt", "--cC", "banane" ] )
        oalCommandLineArgument = ALCommandLineArgument( [ "--DEBUG", "--cB", "1000", "--cA", "file_name.txt", "--cC", "banane", "--importResource", "test-resource-XML.xml", "couocu" ] )



        # Testing resource Framework

        if (ALApplication.existResource( "Application-Test", "0.0.9" )==True):
            ALVerbose.unitTest( "The ALApplication-test resource file: YES - It exists" )
        else:
            ALVerbose.unitTest( "The ALApplication-test resource file: NO - it will be created" )

        if (ALApplication.isFirstExecution( "Application-Test", "0.0.9" )==True):
            ALVerbose.unitTest( "The ALApplication-test First Execution: YES" )
        else:
            ALVerbose.unitTest( "The ALApplication-test First Execution: NO" )



        # Testing Main Framework

        oalApplication = ALApplication( "Application-Test", "0.0.9" )

        ALVerbose.unitTest( "Application Name :: " + ALApplication.getApplicationName() )

        oalApplication.addListArgument( oalCommandLineArgument )

        # FORCING an error to be signalized
#        oalApplication.addCommand( "--cA", objA.methodFirst )

        oalApplication.addCommand( "--cB", objB.methodFirst )
        oalApplication.addCommand( "--cC", objC.methodFirst )


        # FORCING an error to be signalized
       # oalApplication.connectExecute( objA.executeError )

        oalApplication.execute()

        #
        ## Testing Access to resources
        #oalObject = ALApplication.getResource( "application_last_run" )
        #if (oalObject is not None):
        #    oalStringLastRun = ALString( oalObject );
        #    ALVerbose.screen( "Testing application ressource acces: " + oalStringLastRun )
        #
        #
        ## Testing Access to arguments
        #oalArguments = ALApplication.getCommandLineArguments()
        #oalStringArguments = ALString( oalArguments )
        ##ALVerbose.screen( "Testing application ressource acces: " + oalStringArguments )
        #
        #
        #
        ## Testing Access to command line
        #ALApplication.addCommandLineArguments( ALList( [ "primeiro_argument"] ))
        #oalArguments = ALApplication.getCommandLineArguments()
        #oalStringArguments = ALString( oalArguments )
        ##ALVerbose.screen( "Testing application ressource acces2: " + oalStringArguments )

        del oalApplication


        ALVerbose.unitTest()
        ALVerbose.unitTest("\n\n\n")



    def testALApplication3( self ):
        ALVerbose.unitTest( "UnitTest ALApplication3 - Testing TimeOUT" )


        objA = ClassAA()
        objB = ClassBB()
        objC = ClassCC()
        oalCommandLineArgument = ALCommandLineArgument( [ "--DEBUG", "--cB", "1000", "--cA", "file_name.txt", "--cC", "banane", "--importResource", "test-resource-XML.xml", "couocu" ] )



        # Testing resource Framework

        if (ALApplication.existResource( "Application-Test", "0.1.9" )==True):
            ALVerbose.unitTest( "The ALApplication-test resource file: YES - It exists" )
        else:
            ALVerbose.unitTest( "The ALApplication-test resource file: NO - it will be created" )

        if (ALApplication.isFirstExecution( "Application-Test", "0.0.9" )==True):
            ALVerbose.unitTest( "The ALApplication-test First Execution: YES" )
        else:
            ALVerbose.unitTest( "The ALApplication-test First Execution: NO" )



        # Testing Main Framework

        oalApplication = ALApplication( "Application-Test", "0.1.9" )

        ALVerbose.unitTest( "Application Name :: " + ALApplication.getApplicationName() )
        ALVerbose.unitTest( "Testing the Time Out for Plugins and Actions: 3000 ms" )

        oalApplication.setTimeOutPlugins( 3.0 ) # 1000 ms

        oalApplication.addListArgument( oalCommandLineArgument )

        # FORCING an error to be signalized
#        oalApplication.addCommand( "--cA", objA.methodFirst )

        oalApplication.addCommand( "--cB", objB.methodFirst )
        oalApplication.addCommand( "--cC", objC.methodFirst )

        oalApplication.connectExecute( ActionWait().executeAction )

        oalApplication.execute()

        del oalApplication

        ALVerbose.unitTest()
        ALVerbose.unitTest("\n\n\n")





    def testALApplicationCommandLineInterface2( self ):
        ALVerbose.unitTest( "UnitTest ALApplication CommandLineInterface2" )


        # Testing Main Framework
        oalApplication = ALApplication( "ApplicationCommandLineInterface-Test", "3.0" )
#        oalApplication.setCommandLine( "--DEBUG --test banane --test2 1000 --test3 100.01"  )
        oalApplication.setCommandLine( "--test banane --test2 1000 --test3 100.01"  )
#        oalApplication.setCommandLine( "-h --test"  )
#        oalApplication.setCommandLine( "--help --test2"  )
#        oalApplication.setCommandLine( "--man --test2"  )
#        oalApplication.setCommandLine( "-m --test1"  )

        # Using a file to define the command line execution and mapping
        oalApplication.setCommandLineInterfaceFile( "./DataForTests/commandLineInterface.xml"  )

        # Execution Again
        oalApplication.execute()

        del oalApplication

        ALVerbose.unitTest()
        ALVerbose.unitTest("\n\n\n")





    def testALApplicationActionControl( self ):
        ALVerbose.unitTest( "UnitTest ALApplication Action Control:" )
        ALVerbose.unitTest( "Obj: test the ActionListEvent in the Framework control" )


        oalCommandLineArgument = ALCommandLineArgument( [ "--DEBUG", "--cB", "1000" ] )
        oalPlugin = TestPlugin05()


        # Testing Main Framework
        oalApplication = ALApplication( "ApplicationActionControl-Test", "0.1.9" )
        oalApplication.addListArgument( oalCommandLineArgument )

        # Execution
        oalApplication.connectExecute( oalPlugin.executePlugin )
        oalApplication.execute()

        del oalApplication

        ALVerbose.unitTest()
        ALVerbose.unitTest("\n\n\n")






    def testALApplicationSubThreadExecutionAndControl( self ):
        ALVerbose.unitTest( "UnitTest ALApplication SubThreadExecutionAndControl:" )


        oalCommandLineArgument = ALCommandLineArgument( [ "--DEBUGECLIPSE", "--cB", "1000" ] )


        # Testing Main Framework
        oalApplication = ALApplication( "ApplicationActionControl-SubThread", "0.0.9" )
        oalApplication.addListArgument( oalCommandLineArgument )

        oalPlugin7 = TestPlugin07()


        # Execution
        oalApplication.connectExecute( oalPlugin7.executePlugin )
        # hard test
        oalApplication.connectExecute( None )
        oalApplication.execute()

        del oalApplication

        ALVerbose.unitTest()
        ALVerbose.unitTest("\n\n\n")





    def testALApplicationTimerLoop( self ):
        ALVerbose.unitTest( "UnitTest ALApplication timerLoop" )

#        oalCommandLineArgument = ALCommandLineArgument( [ "--DEBUGECLIPSE", "--cB", "1000" ] )
        oalCommandLineArgument = ALCommandLineArgument( [  "--cB", "1000" ] )


        # Testing Main Framework
        oalApplication = ALApplication( "ApplicationTimerLoop", "0.999" )
        oalApplication.addListArgument( oalCommandLineArgument )

        # Execution Looping
        oalPlugin3 = TestPlugin03( "Timer03" )
        oalPlugin4 = TestPlugin04( "Timer04" )

        oalApplication.connectExecuteTimerAction( oalPlugin4.executePluginTimer, 0.6, 15.0 )
        oalApplication.connectExecuteTimerAction( oalPlugin3.executePluginTimer, 0.6, 15.0 )
        oalApplication.connectExecuteTimerAction( printLine, 0.5, 5.0 )
        oalApplication.connectExecuteTimerAction( printLine, 0.6, 12.0 )

#        oalAction = ALActionMethod( oalPlugin3.methodDoPrintAction )
#        oalAction.executeMethod()
#
#        oalActionPure = ActionPure()
#        oalAction = ALActionMethod( oalActionPure.executeAction )
#        oalAction.executeMethod()

        # Test the static class
        ALApplication.executeTimerAction( oalPlugin3.methodDoPrint2, 0.1, 20.0 )


        # Do a wait until the rest is tested
        oalApplication.connectExecute( oalPlugin3.methodDoSleep )
        oalApplication.connectExecute( oalPlugin3.executePlugin )
        oalApplication.execute()

        del oalApplication

        ALVerbose.unitTest()
        ALVerbose.unitTest("\n\n\n")





    def testALFactoryPluginApplication( self ):
        ALVerbose.unitTest( "UnitTest ALFactoryPlugin - ALApplication:" )


        objA = ClassAA()
        objB = ClassBB()
        objC = ClassCC()
        # Workaround to have an command line to test the stuffs
        oalCommandLineArgument = ALCommandLineArgument( [ "--DEBUG", "--cB", "1000", "--cA", "file_name.txt", "--cC", "banane" ] )


        # Testing Main Framework
        oalApplication = ALApplication( "Application-Test-FactoryPlugin", "0.9" )
        oalApplication.addListArgument( oalCommandLineArgument )


        #Plugin factory in the main application class
        oalApplication.addPlugin( None )
        oalApplication.addPlugin( TestPlugin01() )
        oalApplication.addPlugin( TestPlugin02() )
        oalApplication.addPlugin( TestPlugin03( "tokenPlugin03" ) )
        oalApplication.addPlugin( TestPlugin04( "tokenPlugin04" ) )


        #Plugin search and load from files especified in this location
        oalApplication.addPluginFolder( "./DataForTests/FolderPlugins" )
        oalApplication.addPluginFolder( "../plugins" )
        oalApplication.addPluginFile( "./DataForTests/FolderPlugins/PluginTest01.py" )
#        oalApplication.addPluginFile( "./DataForTests/ClassPluginFile.py" )


        if(ALSystem.getInterpreter()=="Java"):
            oalApplication.addPluginFolder( "./DataForTests/FolderPluginJythonJava" )


        # Test
        oalApplication.connectExecute( methodFactory )
        oalApplication.connectExecute( methodFactoryFromFolder )
        oalApplication.connectExecute( methodFactoryFromFile )
        oalApplication.connectExecute( methodFactoryAll )
        oalApplication.execute()

        del oalApplication

        ALVerbose.unitTest()
        ALVerbose.unitTest("\n\n\n")




    def testALFactoryPluginApplicationFactoryPluginStatic( self ):
        ALVerbose.unitTest( "UnitTest ALFactoryPlugin - testALFactoryPluginApplicationFactoryPluginStatic:" )

        # Construct the static plugin factory
        oalFactoryPluginStatic = ALFactoryPluginStatic()

        import ALString
        import ALPlugin
        oalFactoryPluginStatic.addPluginStatic( "ALString", ALString )
        oalFactoryPluginStatic.addPluginStatic( "ALPlugin", ALPlugin )


        oalPlugin = oalFactoryPluginStatic.getPlugin( "ALPlugin" )
        oalPlugin.setPluginName( "PluginTestNameForSTATICFactory" )
        oalPlugin.executePlugin()
        oalPlugin.outputScreenSignature()
        ALVerbose.screen( "Test PluginFactory: " + oalPlugin.getToken() )

        oalCommandLineArgument = ALCommandLineArgument( [  "--cB", "1000", "--plugin", "--pluginGroup", "--pluginGroupTree" ] )

        # Testing Main Framework
        oalApplication = ALApplication( "Application-Test-FactoryPlugiStatic", "1.1" )
        oalApplication.addListArgument( oalCommandLineArgument )

        # change the factory plugin engine
        oalApplication.setFactoryPluginStatic( oalFactoryPluginStatic )

        if (oalApplication.isFactoryPluginDynamic()):
            ALVerbose.unitTest("ALAplication plugin is set to Dynamic" )
        else:
            ALVerbose.unitTest("ALAplication plugin is set to STATIC" )

        #New Thread
        oalPlugin = ALApplication.getPluginObject( "ALPlugin" )
        if (oalPlugin!=None):
            oalApplication.connectExecute( oalPlugin.executePlugin )

        # Execute the application with the static plugin factory
        oalApplication.execute()

        del oalApplication

        ALVerbose.unitTest()
        ALVerbose.unitTest("\n\n\n")



    def testALApplicationCommandLineInterface3( self ):
        ALVerbose.unitTest( "UnitTest ALApplication CommandLineInterface3" )

        # Construct the static plugin factory
        oalFactoryPluginStatic = ALFactoryPluginStatic()

        import ALString
        import ALPlugin
        oalFactoryPluginStatic.addPluginStatic( "ALString", ALString )
        oalFactoryPluginStatic.addPluginStatic( "ALPlugin", ALPlugin )

        # Testing Main Framework
        oalApplication = ALApplication( "ApplicationCommandLineInterface-Test", "1.1" )
#        oalApplication.setCommandLine( "--DEBUG --test banane --test2 1000 --test3 100.01"  )
        oalApplication.setCommandLine( "--test banane --test2 1000 --test3 100.01"  )
#        oalApplication.setCommandLine( "-h --test"  )
#        oalApplication.setCommandLine( "--help --test2"  )
        oalApplication.setCommandLine( "--help --test4"  )
#        oalApplication.setCommandLine( "-m --test1"  )

        # change the factory plugin engine
        oalApplication.setFactoryPluginStatic( oalFactoryPluginStatic )

        # Using a file to define the command line execution and mapping
        oalApplication.setCommandLineInterfaceFile( "./DataForTests/commandLineInterface.xml"  )

        # Execution Again
        oalApplication.execute()


        del oalApplication

        ALVerbose.unitTest()
        ALVerbose.unitTest("\n\n\n")




    def testALFactoryPluginApplicationForceClose( self ):
        ALVerbose.unitTest( "UnitTest Force Close all - ALApplication:" )

#        oalCommandLineArgument = ALCommandLineArgument( [ "--DEBUGECLIPSE", "--cB", "1000" ] )
        oalCommandLineArgument = ALCommandLineArgument( [  "--cB", "1000" ] )


        # Testing Main Framework
        oalApplication = ALApplication( "ApplicationForceCloseTest", "2.0" )
        oalApplication.addListArgument( oalCommandLineArgument )

        # Execution Looping
        oalPlugin3 = TestPlugin03( "Timer03" )
        oalPlugin4 = TestPlugin04( "Timer04" )

        oalApplication.connectExecuteTimerAction( oalPlugin4.executePluginTimer, 0.6, 15.0 )
        oalApplication.connectExecuteTimerAction( oalPlugin3.executePluginTimer, 0.6, 15.0 )
        oalApplication.connectExecuteTimerAction( printLine, 0.5, 5.0 )
        oalApplication.connectExecuteTimerAction( printLine, 0.6, 7.0 )

        # Test the static class
        ALApplication.executeTimerAction( oalPlugin3.methodDoPrint2, 0.1, 5.0 )

        # Do a wait until the rest is tested
        oalApplication.connectExecute( oalPlugin3.methodDoSleep )
        oalApplication.connectExecute( oalPlugin3.executePlugin )
        oalApplication.connectExecute( self.methodForceCloseAll )


        # MAIN Application execution
        oalApplication.execute()

        del oalApplication

        ALVerbose.unitTest()
        ALVerbose.unitTest("\n\n\n")




    def methodForceCloseAll( self, oalObject = None ):
        ALVerbose.screen( "" )
        ALVerbose.screen( "" )
        ALVerbose.screen( "#####################################################" )
        ALVerbose.screen( "executing the ForceClose ALL" )
#        Problem on that routines on Jython - to fix
#        ALApplication.outputScreenPlugin()
#        ALApplication.outputScreenPluginGroup()
#        ALApplication.outputScreenPluginGroupTree()

        # simulating an error to force close all
        _FORCING_an_error_to_force_close_all_actions.forceCloseActions() # FORCING an error to force close all actions




##############################################################################

    """
    Total compatible with Jython
    """
    def process( self ):
        self.addTestMethod( self.testALApplication )
        self.addTestMethod( self.testALApplicationControlDebug )
        self.addTestMethod( self.testALApplicationProfile )
        self.addTestMethod( self.testALApplicationFolderPlugins )
        self.addTestMethod( self.testALApplicationCommandLineInterface )
        self.addTestMethod( self.testALApplicationFolderPluginsGroup )
        self.addTestMethod( self.testALApplication2 )
        self.addTestMethod( self.testALApplication3 )
        self.addTestMethod( self.testALApplicationCommandLineInterface2 )
        self.addTestMethod( self.testALApplicationActionControl )
        self.addTestMethod( self.testALApplicationSubThreadExecutionAndControl )
#        self.addTestMethod( self.testALApplicationTimerLoop )


#Problem with these atuffs related to the plugin signature on the screen
        self.addTestMethod( self.testALFactoryPluginApplication )
        self.addTestMethod( self.testALFactoryPluginApplicationFactoryPluginStatic )

        self.addTestMethod( self.testALApplicationCommandLineInterface3 )
#        self.addTestMethod( self.testALFactoryPluginApplicationForceClose )





##############################################################################


if __name__ == '__main__':

    # JIT compiler accelerator
    ALCompiler.accelerator()

    oalTestCaseALApplication = ALTestCaseALApplication( "TestCase ALApplication")
    oalTestCaseALApplication.execute()
