#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################



from ALImportSystem      import *
from ALImportKernel      import *
from ALImportAction      import *
from ALImportMath        import *



class ALTestCaseALResource( ALTestCase ):
    """
    TestSuite for ALObject Class
    """
    def testALResource( self ):
        ALVerbose.unitTest( "ALTestCase ALResource:" )

        ALVerbose.warning( "" )
        ALVerbose.warning( "Jython and Python Versions Must add all methods" )
        ALVerbose.warning( "" )

        oalResource = ALResource( "test-resource.res", False ) # local file = not in user path

        if (oalResource.exist()==True):
            ALVerbose.unitTest( "The resource file Exist: test-resource.res" )
        else:
            ALVerbose.unitTest( "The resource file Doesn't Exist: test-resource.res" )
            ALVerbose.unitTest( "....it will be created" )


        # To fix for Jython
        oalResource.input()
        oalObject = oalResource.getResource( "local_path" )
        ALVerbose.unitTest( "Getting Resource 1: " + ALString( oalObject ) )
        oalResource.setResource( "local_path", 1000 )
        oalObject = oalResource.getResource( "local_path" )
        ALVerbose.unitTest( "Getting Resource 2: " + ALString( oalObject ) )
        oalResource.output()


        # ASCII
        #oalResource = ALResource( "test-resource-ASCII.res", False ) # local file = not in user path
        #oalResource.setResource( "local_path", 1000 )
        #oalResource.setResource( "banane", "banane" )
        #oalResource.setResource( "pomme", "pomme" )
        #oalResource.outputSerialASCII()
        #
        #oalResource2 = ALResource( "test-resource-ASCII.res", False ) # local file = not in user path
        #oalResource2.inputSerialASCII()
        #oalObject = oalResource2.getResource( "local_path" )
        #ALVerbose.unitTest( "Getting Resource : " + ALString( oalObject ) )
        #oalObject = oalResource2.getResource( "banane" )
        #ALVerbose.unitTest( "Getting Resource : " + ALString( oalObject ) )
        #oalObject = oalResource2.getResource( "pomme" )
        #ALVerbose.unitTest( "Getting Resource : " + ALString( oalObject ) )

        # XML Only
        oalResource3 = ALResource( "test-resource-XML.xml", False ) # local file = not in user path
        oalResource3.setResource( "local_path", 99999 )
        oalResource3.setResource( "banane", "banane-Black" )
        oalResource3.setResource( "pomme", "pomme-Green" )
        oalResource3.output()

        ALVerbose.unitTest( "Testing the import of other resources" )
        ALVerbose.unitTest( "Original Resource : " + ALString( oalResource.getResourceDictionary() ) )

        ALVerbose.unitTest( "Importing.........." )
        oalResource.importResource( "test-resource-XML.xml" )
        ALVerbose.unitTest( "     New Resource : " + ALString( oalResource.getResourceDictionary() ) )

        ALVerbose.unitTest( "Other test" )

        oalResource4 = ALResource( "test-resource-XML.xml", False, False ) # local file = not in user path

        oalResource4.input()
        ALVerbose.unitTest( "     Reading Resource : " + ALString( oalResource4.getResourceDictionary() ) )

        oalObject = oalResource4.getResource( "local_path" )
        ALVerbose.unitTest( "Getting Resource : " + ALString( oalObject ) )
        ALAssert.equal( "99999", oalObject)

        oalObject = oalResource4.getResource( "banane" )
        ALVerbose.unitTest( "Getting Resource : " + ALString( oalObject ) )
        ALAssert.equal( "banane-Black", oalObject)

        oalObject = oalResource4.getResource( "pomme" )
        ALVerbose.unitTest( "Getting Resource : " + ALString( oalObject ) )
        ALAssert.equal( "pomme-Green", oalObject)

        oalObject = oalResource4.getResource( "not_a_valid_key" )
        ALVerbose.unitTest( "Getting Resource : " + ALString( oalObject ) )
        ALAssert.equal( None, oalObject)





    def testALResourceException( self ):
        ALVerbose.unitTest( "ALTestCase ALResource:" )

        oalStreamFile = ALStreamFile( "test-ExceptionRessource.res" )
        oalStreamFile.write( "something wrong in the file" )
        oalStreamFile.close()

        oalResource = ALResource( "test-ExceptionRessource.res", False )

        ALVerbose.warning( "If you have an error message the resource test is [ OK ]" )

        ALVerbose.unitTest()



##############################################################################

    """
    Total compatible with Jython
    """
    def process( self ):
        self.addTestMethod( self.testALResource )
        self.addTestMethod( self.testALResourceException )


##############################################################################


if __name__ == '__main__':

    # JIT compiler accelerator
    ALCompiler.accelerator()

    oalTestCaseALResource = ALTestCaseALResource( "TestCase ALResource" )
    oalTestCaseALResource.execute()
