#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################



from ALImportSystem      import *
from ALImportKernel      import *
from ALImportAction      import *
from ALImportMath        import *

import ALExternalPackGenerateDS  as ALPackXsd


class ALTestCaseALXsd( ALTestCase ):
    """
    TestSuite for ALXsd
    """
    def testALXsd( self ):
        ALVerbose.unitTest( "Test ALXsd" )

        oalStrPyOutputFile = ALString( "module_people.py" )
        oalStrXsdInputFile = ALString( "./DataForTests/people.xsd")
        oalStrXmlInputFile = ALString( "./DataForTests/people.xml" )
        ALXsd().generateModuleBuffer( oalStrPyOutputFile, oalStrXsdInputFile )

        ALTime.wait()
        from module_people import people as ALPeople

        oalPeople = ALPeople()
        oalPeople.outputFile( "testOuputPeopleXsd01.xml" )

        import module_people
        oalPeople2 = module_people.unMarshalling( oalStrXmlInputFile )
        oalPeople2.outputFile( "testOuputPeopleXsd02.xml")

        ALVerbose.unitTest()


    def testALPackXsd( self ):
        ALVerbose.unitTest( "Test ALPackXsd" )

        oalStrPyOutputFile = "people.py"
        oalStrXsdInputFile = "./DataForTests/people.xsd"
        ALPackXsd.generateClassDefinition( oalStrPyOutputFile, oalStrXsdInputFile, "xs:" )

        ALTime.wait()
        ALVerbose.unitTest( "Importing Class" )
        import people

        ALVerbose.unitTest( "unMarshalling Class" )
        oObject = people.unMarshalling( "./DataForTests/people.xml" )
        oObject.outputFile( "test-direct-output.xml" )

        import sys

        print "\n\n\n\n"
        oObject.exportLiteral( sys.stdout, 0 )



    def testModuleXsdClass( self ):
        ALVerbose.unitTest( "Test Module Xsd Class"  )


        oalStrXsdInputFile = "./DataForTests/FolderPlugins/DTCTestReference.xsd"
        oalStrPyOutputFile = "DTCTestReference.py"


        ALXsd().generateModuleBuffer( oalStrPyOutputFile, oalStrXsdInputFile )
#        ALXsd().generateModule( oalStrPyOutputFile, oalStrXsdInputFile )

        ALTime.wait()
        import DTCTestReference
        import sys

        ALVerbose.unitTest( "unMarshalling Class" )
        oObject = DTCTestReference.unMarshalling( "./DataForTests/DTTestReference.xml" )
        oObject.outputFile( "testOutputXsdReference.xml" )
        if ( oObject.getDate() == ALString.empty() ):
            ALVerbose.unitTest( " !!!!!!! = The test date is empty = ALString.empty()" )
        else:
            ALVerbose.unitTest( "TestDate: " + oObject.getDate() )
        ALVerbose.unitTest( ALString( oObject ) )

        oGroupDiffractionImage = oObject.getGroupDiffractionImage()
        oGroupDiffractionImage.outputFile( "groupDiffractionImage.xml" )
        oList = oGroupDiffractionImage.getDiffractionImage()

        for i in range( 0, len( oList ) ):
            print oList[i].getFileName()
            print oList[i].getDate()
            print oList[i].getResolution()


        for i in oList:
            print i.getFileName()
            print i.getDate()
            print i.getResolution()
            i.outputFile( "DiffractionImage-" + i.getFileName() + ".xml" )


        ALVerbose.unitTest( "marshalling Class" )
        DTCTestReference.marshalling( oObject, "testReference.xml" )

        import sys
        oObject.export( sys.stdout )


        oalStrXsdInputFile = "./DataForTests/FolderPlugins/DTCTestGroupDiffractionImage.xsd"
        oalStrPyOutputFile = "DTCTestGroupDiffractionImage.py"
        ALXsd().generateModuleBuffer( oalStrPyOutputFile, oalStrXsdInputFile )

        ALTime.wait()
        import DTCTestGroupDiffractionImage

        oObjectList = DTCTestGroupDiffractionImage.DTCTestGroupDiffractionImage()
        oObjectList.addDiffractionImage( oList[1] )
        oObjectList.outputFile( "groupDiffractionImageInstance.xml" )
        oObject.setGroupDiffractionImage( oObjectList )
        oObject.outputFile( "newReferenceFile.xml" )

        import sys
        oObjectList.export( sys.stdout )
        oObject.export( sys.stdout )
        print "\n\n\n\n"
#        oObject.exportLiteral( sys.stdout )
#        oObject.exportLiteralHtml( sys.stdout )
        oObject.outputPageHtml( "newReferenceFile.html" )
        oObject.outputFileHtml( "newReferenceFile2.html" )

        ALVerbose.unitTest( )



##############################################################################

    """
    Total compatible with Jython
    """
    def process( self ):
        self.addTestMethod( self.testALXsd )
        self.addTestMethod( self.testALPackXsd )
        self.addTestMethod( self.testModuleXsdClass )


##############################################################################


if __name__ == '__main__':

    # JIT compiler accelerator
    ALCompiler.accelerator()

    oalTestCaseALXsd = ALTestCaseALXsd( "TestCase ALXsd" )
    oalTestCaseALXsd.execute()
