#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################


from ALImportSystem      import *
from ALImportKernel      import *
from ALImportAction      import *
from ALImportMath        import *


import ALExternalPackGenerateDS  as ALPackXsd



##############################################################################
"""
This is the example using an external XDS data binding tool.
The method to be used must to be a wrapper with two inputs:
- the method name + the xdsschemafile.
The example used here is using the latest "generateDS" file avaiable.
It is installed on the "/test" folder.
Please modifie the import as you want for your own parser.
"""
try:
    # Here is the parser to be used with the external xds data binding tool
    import generateDS

except:
    ALVerbose.warning( "Please: configure your external XDS data binding tool to be used in the ALTestCaseALXsdDataBindingEngine")

# The method wraper for generateDS
def externalMethodWrapper( _outFilename, _xschemaFileName,\
        _nameSpace = 'xs:',
        subclassFilename = None, prefix = '', \
        behaviorFilename = None, superModule='???' ):
    generateDS.debug = 0
    generateDS.set_type_constants( _nameSpace  )
    generateDS.parseAndGenerate( str( _outFilename ), subclassFilename, prefix,
                     str( _xschemaFileName ) , behaviorFilename, superModule=superModule)

##############################################################################



class ALTestCaseALXsdDataBindingEngine( ALTestCase ):
    """
    TestSuite for ALXsdDataBindingEngine
    """


    def testALXsdObjectDirectEngine( self ):
        ALVerbose.unitTest( "Test ALTestCaseALXsdDataBindingEngine" )

        oalStrXsdInputFile = ALString( "./DataForTests/people.xsd")
        oalStrXmlInputFile = ALString( "./DataForTests/people.xml" )


        ALVerbose.unitTest( "Testing the people.xsd import module people.py" )
        oalXsdDataBinding3 = ALXsdDataBindingEngine( ALPackXsd.generateClassDefinition, oalStrXsdInputFile )
        oalObject3 = oalXsdDataBinding3.getObject()
        oalObject3.outputFile( "testOuputPeopleXsd03.xml" )


        ALVerbose.unitTest( "Testing the empty instance of the object" )
        ALVerbose.unitTest( "Testing the people.xsd import module people.py" )
        oalXsdDataBinding6 = ALXsdDataBindingEngine( ALPackXsd.generateClassDefinition, oalStrXsdInputFile )
        oalObject6 = oalXsdDataBinding6.getObject()
        # oalObject6.setToken00()
        # oalObject6.set.....
        oalObject6.outputFile( "testOuputPeopleXsd06.xml" )

        ALVerbose.unitTest()







    def testALXsdObjectDirectEngineExternalgenerateDS( self ):
        ALVerbose.unitTest( "Test testALXsdObjectDirectEngineExternalgenerateDS" )

        oalStrXsdInputFile = ALString( "./DataForTests/people.xsd")
        oalStrXmlInputFile = ALString( "./DataForTests/people.xml" )


        ALVerbose.unitTest( "Testing the people.xsd import module people.py" )
        ALVerbose.unitTest( "It generates an empty Object instance to be used in the code" )
        oalXsdDataBinding3 = ALXsdDataBindingEngine( externalMethodWrapper, oalStrXsdInputFile )
        oalObject3 = oalXsdDataBinding3.getObject()
        # For standard generateDS only
        # oalObject3.setToken00()
        # oalObject3.set.....
        ALVerbose.unitTest( ALString( oalObject3 ) )

        oalObject4 = oalXsdDataBinding3.getObjectClass( "person" )
        # For standard generateDS only
        # oalObject3.setToken00()
        # oalObject3.set.....
        ALVerbose.unitTest( ALString( oalObject4 ) )



        ALVerbose.unitTest( "=========Testing the import of the generateDS" )
        # or use the code method generated by generateDS
        import people
        people.parse( oalStrXmlInputFile )
        # and use the code as you want


        ALVerbose.unitTest()



    def testEdnaXsd( self ):
        ALVerbose.unitTest( "Test testALXsdObjectDirectEngineExternalgenerateDS" )

        oalStrXsdInputFile = ALString( "./DataForTests/XSDataRaddosev01.exsd")
        oalStrXmlInputFile = ALString( "./DataForTests/people.xml" )


        ALVerbose.unitTest( "Testing the people.xsd import module people.py" )
        ALVerbose.unitTest( "It generates an empty Object instance to be used in the code" )
        oalXsdDataBinding3 = ALXsdDataBindingEngine( externalMethodWrapper, oalStrXsdInputFile )
        oalObject3 = oalXsdDataBinding3.getObjectClass( "XSData" )
        # oalObject3.setToken00()
        # oalObject3.set.....
        ALVerbose.unitTest( ALString( oalObject3 ) )

        oalObject = oalXsdDataBinding3.getObjectClass( "XSDataFlux" )
        ALVerbose.unitTest( ALString( oalObject ) )

        oalObject = oalXsdDataBinding3.getObjectClass( "XSDataWavelength" )
        ALVerbose.unitTest( ALString( oalObject ) )

        oalObject = oalXsdDataBinding3.getObjectClass( "XSDataAtomicComposition" )
        ALVerbose.unitTest( ALString( oalObject ) )



        ALVerbose.unitTest( "=========Testing the import of the generateDS" )
        # or use the code method generated by generateDS
        import XSDataRaddosev01
#        XSDataRaddosev01_include.parse( oalStrXmlInputFile )
        # and use the code as you want


        ALVerbose.unitTest()



##############################################################################

    """
    Total compatible with Jython
    """
    def process( self ):
        self.addTestMethod( self.testALXsdObjectDirectEngine )
        self.addTestMethod( self.testALXsdObjectDirectEngineExternalgenerateDS )
        #self.addTestMethod( self.testEdnaXsd )


##############################################################################


if __name__ == '__main__':

    # JIT compiler accelerator
    ALCompiler.accelerator()

    oalTestCaseALXsdDataBindingEngine = ALTestCaseALXsdDataBindingEngine( "TestCase ALXsdDataBindingEngine" )
    oalTestCaseALXsdDataBindingEngine.execute()
