#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################



from ALImportSystem      import *
from ALImportKernel      import *
from ALImportAction      import *
from ALImportMath        import *


##############################################################################
# Divers



##############################################################################
# ALAssert


class UnitTestALAssert( ALTestCase ):

    def testALAssertEqual( self ):
        # Test equal: case OK: retrieve the proper token
        oalString1 = ALString( "Ma chaine a moi qu'elle est belle!")
        ALAssert.equal( "moi", oalString1.getToken( 3 ))
        # Test equal: case OK: retrieve None since index is outside the string size
        ALAssert.equal( None, oalString1.getToken( 10 ))
        # Test equal: case FAILURE: Not Equal
        ALVerbose.unitTest( "Testing Not Equal Failure...")
        try:
            ALAssert.equal("ma", oalString1.getToken( 3 ))
        except AssertionError:
            ALVerbose.unitTest( "FAILURE is OK, not equal failure is expected" )

    def testALAssertRaiseException( self ):
        # Test raiseException: case OK: catch the proper exception
        ALAssert.raiseException( IOError, open, "toto.txt")

        # Test raiseException: case FAILURE: Unexpected or No Exception
        try:
            ALVerbose.unitTest( "Testing Unexpected Exception Failure...")
            ALAssert.raiseException( TypeError, open, "toto.txt")
        except AssertionError:
            ALVerbose.unitTest( "FAILURE is OK, the unexpected exception failure is expected")

        try:
            ALVerbose.unitTest( "Testing No Exception Failure...")
            ALAssert.raiseException( IOError, str, 0.01 )
        except AssertionError:
            ALVerbose.unitTest( "FAILURE is OK, normally this method shouldn' t raise any exception")








##############################################################################
# ALException

class UnitTestALException( ALTestCase ):
    def testALException( self ):
        ALVerbose.unitTest( "Test ALException Base" )



        try:
            raise ALException, "banane"
        except ALException, _oalException:
            ALVerbose.unitTest( "Testing the ALException : OK" )
            ALVerbose.unitTest( "Exception: " + ALString( _oalException.getException() ) )


        try:
            raise ALExceptionControl, ALObject()
        except ALExceptionControl, _oalExceptionControl:
            ALVerbose.unitTest( "Testing the ALExceptionControl : OK" )
            oalObject    = _oalExceptionControl.getException()
            ALVerbose.unitTest( "Exception: " + ALString( oalObject.getID() ) )


        try:
            raise ALExceptionSignal, ALSignal( "Exception Test argument in Control" )
        except ALExceptionSignal, _oalExceptionSignal:
            ALVerbose.unitTest( "Testing the ALExceptionSignal : OK" )
            oalSignal    = _oalExceptionSignal.getSignal()
            ALVerbose.unitTest( "Exception: " + ALString( oalSignal.getSignal() ) )


        try:
            raise ALExceptionSignal, ALSignal.abort()
        except ALExceptionSignal, _oalExceptionSignal:
            ALVerbose.unitTest( "Testing the ALExceptionSignal : OK" )
            oalSignal    = _oalExceptionSignal.getSignal()
            ALVerbose.unitTest( "Exception: " + ALString( oalSignal.getSignal() ) )

        ALVerbose.unitTest()



##############################################################################
# ALSystem

class UnitTestALSystem( ALTestCase ):


    def testALSystemExtended( self ):
        ALVerbose.unitTest( "Test ALSystem Extended" )




        oalbuu = ALString( "coucou" )
        oaltoto = ALString( oalbuu )
        #print oalbuu, type(oalbuu), type(oaltoto)

        ALVerbose.unitTest( oalbuu )
        ALVerbose.unitTest( type( oalbuu ) )
        ALVerbose.unitTest( type( oaltoto ) )
        #self.failIf( not oaltoto.equals( "coucou" ) )
        ALVerbose.unitTest( "%s equals %s: %s" % ( oaltoto, "coucou", oaltoto.equals( "coucou" ) ) )

        oalList = ALList()
        oalList.append(1)
        #print oalList, type( oalList )
        ALVerbose.unitTest( oalList )
        ALVerbose.unitTest( type( oalList ) )


        oalDict = ALDict()
        oalDict["a"] = "b"
        ALVerbose.unitTest( oalDict )
        ALVerbose.unitTest( type( oalDict ) )


        oalThread = ALThread()
        ALVerbose.unitTest( oalThread.getID() )
        ALVerbose.unitTest( oalThread.getMagicNumber() )
        ALVerbose.unitTest( type( oalThread ) )


        oalThreadingCondition = ALThreadingCondition()
        ALVerbose.unitTest( oalThread.getID() )
        ALVerbose.unitTest( oalThread.getMagicNumber() )
        ALVerbose.unitTest( type( oalThreadingCondition ) )



        ALVerbose.unitTest( "Test executeProcess()" )
        oalStrPython = "C:/Program-Files/Automation/win32/Program-Files/Python-2.4.3/python.exe"
        if (ALSystem.getOSName()=="Linux"):
            oalStrPython = "/sware/exp/scisoft/Automation/suse82-DEV/Program-Files/Python-2.4.3/bin/python"
        oalStrCode   = "./DataForTests/demo-dna.py"
        oalCommand   = oalStrPython + " " + oalStrCode
        ALSystem.executeProcess( oalCommand )


        if (ALSystem.isCommandLine( "--DEBUGECLIPSE" )):
            #raise
            ALVerbose.warning( "Test ok for raise control in the ALSystem class" )

        ALVerbose.unitTest()



#  Deprecated to be compatible to Jython - NO decorators
###############################################################################
## ALMultiMethod
#
#class foo:
#    def __init__( self ):
#        pass
#    @multimethod( int )
#    def bar( a ):
#        print "[PARENT] Integer a: %d" % a
#    @multimethod( float )
#    def bar( a ):
#        print "[PARENT] Float a: %f" % a
#    @multimethod( str, int )
#    def bar( a, b ):
#        print "[PARENT] String a and integer b: %s and %d" % ( a, b )
#    bar = staticmethod( bar )
#
#
#class child( foo ):
#    def __init__( self ):
#        foo.__init__( self )
#    def barbar( self, a, b ):
#         print "[CHILD] String a and integer b: %s and %d" % ( a, b )
#
#    #def bar( self, a, b ):
#    #    print "CHILD: String a and integer b: %s and %d" % ( a, b )
#    #@multimethod( int )
#    #def bar( a ):
#    #    print "[CHILD] Integer a: %d" % a
#    #@multimethod( float )
#    #def bar( a ):
#    #    print "[CHILD]  Float a: %f" % a
#    #@multimethod( str, int )
#    #def bar( a, b ):
#    #    print "[CHILD] String a and integer b: %s and %d" % ( a, b )
#    #bar = staticmethod( bar )
#
#class UnitTestMultiMethod( ALTestCase ):
#    def testMultiMethod( self ):
#        ALVerbose.unitTest( "Test MultiMethod" )
#        e = foo()
#        e.bar(1)
#        e.bar(1.0)
#        e.bar( "coco", 5 )
#        b = child()
#        b.bar(2)
#        b.bar(3.0)
#        b.barbar( "bobo", 1 )
#        ALVerbose.unitTest()
#
#

##############################################################################
# ALThread

class TestALThreadScreen( ALThread ):

    def __init__( self, _iThreadNo, _fSleepTime ):
        ALThread.__init__( self )
        self.m_iThreadNo  = _iThreadNo
        self.m_fSleepTime = _fSleepTime

    def run( self ):
        for i in range( 1, 5 ):
            #while True:
                ALVerbose.screenID( self.getID(), "Output from thread %d" % self.m_iThreadNo )
                ALTime.sleep( self.m_fSleepTime )



class UnitTestALThread( ALTestCase ):
    def testALThread( self ):
        ALVerbose.unitTest( "Test ALThread" )
        thread1 = TestALThreadScreen( 1, 0.1 )
        thread2 = TestALThreadScreen( 2, 0.3 )
        thread3 = TestALThreadScreen( 3, 0.01 )
        thread4 = TestALThreadScreen( 4, 0.2 )
        thread5 = TestALThreadScreen( 5, 0.05 )
        thread1.execute()
        thread2.execute()
        thread3.execute()
        thread4.execute()
        thread5.execute()
        ALVerbose.unitTest( )



##############################################################################
# ALList

class UnitTestALList( ALTestCase ):

    def testALList( self ):
        ALVerbose.unitTest( "UnitTest ALList:" )

        oalList = ALList()
        oalList.append( "A small string" )
        oalList.append( 1 )
        oalList.append( 1.0 )
        oalList.append( { "a": "b" } )
        ALVerbose.unitTest( oalList )


        for i in oalList:
            print i


        for i in range( 0, len( oalList)):
#        for i in range( 0, 3 ):
            print oalList[i]

        ALVerbose.unitTest( "Number of objects in the list: " + ALString( oalList.getNumberObjects() ) )



##############################################################################
# ALDiskExplorer

class UnitTestIteract:
    def printIteract( self, _pyArg, _pyFolder, _pyFiles ):
        oalStrFolder = ALDiskExplorer.normPath( _pyFolder )
        oalStrArg    = ALString( _pyArg )
        oalListFile  = ALList( _pyFiles )
        oalStrCurFolder = ALDiskExplorer.mergePath( ALDiskExplorer.getCurrentFolder(), oalStrFolder )
        print oalStrArg + " > " + oalStrCurFolder
        # Print Folders
        for ifile in oalListFile:
            if ALDiskExplorer.isFolder( ALDiskExplorer.mergePath( oalStrCurFolder, ifile ) ):
                print "              |-->FOLDER::" + ALString( ifile )
       # Print Links
        for ifile in oalListFile:
            if ALDiskExplorer.isLink( ALDiskExplorer.mergePath( oalStrCurFolder, ifile ) ):
                print "              |-->LINK  ::" + ALString( ifile )
        # Print files
        for ifile in oalListFile:
            if ALDiskExplorer.isFile( ALDiskExplorer.mergePath( oalStrCurFolder, ifile ) ):
                print "              |-->FILE  ::" + ALString( ifile )


    def printTree( self, _pyArg, _pyFolder, _pyFiles ):
        oalStrFolder = ALDiskExplorer.normPath( _pyFolder )
        oalStrArg    = ALString( _pyArg )
        oalListFile  = ALList( _pyFiles )
        oalStrCurFolder =  oalStrFolder
        ilenght = len( oalStrCurFolder )
        print "> ./" + oalStrCurFolder
        # Print files
        for ifile in oalListFile:
            if ALDiskExplorer.isFile( ALDiskExplorer.mergePath( oalStrCurFolder, ifile ) ):
                oalString = ALString( " " )
                for i in range( 0, ilenght+2):
                    oalString += ALString( " " )
                print oalString + "|-->" + ALString( ifile )



class UnitTestALDiskExplorer( ALTestCase ):

    def testALDiskExplorer( self ):
        ALVerbose.unitTest( "Test ALDiskExplorer" )


        ALVerbose.unitTest( "       Absolute Path of ../ :" + ALDiskExplorer.absolutePath( "../" ) )
        ALVerbose.unitTest( "       Absolute Path of ./  :" + ALDiskExplorer.absolutePath( "./" ) )
        ALVerbose.unitTest( "    Absolute Path of tests  :" + ALDiskExplorer.absolutePath( "tests" ) )
        ALVerbose.unitTest( "   Absolute Path of /tests  :" + ALDiskExplorer.absolutePath( "/tests" ) )
        ALVerbose.unitTest( "   Absolute Path of ../../  :" + ALDiskExplorer.absolutePath( "../../" ) )
        ALVerbose.unitTest( "Absolute Path of ../modules :" + ALDiskExplorer.absolutePath( "../modules" ) )

        ALDiskExplorer.createFolder( "./bananina/Banane/000-Banane/001Bananina" )
        ALDiskExplorer.removeFolder( "./bananina/Banane/000-Banane/001Bananina"  )



        ALVerbose.unitTest( "Testing copy methods" )
        ALDiskExplorer.copyFolder( "./DataForTests/FolderPlugins", "./TestCopyFolder"  )
        ALDiskExplorer.copyFolder( "./DataForTests/FolderPlugins", "./TestCopyFoldertoDelete"  )
        ALDiskExplorer.copyFile( "./DataForTests/imag01.jpg", "./CopyDataTests/copyImag01.jpg"  )
        ALDiskExplorer.copyFile( "./DataForTests/imag01.jpg", "./TestCopyFolder/copyImag01.jpg"  )
#        ALDiskExplorer.removeFolder( "./TestCopyFoldertoDelete"  )



        oalDiskExplorer = ALDiskExplorer()

        ALVerbose.unitTest( "Creating Folder :" + "./folderCreatedToBeDeleted" )
        oalDiskExplorer.createFolder( "./folderCreatedToBeDeleted" )
        ALVerbose.unitTest( "DELETING Folder :" + "./folderCreatedToBeDeleted" )
        oalDiskExplorer.deleteFolder( "./folderCreatedToBeDeleted" )

#        ALVerbose.unitTest( ALDiskExplorer.normDrive( PyOS.getcwd() ) )
        ALVerbose.unitTest( ALDiskExplorer.getCwd() )
        ALVerbose.unitTest( ALDiskExplorer.getCurrentFolder() )
#        ALVerbose.unitTest( ALDiskExplorer.absPath( PyOS.getcwd()  )  )
        ALVerbose.unitTest( ALDiskExplorer.absPath( ALDiskExplorer.getCurrentFolder() )  )

        ALVerbose.unitTest( ALDiskExplorer.getFileList( "../modules" ) )
        #ALVerbose.unitTest( ALDiskExplorer.getFileList( "./" ) )

       # oalListFile = ALDiskExplorer.getFileList( "./" )
        oalListFile = ALDiskExplorer.getFolderList( "../modules" )
        for ifile in oalListFile:
            if ALDiskExplorer.isFile( ifile ):
                ALVerbose.unitTest( ALString( ifile ) + "->isFile" )
            elif ALDiskExplorer.isDirectory( ifile ):
                ALVerbose.unitTest( ALString( ifile ) + "->isDirectory" )
            elif ALDiskExplorer.isLink( ifile ):
                ALVerbose.unitTest( ALString( ifile ) + "->isLink" )



        ALDiskExplorer.iteractPath( "../modules", UnitTestIteract().printIteract, "Executing::" )
        ALDiskExplorer.iteractPath( "./", UnitTestIteract().printIteract, "TreeView::" )
        ALDiskExplorer.iteractPath( "./", UnitTestIteract().printTree, "TreeView::" )

        try:
            oalStrCurrentFolder = ALDiskExplorer.getCurrentFolder()
            ALVerbose.unitTest( "Current Folder " + oalStrCurrentFolder )
            ALDiskExplorer.setCurrentFolder( "../modules" )
            ALVerbose.unitTest( "New Current Folder " + ALDiskExplorer.getCurrentFolder() )
            ALDiskExplorer.setCurrentFolder( oalStrCurrentFolder )
            ALVerbose.unitTest( "New Current Folder " + ALDiskExplorer.getCurrentFolder() )
        except:
            ALVerbose.warning( "Jython doesn't implements chdir" )
            ALVerbose.warning( "CHDIR test can not be performed" )


        ALVerbose.unitTest()






##############################################################################
# ALDictionary

class UnitTestALDictionary( ALTestCase ):

    def testALDictionary( self ):
        ALVerbose.unitTest( "Test ALDictionary - ordered version" )
        d = ALDictionary()
        d['a'] = 1
        d['b'] = 2
        d['c'] = 3
        d['d'] = 4
        d['e'] = 5

        for i in d:
            print d[i]

        print d.items()

        print d.keys()

        for i, v in d.iteritems():
            print i, v

        print ''
        for i in d.keys():
            print i, d[i]

        print ''
        for i in d.getSequence():
            print i, d[i]

        print ''
        for i in d.ordered():
            print i, d[i]


        d = ALDictionary()
        d.addObject( 'a', 1 )
        d.addObject( 'b', 2 )
        d.addObject( 'c', 3 )
        d.addObject( 'd', 4 )
        d.addObject( 'e', 5 )


        print ''
        for i in d.getSequence():
            print i, d[i]

        print ''
        for i in d.ordered():
            print i, d[i]



        print 'Test original dict'
        e = ALDict()
        e['a'] = 1
        e['b'] = 2
        e['c'] = 3
        e['d'] = 4
        e['e'] = 5

        for i in e:
            print i, e[i]


        c = ALDictionary( d )
        print 'Test copy dictionary Ordered in constructor'
        for i in c.ordered():
            print i, c[i]

        c = ALDictionary( e )
        print 'Test copy dictionary dict in constructor'
        for i in c.ordered():
            print i, c[i]

        ALVerbose.unitTest()






##############################################################################
# Module ALDict

class UnitTestALDict( ALTestCase ):
    def testALDict( self ):
        ALVerbose.unitTest( "Test ALDict" )
        oalDict = ALDict()
        ALVerbose.unitTest( ALString( oalDict ) )

        oalDict.add( "banane", 1000 )
        oalDict.add( "pomme", 2000 )
        oalDict.add( "salade", "pomme and outres salades" )
        ALVerbose.unitTest( ALString( oalDict ) )

        oalObject = oalDict.get( "banane" )
        ALVerbose.unitTest( ALString( oalObject ) )

        oalObject = oalDict.get( "coco" )
        ALVerbose.unitTest( ALString( oalObject ) )

        ALVerbose.unitTest()


##############################################################################


##############################################################################
# Divers

class ALTestCaseDivers( ALTestCase ):

    def process( self ):
        self.addTestMethod( UnitTestALException().testALException )
        self.addTestMethod( UnitTestALSystem().testALSystemExtended )
        self.addTestMethod( UnitTestALAssert().testALAssertEqual )
        self.addTestMethod( UnitTestALAssert().testALAssertRaiseException )
        self.addTestMethod( UnitTestALDict().testALDict )
        self.addTestMethod( UnitTestALDictionary().testALDictionary )
        self.addTestMethod( UnitTestALList().testALList )
        self.addTestMethod( UnitTestALThread().testALThread )
     #   self.addTestMethod( UnitTestALDiskExplorer().testALDiskExplorer )



##############################################################################


if __name__ == '__main__':

    # JIT compiler accelerator
    ALCompiler.accelerator()

    oalTestCaseDivers = ALTestCaseDivers( "TestCase ALTestCaseDivers" )
    oalTestCaseDivers.execute()
