#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################


from ALSystem                    import ALSystem
from ALImportTestCase            import *
from ALImportSystem              import *
from ALImportKernel              import *



class ALTestSuiteSystem( ALTestSuite ):

    def testALSystemBase( self ):
        ALVerbose.unitTest( "Test ALSystem Base" )


        ALVerbose.unitTest( "ALSystem.getInterpreter()    Compiler  :" + ALSystem.getInterpreter() )
        ALVerbose.unitTest( "ALSystem.getOSName()         OS        :" + ALSystem.getOSName() )
        ALVerbose.unitTest( "ALSystem.getOSType()         OSType    :" + ALSystem.getOSType() )
        ALVerbose.unitTest( "ALSystem.getOSVersion()      OSVersion :" + ALSystem.getOSVersion() )
        ALVerbose.unitTest( "ALSystem.getMachineVersion() Machine   :" + ALSystem.getMachineVersion() )
        ALVerbose.unitTest( "ALSystem.getHostName()       HostName  :" + ALSystem.getHostName() )
        ALVerbose.unitTest( "ALSystem.getHost()           Host      :" + ALSystem.getHost() )
        ALVerbose.unitTest( "ALSystem.getHostIP()         HostIP    :" + ALSystem.getHostIP() )
        ALVerbose.unitTest( "ALSystem.getUserName()       UserName  :" + ALSystem.getUserName() )
        ALVerbose.unitTest( "ALSystem.getCompilerName()   Compiler  :" + ALSystem.getCompilerName() )
        ALVerbose.unitTest( "ALSystem.getProgramName()    Program   :" + ALSystem.getProgramName() )


        ALVerbose.unitTest( "ALDiskExplorer.getCwd()      Path      :" + ALDiskExplorer.getCwd() )
        ALVerbose.unitTest( "ALSystem.getUserPath()       USER Path :" + ALSystem.getUserPath() )

        ALVerbose.unitTest( "ALSystem.getCurrentFolder()  Current Folder: " + ALSystem.getCurrentFolder() )
        ALSystem.setCurrentFolder( "../" )
        ALVerbose.unitTest( "ALSystem.getCurrentFolder()  Changed Current Folder to: " + ALSystem.getCurrentFolder() )
        ALSystem.setCurrentFolder( "./tests" )
        ALVerbose.unitTest( "ALSystem.getCurrentFolder()  Changed Current Folder to: " + ALSystem.getCurrentFolder() )
        ALVerbose.unitTest( "ALSystem.getCurrentFolder()  Current Folder: " + ALSystem.getCurrentFolder() )


        ALVerbose.unitTest( "" )


    def testALSysInfoBase( self ):
        ALVerbose.unitTest( "Test ALSysInfo Base" )

        oalSysInfo = ALSysInfo()
        ALVerbose.unitTest( "oalSysInfo.m_interpreter:      " + oalSysInfo.m_interpreter )
        ALVerbose.unitTest( "oalSysInfo.m_machineVirtuel:   " + oalSysInfo.m_machineVirtuel )
        ALVerbose.unitTest( "oalSysInfo.m_OSName:           " + oalSysInfo.m_OSName )
        ALVerbose.unitTest( "oalSysInfo.m_typeOS:           " + oalSysInfo.m_typeOS )
        ALVerbose.unitTest( "oalSysInfo.m_userName:         " + oalSysInfo.m_userName )
        ALVerbose.unitTest( "oalSysInfo.m_hostName:         " + oalSysInfo.m_hostName )
        ALVerbose.unitTest( "oalSysInfo.m_pythonVersion:    " + oalSysInfo.m_pythonVersion )

        ALVerbose.unitTest( "oalSysInfo.m_OSVersion:        " + oalSysInfo.m_OSVersion  )
        ALVerbose.unitTest( "oalSysInfo.m_machineVersion:   " + oalSysInfo.m_machineVersion  )
        ALVerbose.unitTest( "oalSysInfo.m_hostIP:           " + oalSysInfo.m_hostIP )


        ALVerbose.unitTest()



    def process( self ):
        self.addTestMethod( self.testALSystemBase )
        self.addTestMethod( self.testALSysInfoBase )

        self.addTestMethod( ALTestCaseALString().execute )
        self.addTestMethod( ALTestCaseALXml().execute )
        self.addTestMethod( ALTestCaseDivers().execute )

        self.addTestMethod( ALTestCaseALTime().execute )
    #    self.addTestMethod( ALTestCaseALXsd().execute )
    #    self.addTestMethod( ALTestCaseALXsdDataBinding().execute )
    #    self.addTestMethod( ALTestCaseALXsdDataBindingEngine().execute )
        self.addTestMethod( ALTestCaseALThread().execute )
        self.addTestMethod( ALTestCaseALList().execute )
        self.addTestMethod( ALTestCaseALSignal().execute )
        self.addTestMethod( ALTestCaseALManagerProcess().execute )
        self.addTestMethod( ALTestCaseALContainer().execute )
        self.addTestMethod( ALTestCaseALSubProcess().execute )

        if (ALSystem.getInterpreter()=="Jython"):
            self.addTestMethod( ALTestCaseJython().execute )


        # Forcing errors and traces
     #   self.addTestMethod( ALTestCaseALTrace().execute )
     #   self.addTestMethod( ALTestCaseALDebug().execute )



##############################################################################
if __name__ == '__main__':

    # JIT compiler accelerator
    ALCompiler.accelerator()

    oalTestSuiteSystem = ALTestSuiteSystem( "ALTestSuiteSystem" )
    oalTestSuiteSystem.execute()


##############################################################################