#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################


from ALImportPython            import PyThread
from ALString                  import ALString
from ALObject                  import ALObject
from ALVerbose                 import ALVerbose
from ALDebug                   import ALDebug
from ALTrace                   import ALTrace


class ALThread( ALObject, PyThread ):
    """
    Interface for ALThread objects.

    To be modified by the user
    def run( self ):
    """
    def __init__( self ):
        ALObject.__init__( self )
        PyThread.__init__( self )
        self.m_oalObject = None


    def __controlRaise( self ):
        oBoolStatus =  ALDebug.isDebugModeOn()
        ALDebug.controlDebug( self, oBoolStatus )


    def executeKernel( self ):
        """
        This is the method started by the thread model in Synchronous
        It must be modified by the user
        """
        pass



    def execute( self, _oalObject = None ):
        """
        This method starts the thread.
        """
        ALVerbose.aalibDEBUGID( self.getID(), "[Thread] Starting instance: " + ALString.setLogXmlToString( ALString( self ) ) )
        ALVerbose.aalibDEBUGID( self.getID(), "[Thread]         parameter: " + ALString.setLogXmlToString( ALString(  _oalObject )) )

        self.m_oalObject = _oalObject
        try:
            self.start()
        except:
            ALVerbose.errorID( self.getID(), "Exception starting the thread: " + ALString( self.getID() ) )
            ALVerbose.aalibDEBUGID( self.getID(), "   [Thread] Starting instance: " + ALString.setLogXmlToString( ALString( self ) ) )
            ALVerbose.aalibDEBUGID( self.getID(), "   [Thread]         parameter: " + ALString.setLogXmlToString( ALString(  _oalObject ) ) )
            self.__controlRaise()




    def executeSynchronous( self, _oalObject = None ):
        """
        This method starts the Synchronous Thread.
        """
        ALVerbose.aalibDEBUGID( self.getID(), "[Synchronous Thread] Starting instance: " + ALString.setLogXmlToString( ALString( self ) ) )
        ALVerbose.aalibDEBUGID( self.getID(), "[Synchronous Thread]         parameter: " + ALString.setLogXmlToString( ALString( _oalObject ) ) )

        self.m_oalObject = _oalObject
        try:
            self.executeKernel()
        except:
            ALVerbose.errorID( self.getID(), "Exception starting the Synchronous thread: " + ALString( self.getID() ) )
            ALVerbose.aalibDEBUGID( self.getID(), "   [Synchronous Thread] Starting instance: " + ALString.setLogXmlToString( ALString( self ) ) )
            ALVerbose.aalibDEBUGID( self.getID(), "   [Synchronous Thread]         parameter: " + ALString.setLogXmlToString( ALString( _oalObject )) )
            self.__controlRaise()



##############################################################################