#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################


from ALImportPython            import PyTime
from ALString                  import ALString
from ALObject                  import ALObject


class ALTime( ALObject ):
    """
    Interface for AL time objects.

    """
    m_fTimeInit = 0.0
    m_fTimeEnd  = 1.0

    def __init__( self ):
        ALObject.__init__( self )
        self.m_oFloatTimeInit = 0.0
        self.m_oFloatTimeEnd  = 1.0
        self.initTime()


    def initTime( self ):
        """
        Set the time in seconds
        """
        self.m_oFloatTimeInit = ALTime.time()
        return self.m_oFloatTimeInit



    def endTime( self ):
        """
        Set the time in seconds
        """
        self.m_oFloatTimeEnd = ALTime.time()
        return self.m_oFloatTimeEnd


    def getDuration( self ):
        """
        Return the time in seconds
        """
        self.endTime()
        return self.m_oFloatTimeEnd - self.m_oFloatTimeInit



    # Static methods

    def time():
        return PyTime.time()
    time = staticmethod( time )


    def clock():
        return PyTime.clock()
    clock = staticmethod( clock )


    def date():
        osltimestring = ALString( PyTime.strftime( "%Y%m%d", PyTime.localtime( PyTime.time() ) ) )
        return osltimestring
    date = staticmethod( date )


    def getDate():
        return ALTime.date()
    getDate = staticmethod( getDate )


    def hour():
        oslTimeString = ALString( PyTime.strftime( "%H:%M:%S", PyTime.localtime( PyTime.time() ) ) )
        return oslTimeString
    hour = staticmethod( hour )


    def getTimeStringPSP( _ofTime ):
     #   oslTimeString = ALString(  PyTime.strftime( "%dd:%Hh:%Mm:%Ss",  PyTime.localtime(_ofTime) )  )
        ofTime       = _ofTime
        oTotalHour   = 0
        oiHour       = 0
        oiMinute     = 0
        oiDay        = 0
        oiSecond     = 0
        oiMiliSecond = 0
        if (ofTime>3600.0):
            oTotalHour = int(ofTime/3600.0)
            if (oTotalHour>24):
                oiDay = int(oTotalHour/24.0)
                oiHour = int( oTotalHour -(oiDay*24.0) )
            else:
                oiHour = int( oTotalHour )

        ofSecond = ofTime - (3600.0*oTotalHour)
        if (ofSecond>60.0):
            oiMinute = int( ofSecond/60.0)
        ofSecond = ofSecond - (oiMinute*60.0)
        oiSecond = int( ofSecond )
        oiMiliSecond = int( (ofSecond - oiSecond)*1000 )

        if (oiDay<10):
            osDay = ALString( "0" + ALString(oiDay) + "d:" )
        else:
            osDay = ALString( ALString(oiDay) + "d:" )
        if (oiHour<10):
            osHour = ALString( "0" + ALString(oiHour) + "h:" )
        else:
            osHour = ALString( ALString(oiHour) + "h:" )

        if (oiMinute<10):
            osMinute = ALString( "0" + ALString(oiMinute) + "m:" )
        else:
            osMinute = ALString( ALString(oiMinute) + "m:" )

        if (oiSecond<10):
            osSecond = ALString( "0" + ALString(oiSecond) + "s:" )
        else:
            osSecond = ALString( ALString(oiSecond) + "s:" )

        if (oiMiliSecond<10):
            osMiliSecond = ALString( "00" + ALString(oiMiliSecond) + "ms" )
        elif (oiMiliSecond<100):
            osMiliSecond = ALString( "0" + ALString(oiMiliSecond) + "ms" )
        else:
            osMiliSecond = ALString( ALString(oiMiliSecond) + "ms" )

        oslTimeString = osDay + osHour + osMinute + osSecond + osMiliSecond

        return oslTimeString
    getTimeStringPSP = staticmethod( getTimeStringPSP )



    def timeString():
        osltimestring = ALString( PyTime.strftime( "[%Y/%m/%d-%H:%M:%S]", PyTime.localtime( PyTime.time() ) ) )
        return osltimestring
    timeString = staticmethod( timeString )



    def timeStringValue( _fvalue ):
        osltimestring = ALString( PyTime.strftime( "[%Y/%m/%d-%H:%M:%S]", PyTime.localtime( _fvalue ) ) )
        return osltimestring
    timeStringValue = staticmethod( timeStringValue )


    def getTimeString():
        return ALTime.timeString()
    getTimeString = staticmethod( getTimeString )

    def getTimeStringUmt():
        oalStrTime = ALTime.getTimeString()
        oalStrTime = oalStrTime.replace("[","")
        oalStrTime = oalStrTime.replace("]","")
        oalStrTime = oalStrTime.replace("/","-")
        oalStrTime = oalStrTime.replace(":","-")
        oalListTime = oalStrTime.split("-")

        oalStringDataTime = ALString(oalListTime[1]) +" "+ ALString(oalListTime[2]) \
            +" "+ ALString(oalListTime[0])  +" "+ ALString(oalListTime[3]) \
            +":"+ ALString(oalListTime[4])+":"+ ALString(oalListTime[5]) + " GMT-0000"

        return oalStringDataTime
    getTimeStringUmt = staticmethod( getTimeStringUmt )






    def getTimeName():
        osltimestring = ALString( PyTime.strftime( "%Y%m%d-%H%M%S", PyTime.localtime( PyTime.time() ) ) )
        return osltimestring
    getTimeName = staticmethod( getTimeName )



    def getHourName():
        oslHourString = ALTime.getTimeName().split("-")[1]
        return oslHourString
    getHourName = staticmethod( getHourName )


    def sleep( _fTimeInSeconds = 1 ):
        PyTime.sleep( _fTimeInSeconds )
    sleep = staticmethod( sleep )


    def wait( _fTimeInSeconds = 1 ):
        ALTime.sleep( _fTimeInSeconds )
    wait = staticmethod( wait )


    def init():
        """
        Set the time in seconds
        """
        ALTime.m_fTimeInit = ALTime.time()
        return ALTime.m_fTimeInit
    init = staticmethod( init )


    def end():
        """
        Set the time in seconds
        """
        ALTime.m_fTimeEnd = ALTime.time()
        return ALTime.m_fTimeEnd
    end = staticmethod( end )


    def duration():
        """
        Return the time in seconds
        """
        ALTime.end()
        return ALTime.m_fTimeEnd - ALTime.m_fTimeInit
    duration = staticmethod( duration )


    def dif():
        """
        Return the time in seconds
        """
        ALTime.end()
        fdif = ALTime.m_fTimeEnd - ALTime.m_fTimeInit
        ALTime.init()
        return fdif
    dif = staticmethod( dif )



##############################################################################
