#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################


from ALObject                     import ALObject
from ALUnitTest                   import ALUnitTest
from ALCompiler                   import ALCompiler
from ALVerbose                    import ALVerbose
from ALPlugin                     import ALPlugin
from ALTime                       import ALTime
from ALApplication                import ALApplication


class TestClass( ALObject ):

    def methodError( self, _alObject = None ):
        ALVerbose.screen( "TestClass::methodError  - Simulating an Error" )
        simulateAnError()  # simulating an error




class PluginTest( ALPlugin ):

    def process( self, _object = None ):
        ALVerbose.screenID( self.getID(), "Waiting 2 seconds" )
        ALTime.sleep( 2 )
        ALVerbose.screenID( self.getID(), "Finished 2 seconds" )





class ALUnitTestALApplication( ALUnitTest ):
    """
    UnitTest ALApplication models - pure run
    """
    def runApplication01( self ):
        ALVerbose.unitTest( "Testing the runApplication01 = no plugins mode" )

        ALCompiler.accelerator()
        oBoolPlugins = False
        oalApplication = ALApplication( "UnitTestALApplicationDemoNoPlugins", "2.0", oBoolPlugins )
        oalApplication.execute()

        ALVerbose.unitTest( )
        ALVerbose.screen( "\n\n\n" )



    def runApplication02( self ):
        ALVerbose.unitTest( "Testing the runApplication02 = Plugin MODE" )

        ALCompiler.accelerator()

        oalApplication = ALApplication( "UnitTestALApplicationDemo", "2.0" )
        oalApplication.execute()

        ALVerbose.unitTest( )
        ALVerbose.screen( "\n\n\n" )




    def runApplication03( self ):
        ALVerbose.unitTest( "UnitTest ALApplication CommandLineInterface:" )

        # Testing Main Framework
        oalApplication = ALApplication( "ApplicationCommandLineInterface-Test", "1.0" )
#        oalApplication.setCommandLine( "--DEBUG --test banane --test2 1000 --test3 100.01"  )
#        oalApplication.setCommandLine( "--test banane --test2 1000 --test3 100.01"  )
#        oalApplication.setCommandLine( "-h --test"  )
#        oalApplication.setCommandLine( "--help --test2"  )
#        oalApplication.setCommandLine( "--man --test2"  )
#        oalApplication.setCommandLine( "-m --test1"  )
        oalApplication.setCommandLine( "--help --ALL"  )
#        oalApplication.setCommandLine( "--man --all"  )
#        oalApplication.setCommandLine( "--pluginGroup"  )

        # Using a file to define the command line execution and mapping
        oalApplication.setCommandLineInterfaceFile( "./DataForTests/commandLineInterface.xml"  )

        # Execution Again
        oalApplication.execute()


        ALVerbose.unitTest()
        ALVerbose.screen( "\n\n\n" )




    def runApplication04( self ):
        ALVerbose.unitTest( "UnitTest ALApplication Simulate An Error" )

        # Testing Main Framework
        oalApplication = ALApplication( "ApplicationCommandLineInterface-Test", "1.0" )
#        oalApplication.setCommandLine( "--DEBUG --test banane --test2 1000 --test3 100.01"  )
        oalApplication.setCommandLine( "--test banane --test2 1000 --test3 100.01"  )
#        oalApplication.setCommandLine( "-h --test"  )
#        oalApplication.setCommandLine( "--help --test2"  )
#        oalApplication.setCommandLine( "--man --test2"  )
#        oalApplication.setCommandLine( "-m --test1"  )

        # Using a file to define the command line execution and mapping
        oalApplication.setCommandLineInterfaceFile( "./DataForTests/commandLineInterface.xml"  )

        # Connecting the error to  the main loop
        oalApplication.connectExecute( TestClass().methodError )


        # Execution Again
        oalApplication.execute()


        ALVerbose.unitTest()
        ALVerbose.screen( "\n\n\n" )


    def runApplication05( self ):
        ALVerbose.unitTest( "UnitTest ALApplication" )

#        ALApplication.setDebugProfileOff()

        # Testing Main Framework
        oalApplication = ALApplication( "ApplicationCommandLineInterface-Test", "1.0" )

        # Connecting the error to  the main loop
        oalApplication.connectExecute( PluginTest().execute )

        # Execution Again
        oalApplication.execute()


        ALVerbose.unitTest()
        ALVerbose.screen( "\n\n\n" )




    def runApplicationControlDebug( self ):
        ALVerbose.unitTest( "UnitTest ALApplication ControlDebug" )

        # Testing Main Framework
        oalApplication = ALApplication( "ApplicationControlDebug", "2.0" )

        oalApplication.setCommandLine( "--DEBUGFORCE"  )

        # Using a file to define the command line execution and mapping
        oalApplication.setCommandLineInterfaceFile( "./DataForTests/commandLineInterface.xml"  )

        # Execution Again
        oalApplication.execute()


        ALVerbose.unitTest()
        ALVerbose.screen( "\n\n\n" )






    def execute( self ):
        ALVerbose.unitTest( "Testing the Basic ALApplication - Free mode = No test framework - Pure Code" )
        self.runApplication01()
        self.runApplication02()
        self.runApplication03() # testALApplicationCommandLineInterface
        self.runApplication04() # testALApplicationCommandLineInterface
        self.runApplication05()
        self.runApplicationControlDebug()




##############################################################################
if __name__ == '__main__':

    oalUnitTestALApplication = ALUnitTestALApplication()
    oalUnitTestALApplication.execute()


##############################################################################
