#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################


##############################################################################
#
#  Assert code by: Marie-Francoise Incardona
#
##############################################################################



from ALImportPython          import  PySys
from ALDefinition            import  *
from ALObject                import  ALObject
from ALString                import  ALString
from ALDictionary            import  ALDictionary
from ALThreadingCondition    import  ALThreadingCondition
from ALTime                  import  ALTime
from ALLogFile               import  ALLogFile



class ALVerbose( ALObject ):
    """
    Base class to Verbose
    This class control the information flux between the
    application and the outside world (screen, log files, error, std,
    stderror, etc. )

    Note: Designed to be a static class
    Note: Trying to be Thread Safety!!!!
    """

    m_oalThreadingConditionError    = ALThreadingCondition()
    m_oalThreadingConditionUnitTest = ALThreadingCondition()
    m_oalThreadingConditionScreen   = ALThreadingCondition()
    m_oalThreadingConditionLog      = ALThreadingCondition()
    m_oalThreadingConditionLogFile  = ALThreadingCondition()
    m_oalThreadingConditionWrite    = ALThreadingCondition()
    m_oalThreadingConditionProfile  = ALThreadingCondition()
    m_bIsProfile      = False
    m_oalDictionary   = ALDictionary()
    m_oalLogFile      = ALLogFile( AL_FRAMEWORK_LOG_NAME )
    m_bIsLogFile      = True
    m_bIsVerbose      = False
    m_bIsVerboseDebug = False
    m_oalDebugTag     = AL_DEBUG_TAG
    m_iindex          = 0
    m_iConteur        = 0


    def __init__( self ):
        ALObject.__init__( self )



    def __writeError( _oalString ):
        if (ALVerbose.m_bIsVerbose == True or ALVerbose.m_bIsVerboseDebug==True ):
            PySys.stderr.write( ALString( _oalString ) )
    __writeError = staticmethod( __writeError )


    def __write( _oalString ):
        if (ALVerbose.m_bIsVerbose == True or  ALVerbose.m_bIsVerboseDebug==True ):
            PySys.stdout.write( ALString( _oalString ) )
    __write = staticmethod( __write )



    def __toIndex( _oalObject ):
        ALVerbose.m_iConteur += 1
        return ALString( ALString( ALTime.time() ) + AL_TOKEN_SEPARATOR + ALString( _oalObject ) + AL_TOKEN_SEPARATOR + ALString( ALVerbose.m_iConteur ))
    __toIndex = staticmethod( __toIndex )



    def __fromIndex( _oalstring ):
        oList = ALString( _oalstring ).split( AL_TOKEN_SEPARATOR )
        oalStrTime = ALTime.timeStringValue( float( oList[0] ) )
        oalStrInfo = ALString( oList[1] )
        return oalStrTime, oalStrInfo
    __fromIndex = staticmethod( __fromIndex )



    def errorID(  _oalStrID, _oalString ):
        ALVerbose.m_oalThreadingConditionError.synchronizeOn()
        ALVerbose.__addObjectIDInformation( "  ==>[ERROR]: " +  ALString( _oalString ), _oalStrID )
        ALVerbose.m_oalThreadingConditionError.synchronizeOff()
        ALVerbose.error( _oalString )
    errorID = staticmethod( errorID )


    def error( _oalString ):
        ALVerbose.m_oalThreadingConditionError.synchronizeOn()
        oalString = ALString( _oalString )
        oalString = ALString( "  ==>[ERROR]: " + oalString + "\n"  )
        # Force the ERROR output in the screen
        btmp = ALVerbose.isVerbose()
        ALVerbose.setVerboseOn()
        ALVerbose.__writeError( oalString )
        if btmp ==False:
            ALVerbose.setVerboseOff()
        # Force the ERROR output in the screen
        ALVerbose.logFile( oalString )
        ALVerbose.m_oalThreadingConditionError.synchronizeOff()
    error = staticmethod( error )


    def warningID(  _oalStrID, _oalString ):
        ALVerbose.m_oalThreadingConditionScreen.synchronizeOn()
        ALVerbose.__addObjectIDInformation( " [Warning]: " +  ALString(  _oalString ), _oalStrID )
        ALVerbose.m_oalThreadingConditionScreen.synchronizeOff()
        ALVerbose.warning( _oalString )
    warningID = staticmethod( warningID )



    def warning( _oalString ):
        ALVerbose.m_oalThreadingConditionScreen.synchronizeOn()
        oalString = ALString( _oalString )
        oalString = ALString( "  [Warning]: " + oalString + "\n" )
        ALVerbose.__write( oalString )
        ALVerbose.logFile( oalString )
        ALVerbose.m_oalThreadingConditionScreen.synchronizeOff()
    warning = staticmethod( warning )


    def unitTestID( _oalStrID, _oalString = ALString( "[ ok ]" ) ):
        ALVerbose.m_oalThreadingConditionUnitTest.synchronizeOn()
        ALVerbose.__addObjectIDInformation( "  [UnitTest]: " +  ALString( _oalString ), _oalStrID )
        ALVerbose.m_oalThreadingConditionUnitTest.synchronizeOff()
        ALVerbose.unitTest( _oalString )
    unitTestID = staticmethod( unitTestID )

    def unitTest( _oalString = ALString( "[ ok ]" ) ):
        ALVerbose.m_oalThreadingConditionUnitTest.synchronizeOn()
        oalString = ALString( _oalString )
        oalString = ALString( "  [UnitTest]: " + oalString + "\n" )
        ALVerbose.__write( oalString )
        ALVerbose.logFile( oalString )
        ALVerbose.m_oalThreadingConditionUnitTest.synchronizeOff()
    unitTest = staticmethod( unitTest )


    def screenID( _oalStrID, _oalString = ALString( "[ ok ]" ) ):
        ALVerbose.m_oalThreadingConditionScreen.synchronizeOn()
        ALVerbose.__addObjectIDInformation( " " + ALString( _oalString ), _oalStrID )
        ALVerbose.m_oalThreadingConditionScreen.synchronizeOff()
        ALVerbose.screen( _oalString )
    screenID = staticmethod( screenID )

    def screen( _oalString = ALString( "[ ok ]" ) ):
        ALVerbose.m_oalThreadingConditionScreen.synchronizeOn()
        oalString = ALString( _oalString )
        oalString = ALString( "  " + oalString + "\n" )
        ALVerbose.__write( oalString )
        ALVerbose.logFile( oalString )
        ALVerbose.m_oalThreadingConditionScreen.synchronizeOff()
    screen = staticmethod( screen )



    def screenIndexID( _oalStrID, _fvalue, _fref ):
        ALVerbose.m_oalThreadingConditionScreen.synchronizeOn()
        ALVerbose.__addObjectIDInformation( " " + ALString( _fvalue) + " " + ALString( _fref ), _oalStrID )
        ALVerbose.m_oalThreadingConditionScreen.synchronizeOff()
        ALVerbose.screenIndex( _fvalue, _fref )
    screenIndexID = staticmethod( screenIndexID )



    def screenIndex( _fvalue, _fref ):
        ALVerbose.m_oalThreadingConditionScreen.synchronizeOn()
        index = ( float(_fvalue)/ float(_fref) )*100.0
        if ( index >= ALVerbose.m_iindex+1 ):
            ALVerbose.m_iindex = int( index )
            oalString = ALString( ALVerbose.m_iindex )
            oalString = ALString( "  [ " + oalString + "% ]" + "\n" )
            ALVerbose.__write( oalString )
            ALVerbose.logFile( oalString )
        if (index>99.75):  #bug to fix
            ALVerbose.m_iindex = 0
        ALVerbose.m_oalThreadingConditionScreen.synchronizeOff()
    screenIndex = staticmethod( screenIndex )



#=====================================================================================================
    # PUBLIC GENERAL USE
    def DEBUGID( _oalStrID, _oalString = ALString( " DEBUG mode " ), _oalTag = AL_DEBUG_TAG ):
        if (ALVerbose.controllDebugTag( _oalTag )==False ):
            return
        ALVerbose.m_oalThreadingConditionScreen.synchronizeOn()
        ALVerbose.__addObjectIDInformation( " [DEBUG]: "  + _oalString, _oalStrID )
        ALVerbose.m_oalThreadingConditionScreen.synchronizeOff()
        ALVerbose.DEBUG( _oalString, _oalTag )
    DEBUGID = staticmethod( DEBUGID )


    def DEBUG( _oalString = ALString( " DEBUG mode " ), _oalTag = AL_DEBUG_TAG ):
        if (ALVerbose.controllDebugTag( _oalTag )==False ):
            return
        if (ALVerbose.isVerboseDebug()):
            ALVerbose.screen( ALString( "[DEBUG]: " + ALString( _oalString ) ) )
    DEBUG = staticmethod( DEBUG )


    def DEBUGINDEXID( _oalStrID, _fvalue, _fref, _oalTag = AL_DEBUG_TAG ):
        if (ALVerbose.controllDebugTag( _oalTag )==False ):
            return
        ALVerbose.m_oalThreadingConditionScreen.synchronizeOn()
        ALVerbose.__addObjectIDInformation( " [DEBUG]: "  + ALString( _fvalue) + " " + ALString( _fref ), _oalStrID )
        ALVerbose.m_oalThreadingConditionScreen.synchronizeOff()
        ALVerbose.DEBUGINDEX( _fvalue, _fref, _oalTag )
    DEBUGINDEXID = staticmethod( DEBUGINDEXID )


    def DEBUGINDEX( _fvalue, _fref, _oalTag = AL_DEBUG_TAG ):
        if (ALVerbose.controllDebugTag( _oalTag )==False ):
            return
        if (ALVerbose.isVerboseDebug()==True):
            ALVerbose.screenIndex( _fvalue, _fref )
    DEBUGINDEX = staticmethod( DEBUGINDEX )



    def ASSERT( _oalString ):
        if ALVerbose.isVerbose():
            ALVerbose.screen( "[ASSERT]:   "  + ALString( _oalString ) )
    ASSERT = staticmethod( ASSERT )

#=====================================================================================================

    # RESERVED FOR AALIB
    def aalibDEBUGID( _oalStrID, _oalString = ALString( " DEBUG mode " ), _oalTag = AL_DEBUG_TAG_AALIB ):
        ALVerbose.DEBUGID( _oalStrID, _oalString, _oalTag )
    aalibDEBUGID = staticmethod( aalibDEBUGID )


    def aalibDEBUG( _oalString = ALString( " DEBUG mode " ), _oalTag = AL_DEBUG_TAG_AALIB ):
        ALVerbose.DEBUG( _oalString, _oalTag )
    aalibDEBUG = staticmethod( aalibDEBUG )



#=====================================================================================================

    # Private Method
    def logFile( _oalString ):
        if (ALVerbose.m_bIsLogFile == False):
            return
        ALVerbose.m_oalThreadingConditionLogFile.synchronizeOn()
        ALVerbose.m_oalLogFile.log( ALString(  _oalString ) )
        ALVerbose.m_oalThreadingConditionLogFile.synchronizeOff()
    logFile = staticmethod( logFile )



    def log( _oalString ):
        ALVerbose.m_oalThreadingConditionLog.synchronizeOn()
        oalString = ALString( _oalString )
        oalString = ALString( "  " + oalString + "\n" )
        ALVerbose.logFile( oalString )
        ALVerbose.m_oalThreadingConditionLog.synchronizeOff()
    log = staticmethod( log )


    def logID( _oalStrID, _oalString ):
        ALVerbose.m_oalThreadingConditionLog.synchronizeOn()
        oalString = ALString( _oalString )
        oalString = ALString( "  " + oalString + "\n" )
        ALVerbose.__addObjectIDInformation( " " + _oalString, _oalStrID )
        ALVerbose.logFile( oalString )
        ALVerbose.m_oalThreadingConditionLog.synchronizeOff()
    logID = staticmethod( logID )



    def isVerbose():
        bisVerbose = ALVerbose.m_bIsVerbose
        return bisVerbose
    isVerbose = staticmethod( isVerbose )



    def isLogFile( ):
        return ALVerbose.m_bIsLogFile
    isLogFile = staticmethod( isLogFile )


    def setLogFileOn( ):
        ALVerbose.m_bIsLogFile = True
    setLogFileOn = staticmethod( setLogFileOn )


    def setLogFileOff( ):
        ALVerbose.m_bIsLogFile = False
    setLogFileOff = staticmethod( setLogFileOff )


    def setLogFileName( _oalString ):
        ALVerbose.m_oalThreadingConditionLog.synchronizeOn()
        ALVerbose.m_oalLogFile.setFileName( _oalString )
        ALVerbose.m_oalThreadingConditionLog.synchronizeOff()
    setLogFileName = staticmethod( setLogFileName )


    def setLogFileNameUnique( _oalString ):
        ALVerbose.m_oalThreadingConditionLog.synchronizeOn()
        ALVerbose.m_oalLogFile.setUniqueFileName( _oalString )
        ALVerbose.m_oalThreadingConditionLog.synchronizeOff()
    setLogFileNameUnique = staticmethod( setLogFileNameUnique )




    def getLogFileName():
        ALVerbose.m_oalThreadingConditionLog.synchronizeOn()
        oalString = ALVerbose.m_oalLogFile.getFileName()
        ALVerbose.m_oalThreadingConditionLog.synchronizeOff()
        return oalString
    getLogFileName = staticmethod( getLogFileName )



    def setVerboseOn( ):
        ALVerbose.m_bIsVerbose = True
    setVerboseOn = staticmethod( setVerboseOn )


    def setVerboseOff( ):
        ALVerbose.m_bIsVerbose = False
    setVerboseOff = staticmethod( setVerboseOff )


    def isVerboseDebug():
        bisVerboseDebug = ALVerbose.m_bIsVerboseDebug
        return bisVerboseDebug
    isVerboseDebug = staticmethod( isVerboseDebug )


    def setVerboseDebugOn( ):
        ALVerbose.m_bIsVerboseDebug = True
    setVerboseDebugOn = staticmethod( setVerboseDebugOn )


    def setVerboseDebugOff( ):
        ALVerbose.m_bIsVerboseDebug = False
    setVerboseDebugOff = staticmethod( setVerboseDebugOff )


    def setDebugTag( _oalDebugTag ):
        ALVerbose.m_oalDebugTag = _oalDebugTag
    setDebugTag = staticmethod( setDebugTag )


    def getDebugTag():
        return ALVerbose.m_oalDebugTag
    getDebugTag = staticmethod( getDebugTag )


    def controllDebugTag( _oalDebugTag ):
        if (ALVerbose.isVerboseDebug()==False):
            return False
        oBoolStatus = False;
        if ( ALVerbose.getDebugTag()==AL_DEBUG_TAG and _oalDebugTag!=AL_DEBUG_TAG_AALIB ):
            return True

        if ( ALVerbose.getDebugTag()==_oalDebugTag):
            oBoolStatus = True

        return oBoolStatus
    controllDebugTag = staticmethod( controllDebugTag )


############################################################################################

    def outputActionLog( _oalID = None, _oalFileName = None ):
        if (_oalID != None ):
            ALVerbose.__outputActionLogID( _oalID, _oalFileName )
        else:
            ALVerbose.__outputActionLogFinal()
    outputActionLog = staticmethod( outputActionLog )



    def __outputActionLogID( _oalID, _oalFileName = None ):
        ALVerbose.m_oalThreadingConditionLogFile.synchronizeOn()

        oalFileName = _oalFileName
        if ( oalFileName == None ):
            oalFileName = _oalID
        oalLogFile = ALLogFile( oalFileName, True, True )
        oalLogFile.outputVerboseAction( ALVerbose.m_oalDictionary, _oalID )

        ALVerbose.m_oalThreadingConditionLogFile.synchronizeOff()
    __outputActionLogID = staticmethod( __outputActionLogID )



    def __outputActionLogFinal():
        if (ALVerbose.isLogFile()==False or ALVerbose.isProfile()==False):
            return
        ALVerbose.m_oalThreadingConditionLogFile.synchronizeOn()
        ALVerbose.m_oalLogFile.logVerboseTree( ALVerbose.m_oalDictionary )
        del ALVerbose.m_oalDictionary
        ALVerbose.m_oalDictionary = ALDictionary()
        ALVerbose.m_oalThreadingConditionLogFile.synchronizeOff()
    __outputActionLogFinal = staticmethod( __outputActionLogFinal )



    def outputProfileDataBase():
        if (ALVerbose.isLogFile()==False or ALVerbose.isProfile()==False):
            return
        ALVerbose.m_oalThreadingConditionLogFile.synchronizeOn()
        oalFileName = ALVerbose.m_oalLogFile.logVerboseTree( ALVerbose.m_oalDictionary )
        ALVerbose.m_oalThreadingConditionLogFile.synchronizeOff()
        return oalFileName
    outputProfileDataBase = staticmethod( outputProfileDataBase )



    def outputObjectsDEBUG( _oalFile = PySys.stdout ):
        ALVerbose.m_oalThreadingConditionLog.synchronizeOn()
        for i in ALVerbose.m_oalDictionary.ordered():
            oalStrTime, oalStrInfo = ALVerbose.__fromIndex( i )
            _oalFile.write( oalStrTime + " " + oalStrInfo + " " +  ALString( ALVerbose.m_oalDictionary[i] )+ "\n" )
        ALVerbose.m_oalThreadingConditionLog.synchronizeOff()
    outputObjectsDEBUG = staticmethod( outputObjectsDEBUG )




    # Profile Tool
    def setProfileOn( ):
        ALVerbose.m_oalThreadingConditionProfile.synchronizeOn()
        ALVerbose.m_bIsProfile = True
        ALVerbose.m_oalThreadingConditionProfile.synchronizeOff()
    setProfileOn = staticmethod( setProfileOn )


    def setProfileOff( ):
        ALVerbose.m_oalThreadingConditionProfile.synchronizeOn()
        ALVerbose.m_bIsProfile = False
        ALVerbose.m_oalThreadingConditionProfile.synchronizeOff()
    setProfileOff = staticmethod( setProfileOff )


    def isProfile():
        ALVerbose.m_oalThreadingConditionProfile.synchronizeOn()
        bIsProfile = ALVerbose.m_bIsProfile
        ALVerbose.m_oalThreadingConditionProfile.synchronizeOff()
        return bIsProfile
    isProfile = staticmethod( isProfile )


    def getProfileFile():
        if (ALVerbose.isProfile()==True):
            ALVerbose.m_oalThreadingConditionProfile.synchronizeOn()
            ALVerbose.m_oalLogFile.getProfileFile()
            ALVerbose.m_oalThreadingConditionProfile.synchronizeOff()
    getProfileFile = staticmethod( getProfileFile )


    def __addObjectIDInformation( _oalString, _oalStringID ):
        if (ALVerbose.m_bIsProfile==True):
            ALVerbose.m_oalThreadingConditionProfile.synchronizeOn()
            ALVerbose.m_oalDictionary.addObject( ALVerbose.__toIndex( _oalString  ), _oalStringID )
            ALVerbose.m_oalThreadingConditionProfile.synchronizeOff()
    __addObjectIDInformation = staticmethod( __addObjectIDInformation )

##############################################################################


