#
#    Project: mxPluginExec
#             http://www.edna-site.org
#
#    File: "$Id: EDPluginBestv10.py 752 2009-06-04 09:51:15Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors:      Marie-Francoise Incardona (incardon@esrf.fr)
#                            Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License  along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#


from EDImportLib                     import EDVerbose
from EDImportLib                     import EDString
from EDImportLib                     import EDDiskExplorer
from EDImportLib                     import EDList
from EDImportLib                     import EDDict
from EDUtilsPath                     import EDUtilsPath
from EDMessage                       import EDMessage
from EDUtilsFile                     import EDUtilsFile
from EDUtilsTable                    import EDUtilsTable
from EDConfiguration                 import EDConfiguration
from EDPluginExecProcessScript       import EDPluginExecProcessScript

from XSDataCommon import XSDataFloat 
from XSDataCommon import XSDataTime 
from XSDataCommon import XSDataLength 
from XSDataCommon import XSDataAngle 
from XSDataCommon import XSDataInteger 
from XSDataCommon import XSDataString 
from XSDataCommon import XSDataTime 

from XSDataBestv10 import XSDataBestInput
from XSDataBestv10 import XSDataBestOutput 

from XSDataBestv10 import XSDataBestCollectionPlan 
from XSDataBestv10 import XSDataBestStatisticalPrediction 
from XSDataBestv10 import XSDataBestCollectionRun
from XSDataBestv10 import XSDataBestStrategySummary 
from XSDataBestv10 import XSDataBestResolutionBin 

from XSDataDnaTablesBest          import dna_tables


class EDPluginBestv10( EDPluginExecProcessScript ):
    """
    """
    
    def __init__( self ):
        """
        """
        EDPluginExecProcessScript.__init__( self )
        
        self.addCompatibleVersion( EDString( 'Version 3.1.0.d //  16.07.2007' ))
        self.addCompatibleVersion( EDString( 'Version 3.2.0 //  03.11.2008' ))
        
        self.m_edStrCONF_BEST_HOME_LABEL = "besthome"

        # Default value of strategy complexity
        self.m_edStringComplexity = EDString( "none" )
        
        self.m_edStrBestHome = None
        self.m_edStrCommandBestHome = None
        self.m_edStrCommandBest = None
        
        self.m_edStrExposureTime = None
        self.m_edStrDoseRate = None
        self.m_edStrDetectorType = None
        
        self.m_edFileBestDat = None
        self.m_edFileBestPar = None
        self.m_edListFileBestHKL = EDList()
        
        self.setXSDataInputClass( XSDataBestInput )
        
    
    def getComplexity( self ):
        """
        """
        return self.m_edStringComplexity
    
    
    def setComplexity( self, _edStringComplexity ):
        """
        """
        self.m_edStringComplexity = _edStringComplexity

    
    def getBestHome( self ):
        """
        """
        return self.m_edStrBestHome
    
    
    def setBestHome( self, edStrBestHome ):
        """
        """
        self.m_edStrBestHome = edStrBestHome
        
        
    def getCommandBestHome( self ):
        """
        """
        return self.m_edStrCommandBestHome
    
    
    def setCommandBestHome( self, _edStrCommandBestHome ):
        """
        """
        self.m_edStrCommandBestHome = _edStrCommandBestHome
    
    
    def getCommandBest( self ):
        """
        """
        return self.m_edStrCommandBest
    
    
    def setCommandBest( self, _edStrCommandBest ):
        """
        """
        self.m_edStrCommandBest = _edStrCommandBest
        
        
    def getFileBestDat( self ):
        """
        """
        return self.m_edFileBestDat
    
        
    def setFileBestDat( self, _edFileBestDat ):
        """
        """
        self.m_edFileBestDat = _edFileBestDat
        
        
    def getFileBestPar( self ):
        """
        """
        return self.m_edFileBestPar
    
        
    def setFileBestPar( self, _edFileBestPar ):
        """
        """
        self.m_edFileBestPar = _edFileBestPar
        
        
    def getListFileBestHKL( self ):
        """
        """
        return self.m_edListFileBestHKL
    
        
    def setListFileBestHKL( self, _edListFileBestHKL ):
        """
        """
        self.m_edListFileBestHKL = _edListFileBestHKL
    
    
    def configure( self ):
        """
        """
        EDPluginExecProcessScript.configure( self )
        EDVerbose.DEBUG( "*** EDPluginBestv10.configure")
        self.setRequireCCP4( True )
        edStrScriptExecutable = self.getScriptExecutable()
        EDVerbose.DEBUG("*** EDPluginBestv10.configure: Script Executable: " + edStrScriptExecutable )
        edStrBestScriptHome = EDDiskExplorer.getFolderName( edStrScriptExecutable )
        edStrBestHome = None
        
        pluginConfiguration = self.getConfiguration()
        
        if( pluginConfiguration == None):
            edStrBestHome = edStrBestScriptHome           
        else:
            edStrBestHome = EDConfiguration.getStringParamValue( pluginConfiguration, self.m_edStrCONF_BEST_HOME_LABEL )
            if( edStrBestHome == None ):
                edStrBestHome = edStrBestScriptHome
                
        self.setBestHome( edStrBestHome )
        EDVerbose.DEBUG("*** EDPluginBestv10.configure: Best Home: " + edStrBestHome )
        self.setCommandBestHome( "export besthome=" + self.getBestHome() )
        
                
    def preProcess( self, _edObject = None ):
        """
        """
        EDPluginExecProcessScript.preProcess( self )
        
        self.setFileBestDat( EDUtilsPath.mergePath( self.getWorkingDirectory(), "bestfile.dat" ))
        self.setFileBestPar( EDUtilsPath.mergePath( self.getWorkingDirectory(), "bestfile.par" ))
        
        EDUtilsFile.writeFile( self.getFileBestDat(), self.getDataInput().getBestFileContentDat().getValue() )
        EDUtilsFile.writeFile( self.getFileBestPar(), self.getDataInput().getBestFileContentPar().getValue())
        
        listBestFileContentHKL = self.getDataInput().getBestFileContentHKL() 
        
        iterator = 0       
        for bestFileContentHKL in listBestFileContentHKL:
            iterator = iterator + 1
            bestFileHKL = EDUtilsPath.mergePath( self.getWorkingDirectory(), "bestfile" + EDString( iterator ) + ".hkl" )
            self.m_edListFileBestHKL.add( bestFileHKL )
            EDUtilsFile.writeFile( bestFileHKL, bestFileContentHKL.getValue() )
            
                    
        if( self.getDataInput().getComplexity() is not None ):
            self.setComplexity( EDString( self.getDataInput().getComplexity().getValue() ))

        self.initializeCommands()
        
        
    def initializeCommands( self ):
        """
        """
        self.addListCommandPreExecution( self.m_edStrCommandBestHome )
        
        listFileBestHKL = self.getListFileBestHKL()
        listFileBestHKLCommand = ""
        
        for fileBestHKL in listFileBestHKL:
            listFileBestHKLCommand = listFileBestHKLCommand + fileBestHKL + " "
            
        edStrDetectorName = EDString( self.getDataInput().getDetectorType().getValue() )
        edStrExposureTime = EDString( self.getDataInput().getBeamExposureTime().getValue() )
        edStrMaxExposureTime = EDString( self.getDataInput().getBeamMaxExposureTime().getValue() )
        
        self.m_edStrCommandBest = EDString( "-f " + edStrDetectorName + " " + \
                                   "-t " + edStrExposureTime + " " ) 
        
        if( self.getDataInput().getBeamMinExposureTime() is not None ):
            edStrBeamMinExposureTime = EDString( self.getDataInput().getBeamMinExposureTime().getValue() )
            self.m_edStrCommandBest = self.m_edStrCommandBest + EDString( "-M " + edStrBeamMinExposureTime + " " )
            
        if( self.getDataInput().getGoniostatMaxRotationSpeed() is not None ):
            edStrGoniostatMaxRotationSpeed = EDString( self.getDataInput().getGoniostatMaxRotationSpeed().getValue() )
            self.m_edStrCommandBest = self.m_edStrCommandBest + EDString( "-S " + edStrGoniostatMaxRotationSpeed + " " )
            
        if( self.getDataInput().getGoniostatMinRotationWidth() is not None ):
            edStrGoniostatMinRotationWidth = EDString( self.getDataInput().getGoniostatMinRotationWidth().getValue() )
            self.m_edStrCommandBest = self.m_edStrCommandBest + EDString( "-w " + edStrGoniostatMinRotationWidth + " " )
        
        if( self.getDataInput().getAimedResolution() is not None ):
            edStrAimedResolution = EDString( self.getDataInput().getAimedResolution().getValue() )
            self.m_edStrCommandBest = self.m_edStrCommandBest + EDString( "-r " + edStrAimedResolution + " " )
            
        if( self.getDataInput().getAimedRedundancy() is not None ):
            edStrAimedRedundancy = EDString( self.getDataInput().getAimedRedundancy().getValue() )
            self.m_edStrCommandBest = self.m_edStrCommandBest + EDString( "-R " + edStrAimedRedundancy + " " )
            
        if( self.getDataInput().getAimedCompleteness() is not None ):
            edStrAimedCompleteness = EDString( self.getDataInput().getAimedCompleteness().getValue() )
            self.m_edStrCommandBest = self.m_edStrCommandBest + EDString( "-C " + edStrAimedCompleteness + " " )
            
        if( self.getDataInput().getAimedIOverSigma() is not None ):
            edStrAimedIOverSigma = EDString( self.getDataInput().getAimedIOverSigma().getValue() )
            self.m_edStrCommandBest = self.m_edStrCommandBest + EDString( "-i2s " + edStrAimedIOverSigma + " " )
                                   
        if( self.getDataInput().getCrystalAbsorbedDoseRate() is not None ):
            edStrCrystalAbsorbedDoseRate = EDString( self.getDataInput().getCrystalAbsorbedDoseRate().getValue() )
            self.m_edStrCommandBest = self.m_edStrCommandBest + EDString( "-GpS " + edStrCrystalAbsorbedDoseRate + " " )
            
        if( self.getDataInput().getCrystalShape() is not None ):
            edStrCrystalShape = EDString( self.getDataInput().getCrystalShape().getValue() )
            self.m_edStrCommandBest = self.m_edStrCommandBest + EDString( "-sh " + edStrCrystalShape + " " )
            
        if( self.getDataInput().getCrystalSusceptibility() is not None ):
            edStrCrystalSusceptibility = EDString( self.getDataInput().getCrystalSusceptibility().getValue() )
            self.m_edStrCommandBest = self.m_edStrCommandBest + EDString( "-su " + edStrCrystalSusceptibility + " " )
            
                                              
        self.m_edStrCommandBest = self.m_edStrCommandBest + EDString( "-T " + edStrMaxExposureTime + " " + \
                                     "-dna " + self.getScriptBaseName() + "_dnaTables.xml" + " " + \
                                     "-o " + self.getScriptBaseName() + "_plots.mtv " + \
                                     "-e " + self.getComplexity() + " " + \
                                     "-mos " + EDString( self.getFileBestDat() + " " + self.getFileBestPar() + " " + listFileBestHKLCommand ) ) 
                                         
        self.setScriptCommandline( self.m_edStrCommandBest )
    
    
    def postProcess( self, _edObject = None ):
        """
        """    
        EDPluginExecProcessScript.postProcess( self )
        
        edStringError = self.readProcessErrorLogFile()
        if( (edStringError is not None) and ( edStringError != EDString.empty() ) ):
            errorMessage = EDMessage.ERROR_EXECUTION_03 % ('EDPluginBestv10.postProcess', 'EDPluginBestv10', edStringError )
            EDVerbose.error( errorMessage )
            self.addErrorMessage( errorMessage )
            raise RuntimeError, errorMessage
        
        outputData = self.getOutputDataFromDNATableFile( EDUtilsPath.mergePath( self.getWorkingDirectory(), self.getScriptBaseName() + "_dnaTables.xml" ))
        self.setDataOutput( outputData )   
    
    
    def getOutputDataFromDNATableFile( self, _edStringFileName ):
        """
        """
        
        xsDataBestOutput = XSDataBestOutput()
        edStringDnaTablesXML = self.readProcessFile( _edStringFileName )
        xsDataDnaTables = dna_tables.parseString( edStringDnaTablesXML )
        # Loop through all the tables and fill in the relevant parts of xsDataBestOutput
        
        # SubWedges
        xsTablesCollectionStrategy = EDUtilsTable.getTableListFromTables( xsDataDnaTables, "data_collection_strategy" )

        for xsTableCollectionStrategy in xsTablesCollectionStrategy:
        
            xsDataBestCollectionPlan = XSDataBestCollectionPlan()
            xsCollectionRunList = EDUtilsTable.getListsFromTable( xsTableCollectionStrategy, "collection_run" )
                
            for xsCollectionRunItemList in xsCollectionRunList:
                xsDataCollectionRun = self.collectionRunItemListToCollectionRun( xsCollectionRunItemList )
                xsDataBestCollectionPlan.addCollectionRun( xsDataCollectionRun )
                
            # Strategy Summary
            xsStrategySummaryItemList = EDUtilsTable.getListsFromTable( xsTableCollectionStrategy, "summary" )
            xsDataStrategySummary = self.strategySummaryItemListToStrategySummary( xsStrategySummaryItemList[0] )
            # Ranking Resolution
            # Not part of strategySummaryItemListToStrategySummary method since it is in the general_form part
            xsTableGeneralInform = EDUtilsTable.getTableFromTables( xsDataDnaTables, "general_inform" )
            xsRankingResolutionItemList = EDUtilsTable.getListsFromTable( xsTableGeneralInform, "ranking_resolution" )
            xsItemRankingResolution = EDUtilsTable.getItemFromList( xsRankingResolutionItemList[0], "dmin" )
            fRankingResolution = EDString( xsItemRankingResolution.getValueOf_() ).atof()
            xsDataStrategySummary.setRankingResolution( XSDataFloat( fRankingResolution ) )
            
            xsDataBestCollectionPlan.setStrategySummary( xsDataStrategySummary )
            
            # Satistics
            xsTablesStatisticalPrediction = EDUtilsTable.getTableListFromTables( xsDataDnaTables, "statistical_prediction" )
            for xsTableStatisticalPrediction in xsTablesStatisticalPrediction:
                if( xsTableStatisticalPrediction.getIndex() == xsTableCollectionStrategy.getIndex() ):
                    xsResolutionBinList = EDUtilsTable.getListsFromTable( xsTableStatisticalPrediction, "resolution_bin" )
                    xsDataStatisticalPrediction = XSDataBestStatisticalPrediction()
                    for xsResolutionBinItemList in xsResolutionBinList:
                        xsDataResolutionBin = self.resolutionBinItemListToResolutionBin( xsResolutionBinItemList )
                        xsDataStatisticalPrediction.addResolutionBin( xsDataResolutionBin )
                    
                    xsDataBestCollectionPlan.setStatisticalPrediction( xsDataStatisticalPrediction )
        
            xsDataBestOutput.addCollectionPlan( xsDataBestCollectionPlan )
        
        return xsDataBestOutput
            
            
    def collectionRunItemListToCollectionRun( self, _xsCollectionRunItemList ):
        """
        """
        xsDataCollectionRun = XSDataBestCollectionRun()
        
        xsItemExposureTime = EDUtilsTable.getItemFromList( _xsCollectionRunItemList, "exposure_time" )
        fExposureTime = EDString( xsItemExposureTime.getValueOf_() ).atof()
        xsDataCollectionRun.setExposureTime( XSDataTime( fExposureTime ))
        
        xsItemDistance = EDUtilsTable.getItemFromList( _xsCollectionRunItemList, "distance" )
        fDistance = EDString( xsItemDistance.getValueOf_() ).atof()
        xsDataCollectionRun.setDistance( XSDataLength( fDistance ) )
        
        xsItemRotationAxisStart = EDUtilsTable.getItemFromList( _xsCollectionRunItemList, "phi_start" )
        fRotationAxisStart = EDString( xsItemRotationAxisStart.getValueOf_() ).atof()
        xsDataCollectionRun.setPhiStart( XSDataAngle( fRotationAxisStart ))
        
        xsNumberOfImages = EDUtilsTable.getItemFromList( _xsCollectionRunItemList, "number_of_images" )
        iNumberOfImages = EDString( xsNumberOfImages.getValueOf_() ).atoi()
        xsDataCollectionRun.setNumberOfImages( XSDataInteger( iNumberOfImages ))
        
        xsItemPhiWidth = EDUtilsTable.getItemFromList( _xsCollectionRunItemList, "phi_width" )
        fPhiWidth = EDString( xsItemPhiWidth.getValueOf_() ).atof()
        xsDataCollectionRun.setPhiWidth( XSDataAngle( fPhiWidth ))
        
        xsItemOverlaps = EDUtilsTable.getItemFromList(_xsCollectionRunItemList, "overlaps" )
        edStringOverlaps = EDString( xsItemOverlaps.getValueOf_() )
        xsDataCollectionRun.setOverlaps( XSDataString( edStringOverlaps ) )

        return xsDataCollectionRun
    
    
    def strategySummaryItemListToStrategySummary( self, _xsStrategySummaryItemList ):
        """
        """
        xsDataStrategySummary = XSDataBestStrategySummary()
        strItemAttenuation = EDUtilsTable.getItemFromList( _xsStrategySummaryItemList, "attenuation" )
        fAttenuation = EDString( strItemAttenuation.getValueOf_() ).atof()
        xsDataStrategySummary.setAttenuation( XSDataFloat( fAttenuation ))
        
        strItemCompleteness = EDUtilsTable.getItemFromList( _xsStrategySummaryItemList, "completeness" )
        # For homegeneity concerns, EDNA data model should store all the completeness value in fraction
        # ( DNA table xml file stores the summary strategy completeness in percentage whereas
        # the resolution bin completeness are in fraction )
        fCompleteness = EDString( strItemCompleteness.getValueOf_() ).atof() / 100
        xsDataStrategySummary.setCompleteness( XSDataFloat( fCompleteness ))
        
        strItemISigma = EDUtilsTable.getItemFromList( _xsStrategySummaryItemList, "i_sigma" )
        fISigma = EDString( strItemISigma.getValueOf_() ).atof()
        xsDataStrategySummary.setISigma( XSDataFloat( fISigma ))
        
        strItemRedundancy = EDUtilsTable.getItemFromList( _xsStrategySummaryItemList, "redundancy" )
        fRedundancy = EDString( strItemRedundancy.getValueOf_() ).atof()
        xsDataStrategySummary.setRedundancy( XSDataFloat( fRedundancy ))
        
        strItemResolution = EDUtilsTable.getItemFromList( _xsStrategySummaryItemList, "resolution" )
        fResolution = EDString( strItemResolution.getValueOf_() ).atof()
        xsDataStrategySummary.setResolution( XSDataFloat( fResolution ))
        
        strItemResolutionReasoning = EDUtilsTable.getItemFromList( _xsStrategySummaryItemList, "resolution_reasoning" )
        edStringResolutionReasoning = EDString( strItemResolutionReasoning.getValueOf_())
        xsDataStrategySummary.setResolutionReasoning( XSDataString( edStringResolutionReasoning ))
        
        strItemTotalDataCollectionTime = EDUtilsTable.getItemFromList( _xsStrategySummaryItemList, "total_data_collection_time" )
        fTotalDataCollectionTime = EDString( strItemTotalDataCollectionTime.getValueOf_() ).atof()
        xsDataStrategySummary.setTotalDataCollectionTime( XSDataTime( fTotalDataCollectionTime ))
        
        strItemTotalExposureTime = EDUtilsTable.getItemFromList( _xsStrategySummaryItemList, "total_exposure_time" )
        fTotalExposureTime = EDString( strItemTotalExposureTime.getValueOf_() ).atof()
        xsDataStrategySummary.setTotalExposureTime( XSDataTime( fTotalExposureTime ))
        
        return xsDataStrategySummary
    
    
    def resolutionBinItemListToResolutionBin( self, _xsResolutionBinItemList ):
        """
        """
        xsDataResolutionBin = XSDataBestResolutionBin()
        
        strItem = EDUtilsTable.getItemFromList( _xsResolutionBinItemList, "min_resolution" )
        fItem = EDString( strItem.getValueOf_() ).atof()
        xsDataResolutionBin.setMinResolution( XSDataFloat( fItem ))
        
        strItem = EDUtilsTable.getItemFromList( _xsResolutionBinItemList, "max_resolution" )
        fItem = EDString( strItem.getValueOf_() ).atof()
        xsDataResolutionBin.setMaxResolution( XSDataFloat( fItem ))
        
        strItem = EDUtilsTable.getItemFromList( _xsResolutionBinItemList, "completeness" )
        fItem = EDString( strItem.getValueOf_() ).atof()
        xsDataResolutionBin.setCompleteness( XSDataFloat( fItem ))
        
        strItem = EDUtilsTable.getItemFromList( _xsResolutionBinItemList, "redundancy" )
        fItem = EDString( strItem.getValueOf_() ).atof()
        xsDataResolutionBin.setRedundancy( XSDataFloat( fItem ))
        
        strItem = EDUtilsTable.getItemFromList( _xsResolutionBinItemList, "average_intensity" )
        fItem = EDString( strItem.getValueOf_() ).atof()
        xsDataResolutionBin.setAverageIntensity( XSDataFloat( fItem ))
        
        strItem = EDUtilsTable.getItemFromList( _xsResolutionBinItemList, "average_error" )
        fItem = EDString( strItem.getValueOf_() ).atof()
        xsDataResolutionBin.setAverageSigma( XSDataFloat( fItem ))
        
        strItem = EDUtilsTable.getItemFromList( _xsResolutionBinItemList, "average_i_over_sigma" )
        fItem = EDString( strItem.getValueOf_() ).atof()
        xsDataResolutionBin.setIOverSigma( XSDataFloat( fItem ))
        
        strItem = EDUtilsTable.getItemFromList( _xsResolutionBinItemList, "R_factor" )
        fItem = EDString( strItem.getValueOf_() ).atof()
        xsDataResolutionBin.setRFactor( XSDataFloat( fItem ))
        
        strItem = EDUtilsTable.getItemFromList( _xsResolutionBinItemList, "Chi**2" )
        if ( strItem is not None ):
            fItem = EDString( strItem.getValueOf_() ).atof()
            xsDataResolutionBin.setChi2( XSDataFloat( fItem ))
        
        strItem = EDUtilsTable.getItemFromList( _xsResolutionBinItemList, "average_i_over_sigma_Chi**2" )
        if ( strItem is not None ):
            fItem = EDString( strItem.getValueOf_() ).atof()
            xsDataResolutionBin.setIOverSigmaChi( XSDataFloat( fItem ))
        
        strItem = EDUtilsTable.getItemFromList( _xsResolutionBinItemList, "fract_overload" )
        fItem = EDString( strItem.getValueOf_() ).atof()
        xsDataResolutionBin.setPercentageOverload( XSDataFloat( fItem ))
        
        return xsDataResolutionBin
          
        
    def checkParameters( self ):
        """
        Checks the data input object
        """
        # Checks the mandatory parameters:
        self.checkMandatoryParameters( self.getDataInput().getBeamExposureTime(), "beamExposureTime")
        self.checkMandatoryParameters( self.getDataInput().getBeamMaxExposureTime(), "beamMaxExposureTime")
        self.checkMandatoryParameters( self.getDataInput().getDetectorType(), "detectorType")
        
        self.checkImportantParameters( self.getDataInput().getCrystalAbsorbedDoseRate(), "crystalDoseRate")
        self.checkImportantParameters( self.getDataInput().getCrystalShape(), "crystalShape")
        
        
    def generateExecutiveSummary( self, _edPlugin ):
        """
        Generates a summary of the execution of the plugin.
        """
        EDVerbose.DEBUG( "*** EDPluginBestv10.generateExecutiveSummary" )
        if ( self.getStringVersion() is not None ):
            self.addExecutiveSummaryLine( self.getStringVersion() )
            
        xsDataBestv10Output = self.getDataOutput()
        listCollectionPlan = xsDataBestv10Output.getCollectionPlan()
        
        if( len( listCollectionPlan ) > 1 ):
            self.addExecutiveSummaryLine( "" )
            self.addExecutiveSummaryLine( "MULTI-SWEEP COLLECTION PLAN STRATEGY:" )
            self.addExecutiveSummaryLine( "------------------------------------" )
            self.addExecutiveSummaryLine( "" )
            for xsDataCollectionPlan in listCollectionPlan:
                xsDataStrategySummary = xsDataCollectionPlan.getStrategySummary()
                edStringReasoning = EDString( xsDataStrategySummary.getResolutionReasoning().getValue() )
                if( edStringReasoning.existSubString( "Low-resolution" ) == True and listCollectionPlan.index( xsDataCollectionPlan ) != 0 ):
                    listCollectionPlan.remove( xsDataCollectionPlan )
                    listCollectionPlan.insert( 0, xsDataCollectionPlan )
                    
        iSweepNumber = 1        
        for xsDataCollectionPlan in listCollectionPlan:

            if( len( listCollectionPlan ) > 1 ):

                self.addExecutiveSummaryLine( "SWEEP %d:" % iSweepNumber )
                self.addExecutiveSummaryLine( "---------------------------------------------------------------------------------------" )
                iSweepNumber = iSweepNumber + 1
                
            xsDataStrategySummary = xsDataCollectionPlan.getStrategySummary()
            self.addExecutiveSummaryLine( "Resolution reasoning          : %s"    % xsDataStrategySummary.getResolutionReasoning().getValue() )
            self.addExecutiveSummaryLine( "" )
            self.addExecutiveSummaryLine( "Plan for data collection:" )
            self.addExecutiveSummaryLine( "" )
            self.addExecutiveSummaryLine( " N |  Phi_start |  N.of.images | Rot.width |  Exposure | Distance |  Overlap |" )
            iCollectionRunNumber = 1
            iTotalImages = 0
            fTotalRotationRange = 0.0
            for xsDataCollectionRun in xsDataCollectionPlan.getCollectionRun():
                fDistance = xsDataCollectionRun.getDistance().getValue()
                fExposureTime = xsDataCollectionRun.getExposureTime().getValue()
                iNumberOfImages = xsDataCollectionRun.getNumberOfImages().getValue()
                edStringOverlaps = ""
                if ( xsDataCollectionRun.getOverlaps() is not None ):
                    edStringOverlaps = xsDataCollectionRun.getOverlaps().getValue()
                fPhiStart = xsDataCollectionRun.getPhiStart().getValue()
                fPhiWidth = xsDataCollectionRun.getPhiWidth().getValue()
                self.addExecutiveSummaryLine( "%2d | %8.2f   | %7d      |%9.2f  | %9.2f | %7.1f  | %7s  |" % \
                                              ( iCollectionRunNumber, fPhiStart, iNumberOfImages,\
                                                fPhiWidth, fExposureTime, fDistance, edStringOverlaps ) )
                iCollectionRunNumber += 1
                iTotalImages += iNumberOfImages
                fTotalRotationRange += fPhiWidth * iNumberOfImages
                
            
            self.addExecutiveSummaryLine( "" )
            self.addExecutiveSummaryLine( "Attenuation                   : %6.1f" % xsDataStrategySummary.getAttenuation().getValue() )
            self.addExecutiveSummaryLine( "Total rotation range          : %6.1f [degrees]" % fTotalRotationRange )
            self.addExecutiveSummaryLine( "Total number of images        : %6d"   % iTotalImages )
            self.addExecutiveSummaryLine( "Total exposure time           : %6.2f [s]" % xsDataStrategySummary.getTotalExposureTime().getValue() )
            self.addExecutiveSummaryLine( "Total data collection time    : %6.2f [s]" % xsDataStrategySummary.getTotalDataCollectionTime().getValue() )
            self.addExecutiveSummaryLine( "Resolution                    : %6.2f [A]" % xsDataStrategySummary.getResolution().getValue() )
            self.addExecutiveSummaryLine( "Ranking resolution            : %6.2f [A]" % xsDataStrategySummary.getRankingResolution().getValue() )
            self.addExecutiveSummaryLine( "" )
            self.addExecutiveSummaryLine( "Predicted statistics:" )
            self.addExecutiveSummaryLine( "Completeness                  : %6.1f [%%]" % ( xsDataStrategySummary.getCompleteness().getValue() * 100 ))
            self.addExecutiveSummaryLine( "I/sigma at highest resolution : %6.1f" % xsDataStrategySummary.getISigma().getValue() )
            self.addExecutiveSummaryLine( "Multiplicity                  : %6.1f" % xsDataStrategySummary.getRedundancy().getValue() )
    
            self.addExecutiveSummaryLine( "" )
            self.addExecutiveSummaryLine( "Statistics according to the plan:" )
            
            # Retrieve first resolution bin to check if Chi2 is present
            # Could arrive if Radiation damage is not estimated or double sweep strategy
            bChi2Present = False
            xsDataResolutionBin = xsDataCollectionPlan.getStatisticalPrediction().getResolutionBin()[0]
            if( xsDataResolutionBin.getChi2() is not None ):
                bChi2Present = True
            
            if( bChi2Present == True ):
                self.addExecutiveSummaryLine( " Resolution  Compl.       Average     I/Sigma  I/Sigma Chi**2  R-fact  Overload")
                self.addExecutiveSummaryLine( "Lower Upper     %   Intensity  Sigma            /Chi             %        %")
                self.addExecutiveSummaryLine( "-------------------------------------------------------------------------------" )
            else:
                self.addExecutiveSummaryLine( " Resolution  Compl.       Average     I/Sigma   R-fact  Overload")
                self.addExecutiveSummaryLine( "Lower Upper     %   Intensity  Sigma               %        %")
                self.addExecutiveSummaryLine( "-----------------------------------------------------------------")
            
            for xsDataResolutionBin in xsDataCollectionPlan.getStatisticalPrediction().getResolutionBin():
                fMinResolution =  xsDataResolutionBin.getMinResolution().getValue()
                fMaxResolution =  xsDataResolutionBin.getMaxResolution().getValue()
                fCompleteness = xsDataResolutionBin.getCompleteness().getValue()
                fAverageIntensity = xsDataResolutionBin.getAverageIntensity().getValue()
                fAverageSigma = xsDataResolutionBin.getAverageSigma().getValue()
                fIOverSigma = xsDataResolutionBin.getIOverSigma().getValue()
                if( xsDataResolutionBin.getIOverSigmaChi() is not None ):
                    fIOverSigmaChi = xsDataResolutionBin.getIOverSigmaChi().getValue()
                if( xsDataResolutionBin.getChi2() is not None ):
                    fChi2 = xsDataResolutionBin.getChi2().getValue()
                fRFactor = xsDataResolutionBin.getRFactor().getValue()
                fPercentageOverload = xsDataResolutionBin.getPercentageOverload().getValue()
                
                if( bChi2Present == True ):
                    self.addExecutiveSummaryLine( "%5.2f %5.2f %6.1f %7.1f %9.1f %7.1f %7.1f %7.2f %7.1f %7.2f" % \
                                                ( fMinResolution, fMaxResolution, ( fCompleteness * 100 ), fAverageIntensity, fAverageSigma, fIOverSigma, \
                                                  fIOverSigmaChi, fChi2, \
                                                  fRFactor, fPercentageOverload) )
                else:
                    self.addExecutiveSummaryLine( "%5.2f %5.2f %6.1f %7.1f %9.1f %7.1f %9.1f %9.2f" % \
                                                ( fMinResolution, fMaxResolution, ( fCompleteness * 100 ), fAverageIntensity, fAverageSigma, fIOverSigma, \
                                                  fRFactor, fPercentageOverload) )    
        
            self.addExecutiveSummaryLine( "" )
            self.addExecutiveSummaryLine( "" )
            
            
            