#
#    Project: mxPluginExec
#             http://www.edna-site.org
#
#    File: "$Id: EDPluginMOSFLMGeneratePredictionv10.py 675 2009-05-07 08:42:36Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors:      Marie-Francoise Incardona (incardon@esrf.fr)
#                            Olof Svensson (svensson@esrf.fr) 
#
#    Contributing author:    Karl Levik (karl.levik@diamond.ac.uk)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License  along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#

from EDImportLib import EDVerbose
from EDImportLib import EDString
from EDImportLib import EDDiskExplorer

from EDMessage         import EDMessage
from EDUtilsTable      import EDUtilsTable
from EDPluginMOSFLMv10 import EDPluginMOSFLMv10

from XSDataMOSFLMv10 import XSDataMOSFLMInputGeneratePrediction
from XSDataMOSFLMv10 import XSDataMOSFLMOutputGeneratePrediction
from XSDataMOSFLMv10 import XSDataMOSFLMBeamPosition

from XSDataDnaTables import dna_tables

from XSDataCommon import XSDataAngle
from XSDataCommon import XSDataLength
from XSDataCommon import XSDataInteger
from XSDataCommon import XSDataFloat
from XSDataCommon import XSDataString
from XSDataCommon import XSDataImage
from XSDataCommon import XSDataString

class EDPluginMOSFLMGeneratePredictionv10( EDPluginMOSFLMv10 ):


    def __init__( self ):
        EDPluginMOSFLMv10.__init__( self )
        self.setXSDataInputClass( XSDataMOSFLMInputGeneratePrediction )
        self.m_xsdStringPredictionImageFileName = EDString( "EDPluginMOSFLMGeneratePredictionv10.jpg" )


    def preProcess( self, _edObject = None ):
        EDPluginMOSFLMv10.preProcess( self )
        EDVerbose.DEBUG( "*** EDPluginMOSFLMGeneratePredictionv10.preProcess")
        self.generateMOSFLMCommands()


    def postProcess( self, _edObject = None ):
        EDPluginMOSFLMv10.postProcess( self )
        EDVerbose.DEBUG( "*** EDPluginMOSFLMGeneratePredictionv10.postProcess")
        xsDataMOSFLMOutputGeneratePrediction = self.createDataMOSFLMOutputGeneratePrediction()
        self.setDataOutput( xsDataMOSFLMOutputGeneratePrediction )


    def configure( self ):
        EDPluginMOSFLMv10.configure( self )
        EDVerbose.DEBUG( "*** EDPluginMOSFLMGeneratePredictionv10.configure")   
        xsPluginItem = self.getConfiguration()
        if ( xsPluginItem == None ):
            EDVerbose.DEBUG( "*** EDPluginMOSFLMGeneratePredictionv10.configure: xsPluginItem is None")                 
        self.createPredictionImageFileName()


    def checkParameters( self ):
        """
        Checks the mandatory parameters for MOSLFM indexing
        """
        EDPluginMOSFLMv10.checkParameters( self )
        EDVerbose.DEBUG( "*** EDPluginMOSFLMGeneratePredictionv10.checkParameters")
        self.checkMandatoryParameters( self.getDataInput().getImage(), "image")

    
    def getPredictionImageFileName( self ):
        """
        """
        EDVerbose.DEBUG( "*** EDPluginMOSFLMGeneratePredictionv10.getPredictionImageFileName" )
        return EDString( self.m_xsdStringPredictionImageFileName )
        
        
    def setPredictionImageFileName( self, _edStringFileName ):
        """
        """
        EDVerbose.DEBUG( "*** EDPluginMOSFLMGeneratePredictionv10.setPredictionImageFileName : " + _edStringFileName )
        self.m_xsdStringPredictionImageFileName = EDString( _edStringFileName )

    
    def createPredictionImageFileName( self ):
        """
        """
        EDVerbose.DEBUG( "*** EDPluginMOSFLMGeneratePredictionv10.createPredictionImagePath" )
        edStringImageFileName = self.getBaseName() + "_image.jpg" 
        self.setPredictionImageFileName( edStringImageFileName )

        
    def generateMOSFLMCommands( self ):
        """
        This method creates a list of MOSFLM integration commands given a valid
        XSDataMOSFLMInputGeneratePrediction as input to the plugin.
        """
        EDPluginMOSFLMv10.generateMOSFLMCommands( self )
        EDVerbose.DEBUG( "*** EDPluginMOSFLMGeneratePredictionv10.generateMOSFLMCommands")

        xsDataMOSFLMInputGeneratePrediction = self.getDataInput()

        if ( xsDataMOSFLMInputGeneratePrediction is not None):
                                    
            xsDataMOSFLMImage  = xsDataMOSFLMInputGeneratePrediction.getImage()
            iImageNumber       = xsDataMOSFLMImage.getNumber().getValue()
            fRotationAxisStart = xsDataMOSFLMImage.getRotationAxisStart().getValue()
            fRotationAxisEnd   = xsDataMOSFLMImage.getRotationAxisEnd().getValue()
            self.addListCommandExecution( "XGUI ON" )
            self.addListCommandExecution( "IMAGE %d PHI %f TO %f" % ( iImageNumber, fRotationAxisStart, fRotationAxisEnd ) )
            self.addListCommandExecution( "GO" )
            self.addListCommandExecution( "PREDICT_SPOTS" )
            self.addListCommandExecution( "CREATE_IMAGE PREDICTION ON BINARY TRUE FILENAME %s"  % ( self.getPredictionImageFileName() ) )
            self.addListCommandExecution( "RETURN" )
            self.addListCommandExecution( "EXIT" )
            
        EDVerbose.DEBUG( "*** Finished EDPluginMOSFLMGeneratePredictionv10.generateMOSFLMCommands")

            
    def createDataMOSFLMOutputGeneratePrediction( self ):
        EDVerbose.DEBUG( "*** EDPluginMOSFLMGeneratePredictionv10.createDataMOSFLMOutputIntegration")
        xsDataMOSFLMInputGeneratePrediction = self.getDataInput()
        xsDataMOSFLMImage  = xsDataMOSFLMInputGeneratePrediction.getImage()
        iImageNumber = xsDataMOSFLMImage.getNumber().getValue()
        xsDataMOSFLMOutputGeneratePrediction = XSDataMOSFLMOutputGeneratePrediction()
        xsDataImage = XSDataImage()
        xsDataImage.setNumber( XSDataInteger( iImageNumber ) )
        xsDataImage.setPath( XSDataString( EDDiskExplorer.mergePath( self.getWorkingDirectory(), self.getPredictionImageFileName() ) ) )
        xsDataMOSFLMOutputGeneratePrediction.setPredictionImage( xsDataImage )
        return xsDataMOSFLMOutputGeneratePrediction
    