#
#    Project: mxPluginExec
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCasePluginExecuteMOSFLMIndexingv10.py 724 2009-05-20 11:54:26Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors:      Marie-Francoise Incardona (incardon@esrf.fr)
#                            Olof Svensson (svensson@esrf.fr) 
#
#    Contributing author:    Karl Levik (karl.levik@diamond.ac.uk)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License  along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#

from EDImportLib                         import EDCompiler
from EDImportLib                         import EDVerbose
from EDImportLib                         import EDString
from EDImportLib                         import EDDiskExplorer

from EDAssert                            import EDAssert
from EDTestCasePluginExecute             import EDTestCasePluginExecute
from EDUtilsTest                         import EDUtilsTest
from EDUtilsPath                         import EDUtilsPath

from XSDataMOSFLMv10 import XSDataMOSFLMOutputIndexing


class EDTestCasePluginExecuteMOSFLMIndexingv10( EDTestCasePluginExecute ):
    """
    """
    
    def __init__( self, _edStringTestName = None):
        """
        """
        EDTestCasePluginExecute.__init__( self, "EDPluginMOSFLMIndexingv10", "EDPluginGroupMOSFLM-v1.0", _edStringTestName )

        edStringPluginTestDataHome = self.getPluginTestsDataHome()
        self.m_edStringExecutionTestDataInputHome = EDUtilsPath.mergePath( edStringPluginTestDataHome, EDString( "executionTestInput" ) )
        self.m_edStringExecutionTestDataResultHome = EDUtilsPath.mergePath( edStringPluginTestDataHome, EDString( "executionTestResult_v705_20090513" ) )

        self.setConfigurationFile( self.getRefConfigFile() )
        self.setDataInputFile( EDUtilsPath.mergePath( self.m_edStringExecutionTestDataInputHome, "XSDataMOSFLMInputIndexing_reference.xml" ) )
        
        self.setReferenceDataOutputFile( EDUtilsPath.mergePath( self.m_edStringExecutionTestDataResultHome, "XSDataMOSFLMOutputIndexing_reference.xml"))
        self.m_edObtainedOutputDataFile = self.getPluginName() + "_output.xml"
        
        
    def preProcess( self ):
        """
        """
        EDTestCasePluginExecute.preProcess( self )
        self.loadTestImage( [ "ref-testscale_1_001.img", "ref-testscale_1_002.img" ] )
        
        
    def testExecute( self ):
        """
        """ 
        self.run()
        
        # Checks that there are no error messages
        EDAssert.equal( 0, self.getErrorMessages().getNumberObjects() )
        
        # Checks the expected result
        edStringExpectedOutput = EDUtilsTest.readAndParseFile ( self.getReferenceDataOutputFile() )
        edStringObtainedOutput = EDUtilsTest.readAndParseFile ( self.m_edObtainedOutputDataFile )
        xsDataMOSFLMOutputIndexingObtained  = XSDataMOSFLMOutputIndexing.parseString( edStringObtainedOutput )     
        xsDataMOSFLMOutputIndexingReference = XSDataMOSFLMOutputIndexing.parseString( edStringExpectedOutput )     
        EDAssert.equal( xsDataMOSFLMOutputIndexingReference.marshal(), xsDataMOSFLMOutputIndexingObtained.marshal())
        EDDiskExplorer.deleteFile( self.m_edObtainedOutputDataFile )



##############################################################################

    def process( self ):
        """
        """
        self.addTestMethod( self.testExecute )

        
        
##############################################################################


if __name__ == '__main__':

    # JIT compiler accelerator
    EDCompiler.accelerator()
    
    edTestCasePluginExecuteMOSFLMIndexingv10 = EDTestCasePluginExecuteMOSFLMIndexingv10( "EDTestCasePluginExecuteMOSFLMIndexingv10" )
    edTestCasePluginExecuteMOSFLMIndexingv10.execute()