#
#    Project: mxPluginExec
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCasePluginUnitMOSFLMv10.py 675 2009-05-07 08:42:36Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors:      Marie-Francoise Incardona (incardon@esrf.fr)
#                            Olof Svensson (svensson@esrf.fr) 
#
#    Contributing author:    Karl Levik (karl.levik@diamond.ac.uk)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License  along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#


from EDImportLib import EDCompiler
from EDImportLib import EDString
from EDImportLib import EDVerbose
from EDImportLib import EDDiskExplorer
from EDImportLib import EDList

from EDAssert             import EDAssert
from EDTestCasePluginUnit import EDTestCasePluginUnit
from EDUtilsTest          import EDUtilsTest
from EDUtilsPath          import EDUtilsPath

from XSDataCommon import XSDataAngle
from XSDataCommon import XSDataLength
from XSDataCommon import XSDataInteger
from XSDataCommon import XSDataString
from XSDataCommon import XSDataFloat



class EDTestCasePluginUnitMOSFLMv10( EDTestCasePluginUnit ):
    """
    """

    def __init__( self, _edStringTestName = None):
        """
        """
        EDTestCasePluginUnit.__init__( self, "EDPluginMOSFLMv10", "EDPluginGroupMOSFLM-v1.0", _edStringTestName )
        edStringPluginTestDataHome = self.getPluginTestsDataHome()
        self.m_edStringUnitTestDataHome = EDUtilsPath.mergePath( edStringPluginTestDataHome, EDString( "unitTest" ) )
              

    
    def getReferenceDataMOSFLMNewmat( self ):
        from XSDataCommon import XSDataMatrixDouble
        from XSDataMOSFLMv10 import XSDataCell
        from XSDataMOSFLMv10 import XSDataMOSFLMNewmat
        from XSDataMOSFLMv10 import XSDataMOSFLMMissettingsAngles

        xsDataMOSFLMNewmat = XSDataMOSFLMNewmat()

        XSDataMatrixDoubleA = XSDataMatrixDouble()
        XSDataMatrixDoubleA.setM11(  -0.00826416 )
        XSDataMatrixDoubleA.setM12( 0.00885073 )
        XSDataMatrixDoubleA.setM13( 0.00937013 )
        XSDataMatrixDoubleA.setM21( 0.00126554 )
        XSDataMatrixDoubleA.setM22( 0.01251971 )
        XSDataMatrixDoubleA.setM23( -0.00845341 )
        XSDataMatrixDoubleA.setM31( -0.01484956 )
        XSDataMatrixDoubleA.setM32( -0.00385867 )
        XSDataMatrixDoubleA.setM33( -0.00593515 )
        xsDataMOSFLMNewmat.setAMatrix( XSDataMatrixDoubleA )
        
        xsDataMOSFLMMissettingsAngles = XSDataMOSFLMMissettingsAngles()
        xsDataMOSFLMMissettingsAngles.setPhix( XSDataAngle( 1.000 ) )
        xsDataMOSFLMMissettingsAngles.setPhiy( XSDataAngle( 2.000 ) )
        xsDataMOSFLMMissettingsAngles.setPhiz( XSDataAngle( 3.000 ) )
        xsDataMOSFLMNewmat.setMissettingAngles( xsDataMOSFLMMissettingsAngles )

        XSDataMatrixDoubleU = XSDataMatrixDouble()
        XSDataMatrixDoubleU.setM11( -0.4849475 )
        XSDataMatrixDoubleU.setM12( 0.5598049 )
        XSDataMatrixDoubleU.setM13( 0.6718960 )
        XSDataMatrixDoubleU.setM21( 0.0742629 )
        XSDataMatrixDoubleU.setM22( 0.7918670 )
        XSDataMatrixDoubleU.setM23( -0.6061614 )
        XSDataMatrixDoubleU.setM31( -0.8713845 )
        XSDataMatrixDoubleU.setM32( -0.2440595 )
        XSDataMatrixDoubleU.setM33( -0.4255866 )
        xsDataMOSFLMNewmat.setUMatrix( XSDataMatrixDoubleU )
        
        xsDataCellRefined = XSDataCell()
        xsDataCellRefined.setLength_a(    XSDataLength( 54.8079 ) )
        xsDataCellRefined.setLength_b(    XSDataLength(59.0751 ) )
        xsDataCellRefined.setLength_c(    XSDataLength(66.9736 ) )
        xsDataCellRefined.setAngle_alpha( XSDataAngle( 91.0000 ) )
        xsDataCellRefined.setAngle_beta(  XSDataAngle( 92.0000 ) )
        xsDataCellRefined.setAngle_gamma( XSDataAngle( 93.0000 ) )
        xsDataMOSFLMNewmat.setRefinedCell( xsDataCellRefined )

        return xsDataMOSFLMNewmat
    
        
    def testGetDataMOSFLMNewmat( self ):
        edPluginMOSFLMv10 = self.createPlugin()
        edPluginMOSFLMv10.setScriptExecutable("cat")
        edPluginMOSFLMv10.configure()
        oesStringFilename = EDDiskExplorer.mergePath( self.m_edStringUnitTestDataHome,  "EDPluginMOSFLMv10_autoindexMat_test.txt" )
        xsDataMOSFLMNewmat = edPluginMOSFLMv10.getDataMOSFLMNewmat( oesStringFilename )
        xsDataMOSFLMNewmatReference = self.getReferenceDataMOSFLMNewmat()
        xsDataMOSFLMNewmatReference.outputFile( "test_reference.xml" )
        xsDataMOSFLMNewmat.outputFile( "test1.xml" )
        EDAssert.equal( xsDataMOSFLMNewmatReference.marshal(), xsDataMOSFLMNewmat.marshal() )
        self.cleanUp( edPluginMOSFLMv10 )


    def testWriteDataMOSFLMNewmat( self ):
        edPluginMOSFLMv10 = self.createPlugin()
        edPluginMOSFLMv10.setScriptExecutable("cat")
        edPluginMOSFLMv10.configure()
        xsDataMOSFLMNewmatReference = self.getReferenceDataMOSFLMNewmat()
        edStringNewmatFilename = edPluginMOSFLMv10.getScriptBaseName() + "_testWriteDataMOSFLMNewmat.mat"
        edPluginMOSFLMv10.writeDataMOSFLMNewmat( xsDataMOSFLMNewmatReference, edStringNewmatFilename )
        self.cleanUp( edPluginMOSFLMv10 )


    def testGenerateMOSFLMCommands( self ):
        from XSDataMOSFLMv10 import XSDataMOSFLMInput
        from XSDataMOSFLMv10 import XSDataMOSFLMBeamPosition
        from XSDataMOSFLMv10 import XSDataMOSFLMDetector        
        xsDataMOSFLMInput = XSDataMOSFLMInput()
        xsDataMOSFLMBeam = XSDataMOSFLMBeamPosition()
        xsDataMOSFLMBeam.setX( XSDataLength( 1.0 ) )
        xsDataMOSFLMBeam.setY( XSDataLength( 2.0 ) )
        xsDataMOSFLMInput.setBeam( xsDataMOSFLMBeam )
        xsDataMOSFLMDetector = XSDataMOSFLMDetector()
        xsDataMOSFLMDetector.setType( XSDataString( "ADSC" ) )
        xsDataMOSFLMInput.setDetector( xsDataMOSFLMDetector )
        xsDataMOSFLMInput.setDirectory( XSDataString( "/tmp" ) )
        xsDataMOSFLMInput.setTemplate( XSDataString( "testdata_1_###.img" ) )
        xsDataMOSFLMInput.setWavelength( XSDataFloat( 1.1111 ) )
        xsDataMOSFLMInput.setDistance( XSDataLength( 222.22 ) )
        edPluginMOSFLMv10 = self.createPlugin()
        edPluginMOSFLMv10.setScriptExecutable("cat")        
        edPluginMOSFLMv10.configure()
        edPluginMOSFLMv10.setXSDataInputClass( XSDataMOSFLMInput )
        edPluginMOSFLMv10.setDataInput( xsDataMOSFLMInput )
        edPluginMOSFLMv10.generateMOSFLMCommands()
        edListCommands = edPluginMOSFLMv10.getListCommandExecution()
        edListCommandsReference = ['WAVELENGTH 1.1111', 'DISTANCE 222.22', 'BEAM 1.0 2.0', 'DETECTOR ADSC', 'DIRECTORY /tmp', 'TEMPLATE testdata_1_###.img']
        EDAssert.equal( edListCommandsReference, edListCommands )
        self.cleanUp( edPluginMOSFLMv10 )

    
    
    def process( self ):
        self.addTestMethod( self.testGetDataMOSFLMNewmat )
        self.addTestMethod( self.testWriteDataMOSFLMNewmat )
        self.addTestMethod( self.testGenerateMOSFLMCommands )

    

##############################################################################


if __name__ == '__main__':

    # JIT compiler accelerator
    EDCompiler.accelerator()

    edTestCasePluginUnitMOSFLMv10 = EDTestCasePluginUnitMOSFLMv10( "EDTestCasePluginUnitMOSFLMv10" )
    edTestCasePluginUnitMOSFLMv10.execute()
