#
#    Project: mxPluginExec
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCasePluginUnitLabelitv10.py 675 2009-05-07 08:42:36Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors:      Marie-Francoise Incardona (incardon@esrf.fr)
#                            Olof Svensson (svensson@esrf.fr) 
#
#    Contributing author:    Karl Levik (karl.levik@diamond.ac.uk)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License  along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#


from EDImportLib                         import EDCompiler
from EDImportLib                         import EDVerbose
from EDImportLib                         import EDString
from EDImportLib                         import EDDiskExplorer
from EDImportLib                         import EDDict
from EDImportLib                         import EDDictionary
from EDImportLib                         import EDList

from EDUtilsFile                         import EDUtilsFile
from EDAssert                            import EDAssert
from EDTestCasePluginUnit                import EDTestCasePluginUnit
from EDUtilsTest                         import EDUtilsTest

from XSDataLabelitv10 import XSDataInputLabelit 

class EDTestCasePluginUnitLabelitv10( EDTestCasePluginUnit ):
    """
    """
    
    def __init__( self, _edStringTestName = None):
        """
        """
        EDTestCasePluginUnit.__init__( self, "EDPluginLabelitv10", "EDPluginLabelit-v1.0", _edStringTestName )
        
        self.m_edStringReferenceInputFile = EDDiskExplorer.mergePath( self.getPluginTestsDataHome(), "XSDataInputLabelit_reference.xml")
        self.m_edStringReferenceOutputFile = EDDiskExplorer.mergePath( self.getPluginTestsDataHome(), "XSDataResultLabelit_reference.xml")
        
        
        
            
    def testSetDataInput( self ):
        """
        This method test the setDataInput method of the Labelit plugin by providing an XML string
        and then retriving an XSDataInputLabelit object.
        """
        edPluginLabelitv10 = self.createPlugin()
        xsPluginItemGood01 = self.getPluginConfiguration( EDDiskExplorer.mergePath( self.getPluginTestsDataHome(), "XSConfiguration.xml" ))
        edPluginLabelitv10.setConfiguration( xsPluginItemGood01 )
        edPluginLabelitv10.setScriptExecutable( "cat" )
        edPluginLabelitv10.configure()
        xmlInput = EDUtilsTest.readAndParseFile( self.m_edStringReferenceInputFile )
        edPluginLabelitv10.setDataInput( xmlInput )
        xsDataInputLabelitReference = XSDataInputLabelit.parseString( xmlInput )
        xsDataIntegrationv10Input = edPluginLabelitv10.getDataInput()
        EDAssert.equal( xsDataInputLabelitReference.marshal(), xsDataIntegrationv10Input.marshal() )
        self.cleanUp( edPluginLabelitv10 )
        
        
        
    def testInitaliseLabelitCommandLine( self ):
        """
        This method tests the initaliseLabelitCommandLine method of the Labelit plugin.
        """
        edPluginLabelitv10 = self.createPlugin()

        edStringReferenceInputFile = EDDiskExplorer.mergePath( self.getPluginTestsDataHome(), "XSDataInputLabelit_testCommandLine.xml")
        xmlInput = EDUtilsTest.readAndParseFile( edStringReferenceInputFile )
        edPluginLabelitv10.setDataInput( xmlInput )
        edPluginLabelitv10.initaliseLabelitCommandLine()
        edStringScriptCommandLine = edPluginLabelitv10.getScriptCommandline()
        edStringCommandLineExpected = EDString( "--index_only images/ref-testscale_1_001.img images/ref-testscale_1_002.img" )
        EDAssert.equal( edStringCommandLineExpected, edStringScriptCommandLine )
        
        
    def testParseLabelitScreenOutput( self ):
        """
        This method test the parsing of the Labelit results in the log file.
        """
        edPluginLabelitv10 = self.createPlugin()
        edStringPathToLabelitLogText = EDDiskExplorer.mergePath( self.getPluginTestsDataHome(), "labelit_v1000rc11.log")
        edStringLabelitLogText = EDUtilsTest.readAndParseFile( edStringPathToLabelitLogText )
        xsDataLabelitScreenOutput = edPluginLabelitv10.parseLabelitScreenOutput( edStringLabelitLogText )
        edStringLabelitScreenOutputFile = EDDiskExplorer.mergePath( self.getPluginTestsDataHome(), "XSDataLabelitScreenOutput_reference.xml")
        edStringLabelitScreenOutputXML = EDUtilsTest.readAndParseFile( edStringLabelitScreenOutputFile )
        from XSDataLabelitv10 import XSDataLabelitScreenOutput
        xsDataLabelitScreenOutputReference = XSDataLabelitScreenOutput.parseString( edStringLabelitScreenOutputXML )
        EDAssert.equal( xsDataLabelitScreenOutputReference.marshal(), xsDataLabelitScreenOutput.marshal() )
        

    def testParseMosflmScriptsOutput( self ):
        """
        This method tests the parsing of the MOSFLM script generated by Labelit.
        """
        edPluginLabelitv10 = self.createPlugin()
        edStringPathToMOSFLMScript = EDDiskExplorer.mergePath( self.getPluginTestsDataHome(), "integration05_csh_v1000rc11.txt")
        edStringMOSFLMScript = EDUtilsTest.readAndParseFile( edStringPathToMOSFLMScript )
        xsDataLabelitMosflmScriptsOutput = edPluginLabelitv10.parseMosflmScriptsOutput( edStringMOSFLMScript )
        xsDataLabelitMosflmScriptsOutput.outputFile( "XSDataLabelitMosflmScriptsOutput_reference.xml" )
        edStringLabelitMosflmScriptsOutputFile = EDDiskExplorer.mergePath( self.getPluginTestsDataHome(), "XSDataLabelitMosflmScriptsOutput_reference.xml")
        edStringLabelitMosflmScriptsOutputXML = EDUtilsTest.readAndParseFile( edStringLabelitMosflmScriptsOutputFile )
        from XSDataLabelitv10 import XSDataLabelitMosflmScriptsOutput
        xsDataLabelitMosflmScriptsOutputReference = XSDataLabelitMosflmScriptsOutput.parseString( edStringLabelitMosflmScriptsOutputXML )
        EDAssert.equal( xsDataLabelitMosflmScriptsOutputReference.marshal(), xsDataLabelitMosflmScriptsOutput.marshal() )
    
    
    def testGenerateMosflmScriptName( self ):
        """
        This method tests the generateMosflmScriptName method of the Labelit plugin.
        """
        edPluginLabelitv10 = self.createPlugin()
        EDAssert.equal( "integration01.csh", edPluginLabelitv10.generateMosflmScriptName(  1 ) )
        EDAssert.equal( "integration10.csh", edPluginLabelitv10.generateMosflmScriptName( 10 ) )
    
    
    def testGenerateExecutiveSummary( self ):
        """
        This method tests the generateExecutiveSummary of the Labelit plugin.
        It contains no assert call so the contents of the executive summary is not tested.
        """
        edPluginLabelitv10 = self.createPlugin()
        xmlInput = EDUtilsTest.readAndParseFile( self.m_edStringReferenceInputFile )
        edPluginLabelitv10.setDataInput( xmlInput )
        edStringPath = EDDiskExplorer.mergePath( self.getPluginTestsDataHome(),  self.m_edStringReferenceOutputFile )
        edStringXMLResultLabelit = EDUtilsTest.readAndParseFile( edStringPath )
        from XSDataLabelitv10 import XSDataResultLabelit
        xsDataResultLabelit = XSDataResultLabelit.parseString( edStringXMLResultLabelit  )
        edPluginLabelitv10.setDataOutput( xsDataResultLabelit )
        edPluginLabelitv10.generateExecutiveSummary( edPluginLabelitv10 )



##############################################################################
 
    def process( self ):
        """
        """
        self.addTestMethod( self.testSetDataInput )
        self.addTestMethod( self.testInitaliseLabelitCommandLine )
        self.addTestMethod( self.testParseLabelitScreenOutput )
        self.addTestMethod( self.testParseMosflmScriptsOutput )
        self.addTestMethod( self.testGenerateMosflmScriptName )
        self.addTestMethod( self.testGenerateExecutiveSummary )
        
        
##############################################################################


if __name__ == '__main__':

    # JIT compiler accelerator
    EDCompiler.accelerator()
    
    EDTestCasePluginUnitLabelitv10 = EDTestCasePluginUnitLabelitv10( "EDTestCasePluginUnitLabelitv10" )
    EDTestCasePluginUnitLabelitv10.execute()

