#
#    Project: mxPluginExec
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCasePluginExecuteRaddosev10.py 753 2009-06-04 09:54:13Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors:      Marie-Francoise Incardona (incardon@esrf.fr)
#                            Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License  along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#



from EDImportLib                      import EDCompiler
from EDImportLib                      import EDVerbose
from EDImportLib                      import EDString
from EDImportLib                      import EDDiskExplorer

from EDUtilsPath                      import EDUtilsPath
from EDAssert                         import EDAssert
from EDUtilsTest                      import EDUtilsTest
from EDTestCasePluginExecute          import EDTestCasePluginExecute



class EDTestCasePluginExecuteRaddosev10( EDTestCasePluginExecute ):
    """
    """
    
    def __init__( self, _oalStringTestName = None):
        """
        """
        EDTestCasePluginExecute.__init__( self, "EDPluginRaddosev10", "EDPluginRaddose-v1.0", _oalStringTestName )
        self.setConfigurationFile( self.getRefConfigFile() )
        self.setDataInputFile( EDDiskExplorer.mergePath( self.getPluginTestsDataHome(), "XSDataRaddosev10Input_reference.xml" ) )
        # Default result file: No name Raddose version
        self.setReferenceDataOutputFile( EDDiskExplorer.mergePath( self.getPluginTestsDataHome(), "XSDataRaddosev10Output_reference.xml"))
        self.m_oedObtainedOutputDataFile = self.getPluginName() + "_output.xml"
        
        
    def preProcess( self ):
        """
        """
        EDTestCasePluginExecute.preProcess( self )
        
        
        
    def testExecute( self ):
        """
        """ 
        
        self.run()
        
        # Checks that there are no error messages

        EDVerbose.DEBUG("Checking error messages...")
        EDAssert.equal( 0, self.getErrorMessages().getNumberObjects() )
            
        EDVerbose.DEBUG("Checking warning messages...")
        EDAssert.equal( 0, self.getWarningMessages().getNumberObjects() )
        
        edStringRaddoseVersion = self.getPlugin().getStringVersion()
        if( edStringRaddoseVersion.existSubString( "version 2" )) :
            self.setReferenceDataOutputFile( EDDiskExplorer.mergePath( self.getPluginTestsDataHome(), "XSDataRaddosev10Output_reference_Raddosev2.xml"))
        
        # Checks the expected result
        strExpectedOutput = EDUtilsTest.readAndParseFile ( self.getReferenceDataOutputFile() )
        strObtainedOutput = EDUtilsTest.readAndParseFile ( self.m_oedObtainedOutputDataFile )    
        EDVerbose.DEBUG("Checking obtained result...") 
                
        from XSDataRaddosev10 import XSDataRaddoseOutput
        xsDataOutputExpected = XSDataRaddoseOutput.parseString( strExpectedOutput ) 
        xsDataOutputObtained = XSDataRaddoseOutput.parseString( strObtainedOutput )
        
        EDAssert.equal( xsDataOutputExpected.marshal(), xsDataOutputObtained.marshal() )
        #EDAssert.equal( strExpectedOutput, strObtainedOutput)



##############################################################################

    def process( self ):
        """
        """
        self.addTestMethod( self.testExecute )

        
        
##############################################################################


if __name__ == '__main__':

    # JIT compiler accelerator
    EDCompiler.accelerator()
    
    edTestCasePluginExecuteRaddosev10 = EDTestCasePluginExecuteRaddosev10( "EDTestCasePluginExecuteRaddosev10" )
    edTestCasePluginExecuteRaddosev10.execute()