#
#    Project: EDNA MXv1
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCasePluginExecuteControlCharacterisationv10.py 698 2009-05-11 10:43:08Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors:      Marie-Francoise Incardona (incardon@esrf.fr)
#                            Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImportLib                         import EDCompiler
from EDImportLib                         import EDVerbose
from EDImportLib                         import EDString
from EDImportLib                         import EDDiskExplorer

from EDAssert                            import EDAssert
from EDTestCasePluginExecute             import EDTestCasePluginExecute
from EDUtilsTest                         import EDUtilsTest
from EDUtilsPath                         import EDUtilsPath




class EDTestCasePluginExecuteControlCharacterisationv10( EDTestCasePluginExecute ):
    """
    """
    
    def __init__( self, _edStringTestName = None):
        """
        """
        EDTestCasePluginExecute.__init__( self, "EDPluginControlCharacterisationv10", "EDPluginControlCharacterisation-v1.0", _edStringTestName )

        self.setConfigurationFile( self.getRefConfigFile() )
        self.setDataInputFile( EDDiskExplorer.mergePath( self.getPluginTestsDataHome(), "XSDataCollection_reference.xml" ) )
        
        self.setReferenceDataOutputFile( EDDiskExplorer.mergePath( self.getPluginTestsDataHome(), "XSDataCharacterisationOutput_reference.xml"))
        self.m_oedObtainedOutputDataFile = self.getPluginName() + "_output.xml"
        
        
    def preProcess( self ):
        """
        """
        EDTestCasePluginExecute.preProcess( self )
        self.loadTestImage( [ "ref-testscale_1_001.img", "ref-testscale_1_002.img" ] )
         
        
    def testExecute( self ):
        """
        """ 
        self.run()
        
        # Checks that there are no error messages
        
        plugin = self.getPlugin()
        plugin.verboseScreenExecutiveSummary()

        EDVerbose.DEBUG("Checking error messages...")
        EDAssert.equal( 0, self.getErrorMessages().getNumberObjects() )
            
        EDVerbose.DEBUG("Checking warning messages...")
        EDAssert.equal( 0, self.getWarningMessages().getNumberObjects() )
        
        # Checks the expected result
#        strExpectedOutput = EDUtilsTest.readAndParseFile ( self.getReferenceDataOutputFile() )
#        strObtainedOutput = EDUtilsTest.readAndParseFile ( self.m_oedObtainedOutputDataFile )    
#        EDVerbose.DEBUG("Checking obtained result...") 
#        EDAssert.equal( strExpectedOutput, strObtainedOutput)
        EDDiskExplorer.deleteFile( self.m_oedObtainedOutputDataFile )



##############################################################################

    def process( self ):
        """
        """
        self.addTestMethod( self.testExecute )

        
        
##############################################################################


if __name__ == '__main__':

    # JIT compiler accelerator
    EDCompiler.accelerator()
    
    edTestCasePluginExecuteControlCharacterisationv10 = EDTestCasePluginExecuteControlCharacterisationv10( "EDTestCasePluginExecuteControlCharacterisationv10" )
    edTestCasePluginExecuteControlCharacterisationv10.execute()