#
#    Project: EDNA MXv1
#             http://www.edna-site.org
#
#    File: "$Id: EDPluginControlIndexingWrapperMOSFLMv10.py 701 2009-05-13 09:14:46Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors:      Marie-Francoise Incardona (incardon@esrf.fr)
#                            Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImportLib import EDVerbose
from EDImportLib import EDString
from EDImportLib import EDList
from EDImportLib import EDDiskExplorer

from EDUtilsImage    import EDUtilsImage
from EDPluginControl import EDPluginControl
from EDMessage       import EDMessage
from EDUtilsFile     import EDUtilsFile
from EDConfiguration import EDConfiguration

from XSDataCommon import XSPluginItem
from XSDataCommon import XSDataMatrix
from XSDataCommon import XSDataLength
from XSDataCommon import XSDataAngle
from XSDataCommon import XSDataInteger
from XSDataCommon import XSDataFloat
from XSDataCommon import XSDataString
from XSDataCommon import XSDataWavelength
from XSDataCommon import XSDataImage

from XSDataMXv1 import XSDataCrystal
from XSDataMXv1 import XSDataSpaceGroup
from XSDataMXv1 import XSDataDetector
from XSDataMXv1 import XSDataIndexingResult
from XSDataMXv1 import XSDataIndexingSolution
from XSDataMXv1 import XSDataIndexingSolutionSelected
from XSDataMXv1 import XSDataOrientation
from XSDataMXv1 import XSDataStatisticsIndexing
from XSDataMXv1 import XSDataExperimentalCondition
from XSDataMXv1 import XSDataCollection
from XSDataMXv1 import XSDataSubWedge
from XSDataMXv1 import XSDataIndexingInput
from XSDataMXv1 import XSDataIndexingSolutionSelected

from EDPluginControlIndexingv10 import EDPluginControlIndexingv10


class EDPluginControlIndexingWrapperMOSFLMv10( EDPluginControlIndexingv10 ):
    """
    The plugin which controls the indexing execution plugins.
    This plugin accepts several sub-wedges with one or several images each.
    The indexing is performed on all images together.
    One image with prediction is generated for each input image.
    """
    
    def __init__ ( self ):
        """
        """
        EDPluginControlIndexingv10.__init__( self )
               
        self.m_edStringPluginIndexingName = "EDPluginMOSFLMIndexingv10"
        self.m_edPluginIndexing = None
        self.m_edStringPluginGeneratePredictionName = "EDPluginControlGeneratePredictionv10"
        self.m_edPluginGeneratePrediction = None
        self.m_xsDataIndexingOutput = None
        self.m_xsDataExperimentalCondition = None
        self.m_xsDataIndexingResult = None      
        self.m_xsDataCollection = None
        self.m_xsDataMOSFLMIndexingInput = None
        self.m_bGeneratePredictionImage = True
        self.m_edStringPluginIndexingExecutiveSummaryName = "MOSFLM"


    def setDataInput( self, _dataInput ):
        """
        Sets the Plugin Input Data
        _dataInput could be either an String XML or an XSData
        The XML String should be parsed as an Output Object by the final Plugin
        """
        self.verboseDebug( "*** EDPluginControlIndexingWrapperMOSFLMv10.setDataInput" )
        EDPluginControlIndexingv10.setDataInput( self, _dataInput )
        # Convert the input data to MOSFLM specific input data
        from EDHandlerXSDataMOSFLMv10 import EDHandlerXSDataMOSFLMv10
        try:
            self.m_xsDataMOSFLMIndexingInput =  EDHandlerXSDataMOSFLMv10.generateXSDataMOSFLMInputIndexing( self.getDataInput() )
        except Exception, error:
            raise
            # This exception handling needs to be rethought, see bug #43.
            errorMessage = EDMessage.ERROR_DATA_HANDLER_02 % ( "EDPluginControlIndexingv10.preProcess: Unexpected error in MOSFLM handler: ", error )
            EDVerbose.error( errorMessage )
            self.addErrorMessage( errorMessage )
            raise RuntimeError, errorMessage




    def loadPluginIndexingInputData( self ):
        self.verboseDebug( "*** EDPluginControlIndexingWrapperMOSFLMv10.loadPluginIndexingInputData..." )
        self.m_edPluginIndexing.setDataInput( self.m_xsDataMOSFLMIndexingInput  )
        



    def getDataIndexingResult( self, _edPlugin ):
        """
        This method retrieves the indexing results from a MOSFLM indexing plugin.
        """
        self.verboseDebug( "*** EDPluginControlIndexingv10.getDataIndexingResultFromMOSFLM" )
        xsDataIndexingResult = None
        xsDataMOSFLMOutputIndexing = _edPlugin.getDataOutput()
        from EDHandlerXSDataMOSFLMv10 import EDHandlerXSDataMOSFLMv10
        try:
            xsDataIndexingResult = EDHandlerXSDataMOSFLMv10.generateXSDataIndexingResult( xsDataMOSFLMOutputIndexing, self.m_xsDataExperimentalCondition )
        except Exception, error:
            # This exception handling needs to be rethought, see bug #43.
            errorMessage = EDMessage.ERROR_DATA_HANDLER_02 % ( "EDPluginControlIndexingv10.preProcess: Unexpected error in MOSFLM handler: ", error )
            EDVerbose.error( errorMessage )
            self.addErrorMessage( errorMessage )
            raise RuntimeError, errorMessage
        return xsDataIndexingResult
    
    

#    def generateExecutiveSummary( self, _edPlugin ):
#        """
#        Generates a summary of the execution of the plugin.
#        """
#        self.verboseDebug( "*** EDPluginControlIndexingWrapperMOSFLMv10.generateExecutiveSummary" )
#        #EDPluginControlIndexingv10.generateExecutiveSummary( self, _edPlugin )
#        self.addExecutiveSummaryLine( "Summary of indexing with MOSFLM:" )
#        self.addErrorWarningMessagesToExecutiveSummary( "Indexing failure! Error messages: " )
#        if ( self.m_edPluginIndexing is not None ):
#            self.appendExecutiveSummary( self.m_edPluginIndexing, "MOSFLM : " )
        