#
#    Project: EDNA MXv1
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCasePluginExecuteControlIndexingv10.py 724 2009-05-20 11:54:26Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors:      Marie-Francoise Incardona (incardon@esrf.fr)
#                            Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImportLib                         import EDCompiler
from EDImportLib                         import EDVerbose
from EDImportLib                         import EDString
from EDImportLib                         import EDDiskExplorer

from EDAssert                            import EDAssert
from EDTestCasePluginExecute             import EDTestCasePluginExecute
from EDUtilsTest                         import EDUtilsTest
from EDUtilsPath                         import EDUtilsPath




class EDTestCasePluginExecuteControlIndexingv10( EDTestCasePluginExecute ):
    """
    """
    
    def __init__( self, _edStringTestName = None):
        """
        """
        EDTestCasePluginExecute.__init__( self, "EDPluginControlIndexingv10", "EDPluginControlIndexing-v1.0", _edStringTestName )

        self.setConfigurationFile( self.getRefConfigFile() )
        self.setDataInputFile( EDDiskExplorer.mergePath( self.getPluginTestsDataHome(), "XSDataIndexingInput_reference.xml" ) )
        
        self.setReferenceDataOutputFile( EDDiskExplorer.mergePath( self.getPluginTestsDataHome(), "XSDataIndexingResult_reference.xml"))
        self.m_edObtainedOutputDataFile = self.getPluginName() + "_output.xml"
        
        
        
    def preProcess( self ):
        """
        """
        EDTestCasePluginExecute.preProcess( self )
        self.loadTestImage( [ "ref-testscale_1_001.img", "ref-testscale_1_002.img" ] )        
        
        
    def testExecute( self ):
        """
        """ 
        self.run()
        
        # Checks that there are no error messages
        
        plugin = self.getPlugin()

        EDVerbose.DEBUG("Checking error messages...")
        EDAssert.equal( 0, self.getErrorMessages().getNumberObjects() )
            
        EDVerbose.DEBUG("Checking warning messages...")
        EDAssert.equal( 0, self.getWarningMessages().getNumberObjects() )
        EDVerbose.DEBUG("Checking obtained result...")         
        # Checks the expected result
        edStringExpectedOutput = EDUtilsTest.readAndParseFile ( self.getReferenceDataOutputFile() )
        edStringObtainedOutput = EDUtilsTest.readAndParseFile ( self.m_edObtainedOutputDataFile )  
        # Check that all the paths in the prediction results actaully exists
        from XSDataMXv1 import XSDataIndexingResult
        xsDataIndexingResultObtained = XSDataIndexingResult.parseString( edStringObtainedOutput )
        xsDataIndexingResultExpected = XSDataIndexingResult.parseString( edStringExpectedOutput )
        xsDataGeneratePredictionResult = xsDataIndexingResultObtained.getPredictionResult()
        for predictionImage in xsDataGeneratePredictionResult.getPredictionImage():
            edStringImagePath = EDString( predictionImage.getPath().getValue() )
            bImageExists = EDDiskExplorer.isFile( edStringImagePath )
            EDAssert.equal( True, bImageExists)
        # Check that we found the right space group
        xsDataIndexingSolutionSelectedObtained = xsDataIndexingResultObtained.getSelectedSolution()      
        xsDataIndexingSolutionSelectedExpected = xsDataIndexingResultExpected.getSelectedSolution()
        
        xsDataCrystalObtained = xsDataIndexingSolutionSelectedObtained.getCrystal()
        xsDataCrystalExpected = xsDataIndexingSolutionSelectedExpected.getCrystal()
        
        xsDataSpaceGroupObtained = xsDataCrystalObtained.getSpaceGroup()
        xsDataSpaceGroupExpected = xsDataCrystalExpected.getSpaceGroup()
        
        edStringSpacegroupNameObtained = xsDataSpaceGroupObtained.getName().getValue()
        edStringSpacegroupNameExpected = xsDataSpaceGroupExpected.getName().getValue()
        
        EDAssert.equal( edStringSpacegroupNameExpected, edStringSpacegroupNameObtained )
        
        ## Remove the prediction results since it contains absolute paths  
        #xsDataIndexingResultObtained.setPredictionResult( None )
        #EDAssert.equal( xsDataIndexingResultExpected.marshal(), xsDataIndexingResultObtained.marshal() )
        EDDiskExplorer.deleteFile( self.m_edObtainedOutputDataFile )



##############################################################################

    def process( self ):
        """
        """
        self.addTestMethod( self.testExecute )

        
        
##############################################################################


if __name__ == '__main__':

    # JIT compiler accelerator
    EDCompiler.accelerator()
    
    edTestCasePluginExecuteControlIndexingv10 = EDTestCasePluginExecuteControlIndexingv10( "EDTestCasePluginExecuteControlIndexingv10" )
    edTestCasePluginExecuteControlIndexingv10.execute()