#
#    Project: EDNA MXv1
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCasePluginExecuteControlIntegrationv10.py 724 2009-05-20 11:54:26Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors:      Marie-Francoise Incardona (incardon@esrf.fr)
#                            Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImportLib                         import EDCompiler
from EDImportLib                         import EDVerbose
from EDImportLib                         import EDString
from EDImportLib                         import EDDiskExplorer

from EDAssert                            import EDAssert
from EDTestCasePluginExecute             import EDTestCasePluginExecute
from EDUtilsTest                         import EDUtilsTest
from EDUtilsPath                         import EDUtilsPath




class EDTestCasePluginExecuteControlIntegrationv10( EDTestCasePluginExecute ):
    """
    """
    
    def __init__( self, _edStringTestName = None):
        """
        """
        EDTestCasePluginExecute.__init__( self, "EDPluginControlIntegrationv10", "EDPluginControlIntegration-v0.1", _edStringTestName )
        
        self.setConfigurationFile( self.getRefConfigFile() )
        self.setDataInputFile( EDDiskExplorer.mergePath( self.getPluginTestsDataHome(), "XSDataIntegrationInput_reference.xml" ) )
        
        self.setReferenceDataOutputFile( EDDiskExplorer.mergePath( self.getPluginTestsDataHome(), "XSDataIntegrationResult_reference.xml"))
        self.m_edObtainedOutputDataFile = self.getPluginName() + "_output.xml"
                

    def preProcess( self ):
        """
        """
        EDTestCasePluginExecute.preProcess( self )
        self.loadTestImage( [ "ref-testscale_1_001.img", "ref-testscale_1_002.img" ] )        

        
    def testExecute( self ):
        """
        """ 
        self.run()
        
        # Checks that there are no error messages
        
        plugin = self.getPlugin()

        EDVerbose.DEBUG("Checking error messages...")
        EDAssert.equal( 0, self.getErrorMessages().getNumberObjects() )
            
        EDVerbose.DEBUG("Checking warning messages...")
        EDAssert.equal( 0, self.getWarningMessages().getNumberObjects() )
        
        # Checks some parts of the result
        edStringExpectedOutput = EDUtilsTest.readAndParseFile ( self.getReferenceDataOutputFile() )
        edStringObtainedOutput = EDUtilsTest.readAndParseFile ( self.m_edObtainedOutputDataFile ) 
        from XSDataMXv1 import XSDataIntegrationResult
        xsDataIntegrationResultReference = XSDataIntegrationResult.parseString( edStringExpectedOutput )
        xsDataIntegrationResultObtained  = XSDataIntegrationResult.parseString( edStringObtainedOutput )

        # Check that we have at least the best files in each integration subwedge
        for xsDataIntegrationSubWedgeResult in xsDataIntegrationResultObtained.getIntegrationSubWedgeResult():
            if ( ( xsDataIntegrationSubWedgeResult.getBestfileDat() is None ) or \
                 ( xsDataIntegrationSubWedgeResult.getBestfilePar() is None ) or \
                 ( xsDataIntegrationSubWedgeResult.getBestfileHKL() is None ) ):
                EDVerbose.warning( "Missing BEST output data in the integration subwedge results!" )
                EDAssert.equal( True, False )
                
        EDDiskExplorer.deleteFile( self.m_edObtainedOutputDataFile )



##############################################################################

    def process( self ):
        """
        """
        self.addTestMethod( self.testExecute )

        
        
##############################################################################


if __name__ == '__main__':

    # JIT compiler accelerator
    EDCompiler.accelerator()
    
    edTestCasePluginExecuteControlIntegrationv10 = EDTestCasePluginExecuteControlIntegrationv10( "EDTestCasePluginExecuteControlIntegrationv10" )
    edTestCasePluginExecuteControlIntegrationv10.execute()