#
#    Project: EDNA MXv1
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCasePluginUnitISPyBv10.py 624 2009-02-27 10:59:00Z svensson $"
#
#    Copyright (C) 2008 Diamond Light Source
#                       Chilton, Didcot, UK
#
#    Principal author:      Karl Levik (karl.levik@diamond.ac.uk)
#
#    Contributing authors:  Marie-Francoise Incardona (incardon@esrf.fr)
#                           Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from EDImportLib                    import EDCompiler
from EDImportLib                    import EDVerbose
from EDImportLib                    import EDString
from EDAssert                       import EDAssert
from EDImportLib                    import EDDiskExplorer
from EDImportLib                    import EDDict
from EDImportLib                    import EDDictionary
from EDUtilsFile                    import EDUtilsFile
from EDImportLib                    import EDList
from EDTestCasePluginUnit           import EDTestCasePluginUnit
from EDUtilsTest                    import EDUtilsTest

from XSDataCommon                   import XSDataInteger
from XSDataCommon                   import XSDataFloat
from XSDataCommon                   import XSDataString
from XSDataCommon                   import XSDataBoolean
from XSDataCommon                   import XSData
from XSDataISPyBv10                 import XSDataInputISPyB 
from XSDataISPyBv10                 import XSDataScreening 
from XSDataISPyBv10                 import XSDataScreeningInput 
from XSDataISPyBv10                 import XSDataScreeningOutput 
from XSDataISPyBv10                 import XSDataScreeningOutputLattice 
from XSDataISPyBv10                 import XSDataScreeningRank
from XSDataISPyBv10                 import XSDataScreeningRankSet
from XSDataISPyBv10                 import XSDataScreeningStrategy
from XSDataISPyBv10                 import XSDataISPyBv10
from XSDataISPyBv10                 import XSDataResultISPyB
from XSDataISPyBv10                 import XSDatadbstatus

class EDTestCasePluginUnitISPyBv10( EDTestCasePluginUnit ):
     
    def __init__( self, _edStringTestName = None):
       """
       Set up paths, reference files etc.
       """
       EDTestCasePluginUnit.__init__( self, "EDPluginISPyBv10", "EDPluginGroupISPyB-v0.1", _edStringTestName )

       self.m_edStrDataPath = self.getPluginTestsDataHome()
       self.m_edObtainedInputFile = "XSDataInputISPyBv10.xml"
       self.m_edReferenceInputFile = EDDiskExplorer.mergePath( self.m_edStrDataPath, "XSDataISPyBv10Input_reference.xml")

    def preProcess( self ):
        """
        Delete old obtained files.
        """
        EDTestCasePluginUnit.preProcess( self )
        EDDiskExplorer.deleteFile( self.m_edObtainedInputFile )

    def testConfigureOK( self ):
        """
        A test for the configure method of the plugin
        """
        edPluginISPyB = self.createPlugin()
        xsPluginItemISPyB = self.getPluginConfiguration( EDDiskExplorer.mergePath( self.getPluginTestsDataHome(), "XSConfiguration.xml" ))
        edPluginISPyB.setConfiguration( xsPluginItemISPyB )
        edPluginISPyB.configure()
        EDAssert.equal("localhost", edPluginISPyB.getDbserverHost())        
        EDAssert.equal(9090, edPluginISPyB.getDbserverPort())        
        self.cleanUp( edPluginISPyB )        

    def testSetDataModelInput( self ):
        """
        A test for whether we can obtain the expected XML by setting a certain input for the plugin. 
        """
        edPluginISPyB = self.createPlugin()
        xsPluginItemISPyB = self.getPluginConfiguration( EDDiskExplorer.mergePath( self.getPluginTestsDataHome(), "XSConfiguration.xml" ))
        edPluginISPyB.setConfiguration( xsPluginItemISPyB )
        edPluginISPyB.configure()
        
        xsDataInputISPyB = XSDataInputISPyB()
 
        xsDataScreening = XSDataScreening()
        xsDataScreening.setDataCollectionId( XSDataInteger ( 1 ) )
        xsDataScreening.setProgramVersion( XSDataString( "EDNA Prototype" ) )
 
        xsDataScreeningInput = XSDataScreeningInput() 
        xsDataScreeningInput.setBeamX( XSDataFloat( 10.4 ) )
        xsDataScreeningInput.setBeamY( XSDataFloat( 2.31 ) )
        xsDataScreeningInput.setRmsErrorLimits( XSDataFloat( 0.8 ) )        
        xsDataScreeningInput.setMinimumFractionIndexed( XSDataFloat( 0.4 ) )
        xsDataScreeningInput.setMaximumFractionRejected( XSDataFloat( 0.45 ) )
        xsDataScreeningInput.setMinimumSignalToNoise( XSDataFloat( 0.56 ) )
        
        xsDataScreeningOutput = XSDataScreeningOutput() 
        xsDataScreeningOutput.setStatusDescription( XSDataString( "It's just fine." ) )
        xsDataScreeningOutput.setMosaicity( XSDataFloat( 0.25 ) )
        xsDataScreeningOutput.setBeamShiftX( XSDataFloat ( 0.141 ) )
        xsDataScreeningOutput.setBeamShiftY( XSDataFloat ( 0.156 ) )
 
        xsDataScreeningOutputLattice = XSDataScreeningOutputLattice() 
        xsDataScreeningOutputLattice.setSpaceGroup( XSDataString( "P222" ) )
        
        xsDataScreeningStrategy = XSDataScreeningStrategy() 
        xsDataScreeningStrategy.setPhiStart( XSDataFloat( 0 ) )
        xsDataScreeningStrategy.setPhiEnd( XSDataFloat( 20 ) )
        xsDataScreeningStrategy.setRotation( XSDataFloat( 1 ) )
        xsDataScreeningStrategy.setProgram( XSDataString( "EDNA" ) )
        xsDataScreeningStrategy.setAnomalous( XSDataBoolean( 1 ) )
        
        xsDataInputISPyB.setScreening(xsDataScreening)
        xsDataInputISPyB.setScreeningInput(xsDataScreeningInput)
        xsDataInputISPyB.setScreeningOutput(xsDataScreeningOutput)
        xsDataInputISPyB.setScreeningOutputLattice(xsDataScreeningOutputLattice)
        xsDataInputISPyB.setScreeningStrategy(xsDataScreeningStrategy)
        
        xsDataInputISPyB.outputFile( self.m_edObtainedInputFile )

        strExpectedInput = EDUtilsTest.readAndParseFile ( self.m_edReferenceInputFile )
        strObtainedInput = EDUtilsTest.readAndParseFile ( self.m_edObtainedInputFile )  
        xsDataScreeningExpected = XSDataInputISPyB.parseString( strExpectedInput )
        xsDataScreeningObtained = XSDataInputISPyB.parseString( strObtainedInput )
        pyStrExpectedXML = xsDataScreeningExpected.marshal()
        pyStrObtainedXML = xsDataScreeningObtained.marshal()
        EDAssert.equal( pyStrExpectedXML, pyStrObtainedXML )

        self.cleanUp( edPluginISPyB )

    def testHttpPost( self ):
        """
        A test for whether the httpPost method can send a request to the dbserver and receive a response
        """
        edPluginISPyB = self.createPlugin()
        xsPluginItemISPyB = self.getPluginConfiguration( EDDiskExplorer.mergePath( self.getPluginTestsDataHome(), "XSConfiguration.xml" ))
        edPluginISPyB.setConfiguration( xsPluginItemISPyB )
        edPluginISPyB.configure()
        
        edStringXML = EDString( '<?xml version="1.0" encoding="UTF-8"?><Proposal><code>nt</code><number>20</number></Proposal>' )
        edStrResponse = edPluginISPyB.httpPost( edPluginISPyB.getDbserverHost(), edPluginISPyB.getDbserverPort(), EDString('/proposal_request'), edStringXML )
        if edStrResponse != None:
            EDVerbose.DEBUG( "*** EDTestCasePluginUnitISPyBv10.testHttpPost response: " + edStrResponse )

        EDAssert.equal( edStrResponse!=None, True )
        
    def process( self ):
       """
       List of test methods
       """
       EDVerbose.DEBUG( "*** EDTestCasePluginUnitISPyBv10.process" )
       self.addTestMethod( self.testConfigureOK )
       self.addTestMethod( self.testSetDataModelInput )
       #self.addTestMethod( self.testHttpPost )

if __name__ == '__main__':
    # JIT compiler accelerator
    EDCompiler.accelerator()
    edTestCasePluginUnitISPyBv10 = EDTestCasePluginUnitISPyBv10( "EDTestCasePluginUnitISPyBv10" )
    edTestCasePluginUnitISPyBv10.execute()
