#
#    Project: EDNA MXv1
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCaseEDHandlerMOSFLMv10.py 713 2009-05-15 11:32:21Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors:      Marie-Francoise Incardona (incardon@esrf.fr)
#                            Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImportLib import EDCompiler
from EDImportLib import EDString
from EDImportLib import EDVerbose
from EDImportLib import EDDiskExplorer
from EDImportLib import EDList

from EDAssert      import EDAssert
from EDTestCase    import EDTestCase
from EDUtilsTest   import EDUtilsTest
from EDUtilsPath   import EDUtilsPath
from EDApplication import EDApplication

from XSDataCommon import XSDataAngle
from XSDataCommon import XSDataLength
from XSDataCommon import XSDataInteger
from XSDataCommon import XSDataString
from XSDataCommon import XSDataFloat





class EDTestCaseEDHandlerMOSFLMv10( EDTestCase ):
    """
    """

    def __init__( self, _edStringTestName = None):
        """
        """
        EDTestCase.__init__( self, _edStringTestName )        
        edStringMXv1DataHome = EDUtilsTest.getPluginTestDataDirectory( self.getClassName() )
        edStringDataDir = "EDHandlerMOSFLMv10"
        self.m_edStringDataPath = EDUtilsPath.mergePath( edStringMXv1DataHome, edStringDataDir )

    
    def testGenerateXSDataMOSFLMInputIndexing( self ):
        edStringFilename = "XSDataIndexingInput_reference.xml"
        edStringPath = EDDiskExplorer.mergePath( self.m_edStringDataPath,  edStringFilename )
        edStringXMLIndexingInput = EDUtilsTest.readAndParseFile( edStringPath )
        from XSDataMXv1 import XSDataIndexingInput
        xsDataIndexingInput = XSDataIndexingInput.parseString( edStringXMLIndexingInput  )
        from EDHandlerXSDataMOSFLMv10 import EDHandlerXSDataMOSFLMv10
        xsDataMOSFLMInputIndexing = EDHandlerXSDataMOSFLMv10.generateXSDataMOSFLMInputIndexing( xsDataIndexingInput )
        xsDataMOSFLMInputIndexing.outputFile( "XSDataMOSFLMInputIndexing_reference.xml" )                
        edStringReferenceFilename = "XSDataMOSFLMInputIndexing_reference.xml"
        edStringReferencePath = EDDiskExplorer.mergePath( self.m_edStringDataPath,  edStringReferenceFilename )
        edStringXMLMOSFLMInputIndexingReference = EDUtilsTest.readAndParseFile( edStringReferencePath )
        EDApplication.loadModule( "XSDataMOSFLMv10" )
        from XSDataMOSFLMv10 import XSDataMOSFLMInputIndexing
        xsDataMOSFLMInputIndexingReference = XSDataMOSFLMInputIndexing.parseString( edStringXMLMOSFLMInputIndexingReference )
        EDAssert.equal( xsDataMOSFLMInputIndexingReference.marshal(), xsDataMOSFLMInputIndexing.marshal() )


    def testGenerateXSDataIndexingResult( self ):
        edStringFilename = "XSDataMOSFLMOutputIndexing_reference.xml"
        edStringPath = EDDiskExplorer.mergePath( self.m_edStringDataPath,  edStringFilename )
        edStringXMLMOSFLMOutputIndexing = EDUtilsTest.readAndParseFile( edStringPath )
        from XSDataMOSFLMv10 import XSDataMOSFLMOutputIndexing
        xsDataMOSFLMOutputIndexing = XSDataMOSFLMOutputIndexing.parseString(edStringXMLMOSFLMOutputIndexing  )

        from EDHandlerXSDataMOSFLMv10 import EDHandlerXSDataMOSFLMv10
        xsDataIndexingResult = EDHandlerXSDataMOSFLMv10.generateXSDataIndexingResult( xsDataMOSFLMOutputIndexing )
        edStringReferenceFilename = "XSDataIndexingResult_reference.xml"
        edStringReferencePath = EDDiskExplorer.mergePath( self.m_edStringDataPath,  edStringReferenceFilename )
        edStringXMLIndexingOutputReference = EDUtilsTest.readAndParseFile( edStringReferencePath )
        from XSDataMXv1 import  XSDataIndexingResult
        xsdataIndexingResultReference = XSDataIndexingResult.parseString( edStringXMLIndexingOutputReference )
        EDAssert.equal( xsdataIndexingResultReference.marshal(), xsDataIndexingResult.marshal() )

        
    def testGenerateXSDataMOSFLMInputIntegration( self ):
        edStringFilename = "XSDataIntegrationInput_reference.xml"
        edStringPath = EDDiskExplorer.mergePath( self.m_edStringDataPath,  edStringFilename )
        edStringXMLIntegrationInput = EDUtilsTest.readAndParseFile( edStringPath )
        from XSDataMXv1 import XSDataIntegrationInput
        xsDataIntegrationInput = XSDataIntegrationInput.parseString( edStringXMLIntegrationInput  )

        from EDHandlerXSDataMOSFLMv10 import EDHandlerXSDataMOSFLMv10
        xsDataMOSFLMInputIntegration = EDHandlerXSDataMOSFLMv10.generateXSDataMOSFLMInputIntegration( xsDataIntegrationInput )
        edStringReferenceFilename = "XSDataMOSFLMInputIntegration_reference.xml"
        edStringReferencePath = EDDiskExplorer.mergePath( self.m_edStringDataPath,  edStringReferenceFilename )
        edStringXMLIntegrationInputReference = EDUtilsTest.readAndParseFile( edStringReferencePath )
        EDApplication.loadModule( "XSDataMOSFLMv10" )
        from XSDataMOSFLMv10 import XSDataMOSFLMInputIntegration
        xsDataMOSFLMInputIntegrationReference = XSDataMOSFLMInputIntegration.parseString( edStringXMLIntegrationInputReference )
        EDAssert.equal( xsDataMOSFLMInputIntegrationReference.marshal(), xsDataMOSFLMInputIntegration.marshal() )

    
    def testGenerateXSDataIntegrationSubWedgeResult( self ):
        edStringFilename = "XSDataMOSFLMOutputIntegration_reference.xml"
        edStringPath = EDDiskExplorer.mergePath( self.m_edStringDataPath,  edStringFilename )
        edStringXMLMOSFLMOutputIntegration = EDUtilsTest.readAndParseFile( edStringPath )
        from XSDataMOSFLMv10 import XSDataMOSFLMOutputIntegration
        xsDataMOSFLMOutputIntegration = XSDataMOSFLMOutputIntegration.parseString( edStringXMLMOSFLMOutputIntegration  )

        from EDHandlerXSDataMOSFLMv10 import EDHandlerXSDataMOSFLMv10
        xsDataIntegrationSubWedgeResult = EDHandlerXSDataMOSFLMv10.generateXSDataIntegrationSubWedgeResult( xsDataMOSFLMOutputIntegration )
                
        edStringReferenceFilename = "XSDataIntegrationSubWedgeResult_reference.xml"
        edStringReferencePath = EDDiskExplorer.mergePath( self.m_edStringDataPath,  edStringReferenceFilename )
        #xsDataIntegrationSubWedgeResult.outputFile( edStringReferenceFilename )
        #xsDataIntegrationSubWedgeResult.outputFile( edStringReferencePath )
        edStringXMLIntegrationSubWedgeReference = EDUtilsTest.readAndParseFile( edStringReferencePath )
        #EDAssert.equal( edStringXMLIntegrationSubWedgeReference, xsDataIntegrationSubWedgeResult.marshal() )

    
    
    def process( self ):
        self.addTestMethod( self.testGenerateXSDataMOSFLMInputIndexing )
        self.addTestMethod( self.testGenerateXSDataIndexingResult )
        self.addTestMethod( self.testGenerateXSDataMOSFLMInputIntegration )
        self.addTestMethod( self.testGenerateXSDataIntegrationSubWedgeResult )

    

##############################################################################


if __name__ == '__main__':

    # JIT compiler accelerator
    EDCompiler.accelerator()

    edTestCaseEDHandlerMOSFLMv10 = EDTestCaseEDHandlerMOSFLMv10( "EDTestCaseEDHandlerMOSFLMv10" )
    edTestCaseEDHandlerMOSFLMv10.execute()
